@@ -17,15 +17,10 @@
 
 package org.apache.kafka.clients.admin;
 
-import org.apache.kafka.common.annotation.InterfaceStability;
-
 import java.util.List;
 
 /**
  * Options for {@link AdminClient#alterUserScramCredentials(List, AlterUserScramCredentialsOptions)}
- *
- * The API of this class is evolving. See {@link AdminClient} for details.
  */
-@InterfaceStability.Evolving
 public class AlterUserScramCredentialsOptions extends AbstractOptions<AlterUserScramCredentialsOptions> {
 }
