@@ -18,6 +18,7 @@
 package org.apache.kafka.clients.admin;
 
 import org.apache.kafka.common.ConsumerGroupState;
+import org.apache.kafka.common.GroupState;
 import org.apache.kafka.common.GroupType;
 
 import java.util.Objects;
@@ -29,28 +30,30 @@ import java.util.Optional;
 public class ConsumerGroupListing {
     private final String groupId;
     private final boolean isSimpleConsumerGroup;
-    private final Optional<ConsumerGroupState> state;
+    private final Optional<GroupState> groupState;
     private final Optional<GroupType> type;
 
     /**
      * Create an instance with the specified parameters.
      *
-     * @param groupId Group Id
-     * @param isSimpleConsumerGroup If consumer group is simple or not.
+     * @param groupId                   Group Id.
+     * @param isSimpleConsumerGroup     If consumer group is simple or not.
      */
     public ConsumerGroupListing(String groupId, boolean isSimpleConsumerGroup) {
-        this(groupId, isSimpleConsumerGroup, Optional.empty(), Optional.empty());
+        this(groupId, Optional.empty(), Optional.empty(), isSimpleConsumerGroup);
     }
 
     /**
      * Create an instance with the specified parameters.
      *
-     * @param groupId Group Id
-     * @param isSimpleConsumerGroup If consumer group is simple or not.
-     * @param state The state of the consumer group
+     * @param groupId                   Group Id.
+     * @param isSimpleConsumerGroup     If consumer group is simple or not.
+     * @param state                     The state of the consumer group.
+     * @deprecated Since 4.0. Use {@link #ConsumerGroupListing(String, Optional, boolean)} instead.
      */
+    @Deprecated
     public ConsumerGroupListing(String groupId, boolean isSimpleConsumerGroup, Optional<ConsumerGroupState> state) {
-        this(groupId, isSimpleConsumerGroup, state, Optional.empty());
+        this(groupId, Objects.requireNonNull(state).map(state0 -> GroupState.parse(state0.toString())), Optional.empty(), isSimpleConsumerGroup);
     }
 
     /**
@@ -60,17 +63,51 @@ public class ConsumerGroupListing {
      * @param isSimpleConsumerGroup     If consumer group is simple or not.
      * @param state                     The state of the consumer group.
      * @param type                      The type of the consumer group.
+     * @deprecated Since 4.0. Use {@link #ConsumerGroupListing(String, Optional, Optional, boolean)} instead.
      */
+    @Deprecated
     public ConsumerGroupListing(
         String groupId,
         boolean isSimpleConsumerGroup,
         Optional<ConsumerGroupState> state,
         Optional<GroupType> type
+    ) {
+        this(groupId, Objects.requireNonNull(state).map(state0 -> GroupState.parse(state0.toString())), type, isSimpleConsumerGroup);
+    }
+
+    /**
+     * Create an instance with the specified parameters.
+     *
+     * @param groupId                   Group Id.
+     * @param groupState                The state of the consumer group.
+     * @param isSimpleConsumerGroup     If consumer group is simple or not.
+     */
+    public ConsumerGroupListing(
+            String groupId,
+            Optional<GroupState> groupState,
+            boolean isSimpleConsumerGroup
+    ) {
+        this(groupId, groupState, Optional.empty(), isSimpleConsumerGroup);
+    }
+
+    /**
+     * Create an instance with the specified parameters.
+     *
+     * @param groupId                   Group Id.
+     * @param groupState                The state of the consumer group.
+     * @param type                      The type of the consumer group.
+     * @param isSimpleConsumerGroup     If consumer group is simple or not.
+     */
+    public ConsumerGroupListing(
+            String groupId,
+            Optional<GroupState> groupState,
+            Optional<GroupType> type,
+            boolean isSimpleConsumerGroup
     ) {
         this.groupId = groupId;
-        this.isSimpleConsumerGroup = isSimpleConsumerGroup;
-        this.state = Objects.requireNonNull(state);
+        this.groupState = Objects.requireNonNull(groupState);
         this.type = Objects.requireNonNull(type);
+        this.isSimpleConsumerGroup = isSimpleConsumerGroup;
     }
 
     /**
@@ -88,10 +125,19 @@ public class ConsumerGroupListing {
     }
 
     /**
-     * Consumer Group state
+     * Group state
      */
+    public Optional<GroupState> groupState() {
+        return groupState;
+    }
+
+    /**
+     * Consumer Group state
+     * @deprecated Since 4.0. Use {@link #groupState()} instead.
+     */
+    @Deprecated
     public Optional<ConsumerGroupState> state() {
-        return state;
+        return groupState.map(state0 -> ConsumerGroupState.parse(state0.toString()));
     }
 
     /**
@@ -108,14 +154,14 @@ public class ConsumerGroupListing {
         return "(" +
             "groupId='" + groupId + '\'' +
             ", isSimpleConsumerGroup=" + isSimpleConsumerGroup +
-            ", state=" + state +
+            ", groupState=" + groupState +
             ", type=" + type +
             ')';
     }
 
     @Override
     public int hashCode() {
-        return Objects.hash(groupId, isSimpleConsumerGroup(), state, type);
+        return Objects.hash(groupId, isSimpleConsumerGroup(), groupState, type);
     }
 
     @Override
@@ -125,7 +171,7 @@ public class ConsumerGroupListing {
         ConsumerGroupListing that = (ConsumerGroupListing) o;
         return isSimpleConsumerGroup() == that.isSimpleConsumerGroup() &&
             Objects.equals(groupId, that.groupId) &&
-            Objects.equals(state, that.state) &&
+            Objects.equals(groupState, that.groupState) &&
             Objects.equals(type, that.type);
     }
 }
