@@ -20,17 +20,13 @@ package org.apache.kafka.clients.admin;
 import org.apache.kafka.common.KafkaFuture;
 import org.apache.kafka.common.TopicCollection;
 import org.apache.kafka.common.Uuid;
-import org.apache.kafka.common.annotation.InterfaceStability;
 
 import java.util.Collection;
 import java.util.Map;
 
 /**
  * The result of the {@link Admin#deleteTopics(Collection)} call.
- *
- * The API of this class is evolving, see {@link Admin} for details.
  */
-@InterfaceStability.Evolving
 public class DeleteTopicsResult {
     private final Map<Uuid, KafkaFuture<Void>> topicIdFutures;
     private final Map<String, KafkaFuture<Void>> nameFutures;
@@ -70,16 +66,6 @@ public class DeleteTopicsResult {
         return nameFutures;
     }
 
-    /**
-     * @return a map from topic names to futures which can be used to check the status of
-     * individual deletions if the deleteTopics request used topic names. Otherwise return null.
-     * @deprecated Since 3.0 use {@link #topicNameValues} instead
-     */
-    @Deprecated
-    public Map<String, KafkaFuture<Void>> values() {
-        return nameFutures;
-    }
-
     /**
      * @return a future which succeeds only if all the topic deletions succeed.
      */
