@@ -17,18 +17,20 @@
 
 package org.apache.kafka.clients.admin;
 
-import org.apache.kafka.common.annotation.InterfaceStability;
-
 /**
  * Options for {@link Admin#expireDelegationToken(byte[], ExpireDelegationTokenOptions)}.
- *
- * The API of this class is evolving, see {@link Admin} for details.
  */
-@InterfaceStability.Evolving
 public class ExpireDelegationTokenOptions extends AbstractOptions<ExpireDelegationTokenOptions> {
     private long expiryTimePeriodMs = -1L;
 
-    public ExpireDelegationTokenOptions expiryTimePeriodMs(long expiryTimePeriodMs) {
+    /**
+     * @param expiryTimePeriodMs the time period until we should expire this token.
+     * {@code expiryTimePeriodMs} &gt;= 0: the token will update the expiration timestamp to min(now + expiryTimePeriodMs, maxTimestamp)
+     * {@code expiryTimePeriodMs} &lt; 0: token will be expired immediately.
+     */
+    public ExpireDelegationTokenOptions expiryTimePeriodMs(
+        long expiryTimePeriodMs
+    ) {
         this.expiryTimePeriodMs = expiryTimePeriodMs;
         return this;
     }
