@@ -80,9 +80,6 @@ public class BufferPool {
         MetricName rateMetricName = metrics.metricName("bufferpool-wait-ratio",
                                                    metricGrpName,
                                                    "The fraction of time an appender waits for space allocation.");
-        MetricName totalMetricName = metrics.metricName("bufferpool-wait-time-total",
-                                                   metricGrpName,
-                                                   "*Deprecated* The total time an appender waits for space allocation.");
         MetricName totalNsMetricName = metrics.metricName("bufferpool-wait-time-ns-total",
                                                     metricGrpName,
                                                     "The total time in nanoseconds an appender waits for space allocation.");
@@ -92,7 +89,6 @@ public class BufferPool {
         MetricName bufferExhaustedTotalMetricName = metrics.metricName("buffer-exhausted-total", metricGrpName, "The total number of record sends that are dropped due to buffer exhaustion");
         bufferExhaustedRecordSensor.add(new Meter(bufferExhaustedRateMetricName, bufferExhaustedTotalMetricName));
 
-        this.waitTime.add(new Meter(TimeUnit.NANOSECONDS, rateMetricName, totalMetricName));
         this.waitTime.add(new Meter(TimeUnit.NANOSECONDS, rateMetricName, totalNsMetricName));
         this.closed = false;
     }
