@@ -72,9 +72,6 @@ public final class ProducerBatch {
     private final AtomicInteger attempts = new AtomicInteger(0);
     private final boolean isSplitBatch;
     private final AtomicReference<FinalState> finalState = new AtomicReference<>(null);
-    private boolean bufferDeallocated = false;
-    // Tracks if the batch has been sent to the NetworkClient
-    private boolean inflight = false;
 
     int recordCount;
     int maxRecordSize;
@@ -115,7 +112,7 @@ public final class ProducerBatch {
      */
     void maybeUpdateLeaderEpoch(OptionalInt latestLeaderEpoch) {
         if (latestLeaderEpoch.isPresent()
-            && (!currentLeaderEpoch.isPresent() || currentLeaderEpoch.getAsInt() < latestLeaderEpoch.getAsInt())) {
+            && (currentLeaderEpoch.isEmpty() || currentLeaderEpoch.getAsInt() < latestLeaderEpoch.getAsInt())) {
             log.trace("For {}, leader will be updated, currentLeaderEpoch: {}, attemptsWhenLeaderLastChanged:{}, latestLeaderEpoch: {}, current attempt: {}",
                 this, currentLeaderEpoch, attemptsWhenLeaderLastChanged, latestLeaderEpoch, attempts);
             attemptsWhenLeaderLastChanged = attempts();
@@ -324,16 +321,10 @@ public final class ProducerBatch {
     }
 
     public Deque<ProducerBatch> split(int splitBatchSize) {
-        RecordBatch recordBatch = validateAndGetRecordBatch();
-        Deque<ProducerBatch> batches = splitRecordsIntoBatches(recordBatch, splitBatchSize);
-        finalizeSplitBatches(batches);
-        return batches;
-    }
-
-    private RecordBatch validateAndGetRecordBatch() {
+        Deque<ProducerBatch> batches = new ArrayDeque<>();
         MemoryRecords memoryRecords = recordsBuilder.build();
-        Iterator<MutableRecordBatch> recordBatchIter = memoryRecords.batches().iterator();
 
+        Iterator<MutableRecordBatch> recordBatchIter = memoryRecords.batches().iterator();
         if (!recordBatchIter.hasNext())
             throw new IllegalStateException("Cannot split an empty producer batch.");
 
@@ -345,11 +336,6 @@ public final class ProducerBatch {
         if (recordBatchIter.hasNext())
             throw new IllegalArgumentException("A producer batch should only have one record batch.");
 
-        return recordBatch;
-    }
-
-    private Deque<ProducerBatch> splitRecordsIntoBatches(RecordBatch recordBatch, int splitBatchSize) {
-        Deque<ProducerBatch> batches = new ArrayDeque<>();
         Iterator<Thunk> thunkIter = thunks.iterator();
         // We always allocate batch size because we are already splitting a big batch.
         // And we also Retain the create time of the original batch.
@@ -376,23 +362,9 @@ public final class ProducerBatch {
             batch.closeForRecordAppends();
         }
 
-        return batches;
-    }
-
-    private void finalizeSplitBatches(Deque<ProducerBatch> batches) {
-        // Chain all split batch ProduceRequestResults to the original batch's produceFuture
-        // Ensures the original batch's future doesn't complete until all split batches complete
-        for (ProducerBatch splitBatch : batches) {
-            produceFuture.addDependent(splitBatch.produceFuture);
-        }
-
         produceFuture.set(ProduceResponse.INVALID_OFFSET, NO_TIMESTAMP, index -> new RecordBatchTooLargeException());
         produceFuture.done();
 
-        assignProducerStateToBatches(batches);
-    }
-
-    private void assignProducerStateToBatches(Deque<ProducerBatch> batches) {
         if (hasSequence()) {
             int sequence = baseSequence();
             ProducerIdAndEpoch producerIdAndEpoch = new ProducerIdAndEpoch(producerId(), producerEpoch());
@@ -401,6 +373,7 @@ public final class ProducerBatch {
                 sequence += newBatch.recordCount;
             }
         }
+        return batches;
     }
 
     private ProducerBatch createBatchOffAccumulatorForRecord(Record record, int batchSize) {
@@ -584,22 +557,6 @@ public final class ProducerBatch {
         return reopened;
     }
 
-    public boolean isBufferDeallocated() {
-        return bufferDeallocated;
-    }
-
-    public void markBufferDeallocated() {
-        bufferDeallocated = true;
-    }
-
-    public boolean isInflight() {
-        return inflight;
-    }
-
-    public void setInflight(boolean inflight) {
-        this.inflight = inflight;
-    }
-
     // VisibleForTesting
     OptionalInt currentLeaderEpoch() {
         return currentLeaderEpoch;
