@@ -27,7 +27,6 @@ import org.apache.kafka.common.Node;
 import org.apache.kafka.common.PartitionInfo;
 import org.apache.kafka.common.TopicPartition;
 import org.apache.kafka.common.compress.Compression;
-import org.apache.kafka.common.errors.UnsupportedVersionException;
 import org.apache.kafka.common.header.Header;
 import org.apache.kafka.common.metrics.Metrics;
 import org.apache.kafka.common.record.AbstractRecords;
@@ -278,8 +277,6 @@ public class RecordAccumulator {
      * @param headers the Headers for the record
      * @param callbacks The callbacks to execute
      * @param maxTimeToBlock The maximum time in milliseconds to block for buffer memory to be available
-     * @param abortOnNewBatch A boolean that indicates returning before a new batch is created and
-     *                        running the partitioner's onNewBatch method before trying to append again
      * @param nowMs The current time, in milliseconds
      * @param cluster The cluster metadata
      */
@@ -291,7 +288,6 @@ public class RecordAccumulator {
                                      Header[] headers,
                                      AppendCallbacks callbacks,
                                      long maxTimeToBlock,
-                                     boolean abortOnNewBatch,
                                      long nowMs,
                                      Cluster cluster) throws InterruptedException {
         TopicInfo topicInfo = topicInfoMap.computeIfAbsent(topic, k -> new TopicInfo(createBuiltInPartitioner(logContext, k, batchSize)));
@@ -337,15 +333,9 @@ public class RecordAccumulator {
                     }
                 }
 
-                // we don't have an in-progress record batch try to allocate a new batch
-                if (abortOnNewBatch) {
-                    // Return a result that will cause another call to append.
-                    return new RecordAppendResult(null, false, false, true, 0);
-                }
-
                 if (buffer == null) {
-                    byte maxUsableMagic = apiVersions.maxUsableProduceMagic();
-                    int size = Math.max(this.batchSize, AbstractRecords.estimateSizeInBytesUpperBound(maxUsableMagic, compression.type(), key, value, headers));
+                    int size = Math.max(this.batchSize, AbstractRecords.estimateSizeInBytesUpperBound(
+                            RecordBatch.CURRENT_MAGIC_VALUE, compression.type(), key, value, headers));
                     log.trace("Allocating a new {} byte message buffer for topic {} partition {} with remaining timeout {}ms", size, topic, effectivePartition, maxTimeToBlock);
                     // This call may block if we exhausted buffer space.
                     buffer = free.allocate(size, maxTimeToBlock);
@@ -408,7 +398,7 @@ public class RecordAccumulator {
             return appendResult;
         }
 
-        MemoryRecordsBuilder recordsBuilder = recordsBuilder(buffer, apiVersions.maxUsableProduceMagic());
+        MemoryRecordsBuilder recordsBuilder = recordsBuilder(buffer);
         ProducerBatch batch = new ProducerBatch(new TopicPartition(topic, partition), recordsBuilder, nowMs);
         FutureRecordMetadata future = Objects.requireNonNull(batch.tryAppend(timestamp, key, value, headers,
                 callbacks, nowMs));
@@ -416,15 +406,11 @@ public class RecordAccumulator {
         dq.addLast(batch);
         incomplete.add(batch);
 
-        return new RecordAppendResult(future, dq.size() > 1 || batch.isFull(), true, false, batch.estimatedSizeInBytes());
+        return new RecordAppendResult(future, dq.size() > 1 || batch.isFull(), true, batch.estimatedSizeInBytes());
     }
 
-    private MemoryRecordsBuilder recordsBuilder(ByteBuffer buffer, byte maxUsableMagic) {
-        if (transactionManager != null && maxUsableMagic < RecordBatch.MAGIC_VALUE_V2) {
-            throw new UnsupportedVersionException("Attempting to use idempotence with a broker which does not " +
-                "support the required message format (v2). The broker must be version 0.11 or later.");
-        }
-        return MemoryRecords.builder(buffer, maxUsableMagic, compression, TimestampType.CREATE_TIME, 0L);
+    private MemoryRecordsBuilder recordsBuilder(ByteBuffer buffer) {
+        return MemoryRecords.builder(buffer, RecordBatch.CURRENT_MAGIC_VALUE, compression, TimestampType.CREATE_TIME, 0L);
     }
 
     /**
@@ -456,7 +442,7 @@ public class RecordAccumulator {
                 last.closeForRecordAppends();
             } else {
                 int appendedBytes = last.estimatedSizeInBytes() - initialBytes;
-                return new RecordAppendResult(future, deque.size() > 1 || last.isFull(), false, false, appendedBytes);
+                return new RecordAppendResult(future, deque.size() > 1 || last.isFull(), false, appendedBytes);
             }
         }
         return null;
@@ -1044,39 +1030,14 @@ public class RecordAccumulator {
     }
 
     /**
-     * Complete and deallocate the record batch
-     */
-    public void completeAndDeallocateBatch(ProducerBatch batch) {
-        completeBatch(batch);
-        deallocate(batch);
-    }
-
-    /**
-     * Only perform deallocation (and not removal from the incomplete set)
+     * Deallocate the record batch
      */
     public void deallocate(ProducerBatch batch) {
+        incomplete.remove(batch);
         // Only deallocate the batch if it is not a split batch because split batch are allocated outside the
         // buffer pool.
-        if (!batch.isSplitBatch()) {
-            if (batch.isBufferDeallocated()) {
-                log.warn("Skipping deallocating a batch that has already been deallocated. Batch is {}, created time is {}", batch, batch.createdMs);
-            } else {
-                batch.markBufferDeallocated();
-                if (batch.isInflight()) {
-                    // Create a fresh ByteBuffer to give to BufferPool to reuse since we can't safely call deallocate with the ProduceBatch's buffer
-                    free.deallocate(ByteBuffer.allocate(batch.initialCapacity()));
-                    throw new IllegalStateException("Attempting to deallocate a batch that is inflight. Batch is " + batch);
-                }
-                free.deallocate(batch.buffer(), batch.initialCapacity());
-            }
-        }
-    }
-
-    /**
-     * Remove from the incomplete list but do not free memory yet
-     */
-    public void completeBatch(ProducerBatch batch) {
-        incomplete.remove(batch);
+        if (!batch.isSplitBatch())
+            free.deallocate(batch.buffer(), batch.initialCapacity());
     }
 
     /**
@@ -1118,13 +1079,8 @@ public class RecordAccumulator {
             // We must be careful not to hold a reference to the ProduceBatch(s) so that garbage
             // collection can occur on the contents.
             // The sender will remove ProducerBatch(s) from the original incomplete collection.
-            //
-            // We use awaitAllDependents() here instead of await() to ensure that if any batch
-            // was split into multiple batches, we wait for all the split batches to complete.
-            // This is required to guarantee that all records sent before flush()
-            // must be fully complete, including records in split batches.
             for (ProduceRequestResult result : this.incomplete.requestResults())
-                result.awaitAllDependents();
+                result.await();
         } finally {
             this.flushesInProgress.decrementAndGet();
         }
@@ -1174,14 +1130,7 @@ public class RecordAccumulator {
                 dq.remove(batch);
             }
             batch.abort(reason);
-            if (batch.isInflight()) {
-                // KAFKA-19012: if the batch has been sent it might still be in use by the network client so we cannot allow it to be reused yet.
-                // We skip deallocating it now. When the request in network client completes with a response, either Sender.completeBatch() or
-                // Sender.failBatch() will be called with deallocateBatch=true. The buffer associated with the batch will be deallocated then.
-                completeBatch(batch);
-            } else {
-                completeAndDeallocateBatch(batch);
-            }
+            deallocate(batch);
         }
     }
 
@@ -1201,7 +1150,7 @@ public class RecordAccumulator {
             }
             if (aborted) {
                 batch.abort(reason);
-                completeAndDeallocateBatch(batch);
+                deallocate(batch);
             }
         }
     }
@@ -1255,18 +1204,15 @@ public class RecordAccumulator {
         public final FutureRecordMetadata future;
         public final boolean batchIsFull;
         public final boolean newBatchCreated;
-        public final boolean abortForNewBatch;
         public final int appendedBytes;
 
         public RecordAppendResult(FutureRecordMetadata future,
                                   boolean batchIsFull,
                                   boolean newBatchCreated,
-                                  boolean abortForNewBatch,
                                   int appendedBytes) {
             this.future = future;
             this.batchIsFull = batchIsFull;
             this.newBatchCreated = newBatchCreated;
-            this.abortForNewBatch = abortForNewBatch;
             this.appendedBytes = appendedBytes;
         }
     }
