@@ -25,7 +25,6 @@ import org.apache.kafka.common.security.auth.PlaintextAuthenticationContext;
 import org.apache.kafka.common.utils.Utils;
 
 import java.io.Closeable;
-import java.io.IOException;
 import java.net.InetAddress;
 import java.nio.channels.SelectionKey;
 import java.util.Map;
@@ -72,7 +71,7 @@ public class PlaintextChannelBuilder implements ChannelBuilder {
         return new KafkaChannel(id, transportLayer, authenticatorCreator, maxReceiveSize, memoryPool, metadataRegistry);
     }
 
-    protected PlaintextTransportLayer buildTransportLayer(SelectionKey key) throws IOException {
+    protected PlaintextTransportLayer buildTransportLayer(SelectionKey key) {
         return new PlaintextTransportLayer(key);
     }
 
