@@ -17,18 +17,12 @@
 package org.apache.kafka.common.network;
 
 import java.io.IOException;
-import java.nio.channels.GatheringByteChannel;
 
 /**
- * This interface models the in-progress sending of data to a specific destination
+ * This interface models the in-progress sending of data.
  */
 public interface Send {
 
-    /**
-     * The id for the destination of this send
-     */
-    String destination();
-
     /**
      * Is this send complete?
      */
@@ -41,7 +35,7 @@ public interface Send {
      * @return The number of bytes written
      * @throws IOException If the write fails
      */
-    long writeTo(GatheringByteChannel channel) throws IOException;
+    long writeTo(TransferableChannel channel) throws IOException;
 
     /**
      * Size of the send
