@@ -31,7 +31,6 @@ import org.apache.kafka.common.message.ApiVersionsResponseData.SupportedFeatureK
 import org.apache.kafka.common.protocol.ApiKeys;
 import org.apache.kafka.common.protocol.ByteBufferAccessor;
 import org.apache.kafka.common.protocol.Errors;
-import org.apache.kafka.common.record.RecordVersion;
 
 import java.nio.ByteBuffer;
 import java.util.Map;
@@ -172,7 +171,6 @@ public class ApiVersionsResponse extends AbstractResponse {
     }
 
     public static ApiVersionCollection controllerApiVersions(
-        RecordVersion minRecordVersion,
         NodeApiVersions controllerApiVersions,
         ListenerType listenerType,
         boolean enableUnstableLastVersion,
@@ -180,27 +178,23 @@ public class ApiVersionsResponse extends AbstractResponse {
     ) {
         return intersectForwardableApis(
             listenerType,
-            minRecordVersion,
             controllerApiVersions.allSupportedApiVersions(),
             enableUnstableLastVersion,
             clientTelemetryEnabled);
     }
 
     public static ApiVersionCollection brokerApiVersions(
-        RecordVersion minRecordVersion,
         ListenerType listenerType,
         boolean enableUnstableLastVersion,
         boolean clientTelemetryEnabled
     ) {
         return filterApis(
-            minRecordVersion,
             listenerType,
             enableUnstableLastVersion,
             clientTelemetryEnabled);
     }
 
     public static ApiVersionCollection filterApis(
-        RecordVersion minRecordVersion,
         ApiMessageType.ListenerType listenerType,
         boolean enableUnstableLastVersion,
         boolean clientTelemetryEnabled
@@ -210,21 +204,19 @@ public class ApiVersionsResponse extends AbstractResponse {
             // Skip telemetry APIs if client telemetry is disabled.
             if ((apiKey == ApiKeys.GET_TELEMETRY_SUBSCRIPTIONS || apiKey == ApiKeys.PUSH_TELEMETRY) && !clientTelemetryEnabled)
                 continue;
-
-            if (apiKey.minRequiredInterBrokerMagic <= minRecordVersion.value) {
-                apiKey.toApiVersion(enableUnstableLastVersion).ifPresent(apiKeys::add);
-            }
+            apiKey.toApiVersionForApiResponse(enableUnstableLastVersion, listenerType).ifPresent(apiKeys::add);
         }
         return apiKeys;
     }
 
     public static ApiVersionCollection collectApis(
+        ApiMessageType.ListenerType listenerType,
         Set<ApiKeys> apiKeys,
         boolean enableUnstableLastVersion
     ) {
         ApiVersionCollection res = new ApiVersionCollection();
         for (ApiKeys apiKey : apiKeys) {
-            apiKey.toApiVersion(enableUnstableLastVersion).ifPresent(res::add);
+            apiKey.toApiVersionForApiResponse(enableUnstableLastVersion, listenerType).ifPresent(res::add);
         }
         return res;
     }
@@ -234,7 +226,6 @@ public class ApiVersionsResponse extends AbstractResponse {
      * known range and that of another set.
      *
      * @param listenerType the listener type which constrains the set of exposed APIs
-     * @param minRecordVersion min inter broker magic
      * @param activeControllerApiVersions controller ApiVersions
      * @param enableUnstableLastVersion whether unstable versions should be advertised or not
      * @param clientTelemetryEnabled whether client telemetry is enabled or not
@@ -242,42 +233,39 @@ public class ApiVersionsResponse extends AbstractResponse {
      */
     public static ApiVersionCollection intersectForwardableApis(
         final ApiMessageType.ListenerType listenerType,
-        final RecordVersion minRecordVersion,
         final Map<ApiKeys, ApiVersion> activeControllerApiVersions,
         boolean enableUnstableLastVersion,
         boolean clientTelemetryEnabled
     ) {
         ApiVersionCollection apiKeys = new ApiVersionCollection();
         for (ApiKeys apiKey : ApiKeys.apisForListener(listenerType)) {
-            if (apiKey.minRequiredInterBrokerMagic <= minRecordVersion.value) {
-                final Optional<ApiVersion> brokerApiVersion = apiKey.toApiVersion(enableUnstableLastVersion);
-                if (!brokerApiVersion.isPresent()) {
-                    // Broker does not support this API key.
-                    continue;
-                }
-
-                // Skip telemetry APIs if client telemetry is disabled.
-                if ((apiKey == ApiKeys.GET_TELEMETRY_SUBSCRIPTIONS || apiKey == ApiKeys.PUSH_TELEMETRY) && !clientTelemetryEnabled)
-                    continue;
-
-                final ApiVersion finalApiVersion;
-                if (!apiKey.forwardable) {
-                    finalApiVersion = brokerApiVersion.get();
-                } else {
-                    Optional<ApiVersion> intersectVersion = intersect(
-                        brokerApiVersion.get(),
-                        activeControllerApiVersions.getOrDefault(apiKey, null)
-                    );
-                    if (intersectVersion.isPresent()) {
-                        finalApiVersion = intersectVersion.get();
-                    } else {
-                        // Controller doesn't support this API key, or there is no intersection.
-                        continue;
-                    }
-                }
-
-                apiKeys.add(finalApiVersion.duplicate());
+            final Optional<ApiVersion> brokerApiVersion = apiKey.toApiVersionForApiResponse(enableUnstableLastVersion, listenerType);
+            if (brokerApiVersion.isEmpty()) {
+                // Broker does not support this API key.
+                continue;
             }
+
+            // Skip telemetry APIs if client telemetry is disabled.
+            if ((apiKey == ApiKeys.GET_TELEMETRY_SUBSCRIPTIONS || apiKey == ApiKeys.PUSH_TELEMETRY) && !clientTelemetryEnabled)
+                continue;
+
+            final ApiVersion finalApiVersion;
+            if (!apiKey.forwardable) {
+                finalApiVersion = brokerApiVersion.get();
+            } else {
+                Optional<ApiVersion> intersectVersion = intersect(
+                    brokerApiVersion.get(),
+                    activeControllerApiVersions.getOrDefault(apiKey, null)
+                );
+                if (intersectVersion.isPresent()) {
+                    finalApiVersion = intersectVersion.get();
+                } else {
+                    // Controller doesn't support this API key, or there is no intersection.
+                    continue;
+                }
+            }
+
+            apiKeys.add(finalApiVersion.duplicate());
         }
         return apiKeys;
     }
