@@ -22,12 +22,11 @@ import org.apache.kafka.common.security.oauthbearer.internals.OAuthBearerSaslCli
 import java.security.Provider;
 import java.security.Security;
 
-public class OAuthBearerSaslClientProvider extends Provider {
+public final class OAuthBearerSaslClientProvider extends Provider {
     private static final long serialVersionUID = 1L;
 
-    @SuppressWarnings("this-escape")
-    protected OAuthBearerSaslClientProvider() {
-        super("SASL/OAUTHBEARER Client Provider", 1.0, "SASL/OAUTHBEARER Client Provider for Kafka");
+    private OAuthBearerSaslClientProvider() {
+        super("SASL/OAUTHBEARER Client Provider", "1.0", "SASL/OAUTHBEARER Client Provider for Kafka");
         put("SaslClientFactory." + OAuthBearerLoginModule.OAUTHBEARER_MECHANISM,
                 OAuthBearerSaslClientFactory.class.getName());
     }
