@@ -16,6 +16,14 @@
  */
 package org.apache.kafka.common.security.oauthbearer.internals.expiring;
 
+import org.apache.kafka.common.security.auth.AuthenticateCallbackHandler;
+import org.apache.kafka.common.security.auth.Login;
+import org.apache.kafka.common.utils.KafkaThread;
+import org.apache.kafka.common.utils.Time;
+
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
 import java.util.Date;
 import java.util.Objects;
 import java.util.Random;
@@ -25,13 +33,6 @@ import javax.security.auth.login.Configuration;
 import javax.security.auth.login.LoginContext;
 import javax.security.auth.login.LoginException;
 
-import org.apache.kafka.common.security.auth.AuthenticateCallbackHandler;
-import org.apache.kafka.common.security.auth.Login;
-import org.apache.kafka.common.utils.KafkaThread;
-import org.apache.kafka.common.utils.Time;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
-
 /**
  * This class is responsible for refreshing logins for both Kafka client and
  * server when the login is a type that has a limited lifetime/will expire. The
@@ -83,10 +84,10 @@ public abstract class ExpiringCredentialRefreshingLogin implements AutoCloseable
                 }
                 // safety check motivated by KAFKA-7945,
                 // should generally never happen except due to a bug
-                if (nextRefreshMs.longValue() < nowMs) {
+                if (nextRefreshMs < nowMs) {
                     log.warn("[Principal={}]: Expiring credential re-login sleep time was calculated to be in the past! Will explicitly adjust. ({})", principalLogText(),
                             new Date(nextRefreshMs));
-                    nextRefreshMs = Long.valueOf(nowMs + 10 * 1000); // refresh in 10 seconds
+                    nextRefreshMs = nowMs + 10 * 1000; // refresh in 10 seconds
                 }
                 log.info("[Principal={}]: Expiring credential re-login sleeping until: {}", principalLogText(),
                         new Date(nextRefreshMs));
@@ -119,7 +120,7 @@ public abstract class ExpiringCredentialRefreshingLogin implements AutoCloseable
                         if (Thread.currentThread().isInterrupted()) {
                             log.error(
                                     "[Principal={}]: Interrupted while trying to perform a subsequent expiring credential re-login after one or more initial re-login failures: re-login thread exiting now: {}",
-                                    principalLogText(), String.valueOf(loginException.getMessage()));
+                                    principalLogText(), loginException.getMessage());
                             loginContextFactory.refresherThreadDone();
                             return;
                         }
@@ -195,7 +196,7 @@ public abstract class ExpiringCredentialRefreshingLogin implements AutoCloseable
      * instance and starts the thread used to periodically re-login.
      * <p>
      * The synchronized keyword is not necessary because an implementation of
-     * {@link Login} will delegate to this code (e.g. OAuthBearerRefreshingLogin},
+     * {@link Login} will delegate to this code (e.g. OAuthBearerRefreshingLogin),
      * and the {@code login()} method on the delegating class will itself be
      * synchronized if necessary.
      */
@@ -305,16 +306,16 @@ public abstract class ExpiringCredentialRefreshingLogin implements AutoCloseable
             }
         }
         Long absoluteLastRefreshTimeMs = expiringCredential.absoluteLastRefreshTimeMs();
-        if (absoluteLastRefreshTimeMs != null && absoluteLastRefreshTimeMs.longValue() < expireTimeMs) {
+        if (absoluteLastRefreshTimeMs != null && absoluteLastRefreshTimeMs < expireTimeMs) {
             log.warn("[Principal={}]: Expiring credential refresh thread exiting because the"
                     + " expiring credential's current expiration time ({}) exceeds the latest possible refresh time ({})."
                     + " This process will not be able to authenticate new SASL connections after that"
                     + " time (for example, it will not be able to authenticate a new connection with a Kafka Broker).",
-                    principalLogText(), new Date(expireTimeMs), new Date(absoluteLastRefreshTimeMs.longValue()));
+                    principalLogText(), new Date(expireTimeMs), new Date(absoluteLastRefreshTimeMs));
             return null;
         }
         Long optionalStartTime = expiringCredential.startTimeMs();
-        long startMs = optionalStartTime != null ? optionalStartTime.longValue() : relativeToMs;
+        long startMs = optionalStartTime != null ? optionalStartTime : relativeToMs;
         log.info("[Principal={}]: Expiring credential valid from {} to {}", expiringCredential.principalName(),
                 new java.util.Date(startMs), new java.util.Date(expireTimeMs));
 
