@@ -27,8 +27,11 @@ public final class OperatingSystem {
 
     public static final boolean IS_WINDOWS;
 
+    public static final boolean IS_ZOS;
+
     static {
         NAME = System.getProperty("os.name").toLowerCase(Locale.ROOT);
         IS_WINDOWS = NAME.startsWith("windows");
+        IS_ZOS = NAME.startsWith("z/os");
     }
 }
