@@ -19,8 +19,10 @@ package org.apache.kafka.clients.admin;
 import org.apache.kafka.clients.admin.DescribeReplicaLogDirsResult.ReplicaLogDirInfo;
 import org.apache.kafka.clients.admin.internals.CoordinatorKey;
 import org.apache.kafka.clients.consumer.OffsetAndMetadata;
+import org.apache.kafka.clients.consumer.internals.ConsumerProtocol;
 import org.apache.kafka.common.ElectionType;
-import org.apache.kafka.common.KafkaException;
+import org.apache.kafka.common.GroupState;
+import org.apache.kafka.common.GroupType;
 import org.apache.kafka.common.KafkaFuture;
 import org.apache.kafka.common.Metric;
 import org.apache.kafka.common.MetricName;
@@ -36,6 +38,8 @@ import org.apache.kafka.common.acl.AclBinding;
 import org.apache.kafka.common.acl.AclBindingFilter;
 import org.apache.kafka.common.acl.AclOperation;
 import org.apache.kafka.common.config.ConfigResource;
+import org.apache.kafka.common.errors.DelegationTokenNotFoundException;
+import org.apache.kafka.common.errors.InvalidPrincipalTypeException;
 import org.apache.kafka.common.errors.InvalidReplicationFactorException;
 import org.apache.kafka.common.errors.InvalidRequestException;
 import org.apache.kafka.common.errors.InvalidUpdateVersionException;
@@ -47,15 +51,18 @@ import org.apache.kafka.common.errors.UnknownTopicIdException;
 import org.apache.kafka.common.errors.UnknownTopicOrPartitionException;
 import org.apache.kafka.common.errors.UnsupportedVersionException;
 import org.apache.kafka.common.internals.KafkaFutureImpl;
+import org.apache.kafka.common.metrics.KafkaMetric;
 import org.apache.kafka.common.quota.ClientQuotaAlteration;
 import org.apache.kafka.common.quota.ClientQuotaFilter;
 import org.apache.kafka.common.requests.DescribeLogDirsResponse;
-import org.apache.kafka.common.errors.DelegationTokenNotFoundException;
-import org.apache.kafka.common.errors.InvalidPrincipalTypeException;
+import org.apache.kafka.common.security.auth.KafkaPrincipal;
+import org.apache.kafka.common.security.token.delegation.DelegationToken;
+import org.apache.kafka.common.security.token.delegation.TokenInformation;
+import org.apache.kafka.common.utils.Time;
 
 import java.time.Duration;
-import java.util.Arrays;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
@@ -64,10 +71,6 @@ import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
 import java.util.stream.Collectors;
-import org.apache.kafka.common.security.auth.KafkaPrincipal;
-import org.apache.kafka.common.security.token.delegation.DelegationToken;
-import org.apache.kafka.common.security.token.delegation.TokenInformation;
-import org.apache.kafka.common.utils.Time;
 
 public class MockAdminClient extends AdminClient {
     public static final String DEFAULT_CLUSTER_ID = "I4ZmrWqfT2e-upky_4fdPA";
@@ -92,6 +95,9 @@ public class MockAdminClient extends AdminClient {
     private final List<List<String>> brokerLogDirs;
     private final List<Map<String, String>> brokerConfigs;
     private final Map<String, Map<String, String>> clientMetricsConfigs;
+    private final Map<String, Map<String, String>> groupConfigs;
+    private final Map<String, String> defaultGroupConfigs;
+    private final List<KafkaMetric> addedMetrics = new ArrayList<>();
 
     private Node controller;
     private int timeoutNextRequests = 0;
@@ -103,9 +109,7 @@ public class MockAdminClient extends AdminClient {
     private Time mockTime;
     private long blockingTimeMs;
 
-    private KafkaException listConsumerGroupOffsetsException;
-
-    private Map<MetricName, Metric> mockMetrics = new HashMap<>();
+    private final Map<MetricName, Metric> mockMetrics = new HashMap<>();
 
     private final List<DelegationToken> allTokens = new ArrayList<>();
 
@@ -124,8 +128,8 @@ public class MockAdminClient extends AdminClient {
         private Map<String, Short> featureLevels = Collections.emptyMap();
         private Map<String, Short> minSupportedFeatureLevels = Collections.emptyMap();
         private Map<String, Short> maxSupportedFeatureLevels = Collections.emptyMap();
+        private Map<String, String> defaultGroupConfigs = Collections.emptyMap();
 
-        @SuppressWarnings("this-escape")
         public Builder() {
             numBrokers(1);
         }
@@ -141,7 +145,7 @@ public class MockAdminClient extends AdminClient {
             return this;
         }
 
-        public Builder numBrokers(int numBrokers) {
+        public final Builder numBrokers(int numBrokers) {
             if (brokers.size() >= numBrokers) {
                 brokers = brokers.subList(0, numBrokers);
                 brokerLogDirs = brokerLogDirs.subList(0, numBrokers);
@@ -194,6 +198,11 @@ public class MockAdminClient extends AdminClient {
             return this;
         }
 
+        public Builder defaultGroupConfigs(Map<String, String> defaultGroupConfigs) {
+            this.defaultGroupConfigs = defaultGroupConfigs;
+            return this;
+        }
+
         public MockAdminClient build() {
             return new MockAdminClient(brokers,
                 controller == null ? brokers.get(0) : controller,
@@ -204,7 +213,8 @@ public class MockAdminClient extends AdminClient {
                 usingRaftController,
                 featureLevels,
                 minSupportedFeatureLevels,
-                maxSupportedFeatureLevels);
+                maxSupportedFeatureLevels,
+                defaultGroupConfigs);
         }
     }
 
@@ -222,10 +232,10 @@ public class MockAdminClient extends AdminClient {
             false,
             Collections.emptyMap(),
             Collections.emptyMap(),
+            Collections.emptyMap(),
             Collections.emptyMap());
     }
 
-    @SuppressWarnings("this-escape")
     private MockAdminClient(
         List<Node> brokers,
         Node controller,
@@ -236,7 +246,8 @@ public class MockAdminClient extends AdminClient {
         boolean usingRaftController,
         Map<String, Short> featureLevels,
         Map<String, Short> minSupportedFeatureLevels,
-        Map<String, Short> maxSupportedFeatureLevels
+        Map<String, Short> maxSupportedFeatureLevels,
+        Map<String, String> defaultGroupConfigs
     ) {
         this.brokers = brokers;
         controller(controller);
@@ -246,6 +257,8 @@ public class MockAdminClient extends AdminClient {
         this.brokerLogDirs = brokerLogDirs;
         this.brokerConfigs = new ArrayList<>();
         this.clientMetricsConfigs = new HashMap<>();
+        this.groupConfigs = new HashMap<>();
+        this.defaultGroupConfigs = new HashMap<>(defaultGroupConfigs);
         for (int i = 0; i < brokers.size(); i++) {
             final Map<String, String> config = new HashMap<>();
             config.put("default.replication.factor", String.valueOf(defaultReplicationFactor));
@@ -260,7 +273,7 @@ public class MockAdminClient extends AdminClient {
         this.maxSupportedFeatureLevels = new HashMap<>(maxSupportedFeatureLevels);
     }
 
-    public synchronized void controller(Node controller) {
+    public final synchronized void controller(Node controller) {
         if (!brokers.contains(controller))
             throw new IllegalArgumentException("The controller node must be in the list of brokers");
         this.controller = controller;
@@ -273,7 +286,7 @@ public class MockAdminClient extends AdminClient {
         addTopic(internal, name, partitions, configs, true);
     }
 
-    synchronized public void addTopic(boolean internal,
+    public synchronized void addTopic(boolean internal,
                                       String name,
                                       List<TopicPartitionInfo> partitions,
                                       Map<String, String> configs,
@@ -309,7 +322,7 @@ public class MockAdminClient extends AdminClient {
         allTopics.put(name, new TopicMetadata(topicId, internal, partitions, logDirs, configs));
     }
 
-    synchronized public void markTopicForDeletion(final String name) {
+    public synchronized void markTopicForDeletion(final String name) {
         if (!allTopics.containsKey(name)) {
             throw new IllegalArgumentException(String.format("Topic %s did not exist.", name));
         }
@@ -317,12 +330,12 @@ public class MockAdminClient extends AdminClient {
         allTopics.get(name).markedForDeletion = true;
     }
 
-    synchronized public void timeoutNextRequest(int numberOfRequest) {
+    public synchronized void timeoutNextRequest(int numberOfRequest) {
         timeoutNextRequests = numberOfRequest;
     }
 
     @Override
-    synchronized public DescribeClusterResult describeCluster(DescribeClusterOptions options) {
+    public synchronized DescribeClusterResult describeCluster(DescribeClusterOptions options) {
         KafkaFutureImpl<Collection<Node>> nodesFuture = new KafkaFutureImpl<>();
         KafkaFutureImpl<Node> controllerFuture = new KafkaFutureImpl<>();
         KafkaFutureImpl<String> brokerIdFuture = new KafkaFutureImpl<>();
@@ -345,7 +358,7 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public CreateTopicsResult createTopics(Collection<NewTopic> newTopics, CreateTopicsOptions options) {
+    public synchronized CreateTopicsResult createTopics(Collection<NewTopic> newTopics, CreateTopicsOptions options) {
         Map<String, KafkaFuture<CreateTopicsResult.TopicMetadataAndConfig>> createTopicResult = new HashMap<>();
 
         if (timeoutNextRequests > 0) {
@@ -419,7 +432,7 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public ListTopicsResult listTopics(ListTopicsOptions options) {
+    public synchronized ListTopicsResult listTopics(ListTopicsOptions options) {
         Map<String, TopicListing> topicListings = new HashMap<>();
 
         if (timeoutNextRequests > 0) {
@@ -445,7 +458,7 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public DescribeTopicsResult describeTopics(TopicCollection topics, DescribeTopicsOptions options) {
+    public synchronized DescribeTopicsResult describeTopics(TopicCollection topics, DescribeTopicsOptions options) {
         if (topics instanceof TopicIdCollection)
             return DescribeTopicsResult.ofTopicIds(new HashMap<>(handleDescribeTopicsUsingIds(((TopicIdCollection) topics).topicIds(), options)));
         else if (topics instanceof TopicNameCollection)
@@ -494,7 +507,7 @@ public class MockAdminClient extends AdminClient {
         return topicDescriptions;
     }
 
-    synchronized public Map<Uuid, KafkaFuture<TopicDescription>>  handleDescribeTopicsUsingIds(Collection<Uuid> topicIds, DescribeTopicsOptions options) {
+    public synchronized Map<Uuid, KafkaFuture<TopicDescription>>  handleDescribeTopicsUsingIds(Collection<Uuid> topicIds, DescribeTopicsOptions options) {
 
         Map<Uuid, KafkaFuture<TopicDescription>> topicDescriptions = new HashMap<>();
 
@@ -537,7 +550,7 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public DeleteTopicsResult deleteTopics(TopicCollection topics, DeleteTopicsOptions options) {
+    public synchronized DeleteTopicsResult deleteTopics(TopicCollection topics, DeleteTopicsOptions options) {
         DeleteTopicsResult result;
         if (topics instanceof TopicIdCollection)
             result = DeleteTopicsResult.ofTopicIds(new HashMap<>(handleDeleteTopicsUsingIds(((TopicIdCollection) topics).topicIds(), options)));
@@ -608,12 +621,12 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public CreatePartitionsResult createPartitions(Map<String, NewPartitions> newPartitions, CreatePartitionsOptions options) {
+    public synchronized CreatePartitionsResult createPartitions(Map<String, NewPartitions> newPartitions, CreatePartitionsOptions options) {
         throw new UnsupportedOperationException("Not implemented yet");
     }
 
     @Override
-    synchronized public DeleteRecordsResult deleteRecords(Map<TopicPartition, RecordsToDelete> recordsToDelete, DeleteRecordsOptions options) {
+    public synchronized DeleteRecordsResult deleteRecords(Map<TopicPartition, RecordsToDelete> recordsToDelete, DeleteRecordsOptions options) {
         Map<TopicPartition, KafkaFuture<DeletedRecords>> deletedRecordsResult = new HashMap<>();
         if (recordsToDelete.isEmpty()) {
             return new DeleteRecordsResult(deletedRecordsResult);
@@ -623,7 +636,7 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public CreateDelegationTokenResult createDelegationToken(CreateDelegationTokenOptions options) {
+    public synchronized CreateDelegationTokenResult createDelegationToken(CreateDelegationTokenOptions options) {
         KafkaFutureImpl<DelegationToken> future = new KafkaFutureImpl<>();
 
         for (KafkaPrincipal renewer : options.renewers()) {
@@ -634,7 +647,7 @@ public class MockAdminClient extends AdminClient {
         }
 
         String tokenId = Uuid.randomUuid().toString();
-        TokenInformation tokenInfo = new TokenInformation(tokenId, options.renewers().get(0), options.renewers(), System.currentTimeMillis(), options.maxlifeTimeMs(), -1);
+        TokenInformation tokenInfo = new TokenInformation(tokenId, options.renewers().get(0), options.renewers(), System.currentTimeMillis(), options.maxLifetimeMs(), -1);
         DelegationToken token = new DelegationToken(tokenInfo, tokenId.getBytes());
         allTokens.add(token);
         future.complete(token);
@@ -643,11 +656,11 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public RenewDelegationTokenResult renewDelegationToken(byte[] hmac, RenewDelegationTokenOptions options) {
+    public synchronized RenewDelegationTokenResult renewDelegationToken(byte[] hmac, RenewDelegationTokenOptions options) {
         KafkaFutureImpl<Long> future = new KafkaFutureImpl<>();
 
-        Boolean tokenFound = false;
-        Long expiryTimestamp = options.renewTimePeriodMs();
+        boolean tokenFound = false;
+        long expiryTimestamp = options.renewTimePeriodMs();
         for (DelegationToken token : allTokens) {
             if (Arrays.equals(token.hmac(), hmac)) {
                 token.tokenInfo().setExpiryTimestamp(expiryTimestamp);
@@ -665,12 +678,12 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public ExpireDelegationTokenResult expireDelegationToken(byte[] hmac, ExpireDelegationTokenOptions options) {
+    public synchronized ExpireDelegationTokenResult expireDelegationToken(byte[] hmac, ExpireDelegationTokenOptions options) {
         KafkaFutureImpl<Long> future = new KafkaFutureImpl<>();
 
-        Long expiryTimestamp = options.expiryTimePeriodMs();
+        long expiryTimestamp = options.expiryTimePeriodMs();
         List<DelegationToken> tokensToRemove = new ArrayList<>();
-        Boolean tokenFound = false;
+        boolean tokenFound = false;
         for (DelegationToken token : allTokens) {
             if (Arrays.equals(token.hmac(), hmac)) {
                 if (expiryTimestamp == -1 || expiryTimestamp < System.currentTimeMillis()) {
@@ -691,7 +704,7 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public DescribeDelegationTokenResult describeDelegationToken(DescribeDelegationTokenOptions options) {
+    public synchronized DescribeDelegationTokenResult describeDelegationToken(DescribeDelegationTokenOptions options) {
         KafkaFutureImpl<List<DelegationToken>> future = new KafkaFutureImpl<>();
 
         if (options.owners().isEmpty()) {
@@ -710,17 +723,26 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public DescribeConsumerGroupsResult describeConsumerGroups(Collection<String> groupIds, DescribeConsumerGroupsOptions options) {
+    public synchronized ListGroupsResult listGroups(ListGroupsOptions options) {
+        KafkaFutureImpl<Collection<Object>> future = new KafkaFutureImpl<>();
+        future.complete(groupConfigs.keySet().stream().map(g -> new GroupListing(g, Optional.of(GroupType.CONSUMER), ConsumerProtocol.PROTOCOL_TYPE, Optional.of(GroupState.STABLE))).collect(Collectors.toList()));
+        return new ListGroupsResult(future);
+    }
+
+    @Override
+    public synchronized DescribeConsumerGroupsResult describeConsumerGroups(Collection<String> groupIds, DescribeConsumerGroupsOptions options) {
         throw new UnsupportedOperationException("Not implemented yet");
     }
 
     @Override
-    synchronized public ListConsumerGroupsResult listConsumerGroups(ListConsumerGroupsOptions options) {
-        throw new UnsupportedOperationException("Not implemented yet");
+    public synchronized ListConsumerGroupsResult listConsumerGroups(ListConsumerGroupsOptions options) {
+        KafkaFutureImpl<Collection<Object>> future = new KafkaFutureImpl<>();
+        future.complete(groupConfigs.keySet().stream().map(g -> new ConsumerGroupListing(g, false)).collect(Collectors.toList()));
+        return new ListConsumerGroupsResult(future);
     }
 
     @Override
-    synchronized public ListConsumerGroupOffsetsResult listConsumerGroupOffsets(Map<String, ListConsumerGroupOffsetsSpec> groupSpecs, ListConsumerGroupOffsetsOptions options) {
+    public synchronized ListConsumerGroupOffsetsResult listConsumerGroupOffsets(Map<String, ListConsumerGroupOffsetsSpec> groupSpecs, ListConsumerGroupOffsetsOptions options) {
         // ignoring the groups and assume one test would only work on one group only
         if (groupSpecs.size() != 1)
             throw new UnsupportedOperationException("Not implemented yet");
@@ -728,35 +750,24 @@ public class MockAdminClient extends AdminClient {
         String group = groupSpecs.keySet().iterator().next();
         Collection<TopicPartition> topicPartitions = groupSpecs.get(group).topicPartitions();
         final KafkaFutureImpl<Map<TopicPartition, OffsetAndMetadata>> future = new KafkaFutureImpl<>();
-
-        if (listConsumerGroupOffsetsException != null) {
-            future.completeExceptionally(listConsumerGroupOffsetsException);
-        } else {
-            if (topicPartitions.isEmpty()) {
-                future.complete(committedOffsets.entrySet().stream()
-                        .collect(Collectors.toMap(Map.Entry::getKey, entry -> new OffsetAndMetadata(entry.getValue()))));
-            } else {
-                future.complete(committedOffsets.entrySet().stream()
-                        .filter(entry -> topicPartitions.contains(entry.getKey()))
-                        .collect(Collectors.toMap(Map.Entry::getKey, entry -> new OffsetAndMetadata(entry.getValue()))));
-            }
-        }
-
+        future.complete(committedOffsets.entrySet().stream()
+                .filter(entry -> topicPartitions.isEmpty() || topicPartitions.contains(entry.getKey()))
+                .collect(Collectors.toMap(Map.Entry::getKey, entry -> new OffsetAndMetadata(entry.getValue()))));
         return new ListConsumerGroupOffsetsResult(Collections.singletonMap(CoordinatorKey.byGroupId(group), future));
     }
 
     @Override
-    synchronized public DeleteConsumerGroupsResult deleteConsumerGroups(Collection<String> groupIds, DeleteConsumerGroupsOptions options) {
+    public synchronized DeleteConsumerGroupsResult deleteConsumerGroups(Collection<String> groupIds, DeleteConsumerGroupsOptions options) {
         throw new UnsupportedOperationException("Not implemented yet");
     }
 
     @Override
-    synchronized public DeleteConsumerGroupOffsetsResult deleteConsumerGroupOffsets(String groupId, Set<TopicPartition> partitions, DeleteConsumerGroupOffsetsOptions options) {
+    public synchronized DeleteConsumerGroupOffsetsResult deleteConsumerGroupOffsets(String groupId, Set<TopicPartition> partitions, DeleteConsumerGroupOffsetsOptions options) {
         throw new UnsupportedOperationException("Not implemented yet");
     }
 
     @Override
-    synchronized public ElectLeadersResult electLeaders(
+    public synchronized ElectLeadersResult electLeaders(
             ElectionType electionType,
             Set<TopicPartition> partitions,
             ElectLeadersOptions options) {
@@ -764,27 +775,27 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public RemoveMembersFromConsumerGroupResult removeMembersFromConsumerGroup(String groupId, RemoveMembersFromConsumerGroupOptions options) {
+    public synchronized RemoveMembersFromConsumerGroupResult removeMembersFromConsumerGroup(String groupId, RemoveMembersFromConsumerGroupOptions options) {
         throw new UnsupportedOperationException("Not implemented yet");
     }
 
     @Override
-    synchronized public CreateAclsResult createAcls(Collection<AclBinding> acls, CreateAclsOptions options) {
+    public synchronized CreateAclsResult createAcls(Collection<AclBinding> acls, CreateAclsOptions options) {
         throw new UnsupportedOperationException("Not implemented yet");
     }
 
     @Override
-    synchronized public DescribeAclsResult describeAcls(AclBindingFilter filter, DescribeAclsOptions options) {
+    public synchronized DescribeAclsResult describeAcls(AclBindingFilter filter, DescribeAclsOptions options) {
         throw new UnsupportedOperationException("Not implemented yet");
     }
 
     @Override
-    synchronized public DeleteAclsResult deleteAcls(Collection<AclBindingFilter> filters, DeleteAclsOptions options) {
+    public synchronized DeleteAclsResult deleteAcls(Collection<AclBindingFilter> filters, DeleteAclsOptions options) {
         throw new UnsupportedOperationException("Not implemented yet");
     }
 
     @Override
-    synchronized public DescribeConfigsResult describeConfigs(Collection<ConfigResource> resources, DescribeConfigsOptions options) {
+    public synchronized DescribeConfigsResult describeConfigs(Collection<ConfigResource> resources, DescribeConfigsOptions options) {
 
         if (timeoutNextRequests > 0) {
             Map<ConfigResource, KafkaFuture<Config>> configs = new HashMap<>();
@@ -811,7 +822,7 @@ public class MockAdminClient extends AdminClient {
         return new DescribeConfigsResult(results);
     }
 
-    synchronized private Config getResourceDescription(ConfigResource resource) {
+    private synchronized Config getResourceDescription(ConfigResource resource) {
         switch (resource.type()) {
             case BROKER: {
                 int brokerId = Integer.parseInt(resource.name());
@@ -838,6 +849,15 @@ public class MockAdminClient extends AdminClient {
                 }
                 return toConfigObject(clientMetricsConfigs.get(resourceName));
             }
+            case GROUP: {
+                String resourceName = resource.name();
+                if (resourceName.isEmpty()) {
+                    throw new InvalidRequestException("Empty resource name");
+                }
+                Map<String, String> groupConfig = groupConfigs.getOrDefault(resourceName, new HashMap<>());
+                defaultGroupConfigs.forEach(groupConfig::putIfAbsent);
+                return toConfigObject(groupConfig);
+            }
             default:
                 throw new UnsupportedOperationException("Not implemented yet");
         }
@@ -852,13 +872,7 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    @Deprecated
-    synchronized public AlterConfigsResult alterConfigs(Map<ConfigResource, Config> configs, AlterConfigsOptions options) {
-        throw new UnsupportedOperationException("Not implemented yet");
-    }
-
-    @Override
-    synchronized public AlterConfigsResult incrementalAlterConfigs(
+    public synchronized AlterConfigsResult incrementalAlterConfigs(
             Map<ConfigResource, Collection<AlterConfigOp>> configs,
             AlterConfigsOptions options) {
         Map<ConfigResource, KafkaFuture<Void>> futures = new HashMap<>();
@@ -878,7 +892,7 @@ public class MockAdminClient extends AdminClient {
         return new AlterConfigsResult(futures);
     }
 
-    synchronized private Throwable handleIncrementalResourceAlteration(
+    private synchronized Throwable handleIncrementalResourceAlteration(
             ConfigResource resource, Collection<AlterConfigOp> ops) {
         switch (resource.type()) {
             case BROKER: {
@@ -959,13 +973,40 @@ public class MockAdminClient extends AdminClient {
                 clientMetricsConfigs.put(resourceName, newMap);
                 return null;
             }
+            case GROUP: {
+                String resourceName = resource.name();
+                if (resourceName.isEmpty()) {
+                    return new InvalidRequestException("Empty resource name");
+                }
+
+                if (!groupConfigs.containsKey(resourceName)) {
+                    groupConfigs.put(resourceName, new HashMap<>());
+                }
+
+                HashMap<String, String> newMap = new HashMap<>(groupConfigs.get(resourceName));
+                for (AlterConfigOp op : ops) {
+                    switch (op.opType()) {
+                        case SET:
+                            newMap.put(op.configEntry().name(), op.configEntry().value());
+                            break;
+                        case DELETE:
+                            newMap.remove(op.configEntry().name());
+                            break;
+                        default:
+                            return new InvalidRequestException(
+                                "Unsupported op type " + op.opType());
+                    }
+                }
+                groupConfigs.put(resourceName, newMap);
+                return null;
+            }
             default:
                 return new UnsupportedOperationException();
         }
     }
 
     @Override
-    synchronized public AlterReplicaLogDirsResult alterReplicaLogDirs(
+    public synchronized AlterReplicaLogDirsResult alterReplicaLogDirs(
             Map<TopicPartitionReplica, String> replicaAssignment,
             AlterReplicaLogDirsOptions options) {
         Map<TopicPartitionReplica, KafkaFuture<Void>> results = new HashMap<>();
@@ -998,7 +1039,7 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public DescribeLogDirsResult describeLogDirs(Collection<Integer> brokers,
+    public synchronized DescribeLogDirsResult describeLogDirs(Collection<Integer> brokers,
                                                               DescribeLogDirsOptions options) {
         Map<Integer, Map<String, LogDirDescription>> unwrappedResults = new HashMap<>();
 
@@ -1040,7 +1081,7 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public DescribeReplicaLogDirsResult describeReplicaLogDirs(
+    public synchronized DescribeReplicaLogDirsResult describeReplicaLogDirs(
             Collection<TopicPartitionReplica> replicas, DescribeReplicaLogDirsOptions options) {
         Map<TopicPartitionReplica, KafkaFuture<ReplicaLogDirInfo>> results = new HashMap<>();
         for (TopicPartitionReplica replica : replicas) {
@@ -1079,7 +1120,7 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public AlterPartitionReassignmentsResult alterPartitionReassignments(
+    public synchronized AlterPartitionReassignmentsResult alterPartitionReassignments(
             Map<TopicPartition, Optional<NewPartitionReassignment>> newReassignments,
             AlterPartitionReassignmentsOptions options) {
         Map<TopicPartition, KafkaFuture<Void>> futures = new HashMap<>();
@@ -1106,7 +1147,7 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public ListPartitionReassignmentsResult listPartitionReassignments(
+    public synchronized ListPartitionReassignmentsResult listPartitionReassignments(
             Optional<Set<TopicPartition>> partitions,
             ListPartitionReassignmentsOptions options) {
         Map<TopicPartition, PartitionReassignment> map = new HashMap<>();
@@ -1120,7 +1161,7 @@ public class MockAdminClient extends AdminClient {
         return new ListPartitionReassignmentsResult(KafkaFutureImpl.completedFuture(map));
     }
 
-    synchronized private PartitionReassignment findPartitionReassignment(TopicPartition partition) {
+    private synchronized PartitionReassignment findPartitionReassignment(TopicPartition partition) {
         NewPartitionReassignment reassignment = reassignments.get(partition);
         if (reassignment == null) {
             return null;
@@ -1149,12 +1190,12 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public AlterConsumerGroupOffsetsResult alterConsumerGroupOffsets(String groupId, Map<TopicPartition, OffsetAndMetadata> offsets, AlterConsumerGroupOffsetsOptions options) {
+    public synchronized AlterConsumerGroupOffsetsResult alterConsumerGroupOffsets(String groupId, Map<TopicPartition, OffsetAndMetadata> offsets, AlterConsumerGroupOffsetsOptions options) {
         throw new UnsupportedOperationException("Not implement yet");
     }
 
     @Override
-    synchronized public ListOffsetsResult listOffsets(Map<TopicPartition, OffsetSpec> topicPartitionOffsets, ListOffsetsOptions options) {
+    public synchronized ListOffsetsResult listOffsets(Map<TopicPartition, OffsetSpec> topicPartitionOffsets, ListOffsetsOptions options) {
         Map<TopicPartition, KafkaFuture<ListOffsetsResult.ListOffsetsResultInfo>> futures = new HashMap<>();
 
         for (Map.Entry<TopicPartition, OffsetSpec> entry : topicPartitionOffsets.entrySet()) {
@@ -1222,15 +1263,14 @@ public class MockAdminClient extends AdminClient {
         Map<String, FeatureUpdate> featureUpdates,
         UpdateFeaturesOptions options
     ) {
-        Map<String, KafkaFuture<Void>> results = new HashMap<>();
+        Throwable error = null;
         for (Map.Entry<String, FeatureUpdate> entry : featureUpdates.entrySet()) {
-            KafkaFutureImpl<Void> future = new KafkaFutureImpl<>();
             String feature = entry.getKey();
+            short cur = featureLevels.getOrDefault(feature, (short) 0);
+            short next = entry.getValue().maxVersionLevel();
+            short min = minSupportedFeatureLevels.getOrDefault(feature, (short) 0);
+            short max = maxSupportedFeatureLevels.getOrDefault(feature, (short) 0);
             try {
-                short cur = featureLevels.getOrDefault(feature, (short) 0);
-                short next = entry.getValue().maxVersionLevel();
-                short min = minSupportedFeatureLevels.getOrDefault(feature, (short) 0);
-                short max = maxSupportedFeatureLevels.getOrDefault(feature, (short) 0);
                 switch (entry.getValue().upgradeType()) {
                     case UNKNOWN:
                         throw new InvalidRequestException("Invalid upgrade type.");
@@ -1265,18 +1305,32 @@ public class MockAdminClient extends AdminClient {
                 if (next > max) {
                     throw new InvalidUpdateVersionException("Can't upgrade above " + max);
                 }
-                if (!options.validateOnly()) {
-                    featureLevels.put(feature, next);
-                }
-                future.complete(null);
             } catch (Exception e) {
-                future.completeExceptionally(e);
+                error = invalidUpdateVersion(feature, next, e.getMessage());
+                break;
             }
-            results.put(feature, future);
         }
+        Map<String, KafkaFuture<Void>> results = new HashMap<>();
+        for (Map.Entry<String, FeatureUpdate> entry : featureUpdates.entrySet()) {
+            KafkaFutureImpl<Void> future = new KafkaFutureImpl<>();
+            if (error == null) {
+                future.complete(null);
+                if (!options.validateOnly()) {
+                    featureLevels.put(entry.getKey(), entry.getValue().maxVersionLevel());
+                }
+            } else {
+                future.completeExceptionally(error);
+            }
+            results.put(entry.getKey(), future);
+        }
+
         return new UpdateFeaturesResult(results);
     }
 
+    private InvalidRequestException invalidUpdateVersion(String feature, short version, String message) {
+        return new InvalidRequestException(String.format("Invalid update version %d for feature %s. %s", version, feature, message));
+    }
+
     @Override
     public UnregisterBrokerResult unregisterBroker(int brokerId, UnregisterBrokerOptions options) {
         if (usingRaftController) {
@@ -1321,7 +1375,27 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public void close(Duration timeout) {}
+    public AddRaftVoterResult addRaftVoter(int voterId, Uuid voterDirectoryId, Set<RaftVoterEndpoint> endpoints, AddRaftVoterOptions options) {
+        throw new UnsupportedOperationException("Not implemented yet");
+    }
+
+    @Override
+    public RemoveRaftVoterResult removeRaftVoter(int voterId, Uuid voterDirectoryId, RemoveRaftVoterOptions options) {
+        throw new UnsupportedOperationException("Not implemented yet");
+    }
+
+    @Override
+    public synchronized DescribeShareGroupsResult describeShareGroups(Collection<String> groupIds, DescribeShareGroupsOptions options) {
+        throw new UnsupportedOperationException("Not implemented yet");
+    }
+
+    @Override
+    public synchronized DescribeClassicGroupsResult describeClassicGroups(Collection<String> groupIds, DescribeClassicGroupsOptions options) {
+        throw new UnsupportedOperationException("Not implemented yet");
+    }
+
+    @Override
+    public synchronized void close(Duration timeout) {}
 
     public synchronized void updateBeginningOffsets(Map<TopicPartition, Long> newOffsets) {
         beginningOffsets.putAll(newOffsets);
@@ -1335,11 +1409,7 @@ public class MockAdminClient extends AdminClient {
         committedOffsets.putAll(newOffsets);
     }
 
-    public synchronized void throwOnListConsumerGroupOffsets(final KafkaException exception) {
-        listConsumerGroupOffsetsException = exception;
-    }
-
-    private final static class TopicMetadata {
+    private static final class TopicMetadata {
         final Uuid topicId;
         final boolean isInternalTopic;
         final List<TopicPartitionInfo> partitions;
@@ -1364,7 +1434,7 @@ public class MockAdminClient extends AdminClient {
         }
     }
 
-    synchronized public void setMockMetrics(MetricName name, Metric metric) {
+    public synchronized void setMockMetrics(MetricName name, Metric metric) {
         mockMetrics.put(name, metric);
     }
 
@@ -1412,11 +1482,11 @@ public class MockAdminClient extends AdminClient {
     }
 
     @Override
-    synchronized public Map<MetricName, ? extends Metric> metrics() {
+    public synchronized Map<MetricName, ? extends Metric> metrics() {
         return mockMetrics;
     }
 
-    synchronized public void setFetchesRemainingUntilVisible(String topicName, int fetchesRemainingUntilVisible) {
+    public synchronized void setFetchesRemainingUntilVisible(String topicName, int fetchesRemainingUntilVisible) {
         TopicMetadata metadata = allTopics.get(topicName);
         if (metadata == null) {
             throw new RuntimeException("No such topic as " + topicName);
@@ -1424,11 +1494,25 @@ public class MockAdminClient extends AdminClient {
         metadata.fetchesRemainingUntilVisible = fetchesRemainingUntilVisible;
     }
 
-    synchronized public List<Node> brokers() {
+    public synchronized List<Node> brokers() {
         return new ArrayList<>(brokers);
     }
 
-    synchronized public Node broker(int index) {
+    public synchronized Node broker(int index) {
         return brokers.get(index);
     }
+
+    public List<KafkaMetric> addedMetrics() {
+        return Collections.unmodifiableList(addedMetrics);
+    }
+
+    @Override
+    public void registerMetricForSubscription(KafkaMetric metric) {
+        addedMetrics.add(metric);
+    }
+
+    @Override
+    public void unregisterMetricFromSubscription(KafkaMetric metric) {
+        addedMetrics.remove(metric);
+    }
 }
