@@ -22,6 +22,7 @@ import org.apache.kafka.clients.admin.MemberDescription;
 import org.apache.kafka.clients.consumer.ConsumerPartitionAssignor.Assignment;
 import org.apache.kafka.clients.consumer.internals.ConsumerProtocol;
 import org.apache.kafka.common.ConsumerGroupState;
+import org.apache.kafka.common.GroupState;
 import org.apache.kafka.common.GroupType;
 import org.apache.kafka.common.Node;
 import org.apache.kafka.common.TopicPartition;
@@ -29,6 +30,7 @@ import org.apache.kafka.common.Uuid;
 import org.apache.kafka.common.errors.GroupAuthorizationException;
 import org.apache.kafka.common.errors.GroupIdNotFoundException;
 import org.apache.kafka.common.errors.InvalidGroupIdException;
+import org.apache.kafka.common.errors.TopicAuthorizationException;
 import org.apache.kafka.common.errors.UnsupportedVersionException;
 import org.apache.kafka.common.message.ConsumerGroupDescribeRequestData;
 import org.apache.kafka.common.message.ConsumerGroupDescribeResponseData;
@@ -54,6 +56,7 @@ import java.util.Collections;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.LinkedHashSet;
+import java.util.List;
 import java.util.Optional;
 import java.util.Set;
 
@@ -61,7 +64,6 @@ import static java.util.Collections.emptyList;
 import static java.util.Collections.emptySet;
 import static java.util.Collections.singleton;
 import static java.util.Collections.singletonList;
-import static org.apache.kafka.common.utils.Utils.mkSet;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 import static org.junit.jupiter.api.Assertions.assertThrows;
@@ -153,29 +155,46 @@ public class DescribeConsumerGroupsHandlerTest {
     @Test
     public void testSuccessfulHandleConsumerGroupResponse() {
         DescribeConsumerGroupsHandler handler = new DescribeConsumerGroupsHandler(false, logContext);
-        Collection<MemberDescription> members = singletonList(new MemberDescription(
-            "memberId",
-            Optional.of("instanceId"),
-            "clientId",
-            "host",
-            new MemberAssignment(mkSet(
-                new TopicPartition("foo", 0),
-                new TopicPartition("bar",  1))
+        Collection<MemberDescription> members = List.of(
+            new MemberDescription(
+                "memberId",
+                Optional.of("instanceId"),
+                "clientId",
+                "host",
+                new MemberAssignment(Set.of(
+                    new TopicPartition("foo", 0)
+                )),
+                Optional.of(new MemberAssignment(Set.of(
+                    new TopicPartition("foo", 1)
+                ))),
+                Optional.of(10),
+                Optional.of(true)
             ),
-            Optional.of(new MemberAssignment(mkSet(
-                new TopicPartition("foo", 1),
-                new TopicPartition("bar",  2)
-            )))
-        ));
+            new MemberDescription(
+                "memberId-classic",
+                Optional.of("instanceId-classic"),
+                "clientId-classic",
+                "host",
+                new MemberAssignment(Set.of(
+                    new TopicPartition("bar", 0)
+                )),
+                Optional.of(new MemberAssignment(Set.of(
+                    new TopicPartition("bar", 1)
+                ))),
+                Optional.of(9),
+                Optional.of(false)
+            ));
         ConsumerGroupDescription expected = new ConsumerGroupDescription(
             groupId1,
             false,
             members,
             "range",
             GroupType.CONSUMER,
-            ConsumerGroupState.STABLE,
+            GroupState.STABLE,
             coordinator,
-            Collections.emptySet()
+            Collections.emptySet(),
+            Optional.of(10),
+            Optional.of(10)
         );
         AdminApiHandler.ApiResult<CoordinatorKey, ConsumerGroupDescription> result = handler.handleResponse(
             coordinator,
@@ -190,7 +209,7 @@ public class DescribeConsumerGroupsHandlerTest {
                             .setAssignmentEpoch(10)
                             .setAssignorName("range")
                             .setAuthorizedOperations(Utils.to32BitField(emptySet()))
-                            .setMembers(singletonList(
+                            .setMembers(List.of(
                                 new ConsumerGroupDescribeResponseData.Member()
                                     .setMemberId("memberId")
                                     .setInstanceId("instanceId")
@@ -201,27 +220,44 @@ public class DescribeConsumerGroupsHandlerTest {
                                     .setSubscribedTopicNames(singletonList("foo"))
                                     .setSubscribedTopicRegex("regex")
                                     .setAssignment(new ConsumerGroupDescribeResponseData.Assignment()
-                                        .setTopicPartitions(Arrays.asList(
+                                        .setTopicPartitions(List.of(
                                             new ConsumerGroupDescribeResponseData.TopicPartitions()
                                                 .setTopicId(Uuid.randomUuid())
                                                 .setTopicName("foo")
-                                                .setPartitions(Collections.singletonList(0)),
+                                                .setPartitions(Collections.singletonList(0))
+                                        )))
+                                    .setTargetAssignment(new ConsumerGroupDescribeResponseData.Assignment()
+                                        .setTopicPartitions(List.of(
+                                            new ConsumerGroupDescribeResponseData.TopicPartitions()
+                                                .setTopicId(Uuid.randomUuid())
+                                                .setTopicName("foo")
+                                                .setPartitions(Collections.singletonList(1))
+                                        )))
+                                    .setMemberType((byte) 1),
+                                new ConsumerGroupDescribeResponseData.Member()
+                                    .setMemberId("memberId-classic")
+                                    .setInstanceId("instanceId-classic")
+                                    .setClientHost("host")
+                                    .setClientId("clientId-classic")
+                                    .setMemberEpoch(9)
+                                    .setRackId("rackid")
+                                    .setSubscribedTopicNames(singletonList("bar"))
+                                    .setSubscribedTopicRegex("regex")
+                                    .setAssignment(new ConsumerGroupDescribeResponseData.Assignment()
+                                        .setTopicPartitions(List.of(
+                                            new ConsumerGroupDescribeResponseData.TopicPartitions()
+                                                .setTopicId(Uuid.randomUuid())
+                                                .setTopicName("bar")
+                                                .setPartitions(Collections.singletonList(0))
+                                        )))
+                                    .setTargetAssignment(new ConsumerGroupDescribeResponseData.Assignment()
+                                        .setTopicPartitions(List.of(
                                             new ConsumerGroupDescribeResponseData.TopicPartitions()
                                                 .setTopicId(Uuid.randomUuid())
                                                 .setTopicName("bar")
                                                 .setPartitions(Collections.singletonList(1))
                                         )))
-                                    .setTargetAssignment(new ConsumerGroupDescribeResponseData.Assignment()
-                                        .setTopicPartitions(Arrays.asList(
-                                            new ConsumerGroupDescribeResponseData.TopicPartitions()
-                                                .setTopicId(Uuid.randomUuid())
-                                                .setTopicName("foo")
-                                                .setPartitions(Collections.singletonList(1)),
-                                            new ConsumerGroupDescribeResponseData.TopicPartitions()
-                                                .setTopicId(Uuid.randomUuid())
-                                                .setTopicName("bar")
-                                                .setPartitions(Collections.singletonList(2))
-                                        )))
+                                    .setMemberType((byte) 0)
                             ))
                     ))
             )
@@ -233,9 +269,13 @@ public class DescribeConsumerGroupsHandlerTest {
     public void testSuccessfulHandleClassicGroupResponse() {
         Collection<MemberDescription> members = singletonList(new MemberDescription(
                 "memberId",
+                Optional.empty(),
                 "clientId",
                 "host",
-                new MemberAssignment(tps)));
+                new MemberAssignment(tps),
+                Optional.empty(),
+                Optional.empty(),
+                Optional.empty()));
         ConsumerGroupDescription expected = new ConsumerGroupDescription(
                 groupId1,
                 true,
@@ -282,6 +322,7 @@ public class DescribeConsumerGroupsHandlerTest {
     @Test
     public void testFailedHandleConsumerGroupResponse() {
         assertFailed(GroupAuthorizationException.class, handleConsumerGroupWithError(Errors.GROUP_AUTHORIZATION_FAILED));
+        assertFailed(TopicAuthorizationException.class, handleConsumerGroupWithError(Errors.TOPIC_AUTHORIZATION_FAILED));
         assertFailed(InvalidGroupIdException.class, handleConsumerGroupWithError(Errors.INVALID_GROUP_ID));
     }
 
