@@ -36,7 +36,6 @@ import static java.util.Arrays.asList;
 import static java.util.Collections.emptyList;
 import static java.util.Collections.emptyMap;
 import static java.util.Collections.singletonList;
-import static org.apache.kafka.common.utils.Utils.mkSet;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 
@@ -50,12 +49,12 @@ public class DescribeTransactionsHandlerTest {
         String transactionalId2 = "bar";
         String transactionalId3 = "baz";
 
-        Set<String> transactionalIds = mkSet(transactionalId1, transactionalId2, transactionalId3);
+        Set<String> transactionalIds = Set.of(transactionalId1, transactionalId2, transactionalId3);
         DescribeTransactionsHandler handler = new DescribeTransactionsHandler(logContext);
 
         assertLookup(handler, transactionalIds);
-        assertLookup(handler, mkSet(transactionalId1));
-        assertLookup(handler, mkSet(transactionalId2, transactionalId3));
+        assertLookup(handler, Set.of(transactionalId1));
+        assertLookup(handler, Set.of(transactionalId2, transactionalId3));
     }
 
     @Test
@@ -63,7 +62,7 @@ public class DescribeTransactionsHandlerTest {
         String transactionalId1 = "foo";
         String transactionalId2 = "bar";
 
-        Set<String> transactionalIds = mkSet(transactionalId1, transactionalId2);
+        Set<String> transactionalIds = Set.of(transactionalId1, transactionalId2);
         DescribeTransactionsHandler handler = new DescribeTransactionsHandler(logContext);
 
         DescribeTransactionsResponseData.TransactionState transactionState1 =
@@ -105,7 +104,7 @@ public class DescribeTransactionsHandlerTest {
         CoordinatorKey key = CoordinatorKey.byTransactionalId(transactionalId);
         ApiResult<CoordinatorKey, TransactionDescription> result = handleResponseError(handler, transactionalId, error);
         assertEquals(emptyList(), result.unmappedKeys);
-        assertEquals(mkSet(key), result.failedKeys.keySet());
+        assertEquals(Set.of(key), result.failedKeys.keySet());
 
         Throwable throwable = result.failedKeys.get(key);
         assertInstanceOf(error.exception().getClass(), throwable);
@@ -138,7 +137,7 @@ public class DescribeTransactionsHandlerTest {
         Errors error
     ) {
         CoordinatorKey key = CoordinatorKey.byTransactionalId(transactionalId);
-        Set<CoordinatorKey> keys = mkSet(key);
+        Set<CoordinatorKey> keys = Set.of(key);
 
         DescribeTransactionsResponseData.TransactionState transactionState = new DescribeTransactionsResponseData.TransactionState()
             .setErrorCode(error.code())
