@@ -48,13 +48,13 @@ import static java.util.Collections.emptyList;
 import static java.util.Collections.emptyMap;
 import static java.util.Collections.emptySet;
 import static java.util.Collections.singleton;
-import static org.apache.kafka.common.utils.Utils.mkSet;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
 public final class ListOffsetsHandlerTest {
 
     private final LogContext logContext = new LogContext();
+    private final int defaultApiTimeoutMs = 100;
 
     private final TopicPartition t0p0 = new TopicPartition("t0", 0);
     private final TopicPartition t0p1 = new TopicPartition("t0", 1);
@@ -65,7 +65,7 @@ public final class ListOffsetsHandlerTest {
 
     private final Node node = new Node(1, "host", 1234);
 
-    private final Map<TopicPartition, Long> offsetTimestampsByPartition = new HashMap<TopicPartition, Long>() {
+    private final Map<TopicPartition, Long> offsetTimestampsByPartition = new HashMap<>() {
         {
             put(t0p0, ListOffsetsRequest.LATEST_TIMESTAMP);
             put(t0p1, ListOffsetsRequest.EARLIEST_TIMESTAMP);
@@ -79,8 +79,8 @@ public final class ListOffsetsHandlerTest {
     @Test
     public void testBuildRequestSimple() {
         ListOffsetsHandler handler =
-            new ListOffsetsHandler(offsetTimestampsByPartition, new ListOffsetsOptions(), logContext);
-        ListOffsetsRequest request = handler.buildBatchedRequest(node.id(), mkSet(t0p0, t0p1)).build();
+            new ListOffsetsHandler(offsetTimestampsByPartition, new ListOffsetsOptions(), logContext, defaultApiTimeoutMs);
+        ListOffsetsRequest request = handler.buildBatchedRequest(node.id(), Set.of(t0p0, t0p1)).build();
         List<ListOffsetsTopic> topics = request.topics();
         assertEquals(1, topics.size());
         ListOffsetsTopic topic = topics.get(0);
@@ -96,7 +96,7 @@ public final class ListOffsetsHandlerTest {
     public void testBuildRequestMultipleTopicsWithReadCommitted() {
         ListOffsetsHandler handler =
             new ListOffsetsHandler(
-                offsetTimestampsByPartition, new ListOffsetsOptions(IsolationLevel.READ_COMMITTED), logContext);
+                offsetTimestampsByPartition, new ListOffsetsOptions(IsolationLevel.READ_COMMITTED), logContext, defaultApiTimeoutMs);
         ListOffsetsRequest request =
             handler.buildBatchedRequest(node.id(), offsetTimestampsByPartition.keySet()).build();
         List<ListOffsetsTopic> topics = request.topics();
@@ -117,24 +117,24 @@ public final class ListOffsetsHandlerTest {
     @Test
     public void testBuildRequestAllowedVersions() {
         ListOffsetsHandler defaultOptionsHandler =
-            new ListOffsetsHandler(offsetTimestampsByPartition, new ListOffsetsOptions(), logContext);
+            new ListOffsetsHandler(offsetTimestampsByPartition, new ListOffsetsOptions(), logContext, defaultApiTimeoutMs);
         ListOffsetsRequest.Builder builder =
-            defaultOptionsHandler.buildBatchedRequest(node.id(), mkSet(t0p0, t0p1, t1p0));
+            defaultOptionsHandler.buildBatchedRequest(node.id(), Set.of(t0p0, t0p1, t1p0));
         assertEquals(1, builder.oldestAllowedVersion());
 
         ListOffsetsHandler readCommittedHandler =
             new ListOffsetsHandler(
-                offsetTimestampsByPartition, new ListOffsetsOptions(IsolationLevel.READ_COMMITTED), logContext);
-        builder = readCommittedHandler.buildBatchedRequest(node.id(), mkSet(t0p0, t0p1, t1p0));
+                offsetTimestampsByPartition, new ListOffsetsOptions(IsolationLevel.READ_COMMITTED), logContext, defaultApiTimeoutMs);
+        builder = readCommittedHandler.buildBatchedRequest(node.id(), Set.of(t0p0, t0p1, t1p0));
         assertEquals(2, builder.oldestAllowedVersion());
 
-        builder = readCommittedHandler.buildBatchedRequest(node.id(), mkSet(t0p0, t0p1, t1p0, t1p1));
+        builder = readCommittedHandler.buildBatchedRequest(node.id(), Set.of(t0p0, t0p1, t1p0, t1p1));
         assertEquals(7, builder.oldestAllowedVersion());
 
-        builder = readCommittedHandler.buildBatchedRequest(node.id(), mkSet(t0p0, t0p1, t1p0, t1p1, t2p0));
+        builder = readCommittedHandler.buildBatchedRequest(node.id(), Set.of(t0p0, t0p1, t1p0, t1p1, t2p0));
         assertEquals(8, builder.oldestAllowedVersion());
 
-        builder = readCommittedHandler.buildBatchedRequest(node.id(), mkSet(t0p0, t0p1, t1p0, t1p1, t2p0, t2p1));
+        builder = readCommittedHandler.buildBatchedRequest(node.id(), Set.of(t0p0, t0p1, t1p0, t1p1, t2p0, t2p1));
         assertEquals(9, builder.oldestAllowedVersion());
     }
 
@@ -224,7 +224,7 @@ public final class ListOffsetsHandlerTest {
         maxTimestampPartitions.put(t1p1, OffsetSpec.maxTimestamp());
 
         ListOffsetsHandler handler =
-            new ListOffsetsHandler(offsetTimestampsByPartition, new ListOffsetsOptions(), logContext);
+            new ListOffsetsHandler(offsetTimestampsByPartition, new ListOffsetsOptions(), logContext, defaultApiTimeoutMs);
 
         final Map<TopicPartition, Long> nonMaxTimestampPartitions = new HashMap<>(offsetTimestampsByPartition);
         maxTimestampPartitions.forEach((k, v) -> nonMaxTimestampPartitions.remove(k));
@@ -252,6 +252,25 @@ public final class ListOffsetsHandlerTest {
             handler.handleUnsupportedVersionException(brokerId, uve, keysToTest));
     }
 
+    @Test
+    public void testBuildRequestWithDefaultApiTimeoutMs() {
+        ListOffsetsOptions options = new ListOffsetsOptions();
+        ListOffsetsHandler handler =
+                new ListOffsetsHandler(offsetTimestampsByPartition, options, logContext, defaultApiTimeoutMs);
+        ListOffsetsRequest request = handler.buildBatchedRequest(node.id(), Set.of(t0p0, t0p1)).build();
+        assertEquals(defaultApiTimeoutMs, request.timeoutMs());
+    }
+
+    @Test
+    public void testBuildRequestWithTimeoutMs() {
+        Integer timeoutMs = 200;
+        ListOffsetsOptions options = new ListOffsetsOptions().timeoutMs(timeoutMs);
+        ListOffsetsHandler handler =
+                new ListOffsetsHandler(offsetTimestampsByPartition, options, logContext, defaultApiTimeoutMs);
+        ListOffsetsRequest request = handler.buildBatchedRequest(node.id(), Set.of(t0p0, t0p1)).build();
+        assertEquals(timeoutMs, request.timeoutMs());
+    }
+
     private static Map<TopicPartition, Throwable> mapToError(Set<TopicPartition> keys, Throwable t) {
         return keys.stream().collect(Collectors.toMap(k -> k, k -> t));
     }
@@ -288,7 +307,7 @@ public final class ListOffsetsHandlerTest {
 
     private ApiResult<TopicPartition, ListOffsetsResultInfo> handleResponse(ListOffsetsResponse response) {
         ListOffsetsHandler handler =
-            new ListOffsetsHandler(offsetTimestampsByPartition, new ListOffsetsOptions(), logContext);
+            new ListOffsetsHandler(offsetTimestampsByPartition, new ListOffsetsOptions(), logContext, defaultApiTimeoutMs);
         return handler.handleResponse(node, offsetTimestampsByPartition.keySet(), response);
     }
 
