@@ -52,6 +52,7 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Optional;
 import java.util.Set;
 import java.util.UUID;
 
@@ -68,11 +69,11 @@ public class ShareCompletedFetchTest {
 
     @Test
     public void testSimple() {
-        long firstMessageId = 5;
         long startingOffset = 10L;
-        int numRecords = 11;        // Records for 10-20
+        int numRecordsPerBatch = 10;
+        int numRecords = 20;        // Records for 10-29, in 2 equal batches
         ShareFetchResponseData.PartitionData partitionData = new ShareFetchResponseData.PartitionData()
-                .setRecords(newRecords(startingOffset, numRecords, firstMessageId))
+                .setRecords(newRecords(startingOffset, numRecordsPerBatch, 2))
                 .setAcquiredRecords(acquiredRecords(startingOffset, numRecords));
 
         Deserializers<String, String> deserializers = newStringDeserializers();
@@ -84,14 +85,16 @@ public class ShareCompletedFetchTest {
         assertEquals(10, records.size());
         ConsumerRecord<String, String> record = records.get(0);
         assertEquals(10L, record.offset());
+        assertEquals(Optional.of((short) 1), record.deliveryCount());
         Acknowledgements acknowledgements = batch.getAcknowledgements();
         assertEquals(0, acknowledgements.size());
 
         batch = completedFetch.fetchRecords(deserializers, 10, true);
         records = batch.getInFlightRecords();
-        assertEquals(1, records.size());
+        assertEquals(10, records.size());
         record = records.get(0);
         assertEquals(20L, record.offset());
+        assertEquals(Optional.of((short) 1), record.deliveryCount());
         acknowledgements = batch.getAcknowledgements();
         assertEquals(0, acknowledgements.size());
 
@@ -102,13 +105,40 @@ public class ShareCompletedFetchTest {
         assertEquals(0, acknowledgements.size());
     }
 
+    @Test
+    public void testSoftMaxPollRecordLimit() {
+        long startingOffset = 10L;
+        int numRecords = 11;        // Records for 10-20, in a single batch
+        ShareFetchResponseData.PartitionData partitionData = new ShareFetchResponseData.PartitionData()
+            .setRecords(newRecords(startingOffset, numRecords))
+            .setAcquiredRecords(acquiredRecords(startingOffset, numRecords));
+
+        Deserializers<String, String> deserializers = newStringDeserializers();
+
+        ShareCompletedFetch completedFetch = newShareCompletedFetch(partitionData);
+
+        ShareInFlightBatch<String, String> batch = completedFetch.fetchRecords(deserializers, 10, true);
+        List<ConsumerRecord<String, String>> records = batch.getInFlightRecords();
+        assertEquals(11, records.size());
+        ConsumerRecord<String, String> record = records.get(0);
+        assertEquals(10L, record.offset());
+        assertEquals(Optional.of((short) 1), record.deliveryCount());
+        Acknowledgements acknowledgements = batch.getAcknowledgements();
+        assertEquals(0, acknowledgements.size());
+
+        batch = completedFetch.fetchRecords(deserializers, 10, true);
+        records = batch.getInFlightRecords();
+        assertEquals(0, records.size());
+        acknowledgements = batch.getAcknowledgements();
+        assertEquals(0, acknowledgements.size());
+    }
+
     @Test
     public void testUnaligned() {
-        long firstMessageId = 5;
         long startingOffset = 10L;
         int numRecords = 10;
         ShareFetchResponseData.PartitionData partitionData = new ShareFetchResponseData.PartitionData()
-                .setRecords(newRecords(startingOffset, numRecords + 500, firstMessageId))
+                .setRecords(newRecords(startingOffset, numRecords + 500))
                 .setAcquiredRecords(acquiredRecords(startingOffset + 500, numRecords));
 
         Deserializers<String, String> deserializers = newStringDeserializers();
@@ -120,6 +150,7 @@ public class ShareCompletedFetchTest {
         assertEquals(10, records.size());
         ConsumerRecord<String, String> record = records.get(0);
         assertEquals(510L, record.offset());
+        assertEquals(Optional.of((short) 1), record.deliveryCount());
         Acknowledgements acknowledgements = batch.getAcknowledgements();
         assertEquals(0, acknowledgements.size());
 
@@ -149,11 +180,10 @@ public class ShareCompletedFetchTest {
 
     @Test
     public void testNegativeFetchCount() {
-        long firstMessageId = 0;
         int startingOffset = 0;
         int numRecords = 10;
         ShareFetchResponseData.PartitionData partitionData = new ShareFetchResponseData.PartitionData()
-                .setRecords(newRecords(startingOffset, numRecords, firstMessageId))
+                .setRecords(newRecords(startingOffset, numRecords))
                 .setAcquiredRecords(acquiredRecords(0L, 10));
 
         try (final Deserializers<String, String> deserializers = newStringDeserializers()) {
@@ -263,7 +293,6 @@ public class ShareCompletedFetchTest {
 
     @Test
     public void testAcquiredRecords() {
-        long firstMessageId = 5;
         int startingOffset = 0;
         int numRecords = 10;        // Records for 0-9
 
@@ -271,7 +300,7 @@ public class ShareCompletedFetchTest {
         List<ShareFetchResponseData.AcquiredRecords> acquiredRecords = new ArrayList<>(acquiredRecords(0L, 3));
         acquiredRecords.addAll(acquiredRecords(6L, 3));
         ShareFetchResponseData.PartitionData partitionData = new ShareFetchResponseData.PartitionData()
-                .setRecords(newRecords(startingOffset, numRecords, firstMessageId))
+                .setRecords(newRecords(startingOffset, numRecords))
                 .setAcquiredRecords(acquiredRecords);
 
         Deserializers<String, String> deserializers = newStringDeserializers();
@@ -283,9 +312,11 @@ public class ShareCompletedFetchTest {
         // The first offset should be 0
         ConsumerRecord<String, String> record = records.get(0);
         assertEquals(0L, record.offset());
+        assertEquals(Optional.of((short) 1), record.deliveryCount());
         // The third offset should be 6
         record = records.get(3);
         assertEquals(6L, record.offset());
+        assertEquals(Optional.of((short) 1), record.deliveryCount());
 
         records = completedFetch.fetchRecords(deserializers, 10, true).getInFlightRecords();
         assertEquals(0, records.size());
@@ -293,7 +324,6 @@ public class ShareCompletedFetchTest {
 
     @Test
     public void testAcquireOddRecords() {
-        long firstMessageId = 5;
         int startingOffset = 0;
         int numRecords = 10;        // Records for 0-9
 
@@ -304,7 +334,7 @@ public class ShareCompletedFetchTest {
         }
 
         ShareFetchResponseData.PartitionData partitionData = new ShareFetchResponseData.PartitionData()
-                .setRecords(newRecords(startingOffset, numRecords, firstMessageId))
+                .setRecords(newRecords(startingOffset, numRecords))
                 .setAcquiredRecords(acquiredRecords);
 
         Deserializers<String, String> deserializers = newStringDeserializers();
@@ -316,9 +346,11 @@ public class ShareCompletedFetchTest {
         // The first offset should be 1
         ConsumerRecord<String, String> record = records.get(0);
         assertEquals(1L, record.offset());
+        assertEquals(Optional.of((short) 1), record.deliveryCount());
         // The second offset should be 3
         record = records.get(1);
         assertEquals(3L, record.offset());
+        assertEquals(Optional.of((short) 1), record.deliveryCount());
 
         records = completedFetch.fetchRecords(deserializers, 10, true).getInFlightRecords();
         assertEquals(0, records.size());
@@ -349,17 +381,45 @@ public class ShareCompletedFetchTest {
         return new Deserializers<>(new StringDeserializer(), new StringDeserializer());
     }
 
-    private Records newRecords(long baseOffset, int count, long firstMessageId) {
+    private Records newRecords(long baseOffset, int count) {
         try (final MemoryRecordsBuilder builder = MemoryRecords.builder(ByteBuffer.allocate(1024),
                 Compression.NONE,
                 TimestampType.CREATE_TIME,
                 baseOffset)) {
             for (int i = 0; i < count; i++)
-                builder.append(0L, "key".getBytes(), ("value-" + (firstMessageId + i)).getBytes());
+                builder.append(0L, "key".getBytes(), "value-".getBytes());
             return builder.build();
         }
     }
 
+    private Records newRecords(long baseOffset, int numRecordsPerBatch, int batchCount) {
+        Time time = new MockTime();
+        ByteBuffer buffer = ByteBuffer.allocate(1024);
+
+        for (long b = 0; b < batchCount; b++) {
+            try (MemoryRecordsBuilder builder = MemoryRecords.builder(buffer,
+                RecordBatch.CURRENT_MAGIC_VALUE,
+                Compression.NONE,
+                TimestampType.CREATE_TIME,
+                baseOffset + b * numRecordsPerBatch,
+                time.milliseconds(),
+                PRODUCER_ID,
+                PRODUCER_EPOCH,
+                0,
+                true,
+                RecordBatch.NO_PARTITION_LEADER_EPOCH)) {
+                for (int i = 0; i < numRecordsPerBatch; i++)
+                    builder.append(new SimpleRecord(time.milliseconds(), "key".getBytes(), "value".getBytes()));
+
+                builder.build();
+            }
+        }
+
+        buffer.flip();
+
+        return MemoryRecords.readableRecords(buffer);
+    }
+
     public static List<ShareFetchResponseData.AcquiredRecords> acquiredRecords(long firstOffset, int count) {
         ShareFetchResponseData.AcquiredRecords acquiredRecords = new ShareFetchResponseData.AcquiredRecords()
                 .setFirstOffset(firstOffset)
