@@ -59,7 +59,7 @@ public class MockProducerTest {
     private final String groupId = "group";
 
     private void buildMockProducer(boolean autoComplete) {
-        this.producer = new MockProducer<>(autoComplete, new MockSerializer(), new MockSerializer());
+        this.producer = new MockProducer<>(Cluster.empty(), autoComplete, null, new MockSerializer(), new MockSerializer());
     }
 
     @AfterEach
@@ -87,10 +87,16 @@ public class MockProducerTest {
         PartitionInfo partitionInfo1 = new PartitionInfo(topic, 1, null, null, null);
         Cluster cluster = new Cluster(null, new ArrayList<>(0), asList(partitionInfo0, partitionInfo1),
                 Collections.emptySet(), Collections.emptySet());
-        MockProducer<String, String> producer = new MockProducer<>(cluster, true, new StringSerializer(), new StringSerializer());
+        MockProducer<String, String> producer = new MockProducer<>(
+            cluster,
+            true,
+            new org.apache.kafka.clients.producer.RoundRobinPartitioner(),
+            new StringSerializer(),
+            new StringSerializer()
+        );
         ProducerRecord<String, String> record = new ProducerRecord<>(topic, "key", "value");
         Future<RecordMetadata> metadata = producer.send(record);
-        assertEquals(1, metadata.get().partition(), "Partition should be correct");
+        assertEquals(0, metadata.get().partition(), "Partition should be correct");
         producer.clear();
         assertEquals(0, producer.history().size(), "Clear should erase our history");
         producer.close();
@@ -393,14 +399,14 @@ public class MockProducerTest {
         producer.beginTransaction();
 
         String group1 = "g1";
-        Map<TopicPartition, OffsetAndMetadata> group1Commit = new HashMap<TopicPartition, OffsetAndMetadata>() {
+        Map<TopicPartition, OffsetAndMetadata> group1Commit = new HashMap<>() {
             {
                 put(new TopicPartition(topic, 0), new OffsetAndMetadata(42L, null));
                 put(new TopicPartition(topic, 1), new OffsetAndMetadata(73L, null));
             }
         };
         String group2 = "g2";
-        Map<TopicPartition, OffsetAndMetadata> group2Commit = new HashMap<TopicPartition, OffsetAndMetadata>() {
+        Map<TopicPartition, OffsetAndMetadata> group2Commit = new HashMap<>() {
             {
                 put(new TopicPartition(topic, 0), new OffsetAndMetadata(101L, null));
                 put(new TopicPartition(topic, 1), new OffsetAndMetadata(21L, null));
@@ -419,15 +425,6 @@ public class MockProducerTest {
         assertEquals(Collections.singletonList(expectedResult), producer.consumerGroupOffsetsHistory());
     }
 
-    @Deprecated
-    @Test
-    public void shouldThrowOnNullConsumerGroupIdWhenSendOffsetsToTransaction() {
-        buildMockProducer(true);
-        producer.initTransactions();
-        producer.beginTransaction();
-        assertThrows(NullPointerException.class, () -> producer.sendOffsetsToTransaction(Collections.emptyMap(), (String) null));
-    }
-
     @Test
     public void shouldThrowOnNullConsumerGroupMetadataWhenSendOffsetsToTransaction() {
         buildMockProducer(true);
@@ -436,16 +433,6 @@ public class MockProducerTest {
         assertThrows(NullPointerException.class, () -> producer.sendOffsetsToTransaction(Collections.emptyMap(), new ConsumerGroupMetadata(null)));
     }
 
-    @Deprecated
-    @Test
-    public void shouldIgnoreEmptyOffsetsWhenSendOffsetsToTransactionByGroupId() {
-        buildMockProducer(true);
-        producer.initTransactions();
-        producer.beginTransaction();
-        producer.sendOffsetsToTransaction(Collections.emptyMap(), "groupId");
-        assertFalse(producer.sentOffsets());
-    }
-
     @Test
     public void shouldIgnoreEmptyOffsetsWhenSendOffsetsToTransactionByGroupMetadata() {
         buildMockProducer(true);
@@ -454,25 +441,7 @@ public class MockProducerTest {
         producer.sendOffsetsToTransaction(Collections.emptyMap(), new ConsumerGroupMetadata("groupId"));
         assertFalse(producer.sentOffsets());
     }
-
-    @Deprecated
-    @Test
-    public void shouldAddOffsetsWhenSendOffsetsToTransactionByGroupId() {
-        buildMockProducer(true);
-        producer.initTransactions();
-        producer.beginTransaction();
-
-        assertFalse(producer.sentOffsets());
-
-        Map<TopicPartition, OffsetAndMetadata> groupCommit = new HashMap<TopicPartition, OffsetAndMetadata>() {
-            {
-                put(new TopicPartition(topic, 0), new OffsetAndMetadata(42L, null));
-            }
-        };
-        producer.sendOffsetsToTransaction(groupCommit, "groupId");
-        assertTrue(producer.sentOffsets());
-    }
-
+    
     @Test
     public void shouldAddOffsetsWhenSendOffsetsToTransactionByGroupMetadata() {
         buildMockProducer(true);
@@ -481,7 +450,7 @@ public class MockProducerTest {
 
         assertFalse(producer.sentOffsets());
 
-        Map<TopicPartition, OffsetAndMetadata> groupCommit = new HashMap<TopicPartition, OffsetAndMetadata>() {
+        Map<TopicPartition, OffsetAndMetadata> groupCommit = new HashMap<>() {
             {
                 put(new TopicPartition(topic, 0), new OffsetAndMetadata(42L, null));
             }
@@ -498,7 +467,7 @@ public class MockProducerTest {
 
         assertFalse(producer.sentOffsets());
 
-        Map<TopicPartition, OffsetAndMetadata> groupCommit = new HashMap<TopicPartition, OffsetAndMetadata>() {
+        Map<TopicPartition, OffsetAndMetadata> groupCommit = new HashMap<>() {
             {
                 put(new TopicPartition(topic, 0), new OffsetAndMetadata(42L, null));
             }
@@ -525,13 +494,13 @@ public class MockProducerTest {
         producer.beginTransaction();
 
         String group = "g";
-        Map<TopicPartition, OffsetAndMetadata> groupCommit1 = new HashMap<TopicPartition, OffsetAndMetadata>() {
+        Map<TopicPartition, OffsetAndMetadata> groupCommit1 = new HashMap<>() {
             {
                 put(new TopicPartition(topic, 0), new OffsetAndMetadata(42L, null));
                 put(new TopicPartition(topic, 1), new OffsetAndMetadata(73L, null));
             }
         };
-        Map<TopicPartition, OffsetAndMetadata> groupCommit2 = new HashMap<TopicPartition, OffsetAndMetadata>() {
+        Map<TopicPartition, OffsetAndMetadata> groupCommit2 = new HashMap<>() {
             {
                 put(new TopicPartition(topic, 1), new OffsetAndMetadata(101L, null));
                 put(new TopicPartition(topic, 2), new OffsetAndMetadata(21L, null));
@@ -543,7 +512,7 @@ public class MockProducerTest {
         assertTrue(producer.consumerGroupOffsetsHistory().isEmpty());
 
         Map<String, Map<TopicPartition, OffsetAndMetadata>> expectedResult = new HashMap<>();
-        expectedResult.put(group, new HashMap<TopicPartition, OffsetAndMetadata>() {
+        expectedResult.put(group, new HashMap<>() {
             {
                 put(new TopicPartition(topic, 0), new OffsetAndMetadata(42L, null));
                 put(new TopicPartition(topic, 1), new OffsetAndMetadata(101L, null));
@@ -562,7 +531,7 @@ public class MockProducerTest {
         producer.beginTransaction();
 
         String group = "g";
-        Map<TopicPartition, OffsetAndMetadata> groupCommit = new HashMap<TopicPartition, OffsetAndMetadata>() {
+        Map<TopicPartition, OffsetAndMetadata> groupCommit = new HashMap<>() {
             {
                 put(new TopicPartition(topic, 0), new OffsetAndMetadata(42L, null));
                 put(new TopicPartition(topic, 1), new OffsetAndMetadata(73L, null));
@@ -591,7 +560,7 @@ public class MockProducerTest {
         producer.beginTransaction();
 
         String group = "g";
-        Map<TopicPartition, OffsetAndMetadata> groupCommit = new HashMap<TopicPartition, OffsetAndMetadata>() {
+        Map<TopicPartition, OffsetAndMetadata> groupCommit = new HashMap<>() {
             {
                 put(new TopicPartition(topic, 0), new OffsetAndMetadata(42L, null));
                 put(new TopicPartition(topic, 1), new OffsetAndMetadata(73L, null));
@@ -616,7 +585,7 @@ public class MockProducerTest {
         producer.beginTransaction();
 
         String group = "g";
-        Map<TopicPartition, OffsetAndMetadata> groupCommit = new HashMap<TopicPartition, OffsetAndMetadata>() {
+        Map<TopicPartition, OffsetAndMetadata> groupCommit = new HashMap<>() {
             {
                 put(new TopicPartition(topic, 0), new OffsetAndMetadata(42L, null));
                 put(new TopicPartition(topic, 1), new OffsetAndMetadata(73L, null));
@@ -628,7 +597,7 @@ public class MockProducerTest {
         producer.beginTransaction();
 
         String group2 = "g2";
-        Map<TopicPartition, OffsetAndMetadata> groupCommit2 = new HashMap<TopicPartition, OffsetAndMetadata>() {
+        Map<TopicPartition, OffsetAndMetadata> groupCommit2 = new HashMap<>() {
             {
                 put(new TopicPartition(topic, 2), new OffsetAndMetadata(53L, null));
                 put(new TopicPartition(topic, 3), new OffsetAndMetadata(84L, null));
@@ -717,7 +686,7 @@ public class MockProducerTest {
     @Test
     @SuppressWarnings("unchecked")
     public void shouldThrowClassCastException() {
-        try (MockProducer<Integer, String> customProducer = new MockProducer<>(true, new IntegerSerializer(), new StringSerializer())) {
+        try (MockProducer<Integer, String> customProducer = new MockProducer<>(Cluster.empty(), true, null, new IntegerSerializer(), new StringSerializer())) {
             assertThrows(ClassCastException.class, () -> customProducer.send(new ProducerRecord(topic, "key1", "value1")));
         }
     }
