@@ -29,7 +29,6 @@ import org.apache.kafka.common.security.TestSecurityConfig;
 import org.apache.kafka.common.security.auth.SecurityProtocol;
 import org.apache.kafka.common.security.ssl.DefaultSslEngineFactory;
 import org.apache.kafka.common.security.ssl.SslFactory;
-import org.apache.kafka.common.utils.Java;
 import org.apache.kafka.common.utils.LogContext;
 import org.apache.kafka.common.utils.Time;
 import org.apache.kafka.common.utils.Utils;
@@ -56,7 +55,6 @@ import java.nio.channels.SelectionKey;
 import java.nio.channels.SocketChannel;
 import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
@@ -80,7 +78,6 @@ import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
-import static org.junit.jupiter.api.Assumptions.assumeTrue;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.Mockito.doReturn;
 import static org.mockito.Mockito.doThrow;
@@ -147,13 +144,19 @@ public class SslTransportLayerTest {
             List<Arguments> parameters = new ArrayList<>();
             parameters.add(Arguments.of(new Args("TLSv1.2", false)));
             parameters.add(Arguments.of(new Args("TLSv1.2", true)));
-            if (Java.IS_JAVA11_COMPATIBLE) {
-                parameters.add(Arguments.of(new Args("TLSv1.3", false)));
-            }
+            parameters.add(Arguments.of(new Args("TLSv1.3", false)));
             return parameters.stream();
         }
     }
 
+    private static class SslTransportLayerArgumentsForTLS2Provider extends SslTransportLayerArgumentsProvider {
+
+        @Override
+        public Stream<? extends Arguments> provideArguments(ExtensionContext context) throws Exception {
+            return super.provideArguments(context).filter(arg -> ((Args) arg.get()[0]).tlsProtocol.equals("TLSv1.2"));
+        }
+    }
+
     private NioEchoServer server;
     private Selector selector;
 
@@ -469,12 +472,9 @@ public class SslTransportLayerTest {
      * Tests key-pair created using DSA.
      */
     @ParameterizedTest
-    @ArgumentsSource(SslTransportLayerArgumentsProvider.class)
+    @ArgumentsSource(SslTransportLayerArgumentsForTLS2Provider.class)
     public void testDsaKeyPair(Args args) throws Exception {
         // DSA algorithms are not supported for TLSv1.3.
-        assumeTrue(args.tlsProtocol.equals("TLSv1.2"));
-        // Skip test if DSA is not supported by the JVM
-        assumeTrue(isDsaSupported(), "DSA algorithm is not supported by this JVM");
         args.serverCertStores = certBuilder(true, "server", args.useInlinePem).keyAlgorithm("DSA").build();
         args.clientCertStores = certBuilder(false, "client", args.useInlinePem).keyAlgorithm("DSA").build();
         args.sslServerConfigs = args.getTrustingConfig(args.serverCertStores, args.clientCertStores);
@@ -483,39 +483,6 @@ public class SslTransportLayerTest {
         verifySslConfigs(args);
     }
 
-    /**
-     * Check if DSA algorithm is supported by the JVM and if there are compatible cipher suites
-     * available for TLSv1.2. This is important because even if DSA KeyPairGenerator is available,
-     * the SSL handshake may fail if no DSA-compatible cipher suites are available.
-     * @return true if DSA KeyPairGenerator is available and DSA-compatible cipher suites exist, false otherwise
-     */
-    private static boolean isDsaSupported() {
-        // First check if DSA KeyPairGenerator is available
-        try {
-            java.security.KeyPairGenerator.getInstance("DSA");
-        } catch (java.security.NoSuchAlgorithmException e) {
-            return false;
-        }
-
-        // Check if there are DSA-compatible cipher suites available for TLSv1.2
-        // DSA algorithms are not supported for TLSv1.3, so we only check TLSv1.2
-        try {
-            SSLContext context = SSLContext.getInstance("TLSv1.2");
-            context.init(null, null, null);
-            SSLParameters params = context.getDefaultSSLParameters();
-            String[] cipherSuites = params.getCipherSuites();
-
-            // Check if any cipher suite supports DSA
-            // In TLS standards and JVM implementations, DSA signature cipher suites use "_DSS_" naming
-            // Common patterns: TLS_DHE_DSS_*, TLS_DH_DSS_*, SSL_DHE_DSS_*, SSL_DH_DSS_*
-            return Arrays.stream(cipherSuites)
-                    .anyMatch(suite -> suite.contains("_DSS_"));
-        } catch (Exception e) {
-            // If we can't check cipher suites, assume DSA is not fully supported
-            return false;
-        }
-    }
-
     /**
      * Tests key-pair created using EC.
      */
@@ -1389,7 +1356,7 @@ public class SslTransportLayerTest {
     }
 
     private Function<Short, ApiVersionsResponse> defaultApiVersionsSupplier() {
-        return version -> TestUtils.defaultApiVersionsResponse(ApiMessageType.ListenerType.ZK_BROKER);
+        return version -> TestUtils.defaultApiVersionsResponse(ApiMessageType.ListenerType.BROKER);
     }
 
     static class TestSslChannelBuilder extends SslChannelBuilder {
