@@ -62,32 +62,6 @@ public class DeleteAclsRequestTest {
         assertThrows(IllegalArgumentException.class, () -> new DeleteAclsRequest.Builder(requestData(UNKNOWN_FILTER)).build(V1));
     }
 
-    @Test
-    public void shouldRoundTripLiteralV0() {
-        final DeleteAclsRequest original = new DeleteAclsRequest.Builder(requestData(LITERAL_FILTER)).build(V0);
-        final ByteBuffer buffer = original.serialize();
-
-        final DeleteAclsRequest result = DeleteAclsRequest.parse(buffer, V0);
-
-        assertRequestEquals(original, result);
-    }
-
-    @Test
-    public void shouldRoundTripAnyV0AsLiteral() {
-        final DeleteAclsRequest original = new DeleteAclsRequest.Builder(requestData(ANY_FILTER)).build(V0);
-        final DeleteAclsRequest expected = new DeleteAclsRequest.Builder(requestData(
-            new AclBindingFilter(new ResourcePatternFilter(
-                ANY_FILTER.patternFilter().resourceType(),
-                ANY_FILTER.patternFilter().name(),
-                PatternType.LITERAL),
-                ANY_FILTER.entryFilter()))
-        ).build(V0);
-
-        final DeleteAclsRequest result = DeleteAclsRequest.parse(original.serialize(), V0);
-
-        assertRequestEquals(expected, result);
-    }
-
     @Test
     public void shouldRoundTripV1() {
         final DeleteAclsRequest original = new DeleteAclsRequest.Builder(
