@@ -25,7 +25,6 @@ import org.apache.kafka.common.protocol.ApiKeys;
 import org.apache.kafka.common.record.MemoryRecords;
 import org.apache.kafka.common.record.MemoryRecordsBuilder;
 import org.apache.kafka.common.record.RecordBatch;
-import org.apache.kafka.common.record.RecordVersion;
 import org.apache.kafka.common.record.SimpleRecord;
 import org.apache.kafka.common.record.TimestampType;
 
@@ -50,18 +49,18 @@ public class ProduceRequestTest {
     @Test
     public void shouldBeFlaggedAsTransactionalWhenTransactionalRecords() {
         final MemoryRecords memoryRecords = MemoryRecords.withTransactionalRecords(0, Compression.NONE, 1L,
-                (short) 1, 1, 1, simpleRecord);
+            (short) 1, 1, 1, simpleRecord);
 
-        final ProduceRequest request = ProduceRequest.forCurrentMagic(new ProduceRequestData()
-                .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
-                    new ProduceRequestData.TopicProduceData()
-                        .setName("topic")
-                        .setPartitionData(Collections.singletonList(
-                            new ProduceRequestData.PartitionProduceData()
-                                .setIndex(1)
-                                .setRecords(memoryRecords)))).iterator()))
-                .setAcks((short) -1)
-                .setTimeoutMs(10)).build();
+        final ProduceRequest request = ProduceRequest.builder(new ProduceRequestData()
+            .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
+                new ProduceRequestData.TopicProduceData()
+                    .setName("topic")
+                    .setPartitionData(Collections.singletonList(
+                        new ProduceRequestData.PartitionProduceData()
+                            .setIndex(1)
+                            .setRecords(memoryRecords)))).iterator()))
+            .setAcks((short) -1)
+            .setTimeoutMs(10)).build();
         assertTrue(RequestUtils.hasTransactionalRecords(request));
     }
 
@@ -80,52 +79,35 @@ public class ProduceRequestTest {
     @Test
     public void shouldBeFlaggedAsIdempotentWhenIdempotentRecords() {
         final MemoryRecords memoryRecords = MemoryRecords.withIdempotentRecords(1, Compression.NONE, 1L,
-                (short) 1, 1, 1, simpleRecord);
-        final ProduceRequest request = ProduceRequest.forCurrentMagic(new ProduceRequestData()
-                .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
-                    new ProduceRequestData.TopicProduceData()
-                        .setName("topic")
-                        .setPartitionData(Collections.singletonList(
-                            new ProduceRequestData.PartitionProduceData()
-                                .setIndex(1)
-                                .setRecords(memoryRecords)))).iterator()))
-                .setAcks((short) -1)
-                .setTimeoutMs(10)).build();
+            (short) 1, 1, 1, simpleRecord);
+        final ProduceRequest request = ProduceRequest.builder(new ProduceRequestData()
+            .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
+                new ProduceRequestData.TopicProduceData()
+                    .setName("topic")
+                    .setPartitionData(Collections.singletonList(
+                        new ProduceRequestData.PartitionProduceData()
+                            .setIndex(1)
+                            .setRecords(memoryRecords)))).iterator()))
+            .setAcks((short) -1)
+            .setTimeoutMs(10)).build();
         assertTrue(RequestTestUtils.hasIdempotentRecords(request));
     }
 
-    @Test
-    public void testBuildWithOldMessageFormat() {
-        ByteBuffer buffer = ByteBuffer.allocate(256);
-        MemoryRecordsBuilder builder = MemoryRecords.builder(buffer, RecordBatch.MAGIC_VALUE_V1, Compression.NONE,
-                TimestampType.CREATE_TIME, 0L);
-        builder.append(10L, null, "a".getBytes());
-        ProduceRequest.Builder requestBuilder = ProduceRequest.forMagic(RecordBatch.MAGIC_VALUE_V1,
-            new ProduceRequestData()
-                .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
-                    new ProduceRequestData.TopicProduceData().setName("test").setPartitionData(Collections.singletonList(
-                        new ProduceRequestData.PartitionProduceData().setIndex(9).setRecords(builder.build()))))
-                    .iterator()))
-                .setAcks((short) 1)
-                .setTimeoutMs(5000));
-        assertEquals(2, requestBuilder.oldestAllowedVersion());
-        assertEquals(2, requestBuilder.latestAllowedVersion());
-    }
-
     @Test
     public void testBuildWithCurrentMessageFormat() {
         ByteBuffer buffer = ByteBuffer.allocate(256);
         MemoryRecordsBuilder builder = MemoryRecords.builder(buffer, RecordBatch.CURRENT_MAGIC_VALUE,
-                Compression.NONE, TimestampType.CREATE_TIME, 0L);
+            Compression.NONE, TimestampType.CREATE_TIME, 0L);
         builder.append(10L, null, "a".getBytes());
-        ProduceRequest.Builder requestBuilder = ProduceRequest.forMagic(RecordBatch.CURRENT_MAGIC_VALUE,
-                new ProduceRequestData()
-                        .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
-                                new ProduceRequestData.TopicProduceData().setName("test").setPartitionData(Collections.singletonList(
-                                        new ProduceRequestData.PartitionProduceData().setIndex(9).setRecords(builder.build()))))
-                                .iterator()))
-                        .setAcks((short) 1)
-                        .setTimeoutMs(5000));
+        ProduceRequest.Builder requestBuilder = ProduceRequest.builder(
+            new ProduceRequestData()
+                .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
+                    new ProduceRequestData.TopicProduceData().setName("test").setPartitionData(Collections.singletonList(
+                        new ProduceRequestData.PartitionProduceData().setIndex(9).setRecords(builder.build()))))
+                    .iterator()))
+                .setAcks((short) 1)
+                .setTimeoutMs(5000),
+            false);
         assertEquals(3, requestBuilder.oldestAllowedVersion());
         assertEquals(ApiKeys.PRODUCE.latestVersion(), requestBuilder.latestAllowedVersion());
     }
@@ -144,31 +126,31 @@ public class ProduceRequestTest {
 
         buffer.flip();
 
-        ProduceRequest.Builder requestBuilder = ProduceRequest.forCurrentMagic(new ProduceRequestData()
-                .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
-                    new ProduceRequestData.TopicProduceData()
-                        .setName("test")
-                        .setPartitionData(Collections.singletonList(
-                            new ProduceRequestData.PartitionProduceData()
-                                .setIndex(0)
-                                    .setRecords(MemoryRecords.readableRecords(buffer))))).iterator()))
-                .setAcks((short) 1)
-                .setTimeoutMs(5000));
+        ProduceRequest.Builder requestBuilder = ProduceRequest.builder(new ProduceRequestData()
+            .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
+                new ProduceRequestData.TopicProduceData()
+                    .setName("test")
+                    .setPartitionData(Collections.singletonList(
+                        new ProduceRequestData.PartitionProduceData()
+                            .setIndex(0)
+                                .setRecords(MemoryRecords.readableRecords(buffer))))).iterator()))
+            .setAcks((short) 1)
+            .setTimeoutMs(5000));
         assertThrowsForAllVersions(requestBuilder, InvalidRecordException.class);
     }
 
     @Test
     public void testV3AndAboveCannotHaveNoRecordBatches() {
-        ProduceRequest.Builder requestBuilder = ProduceRequest.forCurrentMagic(new ProduceRequestData()
-                .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
-                        new ProduceRequestData.TopicProduceData()
-                                .setName("test")
-                                .setPartitionData(Collections.singletonList(
-                                        new ProduceRequestData.PartitionProduceData()
-                                                .setIndex(0)
-                                                .setRecords(MemoryRecords.EMPTY)))).iterator()))
-                .setAcks((short) 1)
-                .setTimeoutMs(5000));
+        ProduceRequest.Builder requestBuilder = ProduceRequest.builder(new ProduceRequestData()
+            .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
+                new ProduceRequestData.TopicProduceData()
+                    .setName("test")
+                    .setPartitionData(Collections.singletonList(
+                        new ProduceRequestData.PartitionProduceData()
+                            .setIndex(0)
+                            .setRecords(MemoryRecords.EMPTY)))).iterator()))
+            .setAcks((short) 1)
+            .setTimeoutMs(5000));
         assertThrowsForAllVersions(requestBuilder, InvalidRecordException.class);
     }
 
@@ -176,19 +158,19 @@ public class ProduceRequestTest {
     public void testV3AndAboveCannotUseMagicV0() {
         ByteBuffer buffer = ByteBuffer.allocate(256);
         MemoryRecordsBuilder builder = MemoryRecords.builder(buffer, RecordBatch.MAGIC_VALUE_V0, Compression.NONE,
-                TimestampType.NO_TIMESTAMP_TYPE, 0L);
+            TimestampType.NO_TIMESTAMP_TYPE, 0L);
         builder.append(10L, null, "a".getBytes());
 
-        ProduceRequest.Builder requestBuilder = ProduceRequest.forCurrentMagic(new ProduceRequestData()
-                .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
-                        new ProduceRequestData.TopicProduceData()
-                                .setName("test")
-                                .setPartitionData(Collections.singletonList(
-                                        new ProduceRequestData.PartitionProduceData()
-                                                .setIndex(0)
-                                                .setRecords(builder.build())))).iterator()))
-                .setAcks((short) 1)
-                .setTimeoutMs(5000));
+        ProduceRequest.Builder requestBuilder = ProduceRequest.builder(new ProduceRequestData()
+            .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
+                new ProduceRequestData.TopicProduceData()
+                    .setName("test")
+                    .setPartitionData(Collections.singletonList(
+                        new ProduceRequestData.PartitionProduceData()
+                            .setIndex(0)
+                            .setRecords(builder.build())))).iterator()))
+            .setAcks((short) 1)
+            .setTimeoutMs(5000));
         assertThrowsForAllVersions(requestBuilder, InvalidRecordException.class);
     }
 
@@ -196,19 +178,19 @@ public class ProduceRequestTest {
     public void testV3AndAboveCannotUseMagicV1() {
         ByteBuffer buffer = ByteBuffer.allocate(256);
         MemoryRecordsBuilder builder = MemoryRecords.builder(buffer, RecordBatch.MAGIC_VALUE_V1, Compression.NONE,
-                TimestampType.CREATE_TIME, 0L);
+            TimestampType.CREATE_TIME, 0L);
         builder.append(10L, null, "a".getBytes());
 
-        ProduceRequest.Builder requestBuilder = ProduceRequest.forCurrentMagic(new ProduceRequestData()
-                .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
-                        new ProduceRequestData.TopicProduceData()
-                                .setName("test")
-                                .setPartitionData(Collections.singletonList(new ProduceRequestData.PartitionProduceData()
-                                        .setIndex(0)
-                                        .setRecords(builder.build()))))
-                        .iterator()))
-                .setAcks((short) 1)
-                .setTimeoutMs(5000));
+        ProduceRequest.Builder requestBuilder = ProduceRequest.builder(new ProduceRequestData()
+            .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
+                new ProduceRequestData.TopicProduceData()
+                    .setName("test")
+                    .setPartitionData(Collections.singletonList(new ProduceRequestData.PartitionProduceData()
+                        .setIndex(0)
+                        .setRecords(builder.build()))))
+                .iterator()))
+            .setAcks((short) 1)
+            .setTimeoutMs(5000));
         assertThrowsForAllVersions(requestBuilder, InvalidRecordException.class);
     }
 
@@ -220,24 +202,23 @@ public class ProduceRequestTest {
         builder.append(10L, null, "a".getBytes());
 
         ProduceRequestData produceData = new ProduceRequestData()
-                .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
-                    new ProduceRequestData.TopicProduceData()
-                        .setName("test")
-                        .setPartitionData(Collections.singletonList(new ProduceRequestData.PartitionProduceData()
-                            .setIndex(0)
-                            .setRecords(builder.build()))))
-                    .iterator()))
-                .setAcks((short) 1)
-                .setTimeoutMs(1000);
+            .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
+                new ProduceRequestData.TopicProduceData()
+                    .setName("test")
+                    .setPartitionData(Collections.singletonList(new ProduceRequestData.PartitionProduceData()
+                        .setIndex(0)
+                        .setRecords(builder.build()))))
+                .iterator()))
+            .setAcks((short) 1)
+            .setTimeoutMs(1000);
         // Can't create ProduceRequest instance with version within [3, 7)
-        for (short version = 3; version < 7; version++) {
-
+        for (short version = ApiKeys.PRODUCE.oldestVersion(); version < 7; version++) {
             ProduceRequest.Builder requestBuilder = new ProduceRequest.Builder(version, version, produceData);
             assertThrowsForAllVersions(requestBuilder, UnsupportedCompressionTypeException.class);
         }
 
         // Works fine with current version (>= 7)
-        ProduceRequest.forCurrentMagic(produceData);
+        ProduceRequest.builder(produceData);
     }
 
     @Test
@@ -247,20 +228,21 @@ public class ProduceRequestTest {
         final int sequence = 10;
 
         final MemoryRecords nonTxnRecords = MemoryRecords.withRecords(Compression.NONE,
-                new SimpleRecord("foo".getBytes()));
+            new SimpleRecord("foo".getBytes()));
         final MemoryRecords txnRecords = MemoryRecords.withTransactionalRecords(Compression.NONE, producerId,
-                producerEpoch, sequence, new SimpleRecord("bar".getBytes()));
+            producerEpoch, sequence, new SimpleRecord("bar".getBytes()));
 
-        ProduceRequest.Builder builder = ProduceRequest.forMagic(RecordBatch.CURRENT_MAGIC_VALUE,
-                new ProduceRequestData()
-                        .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Arrays.asList(
-                                new ProduceRequestData.TopicProduceData().setName("foo").setPartitionData(Collections.singletonList(
-                                        new ProduceRequestData.PartitionProduceData().setIndex(0).setRecords(txnRecords))),
-                                new ProduceRequestData.TopicProduceData().setName("foo").setPartitionData(Collections.singletonList(
-                                        new ProduceRequestData.PartitionProduceData().setIndex(1).setRecords(nonTxnRecords))))
-                                .iterator()))
-                        .setAcks((short) -1)
-                        .setTimeoutMs(5000));
+        ProduceRequest.Builder builder = ProduceRequest.builder(
+            new ProduceRequestData()
+                .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Arrays.asList(
+                    new ProduceRequestData.TopicProduceData().setName("foo").setPartitionData(Collections.singletonList(
+                        new ProduceRequestData.PartitionProduceData().setIndex(0).setRecords(txnRecords))),
+                    new ProduceRequestData.TopicProduceData().setName("foo").setPartitionData(Collections.singletonList(
+                        new ProduceRequestData.PartitionProduceData().setIndex(1).setRecords(nonTxnRecords))))
+                    .iterator()))
+                .setAcks((short) -1)
+                .setTimeoutMs(5000),
+            true);
         final ProduceRequest request = builder.build();
         assertTrue(RequestUtils.hasTransactionalRecords(request));
         assertTrue(RequestTestUtils.hasIdempotentRecords(request));
@@ -273,26 +255,43 @@ public class ProduceRequestTest {
         final int sequence = 10;
 
         final MemoryRecords nonIdempotentRecords = MemoryRecords.withRecords(Compression.NONE,
-                new SimpleRecord("foo".getBytes()));
+            new SimpleRecord("foo".getBytes()));
         final MemoryRecords idempotentRecords = MemoryRecords.withIdempotentRecords(Compression.NONE, producerId,
-                producerEpoch, sequence, new SimpleRecord("bar".getBytes()));
+            producerEpoch, sequence, new SimpleRecord("bar".getBytes()));
 
-        ProduceRequest.Builder builder = ProduceRequest.forMagic(RecordVersion.current().value,
-                new ProduceRequestData()
-                        .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Arrays.asList(
-                                new ProduceRequestData.TopicProduceData().setName("foo").setPartitionData(Collections.singletonList(
-                                        new ProduceRequestData.PartitionProduceData().setIndex(0).setRecords(idempotentRecords))),
-                                new ProduceRequestData.TopicProduceData().setName("foo").setPartitionData(Collections.singletonList(
-                                        new ProduceRequestData.PartitionProduceData().setIndex(1).setRecords(nonIdempotentRecords))))
-                                .iterator()))
-                        .setAcks((short) -1)
-                        .setTimeoutMs(5000));
+        ProduceRequest.Builder builder = ProduceRequest.builder(
+            new ProduceRequestData()
+                .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Arrays.asList(
+                    new ProduceRequestData.TopicProduceData().setName("foo").setPartitionData(Collections.singletonList(
+                        new ProduceRequestData.PartitionProduceData().setIndex(0).setRecords(idempotentRecords))),
+                    new ProduceRequestData.TopicProduceData().setName("foo").setPartitionData(Collections.singletonList(
+                        new ProduceRequestData.PartitionProduceData().setIndex(1).setRecords(nonIdempotentRecords))))
+                    .iterator()))
+                .setAcks((short) -1)
+                .setTimeoutMs(5000),
+            true);
 
         final ProduceRequest request = builder.build();
         assertFalse(RequestUtils.hasTransactionalRecords(request));
         assertTrue(RequestTestUtils.hasIdempotentRecords(request));
     }
 
+    @Test
+    public void testBuilderOldestAndLatestAllowed() {
+        ProduceRequest.Builder builder = ProduceRequest.builder(new ProduceRequestData()
+            .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
+                new ProduceRequestData.TopicProduceData()
+                    .setName("topic")
+                    .setPartitionData(Collections.singletonList(new ProduceRequestData.PartitionProduceData()
+                        .setIndex(1)
+                        .setRecords(MemoryRecords.withRecords(Compression.NONE, simpleRecord))))
+            ).iterator()))
+            .setAcks((short) -1)
+            .setTimeoutMs(10));
+        assertEquals(ApiKeys.PRODUCE.oldestVersion(), builder.oldestAllowedVersion());
+        assertEquals(ApiKeys.PRODUCE.latestVersion(), builder.latestAllowedVersion());
+    }
+
     private static <T extends Throwable> void assertThrowsForAllVersions(ProduceRequest.Builder builder,
                                                                          Class<T> expectedType) {
         IntStream.range(builder.oldestAllowedVersion(), builder.latestAllowedVersion() + 1)
@@ -300,15 +299,15 @@ public class ProduceRequestTest {
     }
 
     private ProduceRequest createNonIdempotentNonTransactionalRecords() {
-        return ProduceRequest.forCurrentMagic(new ProduceRequestData()
-                .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
-                        new ProduceRequestData.TopicProduceData()
-                                .setName("topic")
-                                .setPartitionData(Collections.singletonList(new ProduceRequestData.PartitionProduceData()
-                                        .setIndex(1)
-                                        .setRecords(MemoryRecords.withRecords(Compression.NONE, simpleRecord)))))
-                        .iterator()))
-                .setAcks((short) -1)
-                .setTimeoutMs(10)).build();
+        return ProduceRequest.builder(new ProduceRequestData()
+            .setTopicData(new ProduceRequestData.TopicProduceDataCollection(Collections.singletonList(
+                new ProduceRequestData.TopicProduceData()
+                    .setName("topic")
+                    .setPartitionData(Collections.singletonList(new ProduceRequestData.PartitionProduceData()
+                        .setIndex(1)
+                        .setRecords(MemoryRecords.withRecords(Compression.NONE, simpleRecord)))))
+                .iterator()))
+            .setAcks((short) -1)
+            .setTimeoutMs(10)).build();
     }
 }
