@@ -27,7 +27,6 @@ import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
-import java.io.UnsupportedEncodingException;
 import java.net.HttpURLConnection;
 import java.nio.charset.StandardCharsets;
 import java.util.Random;
@@ -172,19 +171,19 @@ public class HttpAccessTokenRetrieverTest extends OAuthBearerTest {
     }
 
     @Test
-    public void testFormatAuthorizationHeader() throws UnsupportedEncodingException {
+    public void testFormatAuthorizationHeader() {
         assertAuthorizationHeader("id", "secret", false, "Basic aWQ6c2VjcmV0");
     }
 
     @Test
-    public void testFormatAuthorizationHeaderEncoding() throws UnsupportedEncodingException {
+    public void testFormatAuthorizationHeaderEncoding() {
         // according to RFC-7617, we need to use the *non-URL safe* base64 encoder. See KAFKA-14496.
         assertAuthorizationHeader("SOME_RANDOM_LONG_USER_01234", "9Q|0`8i~ute-n9ksjLWb\\50\"AX@UUED5E", false, "Basic U09NRV9SQU5ET01fTE9OR19VU0VSXzAxMjM0OjlRfDBgOGl+dXRlLW45a3NqTFdiXDUwIkFYQFVVRUQ1RQ==");
         // according to RFC-6749 clientId & clientSecret must be urlencoded, see https://tools.ietf.org/html/rfc6749#section-2.3.1
         assertAuthorizationHeader("user!@~'", "secret-(*)!", true, "Basic dXNlciUyMSU0MCU3RSUyNzpzZWNyZXQtJTI4KiUyOSUyMQ==");
     }
 
-    private void assertAuthorizationHeader(String clientId, String clientSecret, boolean urlencode, String expected) throws UnsupportedEncodingException {
+    private void assertAuthorizationHeader(String clientId, String clientSecret, boolean urlencode, String expected) {
         String actual = HttpAccessTokenRetriever.formatAuthorizationHeader(clientId, clientSecret, urlencode);
         assertEquals(expected, actual, String.format("Expected the HTTP Authorization header generated for client ID \"%s\" and client secret \"%s\" to match", clientId, clientSecret));
     }
@@ -203,14 +202,14 @@ public class HttpAccessTokenRetrieverTest extends OAuthBearerTest {
     }
 
     @Test
-    public void testFormatRequestBody() throws IOException {
+    public void testFormatRequestBody() {
         String expected = "grant_type=client_credentials&scope=scope";
         String actual = HttpAccessTokenRetriever.formatRequestBody("scope");
         assertEquals(expected, actual);
     }
 
     @Test
-    public void testFormatRequestBodyWithEscaped() throws IOException {
+    public void testFormatRequestBodyWithEscaped() {
         String questionMark = "%3F";
         String exclamationMark = "%21";
 
@@ -224,7 +223,7 @@ public class HttpAccessTokenRetrieverTest extends OAuthBearerTest {
     }
 
     @Test
-    public void testFormatRequestBodyMissingValues() throws IOException {
+    public void testFormatRequestBodyMissingValues() {
         String expected = "grant_type=client_credentials";
         String actual = HttpAccessTokenRetriever.formatRequestBody(null);
         assertEquals(expected, actual);
