@@ -30,7 +30,6 @@ import org.apache.kafka.common.security.ssl.DefaultSslEngineFactory.SecurityStor
 import org.apache.kafka.common.security.ssl.mock.TestKeyManagerFactory;
 import org.apache.kafka.common.security.ssl.mock.TestProviderCreator;
 import org.apache.kafka.common.security.ssl.mock.TestTrustManagerFactory;
-import org.apache.kafka.common.utils.Utils;
 import org.apache.kafka.test.TestSslUtils;
 import org.apache.kafka.test.TestUtils;
 
@@ -47,6 +46,7 @@ import java.security.cert.X509Certificate;
 import java.util.Arrays;
 import java.util.Map;
 import java.util.Properties;
+import java.util.Set;
 
 import javax.net.ssl.SSLContext;
 import javax.net.ssl.SSLEngine;
@@ -81,7 +81,7 @@ public abstract class SslFactoryTest {
             //host and port are hints
             SSLEngine engine = sslFactory.createSslEngine("localhost", 0);
             assertNotNull(engine);
-            assertEquals(Utils.mkSet(tlsProtocol), Utils.mkSet(engine.getEnabledProtocols()));
+            assertEquals(Set.of(tlsProtocol), Set.of(engine.getEnabledProtocols()));
             assertFalse(engine.getUseClientMode());
         }
     }
@@ -95,7 +95,7 @@ public abstract class SslFactoryTest {
             sslFactory.configure(serverSslConfig);
             SSLEngine engine = sslFactory.createSslEngine("localhost", 0);
             assertNotNull(engine);
-            assertEquals(Utils.mkSet(tlsProtocol), Utils.mkSet(engine.getEnabledProtocols()));
+            assertEquals(Set.of(tlsProtocol), Set.of(engine.getEnabledProtocols()));
             assertFalse(engine.getUseClientMode());
         }
     }
