@@ -45,7 +45,7 @@ import static org.junit.jupiter.api.Assertions.assertThrows;
 public class SerializationTest {
 
     private final String topic = "testTopic";
-    private final Map<Class<?>, List<Object>> testData = new HashMap<Class<?>, List<Object>>() {
+    private final Map<Class<?>, List<Object>> testData = new HashMap<>() {
         {
             put(String.class, Arrays.asList(null, "my string"));
             put(Short.class, Arrays.asList(null, (short) 32767, (short) -32768));
