@@ -22,6 +22,9 @@ import org.apache.kafka.common.config.ConfigException;
 import org.apache.kafka.connect.runtime.isolation.Plugins;
 import org.apache.kafka.connect.source.SourceTask;
 
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
@@ -30,9 +33,8 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.stream.Collectors;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
 
+import static org.apache.kafka.common.utils.Utils.enumOptions;
 import static org.apache.kafka.connect.runtime.SourceConnectorConfig.ExactlyOnceSupportLevel.REQUESTED;
 import static org.apache.kafka.connect.runtime.SourceConnectorConfig.ExactlyOnceSupportLevel.REQUIRED;
 import static org.apache.kafka.connect.runtime.TopicCreationConfig.DEFAULT_TOPIC_CREATION_GROUP;
@@ -46,13 +48,12 @@ import static org.apache.kafka.connect.source.SourceTask.TransactionBoundary.CON
 import static org.apache.kafka.connect.source.SourceTask.TransactionBoundary.DEFAULT;
 import static org.apache.kafka.connect.source.SourceTask.TransactionBoundary.INTERVAL;
 import static org.apache.kafka.connect.source.SourceTask.TransactionBoundary.POLL;
-import static org.apache.kafka.common.utils.Utils.enumOptions;
 
-public class SourceConnectorConfig extends ConnectorConfig {
+public final class SourceConnectorConfig extends ConnectorConfig {
 
     private static final Logger log = LoggerFactory.getLogger(SourceConnectorConfig.class);
 
-    protected static final String TOPIC_CREATION_GROUP = "Topic Creation";
+    static final String TOPIC_CREATION_GROUP = "Topic Creation";
 
     public static final String TOPIC_CREATION_PREFIX = "topic.creation.";
 
@@ -61,7 +62,7 @@ public class SourceConnectorConfig extends ConnectorConfig {
             + "created by source connectors";
     private static final String TOPIC_CREATION_GROUPS_DISPLAY = "Topic Creation Groups";
 
-    protected static final String EXACTLY_ONCE_SUPPORT_GROUP = "Exactly Once Support";
+    static final String EXACTLY_ONCE_SUPPORT_GROUP = "Exactly Once Support";
 
     public enum ExactlyOnceSupportLevel {
         REQUESTED,
@@ -102,7 +103,7 @@ public class SourceConnectorConfig extends ConnectorConfig {
             + TRANSACTION_BOUNDARY_CONFIG + " is specified.";
     private static final String TRANSACTION_BOUNDARY_INTERVAL_DISPLAY = "Transaction boundary interval";
 
-    protected static final String OFFSETS_TOPIC_GROUP = "offsets.topic";
+    static final String OFFSETS_TOPIC_GROUP = "offsets.topic";
 
     public static final String OFFSETS_TOPIC_CONFIG = "offsets.storage.topic";
     private static final String OFFSETS_TOPIC_DOC = "The name of a separate offsets topic to use for this connector. "
@@ -247,7 +248,6 @@ public class SourceConnectorConfig extends ConnectorConfig {
         return newDef;
     }
 
-    @SuppressWarnings("this-escape")
     public SourceConnectorConfig(Plugins plugins, Map<String, String> props, boolean createTopics) {
         super(plugins, configDef(), props);
         if (createTopics && props.entrySet().stream().anyMatch(e -> e.getKey().startsWith(TOPIC_CREATION_PREFIX))) {
