@@ -177,7 +177,7 @@ public class WorkerSinkTaskThreadedTest {
         workerTask = new WorkerSinkTask(
                 taskId, sinkTask, statusListener, initialState, workerConfig, ClusterConfigState.EMPTY, metrics, keyConverter,
                 valueConverter, errorHandlingMetrics, headerConverter, transformationChain,
-                consumer, pluginLoader, time, RetryWithToleranceOperatorTest.noopOperator(), null, statusBackingStore,
+                consumer, pluginLoader, time, RetryWithToleranceOperatorTest.noneOperator(), null, statusBackingStore,
                 Collections::emptyList);
         recordsReturned = 0;
     }
@@ -598,7 +598,8 @@ public class WorkerSinkTaskThreadedTest {
 
             TopicPartition topicPartition = new TopicPartition(TOPIC, PARTITION);
             ConsumerRecord<byte[], byte[]> consumerRecord = new ConsumerRecord<>(TOPIC, PARTITION, FIRST_OFFSET + recordsReturned, TIMESTAMP, TIMESTAMP_TYPE, 0, 0, RAW_KEY, RAW_VALUE, emptyHeaders(), Optional.empty());
-            ConsumerRecords<byte[], byte[]> records = new ConsumerRecords<>(Collections.singletonMap(topicPartition, singletonList(consumerRecord)));
+            ConsumerRecords<byte[], byte[]> records = new ConsumerRecords<>(Map.of(topicPartition, List.of(consumerRecord)),
+                Map.of(topicPartition, new OffsetAndMetadata(FIRST_OFFSET + recordsReturned + 1, Optional.empty(), "")));
             recordsReturned++;
             return records;
         });
@@ -629,7 +630,8 @@ public class WorkerSinkTaskThreadedTest {
                     TOPIC, PARTITION, FIRST_OFFSET + recordsReturned, TIMESTAMP, TIMESTAMP_TYPE,
                     0, 0, RAW_KEY, RAW_VALUE, emptyHeaders(), Optional.empty());
             ConsumerRecords<byte[], byte[]> records =
-                    new ConsumerRecords<>(Collections.singletonMap(topicPartition, singletonList(consumerRecord)));
+                    new ConsumerRecords<>(Map.of(topicPartition, List.of(consumerRecord)),
+                        Map.of(topicPartition, new OffsetAndMetadata(FIRST_OFFSET + recordsReturned + 1, Optional.empty(), "")));
             recordsReturned++;
             return records;
         });
@@ -639,7 +641,7 @@ public class WorkerSinkTaskThreadedTest {
     }
 
     private void expectPreCommit(ExpectOffsetCommitCommand... commands) {
-        doAnswer(new Answer<Object>() {
+        doAnswer(new Answer<>() {
             int index = 0;
 
             @Override
@@ -660,7 +662,7 @@ public class WorkerSinkTaskThreadedTest {
     }
     
     private void expectOffsetCommit(ExpectOffsetCommitCommand... commands) {
-        doAnswer(new Answer<Object>() {
+        doAnswer(new Answer<>() {
             int index = 0;
 
             @Override
@@ -716,7 +718,6 @@ public class WorkerSinkTaskThreadedTest {
     private abstract static class TestSinkTask extends SinkTask {
     }
 
-    @SuppressWarnings("NewClassNamingConvention")
     private static class ExpectOffsetCommitCommand {
         final long expectedMessages;
         final RuntimeException error;
