@@ -66,7 +66,7 @@ public class WorkerGroupMemberTest {
         workerProps.put("offset.storage.topic", "topic-1");
         workerProps.put("config.storage.topic", "topic-1");
         workerProps.put("status.storage.topic", "topic-1");
-        workerProps.put(CommonClientConfigs.METRIC_REPORTER_CLASSES_CONFIG, MockConnectMetrics.MockMetricsReporter.class.getName());
+        workerProps.put(CommonClientConfigs.METRIC_REPORTER_CLASSES_CONFIG, JmxReporter.class.getName() + "," + MockConnectMetrics.MockMetricsReporter.class.getName());
         DistributedConfig config = spy(new DistributedConfig(workerProps));
         doReturn("cluster-1").when(config).kafkaClusterId();
 
@@ -109,7 +109,7 @@ public class WorkerGroupMemberTest {
         workerProps.put("offset.storage.topic", "topic-1");
         workerProps.put("config.storage.topic", "topic-1");
         workerProps.put("status.storage.topic", "topic-1");
-        workerProps.put("auto.include.jmx.reporter", "false");
+        workerProps.put(CommonClientConfigs.METRIC_REPORTER_CLASSES_CONFIG, "");
         DistributedConfig config = spy(new DistributedConfig(workerProps));
         doReturn("cluster-1").when(config).kafkaClusterId();
 
