@@ -17,15 +17,6 @@
 
 package org.apache.kafka.connect.runtime.isolation;
 
-import java.io.File;
-import java.net.MalformedURLException;
-import java.net.URL;
-import java.net.URLClassLoader;
-import java.nio.file.Path;
-import java.util.Arrays;
-import java.util.Collections;
-import java.util.Map.Entry;
-
 import org.apache.kafka.common.Configurable;
 import org.apache.kafka.common.config.AbstractConfig;
 import org.apache.kafka.common.config.ConfigDef;
@@ -34,10 +25,10 @@ import org.apache.kafka.common.config.provider.ConfigProvider;
 import org.apache.kafka.common.utils.LogCaptureAppender;
 import org.apache.kafka.common.utils.Utils;
 import org.apache.kafka.connect.components.Versioned;
+import org.apache.kafka.connect.connector.Connector;
 import org.apache.kafka.connect.connector.policy.AllConnectorClientConfigOverridePolicy;
 import org.apache.kafka.connect.connector.policy.ConnectorClientConfigOverridePolicy;
 import org.apache.kafka.connect.converters.ByteArrayConverter;
-import org.apache.kafka.connect.connector.Connector;
 import org.apache.kafka.connect.data.Schema;
 import org.apache.kafka.connect.data.SchemaAndValue;
 import org.apache.kafka.connect.errors.ConnectException;
@@ -55,24 +46,33 @@ import org.apache.kafka.connect.storage.ConverterConfig;
 import org.apache.kafka.connect.storage.ConverterType;
 import org.apache.kafka.connect.storage.HeaderConverter;
 import org.apache.kafka.connect.storage.SimpleHeaderConverter;
-import org.junit.Before;
-import org.junit.Test;
 
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+
+import java.io.File;
+import java.net.MalformedURLException;
+import java.net.URL;
+import java.net.URLClassLoader;
+import java.nio.file.Path;
+import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Map.Entry;
 import java.util.Set;
 import java.util.SortedSet;
 import java.util.stream.Collectors;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertSame;
-import static org.junit.Assert.assertThrows;
-import static org.junit.Assert.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertInstanceOf;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertSame;
+import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 public class PluginsTest {
 
@@ -86,7 +86,7 @@ public class PluginsTest {
     private PluginScanResult empty;
     private String missingPluginClass;
 
-    @Before
+    @BeforeEach
     public void setup() {
         Map<String, String> pluginProps = new HashMap<>();
 
@@ -184,10 +184,9 @@ public class PluginsTest {
                                config,
                                ConnectRestExtension.class);
         assertNotNull(connectRestExtensions);
-        assertEquals("One Rest Extension expected", 1, connectRestExtensions.size());
+        assertEquals(1, connectRestExtensions.size(), "One Rest Extension expected");
         assertNotNull(connectRestExtensions.get(0));
-        assertTrue("Should be instance of TestConnectRestExtension",
-                   connectRestExtensions.get(0) instanceof TestConnectRestExtension);
+        assertInstanceOf(TestConnectRestExtension.class, connectRestExtensions.get(0), "Should be instance of TestConnectRestExtension");
         assertNotNull(((TestConnectRestExtension) connectRestExtensions.get(0)).configs);
         assertEquals(config.originals(),
                      ((TestConnectRestExtension) connectRestExtensions.get(0)).configs);
@@ -483,7 +482,7 @@ public class PluginsTest {
                 .stream()
                 .filter(pluginDesc -> ByteArrayConverter.class.equals(pluginDesc.pluginClass()))
                 .collect(Collectors.toSet());
-        assertFalse("Could not find superclass of " + TestPlugin.SUBCLASS_OF_CLASSPATH_CONVERTER + " as plugin", converters.isEmpty());
+        assertFalse(converters.isEmpty(), "Could not find superclass of " + TestPlugin.SUBCLASS_OF_CLASSPATH_CONVERTER + " as plugin");
         for (PluginDesc<Converter> byteArrayConverter : converters) {
             assertEquals("classpath", byteArrayConverter.location());
         }
@@ -495,7 +494,7 @@ public class PluginsTest {
                 .stream()
                 .filter(pluginDesc -> AllConnectorClientConfigOverridePolicy.class.equals(pluginDesc.pluginClass()))
                 .collect(Collectors.toSet());
-        assertFalse("Could not find superclass of " + TestPlugin.SUBCLASS_OF_CLASSPATH_OVERRIDE_POLICY + " as plugin", overridePolicies.isEmpty());
+        assertFalse(overridePolicies.isEmpty(), "Could not find superclass of " + TestPlugin.SUBCLASS_OF_CLASSPATH_OVERRIDE_POLICY + " as plugin");
         for (PluginDesc<ConnectorClientConfigOverridePolicy> allOverridePolicy : overridePolicies) {
             assertEquals("classpath", allOverridePolicy.location());
         }
@@ -640,10 +639,10 @@ public class PluginsTest {
         );
         plugins = new Plugins(pluginProps, parent, new ClassLoaderFactory());
 
-        assertTrue("Should find plugin in plugin classloader",
-                plugins.converters().stream().anyMatch(desc -> desc.loader() instanceof PluginClassLoader));
-        assertTrue("Should find plugin in parent classloader",
-                plugins.converters().stream().anyMatch(desc -> parent.equals(desc.loader())));
+        assertTrue(plugins.converters().stream().anyMatch(desc -> desc.loader() instanceof PluginClassLoader),
+            "Should find plugin in plugin classloader");
+        assertTrue(plugins.converters().stream().anyMatch(desc -> parent.equals(desc.loader())),
+            "Should find plugin in parent classloader");
 
         Converter converter = plugins.newPlugin(
                 className,
