@@ -58,7 +58,7 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.Map;
 
-import javax.ws.rs.core.MediaType;
+import jakarta.ws.rs.core.MediaType;
 
 import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
@@ -184,7 +184,7 @@ public class ConnectRestServerTest {
         response.getEntity().writeTo(baos);
         assertArrayEquals(
             request.getAllowedMethods(response).toArray(),
-            new String(baos.toByteArray(), StandardCharsets.UTF_8).split(", ")
+            baos.toString(StandardCharsets.UTF_8).split(", ")
         );
     }
 
@@ -281,7 +281,7 @@ public class ConnectRestServerTest {
         expectedLogger.put("level", loggingLevel);
         expectedLogger.put("last_modified", lastModified);
         Map<String, Map<String, Object>> expectedLoggers = Collections.singletonMap(logger, expectedLogger);
-        Map<String, Map<String, Object>> actualLoggers = mapper.readValue(responseStr, new TypeReference<Map<String, Map<String, Object>>>() { });
+        Map<String, Map<String, Object>> actualLoggers = mapper.readValue(responseStr, new TypeReference<>() { });
         assertEquals(expectedLoggers, actualLoggers);
     }
 
@@ -353,7 +353,6 @@ public class ConnectRestServerTest {
         server.stop();
 
         Collection<String> logMessages = restServerAppender.getMessages();
-        LogCaptureAppender.unregister(restServerAppender);
         restServerAppender.close();
         String expectedlogContent = "\"GET / HTTP/1.1\" " + response.getStatusLine().getStatusCode();
         assertTrue(logMessages.stream().anyMatch(logMessage -> logMessage.contains(expectedlogContent)));
