@@ -34,7 +34,6 @@ import org.apache.kafka.common.KafkaException
 import org.apache.kafka.common.Node
 import org.apache.kafka.common.TopicPartition
 import org.apache.kafka.common.Uuid
-import org.apache.kafka.common.internals.Topic
 import org.apache.kafka.common.metrics.Metrics
 import org.apache.kafka.common.network.{ChannelBuilders, ListenerName, NetworkReceive, Selectable, Selector}
 import org.apache.kafka.common.protocol.ApiMessage
@@ -43,9 +42,9 @@ import org.apache.kafka.common.requests.RequestHeader
 import org.apache.kafka.common.security.JaasContext
 import org.apache.kafka.common.security.auth.SecurityProtocol
 import org.apache.kafka.common.utils.{LogContext, Time, Utils}
-import org.apache.kafka.raft.{Endpoints, FileQuorumStateStore, KafkaNetworkChannel, KafkaRaftClient, KafkaRaftClientDriver, LeaderAndEpoch, QuorumConfig, RaftClient, ReplicatedLog}
+import org.apache.kafka.raft.{ExternalKRaftMetrics, Endpoints, FileQuorumStateStore, KafkaNetworkChannel, KafkaRaftClient, KafkaRaftClientDriver, LeaderAndEpoch, QuorumConfig, RaftClient, ReplicatedLog}
 import org.apache.kafka.server.ProcessRole
-import org.apache.kafka.server.common.Features
+import org.apache.kafka.server.common.Feature
 import org.apache.kafka.server.common.serialization.RecordSerde
 import org.apache.kafka.server.util.{FileLock, KafkaScheduler}
 import org.apache.kafka.server.fault.FaultHandler
@@ -84,41 +83,6 @@ object KafkaRaftManager {
       .map(Paths.get(_).toAbsolutePath)
       .contains(Paths.get(config.metadataLogDir).toAbsolutePath)
   }
-
-  /**
-   * Obtain the file lock and delete the metadata log directory completely.
-   *
-   * This is only used by ZK brokers that are in pre-migration or hybrid mode of the ZK to KRaft migration.
-   * The rationale for deleting the metadata log in these cases is that it is safe to do on brokers and it
-   * it makes recovery from a failed migration much easier. See KAFKA-16463.
-   *
-   * @param config  The broker config
-   */
-  def maybeDeleteMetadataLogDir(config: KafkaConfig): Unit = {
-    // These constraints are enforced in KafkaServer, but repeating them here to guard against future callers
-    if (config.processRoles.nonEmpty) {
-      throw new RuntimeException("Not deleting metadata log dir since this node is in KRaft mode.")
-    } else if (!config.migrationEnabled) {
-      throw new RuntimeException("Not deleting metadata log dir since migrations are not enabled.")
-    } else {
-      val metadataDir = new File(config.metadataLogDir)
-      val logDirName = UnifiedLog.logDirName(Topic.CLUSTER_METADATA_TOPIC_PARTITION)
-      val metadataPartitionDir = KafkaRaftManager.createLogDirectory(metadataDir, logDirName)
-      val deletionLock = if (hasDifferentLogDir(config)) {
-        Some(KafkaRaftManager.lockDataDir(metadataDir))
-      } else {
-        None
-      }
-
-      try {
-        Utils.delete(metadataPartitionDir)
-      } catch {
-        case t: Throwable => throw new RuntimeException("Failed to delete metadata log", t)
-      } finally {
-        deletionLock.foreach(_.destroy())
-      }
-    }
-  }
 }
 
 trait RaftManager[T] {
@@ -151,6 +115,7 @@ class KafkaRaftManager[T](
   topicId: Uuid,
   time: Time,
   metrics: Metrics,
+  externalKRaftMetrics: ExternalKRaftMetrics,
   threadNamePrefixOpt: Option[String],
   val controllerQuorumVotersFuture: CompletableFuture[JMap[Integer, InetSocketAddress]],
   bootstrapServers: JCollection[InetSocketAddress],
@@ -194,7 +159,8 @@ class KafkaRaftManager[T](
     client.initialize(
       controllerQuorumVotersFuture.get(),
       new FileQuorumStateStore(new File(dataDir, FileQuorumStateStore.DEFAULT_FILE_NAME)),
-      metrics
+      metrics,
+      externalKRaftMetrics
     )
     netChannel.start()
     clientDriver.start()
@@ -202,11 +168,11 @@ class KafkaRaftManager[T](
 
   def shutdown(): Unit = {
     CoreUtils.swallow(expirationService.shutdown(), this)
-    CoreUtils.swallow(expirationTimer.close(), this)
+    Utils.closeQuietly(expirationTimer, "expiration timer")
     CoreUtils.swallow(clientDriver.shutdown(), this)
     CoreUtils.swallow(scheduler.shutdown(), this)
-    CoreUtils.swallow(netChannel.close(), this)
-    CoreUtils.swallow(replicatedLog.close(), this)
+    Utils.closeQuietly(netChannel, "net channel")
+    Utils.closeQuietly(replicatedLog, "replicated log")
     CoreUtils.swallow(dataDirLock.foreach(_.destroy()), this)
   }
 
@@ -240,14 +206,14 @@ class KafkaRaftManager[T](
       clusterId,
       bootstrapServers,
       localListeners,
-      Features.KRAFT_VERSION.supportedVersionRange(),
+      Feature.KRAFT_VERSION.supportedVersionRange(),
       raftConfig
     )
   }
 
   private def buildNetworkChannel(): KafkaNetworkChannel = {
     val (listenerName, netClient) = buildNetworkClient()
-    new KafkaNetworkChannel(time, listenerName, netClient, config.quorumRequestTimeoutMs, threadNamePrefix)
+    new KafkaNetworkChannel(time, listenerName, netClient, config.quorumConfig.requestTimeoutMs, threadNamePrefix)
   }
 
   private def createDataDir(): File = {
@@ -279,7 +245,6 @@ class KafkaRaftManager[T](
       controllerListenerName,
       config.saslMechanismControllerProtocol,
       time,
-      config.saslInterBrokerHandshakeRequestEnable,
       logContext
     )
 
@@ -313,7 +278,7 @@ class KafkaRaftManager[T](
       reconnectBackoffMsMs,
       Selectable.USE_DEFAULT_BUFFER_SIZE,
       config.socketReceiveBufferBytes,
-      config.quorumRequestTimeoutMs,
+      config.quorumConfig.requestTimeoutMs,
       config.connectionSetupTimeoutMs,
       config.connectionSetupTimeoutMaxMs,
       time,
