@@ -16,13 +16,12 @@
  */
 package kafka.server
 
-import kafka.network
-import kafka.network.RequestChannel
 import org.apache.kafka.common.feature.SupportedVersionRange
 import org.apache.kafka.common.message.ApiMessageType.ListenerType
 import org.apache.kafka.common.protocol.ApiKeys
 import org.apache.kafka.common.requests.ApiVersionsResponse
-import org.apache.kafka.server.ClientMetricsManager
+import org.apache.kafka.network.metrics.RequestChannelMetrics
+import org.apache.kafka.server.{BrokerFeatures, ClientMetricsManager}
 import org.apache.kafka.server.common.FinalizedFeatures
 
 import scala.collection.mutable
@@ -38,7 +37,7 @@ trait ApiVersionManager {
   def isApiEnabled(apiKey: ApiKeys, apiVersion: Short): Boolean = {
     apiKey != null && apiKey.inScope(listenerType) && apiKey.isVersionEnabled(apiVersion, enableUnstableLastVersion)
   }
-  def newRequestMetrics: RequestChannel.Metrics = new network.RequestChannel.Metrics(enabledApis)
+  def newRequestMetrics: RequestChannelMetrics = new RequestChannelMetrics(enabledApis.asJava)
 
   def features: FinalizedFeatures
 }
@@ -47,7 +46,7 @@ object ApiVersionManager {
   def apply(
     listenerType: ListenerType,
     config: KafkaConfig,
-    forwardingManager: Option[ForwardingManager],
+    forwardingManager: ForwardingManager,
     supportedFeatures: BrokerFeatures,
     metadataCache: MetadataCache,
     clientMetricsManager: Option[ClientMetricsManager]
@@ -58,7 +57,6 @@ object ApiVersionManager {
       supportedFeatures,
       metadataCache,
       config.unstableApiVersionsEnabled,
-      config.migrationEnabled,
       clientMetricsManager
     )
   }
@@ -72,7 +70,6 @@ object ApiVersionManager {
  * @param enabledApis the enabled apis, which are computed by the listener type
  * @param brokerFeatures the broker features
  * @param enableUnstableLastVersion whether to enable unstable last version, see [[KafkaConfig.unstableApiVersionsEnabled]]
- * @param zkMigrationEnabled whether to enable zk migration, see [[KafkaConfig.migrationEnabled]]
  * @param featuresProvider a provider to the finalized features supported
  */
 class SimpleApiVersionManager(
@@ -80,14 +77,12 @@ class SimpleApiVersionManager(
   val enabledApis: collection.Set[ApiKeys],
   brokerFeatures: org.apache.kafka.common.feature.Features[SupportedVersionRange],
   val enableUnstableLastVersion: Boolean,
-  val zkMigrationEnabled: Boolean,
   val featuresProvider: () => FinalizedFeatures
 ) extends ApiVersionManager {
 
   def this(
     listenerType: ListenerType,
     enableUnstableLastVersion: Boolean,
-    zkMigrationEnabled: Boolean,
     featuresProvider: () => FinalizedFeatures
   ) = {
     this(
@@ -95,12 +90,11 @@ class SimpleApiVersionManager(
       ApiKeys.apisForListener(listenerType).asScala,
       BrokerFeatures.defaultSupportedFeatures(enableUnstableLastVersion),
       enableUnstableLastVersion,
-      zkMigrationEnabled,
       featuresProvider
     )
   }
 
-  private val apiVersions = ApiVersionsResponse.collectApis(enabledApis.asJava, enableUnstableLastVersion)
+  private val apiVersions = ApiVersionsResponse.collectApis(listenerType, enabledApis.asJava, enableUnstableLastVersion)
 
   override def apiVersionResponse(
     throttleTimeMs: Int,
@@ -113,7 +107,7 @@ class SimpleApiVersionManager(
       setSupportedFeatures(brokerFeatures).
       setFinalizedFeatures(currentFeatures.finalizedFeatures()).
       setFinalizedFeaturesEpoch(currentFeatures.finalizedFeaturesEpoch()).
-      setZkMigrationEnabled(zkMigrationEnabled).
+      setZkMigrationEnabled(false).
       setAlterFeatureLevel0(alterFeatureLevel0).
       build()
   }
@@ -131,16 +125,14 @@ class SimpleApiVersionManager(
  * @param brokerFeatures the broker features
  * @param metadataCache the metadata cache, used to get the finalized features and the metadata version
  * @param enableUnstableLastVersion whether to enable unstable last version, see [[KafkaConfig.unstableApiVersionsEnabled]]
- * @param zkMigrationEnabled whether to enable zk migration, see [[KafkaConfig.migrationEnabled]]
  * @param clientMetricsManager the client metrics manager, helps to determine whether client telemetry is enabled
  */
 class DefaultApiVersionManager(
   val listenerType: ListenerType,
-  forwardingManager: Option[ForwardingManager],
+  forwardingManager: ForwardingManager,
   brokerFeatures: BrokerFeatures,
   metadataCache: MetadataCache,
   val enableUnstableLastVersion: Boolean,
-  val zkMigrationEnabled: Boolean = false,
   val clientMetricsManager: Option[ClientMetricsManager] = None
 ) extends ApiVersionManager {
 
@@ -151,21 +143,19 @@ class DefaultApiVersionManager(
     alterFeatureLevel0: Boolean
   ): ApiVersionsResponse = {
     val finalizedFeatures = metadataCache.features()
-    val controllerApiVersions = forwardingManager.flatMap(_.controllerApiVersions)
+    val controllerApiVersions = forwardingManager.controllerApiVersions
     val clientTelemetryEnabled = clientMetricsManager match {
       case Some(manager) => manager.isTelemetryReceiverConfigured
       case None => false
     }
     val apiVersions = if (controllerApiVersions.isDefined) {
       ApiVersionsResponse.controllerApiVersions(
-        finalizedFeatures.metadataVersion().highestSupportedRecordVersion,
         controllerApiVersions.get,
         listenerType,
         enableUnstableLastVersion,
         clientTelemetryEnabled)
     } else {
       ApiVersionsResponse.brokerApiVersions(
-        finalizedFeatures.metadataVersion().highestSupportedRecordVersion,
         listenerType,
         enableUnstableLastVersion,
         clientTelemetryEnabled)
@@ -176,7 +166,7 @@ class DefaultApiVersionManager(
       setSupportedFeatures(brokerFeatures.supportedFeatures).
       setFinalizedFeatures(finalizedFeatures.finalizedFeatures()).
       setFinalizedFeaturesEpoch(finalizedFeatures.finalizedFeaturesEpoch()).
-      setZkMigrationEnabled(zkMigrationEnabled).
+      setZkMigrationEnabled(false).
       setAlterFeatureLevel0(alterFeatureLevel0).
       build()
   }
