@@ -26,13 +26,12 @@ import org.apache.kafka.common.errors.InvalidTopicException
 import org.apache.kafka.common.internals.Topic
 import org.apache.kafka.common.message.DescribeTopicPartitionsResponseData.{Cursor, DescribeTopicPartitionsResponsePartition, DescribeTopicPartitionsResponseTopic}
 import org.apache.kafka.common.message.MetadataResponseData.{MetadataResponsePartition, MetadataResponseTopic}
-import org.apache.kafka.common.message.UpdateMetadataRequestData.UpdateMetadataPartitionState
 import org.apache.kafka.common.message._
 import org.apache.kafka.common.network.ListenerName
 import org.apache.kafka.common.protocol.Errors
 import org.apache.kafka.common.requests.MetadataResponse
 import org.apache.kafka.image.MetadataImage
-import org.apache.kafka.metadata.{BrokerRegistration, PartitionRegistration, Replicas}
+import org.apache.kafka.metadata.{BrokerRegistration, LeaderAndIsr, PartitionRegistration, Replicas}
 import org.apache.kafka.server.common.{FinalizedFeatures, KRaftVersion, MetadataVersion}
 
 import java.util
@@ -41,8 +40,8 @@ import java.util.function.Supplier
 import java.util.{Collections, Properties}
 import scala.collection.mutable.ListBuffer
 import scala.collection.{Map, Seq, Set, mutable}
-import scala.compat.java8.OptionConverters._
 import scala.jdk.CollectionConverters._
+import scala.jdk.OptionConverters.RichOptional
 import scala.util.control.Breaks._
 
 
@@ -238,7 +237,7 @@ class KRaftMetadataCache(
    * @return None if broker is not alive or if the broker does not have a listener named `listenerName`.
    */
   private def getAliveEndpoint(image: MetadataImage, id: Int, listenerName: ListenerName): Option[Node] = {
-    Option(image.cluster().broker(id)).flatMap(_.node(listenerName.value()).asScala)
+    Option(image.cluster().broker(id)).flatMap(_.node(listenerName.value()).toScala)
   }
 
   // errorUnavailableEndpoints exists to support v0 MetadataResponses
@@ -373,27 +372,23 @@ class KRaftMetadataCache(
 
   override def getAliveBrokerNode(brokerId: Int, listenerName: ListenerName): Option[Node] = {
     Option(_currentImage.cluster().broker(brokerId)).filterNot(_.fenced()).
-      flatMap(_.node(listenerName.value()).asScala)
+      flatMap(_.node(listenerName.value()).toScala)
   }
 
   override def getAliveBrokerNodes(listenerName: ListenerName): Seq[Node] = {
     _currentImage.cluster().brokers().values().asScala.filterNot(_.fenced()).
-      flatMap(_.node(listenerName.value()).asScala).toSeq
+      flatMap(_.node(listenerName.value()).toScala).toSeq
   }
 
-  // Does NOT include offline replica metadata
-  override def getPartitionInfo(topicName: String, partitionId: Int): Option[UpdateMetadataPartitionState] = {
+  override def getBrokerNodes(listenerName: ListenerName): Seq[Node] = {
+    _currentImage.cluster().brokers().values().asScala.flatMap(_.node(listenerName.value()).asScala).toSeq
+  }
+
+  override def getLeaderAndIsr(topicName: String, partitionId: Int): Option[LeaderAndIsr] = {
     Option(_currentImage.topics().getTopic(topicName)).
       flatMap(topic => Option(topic.partitions().get(partitionId))).
-      flatMap(partition => Some(new UpdateMetadataPartitionState().
-        setTopicName(topicName).
-        setPartitionIndex(partitionId).
-        setControllerEpoch(-1). // Controller epoch is not stored in the cache.
-        setLeader(partition.leader).
-        setLeaderEpoch(partition.leaderEpoch).
-        setIsr(Replicas.toList(partition.isr)).
-        setZkVersion(partition.partitionEpoch).
-        setReplicas(Replicas.toList(partition.replicas))))
+      flatMap(partition => Some(new LeaderAndIsr(partition.leader, partition.leaderEpoch,
+        util.Arrays.asList(partition.isr.map(i => i: java.lang.Integer): _*), partition.leaderRecoveryState, partition.partitionEpoch)))
   }
 
   override def numPartitions(topicName: String): Option[Int] = {
@@ -478,7 +473,7 @@ class KRaftMetadataCache(
     val nodes = new util.HashMap[Integer, Node]
     image.cluster().brokers().values().forEach { broker =>
       if (!broker.fenced()) {
-        broker.node(listenerName.value()).asScala.foreach { node =>
+        broker.node(listenerName.value()).toScala.foreach { node =>
           nodes.put(broker.id(), node)
         }
       }
@@ -545,7 +540,7 @@ class KRaftMetadataCache(
     _currentImage.scram().describe(request)
   }
 
-  override def metadataVersion(): MetadataVersion = _currentImage.features().metadataVersion()
+  override def metadataVersion(): MetadataVersion = _currentImage.features().metadataVersionOrThrow()
 
   override def features(): FinalizedFeatures = {
     val image = _currentImage
@@ -554,8 +549,8 @@ class KRaftMetadataCache(
     if (kraftVersionLevel > 0) {
       finalizedFeatures.put(KRaftVersion.FEATURE_NAME, kraftVersionLevel)
     }
-
-    new FinalizedFeatures(image.features().metadataVersion(),
+    new FinalizedFeatures(
+      image.features().metadataVersionOrThrow(),
       finalizedFeatures,
       image.highestOffsetAndEpoch().offset,
       true)
