@@ -183,9 +183,13 @@ abstract class AbstractConsumerTest extends BaseRequestTest {
 
   protected def sendRecords(producer: KafkaProducer[Array[Byte], Array[Byte]], numRecords: Int,
                             tp: TopicPartition,
-                            startingTimestamp: Long = System.currentTimeMillis()): Seq[ProducerRecord[Array[Byte], Array[Byte]]] = {
+                            startingTimestamp: Long = System.currentTimeMillis(),
+                            timestampIncrement: Long = -1L): Seq[ProducerRecord[Array[Byte], Array[Byte]]] = {
     val records = (0 until numRecords).map { i =>
-      val timestamp = startingTimestamp + i.toLong
+      val timestamp = if (timestampIncrement > 0)
+        startingTimestamp + i.toLong * timestampIncrement
+      else
+        startingTimestamp + i.toLong
       val record = new ProducerRecord(tp.topic(), tp.partition(), timestamp, s"key $i".getBytes, s"value $i".getBytes)
       producer.send(record)
       record
@@ -202,7 +206,8 @@ abstract class AbstractConsumerTest extends BaseRequestTest {
                                         startingTimestamp: Long = 0L,
                                         timestampType: TimestampType = TimestampType.CREATE_TIME,
                                         tp: TopicPartition = tp,
-                                        maxPollRecords: Int = Int.MaxValue): Unit = {
+                                        maxPollRecords: Int = Int.MaxValue,
+                                        timestampIncrement: Long = -1L): Unit = {
     val records = consumeRecords(consumer, numRecords, maxPollRecords = maxPollRecords)
     val now = System.currentTimeMillis()
     for (i <- 0 until numRecords) {
@@ -212,8 +217,13 @@ abstract class AbstractConsumerTest extends BaseRequestTest {
       assertEquals(tp.partition, record.partition)
       if (timestampType == TimestampType.CREATE_TIME) {
         assertEquals(timestampType, record.timestampType)
-        val timestamp = startingTimestamp + i
-        assertEquals(timestamp, record.timestamp)
+        if (timestampIncrement > 0) {
+          val timestamp = startingTimestamp + i * timestampIncrement
+          assertEquals(timestamp, record.timestamp)
+        } else {
+          val timestamp = startingTimestamp + i
+          assertEquals(timestamp, record.timestamp)
+        }
       } else
         assertTrue(record.timestamp >= startingTimestamp && record.timestamp <= now,
           s"Got unexpected timestamp ${record.timestamp}. Timestamp should be between [$startingTimestamp, $now}]")
