@@ -15,15 +15,15 @@ package kafka.api
 import kafka.utils.{TestInfoUtils, TestUtils}
 import org.apache.kafka.clients.consumer._
 import org.apache.kafka.common.TopicPartition
-import org.apache.kafka.common.errors.InvalidTopicException
+import org.apache.kafka.common.errors.{InvalidRegularExpression, InvalidTopicException}
 import org.junit.jupiter.api.Assertions._
 import org.junit.jupiter.api.Timeout
+import org.junit.jupiter.api.function.Executable
 import org.junit.jupiter.params.ParameterizedTest
-import org.junit.jupiter.params.provider.{Arguments, MethodSource}
+import org.junit.jupiter.params.provider.MethodSource
 
 import java.time.Duration
 import java.util.regex.Pattern
-import java.util.stream.Stream
 import scala.jdk.CollectionConverters._
 
 /**
@@ -178,6 +178,156 @@ class PlaintextConsumerSubscriptionTest extends AbstractConsumerTest {
     assertEquals(0, consumer.assignment().size)
   }
 
+  @ParameterizedTest(name = TestInfoUtils.TestWithParameterizedQuorumAndGroupProtocolNames)
+  @MethodSource(Array("getTestQuorumAndGroupProtocolParametersConsumerGroupProtocolOnly"))
+  def testRe2JPatternSubscription(quorum: String, groupProtocol: String): Unit = {
+    val topic1 = "tblablac" // matches subscribed pattern
+    createTopic(topic1, 2, brokerCount)
+
+    val topic2 = "tblablak" // does not match subscribed pattern
+    createTopic(topic2, 2, brokerCount)
+
+    val topic3 = "tblab1" // does not match subscribed pattern
+    createTopic(topic3, 2, brokerCount)
+
+    val consumer = createConsumer()
+    assertEquals(0, consumer.assignment().size)
+
+    var pattern = new SubscriptionPattern("t.*c")
+    consumer.subscribe(pattern)
+
+    var assignment = Set(
+      new TopicPartition(topic, 0),
+      new TopicPartition(topic, 1),
+      new TopicPartition(topic1, 0),
+      new TopicPartition(topic1, 1))
+    awaitAssignment(consumer, assignment)
+    consumer.unsubscribe()
+    assertEquals(0, consumer.assignment().size)
+
+    // Subscribe to a different pattern to match topic2 (that did not match before)
+    pattern = new SubscriptionPattern(topic2 + ".*")
+    consumer.subscribe(pattern)
+
+    assignment = Set(
+      new TopicPartition(topic2, 0),
+      new TopicPartition(topic2, 1))
+    awaitAssignment(consumer, assignment)
+  }
+
+  @ParameterizedTest(name = TestInfoUtils.TestWithParameterizedQuorumAndGroupProtocolNames)
+  @MethodSource(Array("getTestQuorumAndGroupProtocolParametersConsumerGroupProtocolOnly"))
+  def testRe2JPatternSubscriptionFetch(quorum: String, groupProtocol: String): Unit = {
+    val topic1 = "topic1" // matches subscribed pattern
+    createTopic(topic1, 2, brokerCount)
+
+    val consumer = createConsumer()
+    assertEquals(0, consumer.assignment().size)
+
+    val pattern = new SubscriptionPattern("topic.*")
+    consumer.subscribe(pattern)
+
+    val assignment = Set(
+      new TopicPartition(topic, 0),
+      new TopicPartition(topic, 1),
+      new TopicPartition(topic1, 0),
+      new TopicPartition(topic1, 1))
+    awaitAssignment(consumer, assignment)
+
+    val producer = createProducer()
+    val totalRecords = 10L
+    val startingTimestamp = System.currentTimeMillis()
+    val tp = new TopicPartition(topic1, 0)
+    sendRecords(producer, totalRecords.toInt, tp, startingTimestamp = startingTimestamp)
+    consumeAndVerifyRecords(consumer = consumer, numRecords = totalRecords.toInt, startingOffset = 0, startingTimestamp = startingTimestamp, tp = tp)
+  }
+
+  @ParameterizedTest(name = TestInfoUtils.TestWithParameterizedQuorumAndGroupProtocolNames)
+  @MethodSource(Array("getTestQuorumAndGroupProtocolParametersConsumerGroupProtocolOnly"))
+  def testRe2JPatternExpandSubscription(quorum: String, groupProtocol: String): Unit = {
+    val topic1 = "topic1" // matches first pattern
+    createTopic(topic1, 2, brokerCount)
+
+    val topic2 = "topic2" // does not match first pattern
+    createTopic(topic2, 2, brokerCount)
+
+    val consumer = createConsumer()
+    assertEquals(0, consumer.assignment().size)
+
+    var pattern = new SubscriptionPattern("topic1.*")
+    consumer.subscribe(pattern)
+    val assignment = Set(
+      new TopicPartition(topic1, 0),
+      new TopicPartition(topic1, 1))
+    awaitAssignment(consumer, assignment)
+
+    consumer.unsubscribe()
+    assertEquals(0, consumer.assignment().size)
+
+    // Subscribe to a different pattern that should match
+    // the same topics the member already had plus new ones
+    pattern = new SubscriptionPattern("topic1|topic2")
+    consumer.subscribe(pattern)
+
+    val expandedAssignment = assignment ++ Set(new TopicPartition(topic2, 0), new TopicPartition(topic2, 1))
+    awaitAssignment(consumer, expandedAssignment)
+  }
+
+  @ParameterizedTest(name = TestInfoUtils.TestWithParameterizedQuorumAndGroupProtocolNames)
+  @MethodSource(Array("getTestQuorumAndGroupProtocolParametersConsumerGroupProtocolOnly"))
+  def testRe2JPatternSubscriptionAndTopicSubscription(quorum: String, groupProtocol: String): Unit = {
+    val topic1 = "topic1" // matches subscribed pattern
+    createTopic(topic1, 2, brokerCount)
+
+    val topic11 = "topic11" // matches subscribed pattern
+    createTopic(topic11, 2, brokerCount)
+
+    val topic2 = "topic2" // does not match subscribed pattern
+    createTopic(topic2, 2, brokerCount)
+
+    val consumer = createConsumer()
+    assertEquals(0, consumer.assignment().size)
+
+    // Subscribe to pattern
+    val pattern = new SubscriptionPattern("topic1.*")
+    consumer.subscribe(pattern)
+    val patternAssignment = Set(
+      new TopicPartition(topic1, 0),
+      new TopicPartition(topic1, 1),
+      new TopicPartition(topic11, 0),
+      new TopicPartition(topic11, 1))
+    awaitAssignment(consumer, patternAssignment)
+    consumer.unsubscribe()
+    assertEquals(0, consumer.assignment().size)
+
+    // Subscribe to explicit topic names
+    consumer.subscribe(List(topic2).asJava)
+    val assignment = Set(
+      new TopicPartition(topic2, 0),
+      new TopicPartition(topic2, 1))
+    awaitAssignment(consumer, assignment)
+    consumer.unsubscribe()
+
+    // Subscribe to pattern again
+    consumer.subscribe(pattern)
+    awaitAssignment(consumer, patternAssignment)
+  }
+
+  @ParameterizedTest(name = TestInfoUtils.TestWithParameterizedQuorumAndGroupProtocolNames)
+  @MethodSource(Array("getTestQuorumAndGroupProtocolParametersConsumerGroupProtocolOnly"))
+  def testRe2JPatternSubscriptionInvalidRegex(quorum: String, groupProtocol: String): Unit = {
+    val consumer = createConsumer()
+    assertEquals(0, consumer.assignment().size)
+
+    val pattern = new SubscriptionPattern("(t.*c")
+    consumer.subscribe(pattern)
+
+    TestUtils.tryUntilNoAssertionError() {
+      assertThrows(classOf[InvalidRegularExpression], () => consumer.poll(Duration.ZERO))
+    }
+    consumer.unsubscribe()
+  }
+
   @ParameterizedTest(name = TestInfoUtils.TestWithParameterizedQuorumAndGroupProtocolNames)
   @MethodSource(Array("getTestQuorumAndGroupProtocolParametersAll"))
   def testExpandingTopicSubscriptions(quorum: String, groupProtocol: String): Unit = {
@@ -211,8 +361,10 @@ class PlaintextConsumerSubscriptionTest extends AbstractConsumerTest {
   @ParameterizedTest(name = TestInfoUtils.TestWithParameterizedQuorumAndGroupProtocolNames)
   @MethodSource(Array("getTestQuorumAndGroupProtocolParametersAll"))
   def testUnsubscribeTopic(quorum: String, groupProtocol: String): Unit = {
-    this.consumerConfig.setProperty(ConsumerConfig.SESSION_TIMEOUT_MS_CONFIG, "100") // timeout quickly to avoid slow test
-    this.consumerConfig.setProperty(ConsumerConfig.HEARTBEAT_INTERVAL_MS_CONFIG, "30")
+    if (groupProtocol.equals(GroupProtocol.CLASSIC.name)) {
+      this.consumerConfig.setProperty(ConsumerConfig.SESSION_TIMEOUT_MS_CONFIG, "100") // timeout quickly to avoid slow test
+      this.consumerConfig.setProperty(ConsumerConfig.HEARTBEAT_INTERVAL_MS_CONFIG, "30")
+    }
     val consumer = createConsumer()
 
     val listener = new TestConsumerReassignmentListener()
@@ -227,11 +379,34 @@ class PlaintextConsumerSubscriptionTest extends AbstractConsumerTest {
 
   @ParameterizedTest(name = TestInfoUtils.TestWithParameterizedQuorumAndGroupProtocolNames)
   @MethodSource(Array("getTestQuorumAndGroupProtocolParametersAll"))
-  def testSubscribeInvalidTopic(quorum: String, groupProtocol: String): Unit = {
-    // Invalid topic name due to space
-    val invalidTopicName = "topic abc"
+  def testSubscribeInvalidTopicCanUnsubscribe(quorum: String, groupProtocol: String): Unit = {
     val consumer = createConsumer()
 
+    setupSubscribeInvalidTopic(consumer)
+    if(groupProtocol == "consumer") {
+      // Must ensure memberId is not empty before sending leave group heartbeat. This is a temporary solution before KIP-1082.
+      TestUtils.waitUntilTrue(() => consumer.groupMetadata().memberId().nonEmpty,
+        waitTimeMs = 30000, msg = "Timeout waiting for first consumer group heartbeat response")
+    }
+    assertDoesNotThrow(new Executable {
+      override def execute(): Unit = consumer.unsubscribe()
+    })
+  }
+
+  @ParameterizedTest(name = TestInfoUtils.TestWithParameterizedQuorumAndGroupProtocolNames)
+  @MethodSource(Array("getTestQuorumAndGroupProtocolParametersAll"))
+  def testSubscribeInvalidTopicCanClose(quorum: String, groupProtocol: String): Unit = {
+    val consumer = createConsumer()
+
+    setupSubscribeInvalidTopic(consumer)
+    assertDoesNotThrow(new Executable {
+      override def execute(): Unit = consumer.close()
+    })
+  }
+
+  def setupSubscribeInvalidTopic(consumer: Consumer[Array[Byte], Array[Byte]]): Unit = {
+    // Invalid topic name due to space
+    val invalidTopicName = "topic abc"
     consumer.subscribe(List(invalidTopicName).asJava)
 
     var exception : InvalidTopicException = null
@@ -246,8 +421,3 @@ class PlaintextConsumerSubscriptionTest extends AbstractConsumerTest {
     assertEquals(s"Invalid topics: [${invalidTopicName}]", exception.getMessage)
   }
 }
-
-object PlaintextConsumerSubscriptionTest {
-  def getTestQuorumAndGroupProtocolParametersAll: Stream[Arguments] =
-    BaseConsumerTest.getTestQuorumAndGroupProtocolParametersAll()
-}
