@@ -16,24 +16,20 @@
   */
 package kafka.api
 
-import kafka.security.authorizer.AclAuthorizer
-import kafka.utils.JaasTestUtils
+import kafka.security.JaasTestUtils
 import org.apache.kafka.common.config.SslConfigs
 import org.apache.kafka.common.config.internals.BrokerSecurityConfigs
 import org.apache.kafka.common.security.auth._
 import org.junit.jupiter.api.Assertions.assertNull
 
-import scala.collection.immutable.List
-
 class SaslGssapiSslEndToEndAuthorizationTest extends SaslEndToEndAuthorizationTest {
   override val clientPrincipal = new KafkaPrincipal(KafkaPrincipal.USER_TYPE,
-    JaasTestUtils.KafkaClientPrincipalUnqualifiedName)
+    JaasTestUtils.KAFKA_CLIENT_PRINCIPAL_UNQUALIFIED_NAME)
   override val kafkaPrincipal = new KafkaPrincipal(KafkaPrincipal.USER_TYPE,
-    JaasTestUtils.KafkaServerPrincipalUnqualifiedName)
+    JaasTestUtils.KAFKA_SERVER_PRINCIPAL_UNQUALIFIED_NAME)
 
   override protected def kafkaClientSaslMechanism = "GSSAPI"
   override protected def kafkaServerSaslMechanisms = List("GSSAPI")
-  override protected def authorizerClass = classOf[AclAuthorizer]
 
   // Configure brokers to require SSL client authentication in order to verify that SASL_SSL works correctly even if the
   // client doesn't have a keystore. We want to cover the scenario where a broker requires either SSL client
