@@ -25,9 +25,11 @@ import org.apache.kafka.common.protocol.ApiKeys
 import org.apache.kafka.common.requests.{RequestContext, RequestHeader}
 import org.apache.kafka.common.security.auth.{KafkaPrincipal, SecurityProtocol}
 import org.apache.kafka.common.utils.{BufferSupplier, MockTime, Time}
+import org.apache.kafka.network.metrics.RequestChannelMetrics
+import org.apache.kafka.server.common.RequestLocal
 import org.apache.kafka.server.log.remote.storage.RemoteStorageMetrics
 import org.apache.kafka.server.metrics.KafkaYammerMetrics
-import org.apache.kafka.storage.log.metrics.BrokerTopicMetrics
+import org.apache.kafka.storage.log.metrics.{BrokerTopicMetrics, BrokerTopicStats}
 import org.junit.jupiter.api.Assertions.{assertEquals, assertFalse, assertTrue}
 import org.junit.jupiter.api.Test
 import org.junit.jupiter.params.ParameterizedTest
@@ -38,6 +40,7 @@ import org.mockito.Mockito.{mock, times, verify, when}
 
 import java.net.InetAddress
 import java.nio.ByteBuffer
+import java.util.Collections
 import java.util.concurrent.CompletableFuture
 import java.util.concurrent.atomic.AtomicInteger
 import java.util.stream.Collectors
@@ -53,7 +56,7 @@ class KafkaRequestHandlerTest {
   def testCallbackTiming(): Unit = {
     val time = new MockTime()
     val startTime = time.nanoseconds()
-    val metrics = new RequestChannel.Metrics(None)
+    val metrics = new RequestChannelMetrics(Collections.emptySet[ApiKeys])
     val requestChannel = new RequestChannel(10, "", time, metrics)
     val apiHandler = mock(classOf[ApiRequestHandler])
     try {
@@ -70,7 +73,7 @@ class KafkaRequestHandlerTest {
             time.sleep(ms)
             handler.stop()
           },
-          RequestLocal.NoCaching)
+          RequestLocal.noCaching)
         // Execute the callback asynchronously.
         CompletableFuture.runAsync(() => callback(1))
         request.apiLocalCompleteTimeNanos = time.nanoseconds
@@ -90,7 +93,7 @@ class KafkaRequestHandlerTest {
   @Test
   def testCallbackTryCompleteActions(): Unit = {
     val time = new MockTime()
-    val metrics = mock(classOf[RequestChannel.Metrics])
+    val metrics = mock(classOf[RequestChannelMetrics])
     val apiHandler = mock(classOf[ApiRequestHandler])
     val requestChannel = new RequestChannel(10, "", time, metrics)
     val handler = new KafkaRequestHandler(0, 0, mock(classOf[Meter]), new AtomicInteger(1), requestChannel, apiHandler, time)
@@ -108,7 +111,7 @@ class KafkaRequestHandlerTest {
         (_: RequestLocal, _: Int) => {
           handler.stop()
         },
-        RequestLocal.NoCaching)
+        RequestLocal.noCaching)
       // Execute the callback asynchronously.
       CompletableFuture.runAsync(() => callback(1))
     }
@@ -126,7 +129,7 @@ class KafkaRequestHandlerTest {
   @Test
   def testHandlingCallbackOnNewThread(): Unit = {
     val time = new MockTime()
-    val metrics = mock(classOf[RequestChannel.Metrics])
+    val metrics = mock(classOf[RequestChannelMetrics])
     val apiHandler = mock(classOf[ApiRequestHandler])
     val requestChannel = new RequestChannel(10, "", time, metrics)
     val handler = new KafkaRequestHandler(0, 0, mock(classOf[Meter]), new AtomicInteger(1), requestChannel, apiHandler, time)
@@ -160,7 +163,7 @@ class KafkaRequestHandlerTest {
   @Test
   def testCallbackOnSameThread(): Unit = {
     val time = new MockTime()
-    val metrics = mock(classOf[RequestChannel.Metrics])
+    val metrics = mock(classOf[RequestChannelMetrics])
     val apiHandler = mock(classOf[ApiRequestHandler])
     val requestChannel = new RequestChannel(10, "", time, metrics)
     val handler = new KafkaRequestHandler(0, 0, mock(classOf[Meter]), new AtomicInteger(1), requestChannel, apiHandler, time)
@@ -228,7 +231,7 @@ class KafkaRequestHandlerTest {
     })
   }
 
-  def makeRequest(time: Time, metrics: RequestChannel.Metrics): RequestChannel.Request = {
+  def makeRequest(time: Time, metrics: RequestChannelMetrics): RequestChannel.Request = {
     // Make unsupported API versions request to avoid having to parse a real request
     val requestHeader = mock(classOf[RequestHeader])
     when(requestHeader.apiKey()).thenReturn(ApiKeys.API_VERSIONS)
