@@ -31,7 +31,7 @@ import org.apache.kafka.common.requests.{AbstractRequest, EnvelopeRequest, Envel
 import org.apache.kafka.common.security.auth.{KafkaPrincipal, SecurityProtocol}
 import org.apache.kafka.common.security.authenticator.DefaultKafkaPrincipalBuilder
 import org.apache.kafka.common.utils.MockTime
-import org.apache.kafka.server.ControllerRequestCompletionHandler
+import org.apache.kafka.server.common.ControllerRequestCompletionHandler
 import org.junit.jupiter.api.Assertions._
 import org.junit.jupiter.api.Test
 import org.mockito.Mockito._
@@ -40,7 +40,7 @@ import org.mockito.Mockito._
 class NodeToControllerRequestThreadTest {
 
   private def controllerInfo(node: Option[Node]): ControllerInformation = {
-    ControllerInformation(node, new ListenerName(""), SecurityProtocol.PLAINTEXT, "", isZkController = true)
+    ControllerInformation(node, new ListenerName(""), SecurityProtocol.PLAINTEXT, "")
   }
 
   private def emptyControllerInfo: ControllerInformation = {
@@ -50,7 +50,7 @@ class NodeToControllerRequestThreadTest {
   @Test
   def testRetryTimeoutWhileControllerNotAvailable(): Unit = {
     val time = new MockTime()
-    val config = new KafkaConfig(TestUtils.createBrokerConfig(1, "localhost:2181"))
+    val config = new KafkaConfig(TestUtils.createBrokerConfig(1))
     val metadata = mock(classOf[Metadata])
     val mockClient = new MockClient(time, metadata)
     val controllerNodeProvider = mock(classOf[ControllerNodeProvider])
@@ -59,7 +59,7 @@ class NodeToControllerRequestThreadTest {
 
     val retryTimeoutMs = 30000
     val testRequestThread = new NodeToControllerRequestThread(
-      mockClient, isNetworkClientForZkController = true, _ => mockClient, new ManualMetadataUpdater(),
+      mockClient,  new ManualMetadataUpdater(),
       controllerNodeProvider, config, time, "", retryTimeoutMs)
     testRequestThread.started = true
 
@@ -84,7 +84,7 @@ class NodeToControllerRequestThreadTest {
   def testRequestsSent(): Unit = {
     // just a simple test that tests whether the request from 1 -> 2 is sent and the response callback is called
     val time = new MockTime()
-    val config = new KafkaConfig(TestUtils.createBrokerConfig(1, "localhost:2181"))
+    val config = new KafkaConfig(TestUtils.createBrokerConfig(1))
     val controllerId = 2
 
     val metadata = mock(classOf[Metadata])
@@ -97,7 +97,7 @@ class NodeToControllerRequestThreadTest {
 
     val expectedResponse = RequestTestUtils.metadataUpdateWith(2, Collections.singletonMap("a", 2))
     val testRequestThread = new NodeToControllerRequestThread(
-      mockClient, isNetworkClientForZkController = true, _ => mockClient, new ManualMetadataUpdater(),
+      mockClient,  new ManualMetadataUpdater(),
       controllerNodeProvider, config, time, "", retryTimeoutMs = Long.MaxValue)
     testRequestThread.started = true
     mockClient.prepareResponse(expectedResponse)
@@ -125,7 +125,7 @@ class NodeToControllerRequestThreadTest {
   def testControllerChanged(): Unit = {
     // in this test the current broker is 1, and the controller changes from 2 -> 3 then back: 3 -> 2
     val time = new MockTime()
-    val config = new KafkaConfig(TestUtils.createBrokerConfig(1, "localhost:2181"))
+    val config = new KafkaConfig(TestUtils.createBrokerConfig(1))
     val oldControllerId = 1
     val newControllerId = 2
 
@@ -141,7 +141,7 @@ class NodeToControllerRequestThreadTest {
 
     val expectedResponse = RequestTestUtils.metadataUpdateWith(3, Collections.singletonMap("a", 2))
     val testRequestThread = new NodeToControllerRequestThread(
-      mockClient, isNetworkClientForZkController = true, _ => mockClient, new ManualMetadataUpdater(),
+      mockClient, new ManualMetadataUpdater(),
       controllerNodeProvider, config, time, "", retryTimeoutMs = Long.MaxValue)
     testRequestThread.started = true
 
@@ -173,7 +173,7 @@ class NodeToControllerRequestThreadTest {
   @Test
   def testNotController(): Unit = {
     val time = new MockTime()
-    val config = new KafkaConfig(TestUtils.createBrokerConfig(1, "localhost:2181"))
+    val config = new KafkaConfig(TestUtils.createBrokerConfig(1))
     val oldControllerId = 1
     val newControllerId = 2
 
@@ -193,7 +193,7 @@ class NodeToControllerRequestThreadTest {
       Collections.singletonMap("a", 2))
     val expectedResponse = RequestTestUtils.metadataUpdateWith(3, Collections.singletonMap("a", 2))
     val testRequestThread = new NodeToControllerRequestThread(
-      mockClient, isNetworkClientForZkController = true,_ => mockClient, new ManualMetadataUpdater(),
+      mockClient, new ManualMetadataUpdater(),
       controllerNodeProvider, config, time, "", retryTimeoutMs = Long.MaxValue)
     testRequestThread.started = true
 
@@ -233,7 +233,7 @@ class NodeToControllerRequestThreadTest {
   @Test
   def testEnvelopeResponseWithNotControllerError(): Unit = {
     val time = new MockTime()
-    val config = new KafkaConfig(TestUtils.createBrokerConfig(1, "localhost:2181"))
+    val config = new KafkaConfig(TestUtils.createBrokerConfig(1))
     val oldControllerId = 1
     val newControllerId = 2
 
@@ -260,7 +260,7 @@ class NodeToControllerRequestThreadTest {
     val expectedResponse = RequestTestUtils.metadataUpdateWith(3, Collections.singletonMap("a", 2))
 
     val testRequestThread = new NodeToControllerRequestThread(
-      mockClient, isNetworkClientForZkController = true, _ => mockClient, new ManualMetadataUpdater(),
+      mockClient,  new ManualMetadataUpdater(),
       controllerNodeProvider, config, time, "", retryTimeoutMs = Long.MaxValue)
     testRequestThread.started = true
 
@@ -307,7 +307,7 @@ class NodeToControllerRequestThreadTest {
   @Test
   def testRetryTimeout(): Unit = {
     val time = new MockTime()
-    val config = new KafkaConfig(TestUtils.createBrokerConfig(1, "localhost:2181"))
+    val config = new KafkaConfig(TestUtils.createBrokerConfig(1))
     val controllerId = 1
 
     val metadata = mock(classOf[Metadata])
@@ -323,7 +323,7 @@ class NodeToControllerRequestThreadTest {
       Collections.singletonMap("a", Errors.NOT_CONTROLLER),
       Collections.singletonMap("a", 2))
     val testRequestThread = new NodeToControllerRequestThread(
-      mockClient, isNetworkClientForZkController = true,_ => mockClient, new ManualMetadataUpdater(),
+      mockClient, new ManualMetadataUpdater(),
       controllerNodeProvider, config, time, "", retryTimeoutMs)
     testRequestThread.started = true
 
@@ -356,7 +356,7 @@ class NodeToControllerRequestThreadTest {
   @Test
   def testUnsupportedVersionHandling(): Unit = {
     val time = new MockTime()
-    val config = new KafkaConfig(TestUtils.createBrokerConfig(1, "localhost:2181"))
+    val config = new KafkaConfig(TestUtils.createBrokerConfig(1))
     val controllerId = 2
 
     val metadata = mock(classOf[Metadata])
@@ -382,7 +382,7 @@ class NodeToControllerRequestThreadTest {
     mockClient.prepareUnsupportedVersionResponse(request => request.apiKey == ApiKeys.METADATA)
 
     val testRequestThread = new NodeToControllerRequestThread(
-      mockClient, isNetworkClientForZkController = true, _ => mockClient, new ManualMetadataUpdater(),
+      mockClient,   new ManualMetadataUpdater(),
       controllerNodeProvider, config, time, "", retryTimeoutMs = Long.MaxValue)
     testRequestThread.started = true
 
@@ -394,7 +394,7 @@ class NodeToControllerRequestThreadTest {
   @Test
   def testAuthenticationExceptionHandling(): Unit = {
     val time = new MockTime()
-    val config = new KafkaConfig(TestUtils.createBrokerConfig(1, "localhost:2181"))
+    val config = new KafkaConfig(TestUtils.createBrokerConfig(1))
     val controllerId = 2
 
     val metadata = mock(classOf[Metadata])
@@ -420,7 +420,7 @@ class NodeToControllerRequestThreadTest {
     mockClient.createPendingAuthenticationError(activeController, 50)
 
     val testRequestThread = new NodeToControllerRequestThread(
-      mockClient, isNetworkClientForZkController = true, _ => mockClient, new ManualMetadataUpdater(),
+      mockClient,  new ManualMetadataUpdater(),
       controllerNodeProvider, config, time, "", retryTimeoutMs = Long.MaxValue)
     testRequestThread.started = true
 
@@ -434,7 +434,7 @@ class NodeToControllerRequestThreadTest {
   def testThreadNotStarted(): Unit = {
     // Make sure we throw if we enqueue anything while the thread is not running
     val time = new MockTime()
-    val config = new KafkaConfig(TestUtils.createBrokerConfig(1, "localhost:2181"))
+    val config = new KafkaConfig(TestUtils.createBrokerConfig(1))
 
     val metadata = mock(classOf[Metadata])
     val mockClient = new MockClient(time, metadata)
@@ -443,7 +443,7 @@ class NodeToControllerRequestThreadTest {
     when(controllerNodeProvider.getControllerInfo()).thenReturn(emptyControllerInfo)
 
     val testRequestThread = new NodeToControllerRequestThread(
-      mockClient, isNetworkClientForZkController = true, _ => mockClient, new ManualMetadataUpdater(),
+      mockClient,   new ManualMetadataUpdater(),
       controllerNodeProvider, config, time, "", retryTimeoutMs = Long.MaxValue)
 
     val completionHandler = new TestControllerRequestCompletionHandler(None)
