@@ -24,9 +24,10 @@ import org.apache.kafka.common.errors.NotLeaderOrFollowerException
 import org.apache.kafka.common.record.{ControlRecordType, EndTransactionMarker, FileRecords, MemoryRecords, RecordBatch, SimpleRecord}
 import org.apache.kafka.common.requests.TransactionResult
 import org.apache.kafka.common.utils.{MockTime, Time}
-import org.apache.kafka.coordinator.group.runtime.CoordinatorLoader.UnknownRecordTypeException
-import org.apache.kafka.coordinator.group.runtime.{CoordinatorPlayback, Deserializer}
-import org.apache.kafka.storage.internals.log.{FetchDataInfo, FetchIsolation, LogOffsetMetadata}
+import org.apache.kafka.coordinator.common.runtime.Deserializer.UnknownRecordTypeException
+import org.apache.kafka.coordinator.common.runtime.{CoordinatorPlayback, Deserializer}
+import org.apache.kafka.server.storage.log.FetchIsolation
+import org.apache.kafka.storage.internals.log.{FetchDataInfo, LogOffsetMetadata}
 import org.apache.kafka.test.TestUtils.assertFutureThrows
 import org.junit.jupiter.api.Assertions.{assertEquals, assertNotNull}
 import org.junit.jupiter.api.{Test, Timeout}
@@ -312,7 +313,7 @@ class CoordinatorLoaderImplTest {
 
       val ex = assertFutureThrows(loader.load(tp, coordinator), classOf[RuntimeException])
 
-      assertEquals(s"Error!", ex.getMessage)
+      assertEquals(s"Deserializing record DefaultRecord(offset=0, timestamp=-1, key=2 bytes, value=2 bytes) from $tp failed due to: Error!", ex.getMessage)
     }
   }
 
