@@ -17,14 +17,13 @@
 
 package kafka.coordinator.group
 
-import java.util.Properties
+import java.util.{OptionalInt, OptionalLong, Properties}
 import java.util.concurrent.locks.{Lock, ReentrantLock}
 import java.util.concurrent.{ConcurrentHashMap, TimeUnit}
-import kafka.common.OffsetAndMetadata
 import kafka.coordinator.AbstractCoordinatorConcurrencyTest
 import kafka.coordinator.AbstractCoordinatorConcurrencyTest._
 import kafka.coordinator.group.GroupCoordinatorConcurrencyTest._
-import kafka.server.{DelayedOperationPurgatory, KafkaConfig, KafkaRequestHandler, RequestLocal}
+import kafka.server.{KafkaConfig, KafkaRequestHandler}
 import kafka.utils.CoreUtils
 import org.apache.kafka.common.{TopicIdPartition, TopicPartition, Uuid}
 import org.apache.kafka.common.internals.Topic
@@ -32,11 +31,12 @@ import org.apache.kafka.common.metrics.Metrics
 import org.apache.kafka.common.message.LeaveGroupRequestData.MemberIdentity
 import org.apache.kafka.common.protocol.{ApiKeys, Errors}
 import org.apache.kafka.common.requests.{JoinGroupRequest, OffsetFetchResponse}
-import org.apache.kafka.common.utils.Time
-import org.apache.kafka.coordinator.group.GroupCoordinatorConfig
+import org.apache.kafka.common.utils.{Time, Utils}
+import org.apache.kafka.coordinator.group.{GroupCoordinatorConfig, OffsetAndMetadata}
+import org.apache.kafka.server.common.RequestLocal
+import org.apache.kafka.server.purgatory.DelayedOperationPurgatory
 import org.junit.jupiter.api.Assertions._
 import org.junit.jupiter.api.{AfterEach, BeforeEach, Test}
-import org.mockito.Mockito.when
 
 import scala.collection._
 import scala.concurrent.duration.Duration
@@ -70,22 +70,18 @@ class GroupCoordinatorConcurrencyTest extends AbstractCoordinatorConcurrencyTest
   override def setUp(): Unit = {
     super.setUp()
 
-    when(zkClient.getTopicPartitionCount(Topic.GROUP_METADATA_TOPIC_NAME))
-      .thenReturn(Some(numPartitions))
-
     serverProps.setProperty(GroupCoordinatorConfig.GROUP_MIN_SESSION_TIMEOUT_MS_CONFIG, ConsumerMinSessionTimeout.toString)
     serverProps.setProperty(GroupCoordinatorConfig.GROUP_MAX_SESSION_TIMEOUT_MS_CONFIG, ConsumerMaxSessionTimeout.toString)
     serverProps.setProperty(GroupCoordinatorConfig.GROUP_INITIAL_REBALANCE_DELAY_MS_CONFIG, GroupInitialRebalanceDelay.toString)
 
     val config = KafkaConfig.fromProps(serverProps)
 
-    heartbeatPurgatory = new DelayedOperationPurgatory[DelayedHeartbeat]("Heartbeat", timer, config.brokerId, reaperEnabled = false)
-    rebalancePurgatory = new DelayedOperationPurgatory[DelayedRebalance]("Rebalance", timer, config.brokerId, reaperEnabled = false)
+    heartbeatPurgatory = new DelayedOperationPurgatory[DelayedHeartbeat]("Heartbeat", timer, config.brokerId, 1000, false, true)
+    rebalancePurgatory = new DelayedOperationPurgatory[DelayedRebalance]("Rebalance", timer, config.brokerId, 1000, false, true)
 
     metrics = new Metrics
     groupCoordinator = GroupCoordinator(config, replicaManager, heartbeatPurgatory, rebalancePurgatory, timer.time, metrics)
-    groupCoordinator.startup(() => zkClient.getTopicPartitionCount(Topic.GROUP_METADATA_TOPIC_NAME).getOrElse(config.offsetsTopicPartitions),
-      enableMetadataExpiration = false)
+    groupCoordinator.startup(() => numPartitions, enableMetadataExpiration = false)
 
     // Transactional appends attempt to schedule to the request handler thread using
     // a non request handler thread. Set this to avoid error.
@@ -96,7 +92,7 @@ class GroupCoordinatorConcurrencyTest extends AbstractCoordinatorConcurrencyTest
   override def tearDown(): Unit = {
     try {
       CoreUtils.swallow(groupCoordinator.shutdown(), this)
-      CoreUtils.swallow(metrics.close(), this)
+      Utils.closeQuietly(metrics, "metrics")
     } finally {
       super.tearDown()
     }
@@ -104,7 +100,7 @@ class GroupCoordinatorConcurrencyTest extends AbstractCoordinatorConcurrencyTest
 
   def createGroupMembers(groupPrefix: String): Set[GroupMember] = {
     (0 until nGroups).flatMap { i =>
-      new Group(s"$groupPrefix$i", nMembersPerGroup, groupCoordinator, replicaManager).members
+      new Group(s"$groupPrefix$i", nMembersPerGroup, groupCoordinator).members
     }.toSet
   }
 
@@ -155,10 +151,9 @@ class GroupCoordinatorConcurrencyTest extends AbstractCoordinatorConcurrencyTest
       groupCoordinator.shutdown()
     groupCoordinator = GroupCoordinator(config, replicaManager, heartbeatPurgatory,
       rebalancePurgatory, timer.time, new Metrics())
-    groupCoordinator.startup(() => zkClient.getTopicPartitionCount(Topic.GROUP_METADATA_TOPIC_NAME).getOrElse(config.offsetsTopicPartitions),
-      enableMetadataExpiration = false)
+    groupCoordinator.startup(() => numPartitions, enableMetadataExpiration = false)
 
-    val members = new Group(s"group", nMembersPerGroup, groupCoordinator, replicaManager)
+    val members = new Group(s"group", nMembersPerGroup, groupCoordinator)
       .members
     val joinOp = new JoinGroupOperation()
 
@@ -209,7 +204,7 @@ class GroupCoordinatorConcurrencyTest extends AbstractCoordinatorConcurrencyTest
 
   class JoinGroupOperation extends GroupOperation[JoinGroupCallbackParams, JoinGroupCallback] {
     override def responseCallback(responsePromise: Promise[JoinGroupCallbackParams]): JoinGroupCallback = {
-      val callback: JoinGroupCallback = responsePromise.success(_)
+      val callback: JoinGroupCallback = responsePromise.success
       callback
     }
     override def runWithCallback(member: GroupMember, responseCallback: JoinGroupCallback): Unit = {
@@ -290,7 +285,7 @@ class GroupCoordinatorConcurrencyTest extends AbstractCoordinatorConcurrencyTest
     }
     override def runWithCallback(member: GroupMember, responseCallback: CommitOffsetCallback): Unit = {
       val tip = new TopicIdPartition(Uuid.randomUuid(), 0, "topic")
-      val offsets = immutable.Map(tip -> OffsetAndMetadata(1, "", Time.SYSTEM.milliseconds()))
+      val offsets = immutable.Map(tip -> new OffsetAndMetadata(1, OptionalInt.empty(), "", Time.SYSTEM.milliseconds(), OptionalLong.empty()))
       groupCoordinator.handleCommitOffsets(member.groupId, member.memberId,
         member.groupInstanceId, member.generationId, offsets, responseCallback)
       replicaManager.tryCompleteActions()
@@ -303,7 +298,7 @@ class GroupCoordinatorConcurrencyTest extends AbstractCoordinatorConcurrencyTest
 
   class CommitTxnOffsetsOperation(lock: Option[Lock] = None) extends CommitOffsetsOperation {
     override def runWithCallback(member: GroupMember, responseCallback: CommitOffsetCallback): Unit = {
-      val offsets = immutable.Map(new TopicIdPartition(Uuid.randomUuid(), 0, "topic") -> OffsetAndMetadata(1, "", Time.SYSTEM.milliseconds()))
+      val offsets = immutable.Map(new TopicIdPartition(Uuid.randomUuid(), 0, "topic") -> new OffsetAndMetadata(1, OptionalInt.empty(), "", Time.SYSTEM.milliseconds(), OptionalLong.empty()))
       val producerId = 1000L
       val producerEpoch : Short = 2
       // When transaction offsets are appended to the log, transactions may be scheduled for
@@ -320,7 +315,7 @@ class GroupCoordinatorConcurrencyTest extends AbstractCoordinatorConcurrencyTest
         // Since the replica manager is mocked we can use a dummy value for transactionalId.
         groupCoordinator.handleTxnCommitOffsets(member.group.groupId, "dummy-txn-id", producerId, producerEpoch,
           JoinGroupRequest.UNKNOWN_MEMBER_ID, Option.empty, JoinGroupRequest.UNKNOWN_GENERATION_ID,
-          offsets, callbackWithTxnCompletion, RequestLocal.NoCaching, ApiKeys.TXN_OFFSET_COMMIT.latestVersion())
+          offsets, callbackWithTxnCompletion, RequestLocal.noCaching, ApiKeys.TXN_OFFSET_COMMIT.latestVersion())
         replicaManager.tryCompleteActions()
       } finally lock.foreach(_.unlock())
     }
@@ -392,11 +387,10 @@ object GroupCoordinatorConcurrencyTest {
   private val DefaultSessionTimeout = 60 * 1000
   private val GroupInitialRebalanceDelay = 50
 
-  class Group(val groupId: String, nMembers: Int,
-      groupCoordinator: GroupCoordinator, replicaManager: TestReplicaManager) {
-    val groupPartitionId = groupCoordinator.partitionFor(groupId)
+  class Group(val groupId: String, nMembers: Int, groupCoordinator: GroupCoordinator) {
+    val groupPartitionId: Int = groupCoordinator.partitionFor(groupId)
     groupCoordinator.groupManager.addOwnedPartition(groupPartitionId)
-    val members = (0 until nMembers).map { i =>
+    val members: Seq[GroupMember] = (0 until nMembers).map { i =>
       new GroupMember(this, groupPartitionId, i == 0)
     }
     def assignment: Map[String, Array[Byte]] = members.map { m => (m.memberId, Array[Byte]()) }.toMap
