@@ -20,22 +20,20 @@ package kafka.server
 import java.net.InetAddress
 import java.util
 import java.util.concurrent.{ExecutionException, TimeUnit}
-import kafka.test.ClusterInstance
-import kafka.test.annotation.{ClusterTest, Type}
-import kafka.test.junit.ClusterTestExtensions
+import org.apache.kafka.common.test.api.ClusterTest
 import kafka.utils.TestUtils
 import org.apache.kafka.clients.admin.{ScramCredentialInfo, ScramMechanism, UserScramCredentialUpsertion}
 import org.apache.kafka.common.errors.{InvalidRequestException, UnsupportedVersionException}
 import org.apache.kafka.common.internals.KafkaFutureImpl
 import org.apache.kafka.common.quota.{ClientQuotaAlteration, ClientQuotaEntity, ClientQuotaFilter, ClientQuotaFilterComponent}
 import org.apache.kafka.common.requests.{AlterClientQuotasRequest, AlterClientQuotasResponse, DescribeClientQuotasRequest, DescribeClientQuotasResponse}
-import org.apache.kafka.server.config.{QuotaConfigs, ZooKeeperInternals}
+import org.apache.kafka.common.test.ClusterInstance
+import org.apache.kafka.server.config.QuotaConfig
 import org.junit.jupiter.api.Assertions._
-import org.junit.jupiter.api.extension.ExtendWith
+import org.junit.jupiter.api.Disabled
 
 import scala.jdk.CollectionConverters._
 
-@ExtendWith(value = Array(classOf[ClusterTestExtensions]))
 class ClientQuotasRequestTest(cluster: ClusterInstance) {
   @ClusterTest
   def testAlterClientQuotasRequest(): Unit = {
@@ -47,63 +45,63 @@ class ClientQuotasRequestTest(cluster: ClusterInstance) {
 
     // Add two configuration entries.
     alterEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(10000.0),
-      QuotaConfigs.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> Some(20000.0)
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(10000.0),
+      QuotaConfig.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> Some(20000.0)
     ), validateOnly = false)
 
     verifyDescribeEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 10000.0,
-      QuotaConfigs.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 10000.0,
+      QuotaConfig.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0
     ))
 
     // Update an existing entry.
     alterEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(15000.0)
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(15000.0)
     ), validateOnly = false)
 
     verifyDescribeEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 15000.0,
-      QuotaConfigs.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 15000.0,
+      QuotaConfig.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0
     ))
 
     // Remove an existing configuration entry.
     alterEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> None
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> None
     ), validateOnly = false)
 
     verifyDescribeEntityQuotas(entity, Map(
-      QuotaConfigs.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0
+      QuotaConfig.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0
     ))
 
     // Remove a non-existent configuration entry.  This should make no changes.
     alterEntityQuotas(entity, Map(
-      QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> None
+      QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> None
     ), validateOnly = false)
 
     verifyDescribeEntityQuotas(entity, Map(
-      QuotaConfigs.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0
+      QuotaConfig.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0
     ))
 
     // Add back a deleted configuration entry.
     alterEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(5000.0)
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(5000.0)
     ), validateOnly = false)
 
     verifyDescribeEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 5000.0,
-      QuotaConfigs.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 5000.0,
+      QuotaConfig.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0
     ))
 
     // Perform a mixed update.
     alterEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(20000.0),
-      QuotaConfigs.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> None,
-      QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(12.3)
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(20000.0),
+      QuotaConfig.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> None,
+      QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(12.3)
     ), validateOnly = false)
 
     verifyDescribeEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
-      QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> 12.3
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
+      QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> 12.3
     ))
   }
 
@@ -113,79 +111,85 @@ class ClientQuotasRequestTest(cluster: ClusterInstance) {
 
     // Set up a configuration.
     alterEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(20000.0),
-      QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(23.45)
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(20000.0),
+      QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(23.45)
     ), validateOnly = false)
 
     verifyDescribeEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
-      QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> 23.45
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
+      QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> 23.45
     ))
 
     // Validate-only addition.
     alterEntityQuotas(entity, Map(
-      QuotaConfigs.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> Some(50000.0)
+      QuotaConfig.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> Some(50000.0)
     ), validateOnly = true)
 
     verifyDescribeEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
-      QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> 23.45
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
+      QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> 23.45
     ))
 
     // Validate-only modification.
     alterEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(10000.0)
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(10000.0)
     ), validateOnly = true)
 
     verifyDescribeEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
-      QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> 23.45
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
+      QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> 23.45
     ))
 
     // Validate-only removal.
     alterEntityQuotas(entity, Map(
-      QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> None
+      QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> None
     ), validateOnly = true)
 
     verifyDescribeEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
-      QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> 23.45
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
+      QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> 23.45
     ))
 
     // Validate-only mixed update.
     alterEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(10000.0),
-      QuotaConfigs.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> Some(50000.0),
-      QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> None
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(10000.0),
+      QuotaConfig.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> Some(50000.0),
+      QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> None
     ), validateOnly = true)
 
     verifyDescribeEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
-      QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> 23.45
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
+      QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> 23.45
     ))
   }
 
-  @ClusterTest(types = Array(Type.ZK)) // No SCRAM for Raft yet
+  @Disabled("TODO: KAFKA-17630 -  Convert ClientQuotasRequestTest#testClientQuotasForScramUsers to kraft")
+  @ClusterTest
   def testClientQuotasForScramUsers(): Unit = {
     val userName = "user"
 
-    val results = cluster.createAdminClient().alterUserScramCredentials(util.Arrays.asList(
-      new UserScramCredentialUpsertion(userName, new ScramCredentialInfo(ScramMechanism.SCRAM_SHA_256, 4096), "password")))
-    results.all.get
+    val admin = cluster.admin()
+    try {
+      val results = admin.alterUserScramCredentials(util.Arrays.asList(
+        new UserScramCredentialUpsertion(userName, new ScramCredentialInfo(ScramMechanism.SCRAM_SHA_256, 4096), "password")))
+      results.all.get
 
-    val entity = new ClientQuotaEntity(Map(ClientQuotaEntity.USER -> userName).asJava)
+      val entity = new ClientQuotaEntity(Map(ClientQuotaEntity.USER -> userName).asJava)
 
-    verifyDescribeEntityQuotas(entity, Map.empty)
+      verifyDescribeEntityQuotas(entity, Map.empty)
 
-    alterEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(10000.0),
-      QuotaConfigs.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> Some(20000.0)
-    ), validateOnly = false)
+      alterEntityQuotas(entity, Map(
+        QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(10000.0),
+        QuotaConfig.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> Some(20000.0)
+      ), validateOnly = false)
 
-    verifyDescribeEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 10000.0,
-      QuotaConfigs.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0
-    ))
+      verifyDescribeEntityQuotas(entity, Map(
+        QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 10000.0,
+        QuotaConfig.CONSUMER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0
+      ))
+    } finally {
+      admin.close()
+    }
   }
 
   @ClusterTest
@@ -203,8 +207,8 @@ class ClientQuotasRequestTest(cluster: ClusterInstance) {
         val result = describeClientQuotas(ClientQuotaFilter.containsOnly(List(entityFilter).asJava))
         assertEquals(expectedMatches.keySet, result.asScala.keySet)
         result.asScala.foreach { case (entity, props) =>
-          assertEquals(Set(QuotaConfigs.IP_CONNECTION_RATE_OVERRIDE_CONFIG), props.asScala.keySet)
-          assertEquals(expectedMatches(entity), props.get(QuotaConfigs.IP_CONNECTION_RATE_OVERRIDE_CONFIG))
+          assertEquals(Set(QuotaConfig.IP_CONNECTION_RATE_OVERRIDE_CONFIG), props.asScala.keySet)
+          assertEquals(expectedMatches(entity), props.get(QuotaConfig.IP_CONNECTION_RATE_OVERRIDE_CONFIG))
           val entityName = entity.entries.get(ClientQuotaEntity.IP)
           // ClientQuotaEntity with null name maps to default entity
           val entityIp = if (entityName == null)
@@ -223,48 +227,48 @@ class ClientQuotasRequestTest(cluster: ClusterInstance) {
     verifyIpQuotas(allIpEntityFilter, Map.empty)
 
     // Add a configuration entry.
-    alterEntityQuotas(entity, Map(QuotaConfigs.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> Some(100.0)), validateOnly = false)
+    alterEntityQuotas(entity, Map(QuotaConfig.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> Some(100.0)), validateOnly = false)
     verifyIpQuotas(entityFilter, Map(entity -> 100.0))
 
     // update existing entry
-    alterEntityQuotas(entity, Map(QuotaConfigs.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> Some(150.0)), validateOnly = false)
+    alterEntityQuotas(entity, Map(QuotaConfig.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> Some(150.0)), validateOnly = false)
     verifyIpQuotas(entityFilter, Map(entity -> 150.0))
 
     // update default value
-    alterEntityQuotas(defaultEntity, Map(QuotaConfigs.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> Some(200.0)), validateOnly = false)
+    alterEntityQuotas(defaultEntity, Map(QuotaConfig.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> Some(200.0)), validateOnly = false)
     verifyIpQuotas(defaultEntityFilter, Map(defaultEntity -> 200.0))
 
     // describe all IP quotas
     verifyIpQuotas(allIpEntityFilter, Map(entity -> 150.0, defaultEntity -> 200.0))
 
     // remove entry
-    alterEntityQuotas(entity, Map(QuotaConfigs.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> None), validateOnly = false)
+    alterEntityQuotas(entity, Map(QuotaConfig.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> None), validateOnly = false)
     verifyIpQuotas(entityFilter, Map.empty)
 
     // remove default value
-    alterEntityQuotas(defaultEntity, Map(QuotaConfigs.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> None), validateOnly = false)
+    alterEntityQuotas(defaultEntity, Map(QuotaConfig.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> None), validateOnly = false)
     verifyIpQuotas(allIpEntityFilter, Map.empty)
   }
 
   @ClusterTest
   def testAlterClientQuotasInvalidRequests(): Unit = {
     var entity = new ClientQuotaEntity(Map(ClientQuotaEntity.USER -> "").asJava)
-    assertThrows(classOf[InvalidRequestException], () => alterEntityQuotas(entity, Map(QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(12.34)), validateOnly = true))
+    assertThrows(classOf[InvalidRequestException], () => alterEntityQuotas(entity, Map(QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(12.34)), validateOnly = true))
 
     entity = new ClientQuotaEntity(Map(ClientQuotaEntity.CLIENT_ID -> "").asJava)
-    assertThrows(classOf[InvalidRequestException], () => alterEntityQuotas(entity, Map(QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(12.34)), validateOnly = true))
+    assertThrows(classOf[InvalidRequestException], () => alterEntityQuotas(entity, Map(QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(12.34)), validateOnly = true))
 
     entity = new ClientQuotaEntity(Map("" -> "name").asJava)
-    assertThrows(classOf[InvalidRequestException], () => alterEntityQuotas(entity, Map(QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(12.34)), validateOnly = true))
+    assertThrows(classOf[InvalidRequestException], () => alterEntityQuotas(entity, Map(QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(12.34)), validateOnly = true))
 
     entity = new ClientQuotaEntity(Map.empty.asJava)
-    assertThrows(classOf[InvalidRequestException], () => alterEntityQuotas(entity, Map(QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(10000.5)), validateOnly = true))
+    assertThrows(classOf[InvalidRequestException], () => alterEntityQuotas(entity, Map(QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(10000.5)), validateOnly = true))
 
     entity = new ClientQuotaEntity(Map(ClientQuotaEntity.USER -> "user").asJava)
     assertThrows(classOf[InvalidRequestException], () => alterEntityQuotas(entity, Map("bad" -> Some(1.0)), validateOnly = true))
 
     entity = new ClientQuotaEntity(Map(ClientQuotaEntity.USER -> "user").asJava)
-    assertThrows(classOf[InvalidRequestException], () => alterEntityQuotas(entity, Map(QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(10000.5)), validateOnly = true))
+    assertThrows(classOf[InvalidRequestException], () => alterEntityQuotas(entity, Map(QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(10000.5)), validateOnly = true))
   }
 
   private def expectInvalidRequestWithMessage(runnable: => Unit, expectedMessage: String): Unit = {
@@ -277,9 +281,9 @@ class ClientQuotasRequestTest(cluster: ClusterInstance) {
     val userAndIpEntity = new ClientQuotaEntity(Map(ClientQuotaEntity.USER -> "user", ClientQuotaEntity.IP -> "1.2.3.4").asJava)
     val clientAndIpEntity = new ClientQuotaEntity(Map(ClientQuotaEntity.CLIENT_ID -> "client", ClientQuotaEntity.IP -> "1.2.3.4").asJava)
     val expectedExceptionMessage = "Invalid quota entity combination"
-    expectInvalidRequestWithMessage(alterEntityQuotas(userAndIpEntity, Map(QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(12.34)),
+    expectInvalidRequestWithMessage(alterEntityQuotas(userAndIpEntity, Map(QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(12.34)),
       validateOnly = true), expectedExceptionMessage)
-    expectInvalidRequestWithMessage(alterEntityQuotas(clientAndIpEntity, Map(QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(12.34)),
+    expectInvalidRequestWithMessage(alterEntityQuotas(clientAndIpEntity, Map(QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG -> Some(12.34)),
       validateOnly = true), expectedExceptionMessage)
   }
 
@@ -288,9 +292,9 @@ class ClientQuotasRequestTest(cluster: ClusterInstance) {
     val invalidHostPatternEntity = new ClientQuotaEntity(Map(ClientQuotaEntity.IP -> "not a valid host because it has spaces").asJava)
     val unresolvableHostEntity = new ClientQuotaEntity(Map(ClientQuotaEntity.IP ->  "RFC2606.invalid").asJava)
     val expectedExceptionMessage = "not a valid IP"
-    expectInvalidRequestWithMessage(alterEntityQuotas(invalidHostPatternEntity, Map(QuotaConfigs.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> Some(50.0)),
+    expectInvalidRequestWithMessage(alterEntityQuotas(invalidHostPatternEntity, Map(QuotaConfig.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> Some(50.0)),
       validateOnly = true), expectedExceptionMessage)
-    expectInvalidRequestWithMessage(alterEntityQuotas(unresolvableHostEntity, Map(QuotaConfigs.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> Some(50.0)),
+    expectInvalidRequestWithMessage(alterEntityQuotas(unresolvableHostEntity, Map(QuotaConfig.IP_CONNECTION_RATE_OVERRIDE_CONFIG -> Some(50.0)),
       validateOnly = true), expectedExceptionMessage)
   }
 
@@ -328,10 +332,10 @@ class ClientQuotasRequestTest(cluster: ClusterInstance) {
 
   private def setupDescribeClientQuotasMatchTest(): Unit = {
     val userClientQuotas = matchUserClientEntities.map { case (e, v) =>
-      e -> Map((QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG, Some(v)))
+      e -> Map((QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG, Some(v)))
     }.toMap
     val ipQuotas = matchIpEntities.map { case (e, v) =>
-      e -> Map((QuotaConfigs.IP_CONNECTION_RATE_OVERRIDE_CONFIG, Some(v)))
+      e -> Map((QuotaConfig.IP_CONNECTION_RATE_OVERRIDE_CONFIG, Some(v)))
     }.toMap
     val result = alterClientQuotas(userClientQuotas ++ ipQuotas, validateOnly = false)
     (matchUserClientEntities ++ matchIpEntities).foreach(e => result(e._1).get(10, TimeUnit.SECONDS))
@@ -357,7 +361,7 @@ class ClientQuotasRequestTest(cluster: ClusterInstance) {
         val result = matchEntity(e)
         assertEquals(1, result.size)
         assertTrue(result.get(e) != null)
-        val value = result.get(e).get(QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG)
+        val value = result.get(e).get(QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG)
         assertNotNull(value)
         assertEquals(value, v, 1e-6)
       }
@@ -399,7 +403,7 @@ class ClientQuotasRequestTest(cluster: ClusterInstance) {
           if (expectedMatchesMap.contains(entity)) {
             val config = result.get(entity)
             assertNotNull(config)
-            val value = config.get(QuotaConfigs.REQUEST_PERCENTAGE_OVERRIDE_CONFIG)
+            val value = config.get(QuotaConfig.REQUEST_PERCENTAGE_OVERRIDE_CONFIG)
             assertNotNull(value)
             assertEquals(expectedValue, value, 1e-6)
           } else {
@@ -410,7 +414,7 @@ class ClientQuotasRequestTest(cluster: ClusterInstance) {
           if (expectedMatchesMap.contains(entity)) {
             val config = result.get(entity)
             assertNotNull(config)
-            val value = config.get(QuotaConfigs.IP_CONNECTION_RATE_OVERRIDE_CONFIG)
+            val value = config.get(QuotaConfig.IP_CONNECTION_RATE_OVERRIDE_CONFIG)
             assertNotNull(value)
             assertEquals(expectedValue, value, 1e-6)
           } else {
@@ -505,28 +509,14 @@ class ClientQuotasRequestTest(cluster: ClusterInstance) {
     val entity = new ClientQuotaEntity(Map(ClientQuotaEntity.USER -> "user with spaces").asJava)
 
     alterEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(20000.0),
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(20000.0),
     ), validateOnly = false)
 
     verifyDescribeEntityQuotas(entity, Map(
-      QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
+      QuotaConfig.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0,
     ))
   }
 
-  @ClusterTest
-  def testClientQuotasWithDefaultName(): Unit = {
-    // An entity using the name associated with the default entity name. The entity's name should be sanitized so
-    // that it does not conflict with the default entity name.
-    val entity = new ClientQuotaEntity(Map(ClientQuotaEntity.CLIENT_ID -> ZooKeeperInternals.DEFAULT_STRING).asJava)
-    alterEntityQuotas(entity, Map(QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> Some(20000.0)), validateOnly = false)
-    verifyDescribeEntityQuotas(entity, Map(QuotaConfigs.PRODUCER_BYTE_RATE_OVERRIDE_CONFIG -> 20000.0))
-
-    // This should not match.
-    val result = describeClientQuotas(
-      ClientQuotaFilter.containsOnly(List(ClientQuotaFilterComponent.ofDefaultEntity(ClientQuotaEntity.CLIENT_ID)).asJava))
-    assert(result.isEmpty)
-  }
-
   private def verifyDescribeEntityQuotas(entity: ClientQuotaEntity, quotas: Map[String, Double]): Unit = {
     TestUtils.tryUntilNoAssertionError(waitTime = 5000L) {
       val components = entity.entries.asScala.map { case (entityType, entityName) =>
