@@ -20,16 +20,15 @@ import kafka.utils.TestUtils
 import org.apache.kafka.clients.producer.ProducerRecord
 import org.apache.kafka.common.config.TopicConfig
 import org.apache.kafka.common.protocol.{ApiKeys, Errors}
-import org.apache.kafka.common.record.{CompressionType, RecordBatch}
+import org.apache.kafka.common.record.CompressionType
 import org.apache.kafka.common.requests.{FetchRequest, FetchResponse, FetchMetadata => JFetchMetadata}
-import org.apache.kafka.common.serialization.{ByteArraySerializer, StringSerializer}
+import org.apache.kafka.common.serialization.StringSerializer
 import org.apache.kafka.common.{IsolationLevel, TopicIdPartition, TopicPartition, Uuid}
 import org.apache.kafka.server.record.BrokerCompressionType
 import org.junit.jupiter.api.Assertions._
 import org.junit.jupiter.params.ParameterizedTest
 import org.junit.jupiter.params.provider.ValueSource
 
-import java.io.DataInputStream
 import java.util
 import java.util.Optional
 import scala.collection.Seq
@@ -43,7 +42,7 @@ import scala.util.Random
 class FetchRequestTest extends BaseFetchRequestTest {
 
   @ParameterizedTest
-  @ValueSource(strings = Array("zk","kraft"))
+  @ValueSource(strings = Array("kraft"))
   def testBrokerRespectsPartitionsOrderAndSizeLimits(quorum: String): Unit = {
     initProducer()
 
@@ -146,7 +145,7 @@ class FetchRequestTest extends BaseFetchRequestTest {
   }
 
   @ParameterizedTest
-  @ValueSource(strings = Array("zk","kraft"))
+  @ValueSource(strings = Array("kraft"))
   def testFetchRequestV4WithReadCommitted(quorum: String): Unit = {
     initProducer()
     val maxPartitionBytes = 200
@@ -165,7 +164,7 @@ class FetchRequestTest extends BaseFetchRequestTest {
   }
 
   @ParameterizedTest
-  @ValueSource(strings = Array("zk","kraft"))
+  @ValueSource(strings = Array("kraft"))
   def testFetchRequestToNonReplica(quorum: String): Unit = {
     val topic = "topic"
     val partition = 0
@@ -196,13 +195,13 @@ class FetchRequestTest extends BaseFetchRequestTest {
   }
 
   @ParameterizedTest
-  @ValueSource(strings = Array("zk", "kraft"))
+  @ValueSource(strings = Array("kraft"))
   def testLastFetchedEpochValidation(quorum: String): Unit = {
     checkLastFetchedEpochValidation(ApiKeys.FETCH.latestVersion())
   }
 
   @ParameterizedTest
-  @ValueSource(strings = Array("zk", "kraft"))
+  @ValueSource(strings = Array("kraft"))
   def testLastFetchedEpochValidationV12(quorum: String): Unit = {
     checkLastFetchedEpochValidation(12)
   }
@@ -251,13 +250,13 @@ class FetchRequestTest extends BaseFetchRequestTest {
   }
 
   @ParameterizedTest
-  @ValueSource(strings = Array("zk","kraft"))
+  @ValueSource(strings = Array("kraft"))
   def testCurrentEpochValidation(quorum: String): Unit = {
     checkCurrentEpochValidation(ApiKeys.FETCH.latestVersion())
   }
 
   @ParameterizedTest
-  @ValueSource(strings = Array("zk","kraft"))
+  @ValueSource(strings = Array("kraft"))
   def testCurrentEpochValidationV12(quorum: String): Unit = {
     checkCurrentEpochValidation(12)
   }
@@ -301,13 +300,13 @@ class FetchRequestTest extends BaseFetchRequestTest {
   }
 
   @ParameterizedTest
-  @ValueSource(strings = Array("zk","kraft"))
+  @ValueSource(strings = Array("kraft"))
   def testEpochValidationWithinFetchSession(quorum: String): Unit = {
     checkEpochValidationWithinFetchSession(ApiKeys.FETCH.latestVersion())
   }
 
   @ParameterizedTest
-  @ValueSource(strings = Array("zk","kraft"))
+  @ValueSource(strings = Array("kraft"))
   def testEpochValidationWithinFetchSessionV12(quorum: String): Unit = {
     checkEpochValidationWithinFetchSession(12)
   }
@@ -363,163 +362,13 @@ class FetchRequestTest extends BaseFetchRequestTest {
     assertResponseErrorForEpoch(Errors.UNKNOWN_LEADER_EPOCH, 2, Optional.of(leaderEpoch + 1))
   }
 
-  /**
-   * Tests that down-conversions don't leak memory. Large down conversions are triggered
-   * in the server. The client closes its connection after reading partial data when the
-   * channel is muted in the server. If buffers are not released this will result in OOM.
-   */
-  @ParameterizedTest
-  @ValueSource(strings = Array("zk","kraft"))
-  def testDownConversionWithConnectionFailure(quorum: String): Unit = {
-    val (topicPartition, leaderId) = createTopics(numTopics = 1, numPartitions = 1).head
-    val topicIds = getTopicIds().asJava
-    val topicNames = topicIds.asScala.map(_.swap).asJava
-
-    val msgValueLen = 100 * 1000
-    val batchSize = 4 * msgValueLen
-    val producer = TestUtils.createProducer(
-      bootstrapServers(),
-      lingerMs = Int.MaxValue,
-      deliveryTimeoutMs = Int.MaxValue,
-      batchSize = batchSize,
-      keySerializer = new StringSerializer,
-      valueSerializer = new ByteArraySerializer)
-    val bytes = new Array[Byte](msgValueLen)
-    val futures = try {
-      (0 to 1000).map { _ =>
-        producer.send(new ProducerRecord(topicPartition.topic, topicPartition.partition, "key", bytes))
-      }
-    } finally {
-      producer.close()
-    }
-    // Check futures to ensure sends succeeded, but do this after close since the last
-    // batch is not complete, but sent when the producer is closed
-    futures.foreach(_.get)
-
-    def fetch(version: Short, maxPartitionBytes: Int, closeAfterPartialResponse: Boolean): Option[FetchResponse] = {
-      val fetchRequest = FetchRequest.Builder.forConsumer(version, Int.MaxValue, 0, createPartitionMap(maxPartitionBytes,
-        Seq(topicPartition))).build(version)
-
-      val socket = connect(brokerSocketServer(leaderId))
-      try {
-        send(fetchRequest, socket)
-        if (closeAfterPartialResponse) {
-          // read some data to ensure broker has muted this channel and then close socket
-          val size = new DataInputStream(socket.getInputStream).readInt()
-          // Check that we have received almost `maxPartitionBytes` (minus a tolerance) since in
-          // the case of OOM, the size will be significantly smaller. We can't check for exactly
-          // maxPartitionBytes since we use approx message sizes that include only the message value.
-          assertTrue(size > maxPartitionBytes - batchSize,
-              s"Fetch size too small $size, broker may have run out of memory")
-          None
-        } else {
-          Some(receive[FetchResponse](socket, ApiKeys.FETCH, version))
-        }
-      } finally {
-        socket.close()
-      }
-    }
-
-    val version = 1.toShort
-    (0 to 15).foreach(_ => fetch(version, maxPartitionBytes = msgValueLen * 1000, closeAfterPartialResponse = true))
-
-    val response = fetch(version, maxPartitionBytes = batchSize, closeAfterPartialResponse = false)
-    val fetchResponse = response.getOrElse(throw new IllegalStateException("No fetch response"))
-    val partitionData = fetchResponse.responseData(topicNames, version).get(topicPartition)
-    assertEquals(Errors.NONE.code, partitionData.errorCode)
-    val batches = FetchResponse.recordsOrFail(partitionData).batches.asScala.toBuffer
-    assertEquals(3, batches.size) // size is 3 (not 4) since maxPartitionBytes=msgValueSize*4, excluding key and headers
-  }
-
-  /**
-    * Ensure that we respect the fetch offset when returning records that were converted from an uncompressed v2
-    * record batch to multiple v0/v1 record batches with size 1. If the fetch offset points to inside the record batch,
-    * some records have to be dropped during the conversion.
-    */
-  @ParameterizedTest
-  @ValueSource(strings = Array("zk","kraft"))
-  def testDownConversionFromBatchedToUnbatchedRespectsOffset(quorum: String): Unit = {
-    // Increase linger so that we have control over the batches created
-    producer = TestUtils.createProducer(bootstrapServers(),
-      retries = 5,
-      keySerializer = new StringSerializer,
-      valueSerializer = new StringSerializer,
-      lingerMs = 30 * 1000,
-      deliveryTimeoutMs = 60 * 1000)
-
-    val (topicPartition, leaderId) = createTopics(numTopics = 1, numPartitions = 1).head
-    val topic = topicPartition.topic
-    val topicIds = getTopicIds().asJava
-    val topicNames = topicIds.asScala.map(_.swap).asJava
-
-    val firstBatchFutures = (0 until 10).map(i => producer.send(new ProducerRecord(topic, s"key-$i", s"value-$i")))
-    producer.flush()
-    val secondBatchFutures = (10 until 25).map(i => producer.send(new ProducerRecord(topic, s"key-$i", s"value-$i")))
-    producer.flush()
-
-    firstBatchFutures.foreach(_.get)
-    secondBatchFutures.foreach(_.get)
-
-    def check(fetchOffset: Long, requestVersion: Short, expectedOffset: Long, expectedNumBatches: Int, expectedMagic: Byte): Unit = {
-      var batchesReceived = 0
-      var currentFetchOffset = fetchOffset
-      var currentExpectedOffset = expectedOffset
-
-      // With KIP-283, we might not receive all batches in a single fetch request so loop through till we have consumed
-      // all batches we are interested in.
-      while (batchesReceived < expectedNumBatches) {
-        val fetchRequest = FetchRequest.Builder.forConsumer(requestVersion, Int.MaxValue, 0, createPartitionMap(Int.MaxValue,
-          Seq(topicPartition), Map(topicPartition -> currentFetchOffset))).build(requestVersion)
-        val fetchResponse = sendFetchRequest(leaderId, fetchRequest)
-
-        // validate response
-        val partitionData = fetchResponse.responseData(topicNames, requestVersion).get(topicPartition)
-        assertEquals(Errors.NONE.code, partitionData.errorCode)
-        assertTrue(partitionData.highWatermark > 0)
-        val batches = FetchResponse.recordsOrFail(partitionData).batches.asScala.toBuffer
-        val batch = batches.head
-        assertEquals(expectedMagic, batch.magic)
-        assertEquals(currentExpectedOffset, batch.baseOffset)
-
-        currentFetchOffset = batches.last.lastOffset + 1
-        currentExpectedOffset += (batches.last.lastOffset - batches.head.baseOffset + 1)
-        batchesReceived += batches.size
-      }
-
-      assertEquals(expectedNumBatches, batchesReceived)
-    }
-
-    // down conversion to message format 0, batches of 1 message are returned so we receive the exact offset we requested
-    check(fetchOffset = 3, expectedOffset = 3, requestVersion = 1, expectedNumBatches = 22,
-      expectedMagic = RecordBatch.MAGIC_VALUE_V0)
-    check(fetchOffset = 15, expectedOffset = 15, requestVersion = 1, expectedNumBatches = 10,
-      expectedMagic = RecordBatch.MAGIC_VALUE_V0)
-
-    // down conversion to message format 1, batches of 1 message are returned so we receive the exact offset we requested
-    check(fetchOffset = 3, expectedOffset = 3, requestVersion = 3, expectedNumBatches = 22,
-      expectedMagic = RecordBatch.MAGIC_VALUE_V1)
-    check(fetchOffset = 15, expectedOffset = 15, requestVersion = 3, expectedNumBatches = 10,
-      expectedMagic = RecordBatch.MAGIC_VALUE_V1)
-
-    // no down conversion, we receive a single batch so the received offset won't necessarily be the same
-    check(fetchOffset = 3, expectedOffset = 0, requestVersion = 4, expectedNumBatches = 2,
-      expectedMagic = RecordBatch.MAGIC_VALUE_V2)
-    check(fetchOffset = 15, expectedOffset = 10, requestVersion = 4, expectedNumBatches = 1,
-      expectedMagic = RecordBatch.MAGIC_VALUE_V2)
-
-    // no down conversion, we receive a single batch and the exact offset we requested because it happens to be the
-    // offset of the first record in the batch
-    check(fetchOffset = 10, expectedOffset = 10, requestVersion = 4, expectedNumBatches = 1,
-      expectedMagic = RecordBatch.MAGIC_VALUE_V2)
-  }
-
   /**
    * Test that when an incremental fetch session contains partitions with an error,
    * those partitions are returned in all incremental fetch requests.
    * This tests using FetchRequests that don't use topic IDs
    */
   @ParameterizedTest
-  @ValueSource(strings = Array("zk","kraft"))
+  @ValueSource(strings = Array("kraft"))
   def testCreateIncrementalFetchWithPartitionsInErrorV12(quorum: String): Unit = {
     def createConsumerFetchRequest(topicPartitions: Seq[TopicPartition],
                            metadata: JFetchMetadata,
@@ -582,7 +431,7 @@ class FetchRequestTest extends BaseFetchRequestTest {
    * Test that when a Fetch Request receives an unknown topic ID, it returns a top level error.
    */
   @ParameterizedTest
-  @ValueSource(strings = Array("zk","kraft"))
+  @ValueSource(strings = Array("kraft"))
   def testFetchWithPartitionsWithIdError(quorum: String): Unit = {
     def createConsumerFetchRequest(fetchData: util.LinkedHashMap[TopicPartition, FetchRequest.PartitionData],
                            metadata: JFetchMetadata,
@@ -627,7 +476,7 @@ class FetchRequestTest extends BaseFetchRequestTest {
   }
 
   @ParameterizedTest
-  @ValueSource(strings = Array("zk","kraft"))
+  @ValueSource(strings = Array("kraft"))
   def testZStdCompressedTopic(quorum: String): Unit = {
     // ZSTD compressed topic
     val topicConfig = Map(TopicConfig.COMPRESSION_TYPE_CONFIG -> BrokerCompressionType.ZSTD.name)
@@ -654,7 +503,7 @@ class FetchRequestTest extends BaseFetchRequestTest {
 
     val res0 = sendFetchRequest(leaderId, req0)
     val data0 = res0.responseData(topicNames, 9).get(topicPartition)
-    assertEquals(Errors.UNSUPPORTED_COMPRESSION_TYPE.code, data0.errorCode)
+    assertEquals(Errors.NONE.code, data0.errorCode)
 
     // fetch request with version 10: works fine!
     val req1= new FetchRequest.Builder(0, 10, -1, -1, Int.MaxValue, 0,
@@ -675,7 +524,7 @@ class FetchRequestTest extends BaseFetchRequestTest {
   }
 
   @ParameterizedTest
-  @ValueSource(strings = Array("zk","kraft"))
+  @ValueSource(strings = Array("kraft"))
   def testZStdCompressedRecords(quorum: String): Unit = {
     // Producer compressed topic
     val topicConfig = Map(TopicConfig.COMPRESSION_TYPE_CONFIG -> BrokerCompressionType.PRODUCER.name)
@@ -702,63 +551,34 @@ class FetchRequestTest extends BaseFetchRequestTest {
       "key3", "value3")).get
     producer2.close()
 
-    // fetch request with fetch version v1 (magic 0):
-    // gzip compressed record is returned with down-conversion.
-    // zstd compressed record raises UNSUPPORTED_COMPRESSION_TYPE error.
-    val req0 = new FetchRequest.Builder(0, 1, -1, -1, Int.MaxValue, 0,
+    // fetch request with version 4: even though zstd is officially only supported from v10, this actually succeeds
+    // since the server validation is only active when zstd is configured via a topic config, the server doesn't
+    // check the record batches and hence has no mechanism to detect the case where the producer sent record batches
+    // compressed with zstd and the topic config for compression is the default
+    val req0 = new FetchRequest.Builder(0, 4, -1, -1, Int.MaxValue, 0,
       createPartitionMap(300, Seq(topicPartition), Map.empty))
-      .setMaxBytes(800)
-      .build()
-
+      .setMaxBytes(800).build()
     val res0 = sendFetchRequest(leaderId, req0)
-    val data0 = res0.responseData(topicNames, 1).get(topicPartition)
+    val data0 = res0.responseData(topicNames, 10).get(topicPartition)
     assertEquals(Errors.NONE.code, data0.errorCode)
-    assertEquals(1, records(data0).size)
-
-    val req1 = new FetchRequest.Builder(0, 1, -1, -1, Int.MaxValue, 0,
-      createPartitionMap(300, Seq(topicPartition), Map(topicPartition -> 1L)))
-      .setMaxBytes(800).build()
-
-    val res1 = sendFetchRequest(leaderId, req1)
-    val data1 = res1.responseData(topicNames, 1).get(topicPartition)
-    assertEquals(Errors.UNSUPPORTED_COMPRESSION_TYPE.code, data1.errorCode)
-
-    // fetch request with fetch version v3 (magic 1):
-    // gzip compressed record is returned with down-conversion.
-    // zstd compressed record raises UNSUPPORTED_COMPRESSION_TYPE error.
-    val req2 = new FetchRequest.Builder(2, 3, -1, -1, Int.MaxValue, 0,
-      createPartitionMap(300, Seq(topicPartition), Map.empty))
-      .setMaxBytes(800).build()
-
-    val res2 = sendFetchRequest(leaderId, req2)
-    val data2 = res2.responseData(topicNames, 3).get(topicPartition)
-    assertEquals(Errors.NONE.code, data2.errorCode)
-    assertEquals(1, records(data2).size)
-
-    val req3 = new FetchRequest.Builder(0, 1, -1, -1, Int.MaxValue, 0,
-      createPartitionMap(300, Seq(topicPartition), Map(topicPartition -> 1L)))
-      .setMaxBytes(800).build()
-
-    val res3 = sendFetchRequest(leaderId, req3)
-    val data3 = res3.responseData(topicNames, 1).get(topicPartition)
-    assertEquals(Errors.UNSUPPORTED_COMPRESSION_TYPE.code, data3.errorCode)
+    assertEquals(3, records(data0).size)
 
     // fetch request with version 10: works fine!
-    val req4 = new FetchRequest.Builder(0, 10, -1, -1, Int.MaxValue, 0,
+    val req1 = new FetchRequest.Builder(0, 10, -1, -1, Int.MaxValue, 0,
       createPartitionMap(300, Seq(topicPartition), Map.empty))
       .setMaxBytes(800).build()
-    val res4 = sendFetchRequest(leaderId, req4)
-    val data4 = res4.responseData(topicNames, 10).get(topicPartition)
-    assertEquals(Errors.NONE.code, data4.errorCode)
-    assertEquals(3, records(data4).size)
+    val res1 = sendFetchRequest(leaderId, req1)
+    val data1 = res1.responseData(topicNames, 10).get(topicPartition)
+    assertEquals(Errors.NONE.code, data1.errorCode)
+    assertEquals(3, records(data1).size)
 
-    val req5 = new FetchRequest.Builder(0, ApiKeys.FETCH.latestVersion(), -1, -1, Int.MaxValue, 0,
+    val req2 = new FetchRequest.Builder(0, ApiKeys.FETCH.latestVersion(), -1, -1, Int.MaxValue, 0,
       createPartitionMap(300, Seq(topicPartition), Map.empty))
       .setMaxBytes(800).build()
-    val res5 = sendFetchRequest(leaderId, req5)
-    val data5 = res5.responseData(topicNames, ApiKeys.FETCH.latestVersion()).get(topicPartition)
-    assertEquals(Errors.NONE.code, data5.errorCode)
-    assertEquals(3, records(data5).size)
+    val res2 = sendFetchRequest(leaderId, req2)
+    val data2 = res2.responseData(topicNames, ApiKeys.FETCH.latestVersion()).get(topicPartition)
+    assertEquals(Errors.NONE.code, data2.errorCode)
+    assertEquals(3, records(data2).size)
   }
 
   private def checkFetchResponse(expectedPartitions: Seq[TopicPartition], fetchResponse: FetchResponse,
