@@ -16,20 +16,27 @@
  */
 package kafka.server
 
-import kafka.test.ClusterInstance
+import kafka.network.SocketServer
 import kafka.utils.TestUtils
-import org.apache.kafka.common.TopicPartition
+import org.apache.kafka.clients.producer.{KafkaProducer, ProducerRecord, RecordMetadata}
+import org.apache.kafka.common.{TopicIdPartition, TopicPartition, Uuid}
 import org.apache.kafka.common.message.DeleteGroupsResponseData.{DeletableGroupResult, DeletableGroupResultCollection}
 import org.apache.kafka.common.message.LeaveGroupRequestData.MemberIdentity
 import org.apache.kafka.common.message.LeaveGroupResponseData.MemberResponse
 import org.apache.kafka.common.message.SyncGroupRequestData.SyncGroupRequestAssignment
-import org.apache.kafka.common.message.{ConsumerGroupDescribeRequestData, ConsumerGroupDescribeResponseData, ConsumerGroupHeartbeatRequestData, ConsumerGroupHeartbeatResponseData, DeleteGroupsRequestData, DeleteGroupsResponseData, DescribeGroupsRequestData, DescribeGroupsResponseData, HeartbeatRequestData, HeartbeatResponseData, JoinGroupRequestData, JoinGroupResponseData, LeaveGroupResponseData, ListGroupsRequestData, ListGroupsResponseData, OffsetCommitRequestData, OffsetCommitResponseData, OffsetDeleteRequestData, OffsetDeleteResponseData, OffsetFetchResponseData, SyncGroupRequestData, SyncGroupResponseData}
+import org.apache.kafka.common.message.{AddOffsetsToTxnRequestData, AddOffsetsToTxnResponseData, ConsumerGroupDescribeRequestData, ConsumerGroupDescribeResponseData, ConsumerGroupHeartbeatRequestData, ConsumerGroupHeartbeatResponseData, DeleteGroupsRequestData, DeleteGroupsResponseData, DescribeGroupsRequestData, DescribeGroupsResponseData, EndTxnRequestData, HeartbeatRequestData, HeartbeatResponseData, InitProducerIdRequestData, JoinGroupRequestData, JoinGroupResponseData, LeaveGroupResponseData, ListGroupsRequestData, ListGroupsResponseData, OffsetCommitRequestData, OffsetCommitResponseData, OffsetDeleteRequestData, OffsetDeleteResponseData, OffsetFetchResponseData, ShareGroupDescribeRequestData, ShareGroupDescribeResponseData, ShareGroupHeartbeatRequestData, ShareGroupHeartbeatResponseData, SyncGroupRequestData, SyncGroupResponseData, TxnOffsetCommitRequestData, TxnOffsetCommitResponseData}
 import org.apache.kafka.common.protocol.{ApiKeys, Errors}
-import org.apache.kafka.common.requests.{AbstractRequest, AbstractResponse, ConsumerGroupDescribeRequest, ConsumerGroupDescribeResponse, ConsumerGroupHeartbeatRequest, ConsumerGroupHeartbeatResponse, DeleteGroupsRequest, DeleteGroupsResponse, DescribeGroupsRequest, DescribeGroupsResponse, HeartbeatRequest, HeartbeatResponse, JoinGroupRequest, JoinGroupResponse, LeaveGroupRequest, LeaveGroupResponse, ListGroupsRequest, ListGroupsResponse, OffsetCommitRequest, OffsetCommitResponse, OffsetDeleteRequest, OffsetDeleteResponse, OffsetFetchRequest, OffsetFetchResponse, SyncGroupRequest, SyncGroupResponse}
+import org.apache.kafka.common.requests.{AbstractRequest, AbstractResponse, AddOffsetsToTxnRequest, AddOffsetsToTxnResponse, ConsumerGroupDescribeRequest, ConsumerGroupDescribeResponse, ConsumerGroupHeartbeatRequest, ConsumerGroupHeartbeatResponse, DeleteGroupsRequest, DeleteGroupsResponse, DescribeGroupsRequest, DescribeGroupsResponse, EndTxnRequest, EndTxnResponse, HeartbeatRequest, HeartbeatResponse, InitProducerIdRequest, InitProducerIdResponse, JoinGroupRequest, JoinGroupResponse, LeaveGroupRequest, LeaveGroupResponse, ListGroupsRequest, ListGroupsResponse, OffsetCommitRequest, OffsetCommitResponse, OffsetDeleteRequest, OffsetDeleteResponse, OffsetFetchRequest, OffsetFetchResponse, ShareGroupDescribeRequest, ShareGroupDescribeResponse, ShareGroupHeartbeatRequest, ShareGroupHeartbeatResponse, SyncGroupRequest, SyncGroupResponse, TxnOffsetCommitRequest, TxnOffsetCommitResponse}
+import org.apache.kafka.common.serialization.StringSerializer
+import org.apache.kafka.common.test.ClusterInstance
+import org.apache.kafka.common.utils.ProducerIdAndEpoch
+import org.apache.kafka.controller.ControllerRequestContextUtil.ANONYMOUS_CONTEXT
 import org.junit.jupiter.api.Assertions.{assertEquals, fail}
 
-import java.util.Comparator
+import java.util.{Comparator, Properties}
 import java.util.stream.Collectors
+import scala.collection.Seq
+import scala.collection.convert.ImplicitConversions.{`collection AsScalaIterable`, `map AsScala`}
 import scala.jdk.CollectionConverters._
 import scala.reflect.ClassTag
 
@@ -38,34 +45,125 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
 
   private def controllerServers(): Seq[ControllerServer] = cluster.controllers().values().asScala.toSeq
 
+  protected var producer: KafkaProducer[String, String] = _
+
   protected def createOffsetsTopic(): Unit = {
-    TestUtils.createOffsetsTopicWithAdmin(
-      admin = cluster.createAdminClient(),
-      brokers = brokers(),
-      controllers = controllerServers()
-    )
+    val admin = cluster.admin()
+    try {
+      TestUtils.createOffsetsTopicWithAdmin(
+        admin = admin,
+        brokers = brokers(),
+        controllers = controllerServers()
+      )
+    } finally {
+      admin.close()
+    }
+  }
+
+  protected def createTransactionStateTopic(): Unit = {
+    val admin = cluster.admin()
+    try {
+      TestUtils.createTransactionStateTopicWithAdmin(
+        admin = admin,
+        brokers = brokers(),
+        controllers = controllerServers()
+      )
+    } finally {
+      admin.close()
+    }
   }
 
   protected def createTopic(
     topic: String,
     numPartitions: Int
   ): Unit = {
-    TestUtils.createTopicWithAdmin(
-      admin = cluster.createAdminClient(),
-      brokers = brokers(),
-      controllers = controllerServers(),
-      topic = topic,
-      numPartitions = numPartitions
-    )
+    val admin = cluster.admin()
+    try {
+      TestUtils.createTopicWithAdmin(
+        admin = admin,
+        brokers = brokers(),
+        controllers = controllerServers(),
+        topic = topic,
+        numPartitions = numPartitions
+      )
+    } finally {
+      admin.close()
+    }
+  }
+
+  protected def createTopicAndReturnLeaders(
+    topic: String,
+    numPartitions: Int = 1,
+    replicationFactor: Int = 1,
+    topicConfig: Properties = new Properties
+  ): Map[TopicIdPartition, Int] = {
+    val admin = cluster.admin()
+    try {
+      val partitionToLeader = TestUtils.createTopicWithAdmin(
+        admin = admin,
+        topic = topic,
+        brokers = brokers(),
+        controllers = controllerServers(),
+        numPartitions = numPartitions,
+        replicationFactor = replicationFactor,
+        topicConfig = topicConfig
+      )
+      partitionToLeader.map { case (partition, leader) => new TopicIdPartition(getTopicIds(topic), new TopicPartition(topic, partition)) -> leader }
+    } finally {
+      admin.close()
+    }
   }
 
   protected def isUnstableApiEnabled: Boolean = {
-    cluster.config.serverProperties.get("unstable.api.versions.enable") == "true"
+    cluster.brokers.values.stream.allMatch(b => b.config.unstableApiVersionsEnabled)
   }
 
   protected def isNewGroupCoordinatorEnabled: Boolean = {
-    cluster.config.serverProperties.get("group.coordinator.new.enable") == "true" ||
-      cluster.config.serverProperties.get("group.coordinator.rebalance.protocols").contains("consumer")
+    cluster.brokers.values.stream.allMatch(b => b.config.isNewGroupCoordinatorEnabled)
+  }
+
+  protected def getTopicIds: Map[String, Uuid] = {
+    cluster.controllers().get(cluster.controllerIds().iterator().next()).controller.findAllTopicIds(ANONYMOUS_CONTEXT).get().toMap
+  }
+
+  protected def getBrokers: Seq[KafkaBroker] = {
+    cluster.brokers.values().stream().collect(Collectors.toList[KafkaBroker]).toSeq
+  }
+
+  protected def bootstrapServers(): String = {
+    TestUtils.plaintextBootstrapServers(getBrokers)
+  }
+
+  protected def initProducer(): Unit = {
+    producer = TestUtils.createProducer(cluster.bootstrapServers(),
+      keySerializer = new StringSerializer, valueSerializer = new StringSerializer)
+  }
+
+  protected def closeProducer(): Unit = {
+    if( producer != null )
+      producer.close()
+  }
+
+  protected def produceData(
+    topicIdPartition: TopicIdPartition, 
+    numMessages: Int
+  ): Seq[RecordMetadata] = {
+    val records = for {
+      messageIndex <- 0 until numMessages
+    } yield {
+      val suffix = s"$topicIdPartition-$messageIndex"
+      new ProducerRecord(topicIdPartition.topic, topicIdPartition.partition, s"key $suffix", s"value $suffix")
+    }
+    records.map(producer.send(_).get)
+  }
+
+  protected def produceData(
+    topicIdPartition: TopicIdPartition, 
+    key: String, 
+    value: String
+  ): RecordMetadata = {
+    producer.send(new ProducerRecord(topicIdPartition.topic, topicIdPartition.partition,
+      key, value)).get
   }
 
   protected def commitOffset(
@@ -76,7 +174,7 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
     partition: Int,
     offset: Long,
     expectedError: Errors,
-    version: Short
+    version: Short = ApiKeys.OFFSET_COMMIT.latestVersion(isUnstableApiEnabled)
   ): Unit = {
     val request = new OffsetCommitRequest.Builder(
       new OffsetCommitRequestData()
@@ -110,6 +208,114 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
     assertEquals(expectedResponse, response.data)
   }
 
+  protected def commitTxnOffset(
+     groupId: String,
+     memberId: String,
+     generationId: Int,
+     producerId: Long,
+     producerEpoch: Short,
+     transactionalId: String,
+     topic: String,
+     partition: Int,
+     offset: Long,
+     expectedError: Errors,
+     version: Short = ApiKeys.TXN_OFFSET_COMMIT.latestVersion(isUnstableApiEnabled)
+  ): Unit = {
+    val request = new TxnOffsetCommitRequest.Builder(
+      new TxnOffsetCommitRequestData()
+        .setGroupId(groupId)
+        .setMemberId(memberId)
+        .setGenerationId(generationId)
+        .setProducerId(producerId)
+        .setProducerEpoch(producerEpoch)
+        .setTransactionalId(transactionalId)
+        .setTopics(List(
+          new TxnOffsetCommitRequestData.TxnOffsetCommitRequestTopic()
+            .setName(topic)
+            .setPartitions(List(
+              new TxnOffsetCommitRequestData.TxnOffsetCommitRequestPartition()
+                .setPartitionIndex(partition)
+                .setCommittedOffset(offset)
+            ).asJava)
+        ).asJava)
+    ).build(version)
+
+    val expectedResponse = new TxnOffsetCommitResponseData()
+      .setTopics(List(
+        new TxnOffsetCommitResponseData.TxnOffsetCommitResponseTopic()
+          .setName(topic)
+          .setPartitions(List(
+            new TxnOffsetCommitResponseData.TxnOffsetCommitResponsePartition()
+              .setPartitionIndex(partition)
+              .setErrorCode(expectedError.code)
+            ).asJava)
+        ).asJava)
+
+    val response = connectAndReceive[TxnOffsetCommitResponse](request)
+    assertEquals(expectedResponse, response.data)
+  }
+
+  protected def addOffsetsToTxn(
+    groupId: String,
+    producerId: Long,
+    producerEpoch: Short,
+    transactionalId: String,
+    version: Short = ApiKeys.ADD_OFFSETS_TO_TXN.latestVersion(isUnstableApiEnabled)
+  ): Unit = {
+    val request = new AddOffsetsToTxnRequest.Builder(
+      new AddOffsetsToTxnRequestData()
+        .setTransactionalId(transactionalId)
+        .setProducerId(producerId)
+        .setProducerEpoch(producerEpoch)
+        .setGroupId(groupId)
+    ).build(version)
+
+    val response = connectAndReceive[AddOffsetsToTxnResponse](request)
+    assertEquals(new AddOffsetsToTxnResponseData(), response.data)
+  }
+
+  protected def initProducerId(
+    transactionalId: String,
+    transactionTimeoutMs: Int = 60000,
+    producerIdAndEpoch: ProducerIdAndEpoch,
+    expectedError: Errors,
+    version: Short = ApiKeys.INIT_PRODUCER_ID.latestVersion(isUnstableApiEnabled)
+  ): ProducerIdAndEpoch = {
+    val request = new InitProducerIdRequest.Builder(
+      new InitProducerIdRequestData()
+        .setTransactionalId(transactionalId)
+        .setTransactionTimeoutMs(transactionTimeoutMs)
+        .setProducerId(producerIdAndEpoch.producerId)
+        .setProducerEpoch(producerIdAndEpoch.epoch))
+      .build(version)
+
+    val response = connectAndReceive[InitProducerIdResponse](request).data
+    assertEquals(expectedError.code, response.errorCode)
+    new ProducerIdAndEpoch(response.producerId, response.producerEpoch)
+  }
+
+  protected def endTxn(
+    producerId: Long,
+    producerEpoch: Short,
+    transactionalId: String,
+    isTransactionV2Enabled: Boolean,
+    committed: Boolean,
+    expectedError: Errors,
+    version: Short = ApiKeys.END_TXN.latestVersion(isUnstableApiEnabled)
+  ): Unit = {
+    val request = new EndTxnRequest.Builder(
+      new EndTxnRequestData()
+        .setProducerId(producerId)
+        .setProducerEpoch(producerEpoch)
+        .setTransactionalId(transactionalId)
+        .setCommitted(committed),
+      isUnstableApiEnabled,
+      isTransactionV2Enabled
+    ).build(version)
+
+    assertEquals(expectedError, connectAndReceive[EndTxnResponse](request).error)
+  }
+
   protected def fetchOffsets(
     groupId: String,
     memberId: String,
@@ -236,7 +442,7 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
     }
   }
 
-  protected def syncGroupWithOldProtocol(
+  protected def verifySyncGroupWithOldProtocol(
     groupId: String,
     memberId: String,
     generationId: Int,
@@ -248,6 +454,37 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
     expectedAssignment: Array[Byte] = Array.empty,
     expectedError: Errors = Errors.NONE,
     version: Short = ApiKeys.SYNC_GROUP.latestVersion(isUnstableApiEnabled)
+  ): SyncGroupResponseData = {
+    val syncGroupResponseData = syncGroupWithOldProtocol(
+      groupId = groupId,
+      memberId = memberId,
+      generationId = generationId,
+      protocolType = protocolType,
+      protocolName = protocolName,
+      assignments = assignments,
+      version = version
+    )
+
+    assertEquals(
+      new SyncGroupResponseData()
+        .setErrorCode(expectedError.code)
+        .setProtocolType(if (version >= 5) expectedProtocolType else null)
+        .setProtocolName(if (version >= 5) expectedProtocolName else null)
+        .setAssignment(expectedAssignment),
+      syncGroupResponseData
+    )
+
+    syncGroupResponseData
+  }
+
+  protected def syncGroupWithOldProtocol(
+    groupId: String,
+    memberId: String,
+    generationId: Int,
+    protocolType: String = "consumer",
+    protocolName: String = "consumer-range",
+    assignments: List[SyncGroupRequestData.SyncGroupRequestAssignment] = List.empty,
+    version: Short = ApiKeys.SYNC_GROUP.latestVersion(isUnstableApiEnabled)
   ): SyncGroupResponseData = {
     val syncGroupRequestData = new SyncGroupRequestData()
       .setGroupId(groupId)
@@ -259,16 +496,6 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
 
     val syncGroupRequest = new SyncGroupRequest.Builder(syncGroupRequestData).build(version)
     val syncGroupResponse = connectAndReceive[SyncGroupResponse](syncGroupRequest)
-    
-    assertEquals(
-      new SyncGroupResponseData()
-        .setErrorCode(expectedError.code)
-        .setProtocolType(if (version >= 5) expectedProtocolType else null)
-        .setProtocolName(if (version >= 5) expectedProtocolName else null)
-        .setAssignment(expectedAssignment),
-      syncGroupResponse.data
-    )
-
     syncGroupResponse.data
   }
 
@@ -330,7 +557,7 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
 
     if (completeRebalance) {
       // Send the sync group request to complete the rebalance.
-      syncGroupWithOldProtocol(
+      verifySyncGroupWithOldProtocol(
         groupId = groupId,
         memberId = rejoinGroupResponseData.memberId,
         generationId = rejoinGroupResponseData.generationId,
@@ -348,6 +575,7 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
     groupId: String,
     groupInstanceId: String,
     metadata: Array[Byte] = Array.empty,
+    assignment: Array[Byte] = Array.empty,
     completeRebalance: Boolean = true
   ): (String, Int) = {
     val joinGroupResponseData = sendJoinRequest(
@@ -358,19 +586,24 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
 
     if (completeRebalance) {
       // Send the sync group request to complete the rebalance.
-      syncGroupWithOldProtocol(
+      verifySyncGroupWithOldProtocol(
         groupId = groupId,
         memberId = joinGroupResponseData.memberId,
-        generationId = joinGroupResponseData.generationId
+        generationId = joinGroupResponseData.generationId,
+        assignments = List(new SyncGroupRequestAssignment()
+          .setMemberId(joinGroupResponseData.memberId)
+          .setAssignment(assignment)),
+        expectedAssignment = assignment
       )
     }
 
     (joinGroupResponseData.memberId, joinGroupResponseData.generationId)
   }
 
-  protected def joinConsumerGroupWithNewProtocol(groupId: String): (String, Int) = {
+  protected def joinConsumerGroupWithNewProtocol(groupId: String, memberId: String = ""): (String, Int) = {
     val consumerGroupHeartbeatResponseData = consumerGroupHeartbeat(
       groupId = groupId,
+      memberId = memberId,
       rebalanceTimeoutMs = 5 * 60 * 1000,
       subscribedTopicNames = List("foo"),
       topicPartitions = List.empty
@@ -382,7 +615,7 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
     if (useNewProtocol) {
       // Note that we heartbeat only once to join the group and assume
       // that the test will complete within the session timeout.
-      joinConsumerGroupWithNewProtocol(groupId)
+      joinConsumerGroupWithNewProtocol(groupId, Uuid.randomUuid().toString)
     } else {
       // Note that we don't heartbeat and assume that the test will
       // complete within the session timeout.
@@ -421,7 +654,7 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
 
   protected def consumerGroupDescribe(
     groupIds: List[String],
-    includeAuthorizedOperations: Boolean,
+    includeAuthorizedOperations: Boolean = false,
     version: Short = ApiKeys.CONSUMER_GROUP_DESCRIBE.latestVersion(isUnstableApiEnabled)
   ): List[ConsumerGroupDescribeResponseData.DescribedGroup] = {
     val consumerGroupDescribeRequest = new ConsumerGroupDescribeRequest.Builder(
@@ -434,13 +667,29 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
     consumerGroupDescribeResponse.data.groups.asScala.toList
   }
 
+  protected def shareGroupDescribe(
+     groupIds: List[String],
+     includeAuthorizedOperations: Boolean,
+     version: Short = ApiKeys.SHARE_GROUP_DESCRIBE.latestVersion(isUnstableApiEnabled)
+   ): List[ShareGroupDescribeResponseData.DescribedGroup] = {
+    val shareGroupDescribeRequest = new ShareGroupDescribeRequest.Builder(
+      new ShareGroupDescribeRequestData()
+        .setGroupIds(groupIds.asJava)
+        .setIncludeAuthorizedOperations(includeAuthorizedOperations),
+      true
+    ).build(version)
+
+    val shareGroupDescribeResponse = connectAndReceive[ShareGroupDescribeResponse](shareGroupDescribeRequest)
+    shareGroupDescribeResponse.data.groups.asScala.toList
+  }
+
   protected def heartbeat(
     groupId: String,
     generationId: Int,
     memberId: String,
     groupInstanceId: String = null,
     expectedError: Errors = Errors.NONE,
-    version: Short
+    version: Short = ApiKeys.HEARTBEAT.latestVersion(isUnstableApiEnabled)
   ): HeartbeatResponseData = {
     val heartbeatRequest = new HeartbeatRequest.Builder(
       new HeartbeatRequestData()
@@ -466,7 +715,8 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
     serverAssignor: String = null,
     subscribedTopicNames: List[String] = null,
     topicPartitions: List[ConsumerGroupHeartbeatRequestData.TopicPartitions] = null,
-    expectedError: Errors = Errors.NONE
+    expectedError: Errors = Errors.NONE,
+    version: Short = ApiKeys.CONSUMER_GROUP_HEARTBEAT.latestVersion(isUnstableApiEnabled)
   ): ConsumerGroupHeartbeatResponseData = {
     val consumerGroupHeartbeatRequest = new ConsumerGroupHeartbeatRequest.Builder(
       new ConsumerGroupHeartbeatRequestData()
@@ -478,9 +728,8 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
         .setRebalanceTimeoutMs(rebalanceTimeoutMs)
         .setSubscribedTopicNames(subscribedTopicNames.asJava)
         .setServerAssignor(serverAssignor)
-        .setTopicPartitions(topicPartitions.asJava),
-      true
-    ).build()
+        .setTopicPartitions(topicPartitions.asJava)
+    ).build(version)
 
     // Send the request until receiving a successful response. There is a delay
     // here because the group coordinator is loaded in the background.
@@ -493,6 +742,35 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
     consumerGroupHeartbeatResponse.data
   }
 
+  protected def shareGroupHeartbeat(
+    groupId: String,
+    memberId: String = Uuid.randomUuid.toString,
+    memberEpoch: Int = 0,
+    rackId: String = null,
+    subscribedTopicNames: List[String] = null,
+    expectedError: Errors = Errors.NONE
+  ): ShareGroupHeartbeatResponseData = {
+    val shareGroupHeartbeatRequest = new ShareGroupHeartbeatRequest.Builder(
+      new ShareGroupHeartbeatRequestData()
+        .setGroupId(groupId)
+        .setMemberId(memberId)
+        .setMemberEpoch(memberEpoch)
+        .setRackId(rackId)
+        .setSubscribedTopicNames(subscribedTopicNames.asJava),
+      true
+    ).build()
+
+    // Send the request until receiving a successful response. There is a delay
+    // here because the group coordinator is loaded in the background.
+    var shareGroupHeartbeatResponse: ShareGroupHeartbeatResponse = null
+    TestUtils.waitUntilTrue(() => {
+      shareGroupHeartbeatResponse = connectAndReceive[ShareGroupHeartbeatResponse](shareGroupHeartbeatRequest)
+      shareGroupHeartbeatResponse.data.errorCode == expectedError.code
+    }, msg = s"Could not heartbeat successfully. Last response $shareGroupHeartbeatResponse.")
+
+    shareGroupHeartbeatResponse.data
+  }
+
   protected def leaveGroupWithNewProtocol(
     groupId: String,
     memberId: String
@@ -504,14 +782,12 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
     )
   }
 
-  protected def leaveGroupWithOldProtocol(
+  protected def classicLeaveGroup(
     groupId: String,
     memberIds: List[String],
     groupInstanceIds: List[String] = null,
-    expectedLeaveGroupError: Errors,
-    expectedMemberErrors: List[Errors],
     version: Short = ApiKeys.LEAVE_GROUP.latestVersion(isUnstableApiEnabled)
-  ): Unit = {
+  ): LeaveGroupResponseData = {
     val leaveGroupRequest = new LeaveGroupRequest.Builder(
       groupId,
       List.tabulate(memberIds.length) { i =>
@@ -521,6 +797,24 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
       }.asJava
     ).build(version)
 
+    connectAndReceive[LeaveGroupResponse](leaveGroupRequest).data
+  }
+
+  protected def leaveGroupWithOldProtocol(
+    groupId: String,
+    memberIds: List[String],
+    groupInstanceIds: List[String] = null,
+    expectedLeaveGroupError: Errors,
+    expectedMemberErrors: List[Errors],
+    version: Short = ApiKeys.LEAVE_GROUP.latestVersion(isUnstableApiEnabled)
+  ): Unit = {
+    val leaveGroupResponse = classicLeaveGroup(
+      groupId,
+      memberIds,
+      groupInstanceIds,
+      version
+    )
+
     val expectedResponseData = new LeaveGroupResponseData()
     if (expectedLeaveGroupError != Errors.NONE) {
       expectedResponseData.setErrorCode(expectedLeaveGroupError.code)
@@ -534,8 +828,7 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
         }.asJava)
     }
 
-    val leaveGroupResponse = connectAndReceive[LeaveGroupResponse](leaveGroupRequest)
-    assertEquals(expectedResponseData, leaveGroupResponse.data)
+    assertEquals(expectedResponseData, leaveGroupResponse)
   }
 
   protected def leaveGroup(
@@ -585,4 +878,21 @@ class GroupCoordinatorBaseRequestTest(cluster: ClusterInstance) {
       cluster.clientListener()
     )
   }
+
+  protected def connectAndReceive[T <: AbstractResponse](
+    request: AbstractRequest,
+    destination: Int
+  )(implicit classTag: ClassTag[T]): T = {
+    IntegrationTestUtils.connectAndReceive[T](
+      request,
+      brokerSocketServer(destination),
+      cluster.clientListener()
+    )
+  }
+
+  private def brokerSocketServer(brokerId: Int): SocketServer = {
+    getBrokers.find { broker =>
+      broker.config.brokerId == brokerId
+    }.map(_.socketServer).getOrElse(throw new IllegalStateException(s"Could not find broker with id $brokerId"))
+  }
 }
