@@ -27,10 +27,11 @@ import kafka.utils.TestUtils
 import net.sourceforge.argparse4j.inf.ArgumentParserException
 import org.apache.kafka.common.metadata.UserScramCredentialRecord
 import org.apache.kafka.common.utils.Utils
-import org.apache.kafka.server.common.Features
+import org.apache.kafka.server.common.{Feature, MetadataVersion}
 import org.apache.kafka.metadata.bootstrap.BootstrapDirectory
 import org.apache.kafka.metadata.properties.{MetaPropertiesEnsemble, PropertiesUtils}
 import org.apache.kafka.metadata.storage.FormatterException
+import org.apache.kafka.network.SocketServerConfigs
 import org.apache.kafka.raft.QuorumConfig
 import org.apache.kafka.server.config.{KRaftConfigs, ServerConfigs, ServerLogConfigs}
 import org.junit.jupiter.api.Assertions.{assertEquals, assertFalse, assertThrows, assertTrue}
@@ -50,11 +51,12 @@ class StorageToolTest {
     properties.setProperty(KRaftConfigs.PROCESS_ROLES_CONFIG, "controller")
     properties.setProperty(KRaftConfigs.NODE_ID_CONFIG, "2")
     properties.setProperty(QuorumConfig.QUORUM_VOTERS_CONFIG, s"2@localhost:9092")
-    properties.setProperty(KRaftConfigs.CONTROLLER_LISTENER_NAMES_CONFIG, "PLAINTEXT")
+    properties.put(KRaftConfigs.CONTROLLER_LISTENER_NAMES_CONFIG, "CONTROLLER")
+    properties.put(SocketServerConfigs.LISTENERS_CONFIG, "CONTROLLER://:9092")
     properties
   }
 
-  val allFeatures = Features.FEATURES.toList
+  val testingFeatures = Feature.FEATURES.toList.asJava
 
   @Test
   def testConfigToLogDirectories(): Unit = {
@@ -292,19 +294,6 @@ Found problem:
         "Failed to find content in output: " + stream.toString())
   }
 
-  @Test
-  def testFormatFailsInZkMode(): Unit = {
-    val availableDirs = Seq(TestUtils.tempDir())
-    val properties = new Properties()
-    properties.setProperty("log.dirs", availableDirs.mkString(","))
-    properties.setProperty("zookeeper.connect", "localhost:2181")
-    val stream = new ByteArrayOutputStream()
-    assertEquals("The kafka configuration file appears to be for a legacy cluster. " +
-      "Formatting is only supported for clusters in KRaft mode.",
-        assertThrows(classOf[TerseFailure],
-          () => runFormatCommand(stream, properties)).getMessage)
-  }
-
   @Test
   def testFormatWithReleaseVersion(): Unit = {
     val availableDirs = Seq(TestUtils.tempDir())
@@ -336,7 +325,7 @@ Found problem:
     properties.putAll(defaultStaticQuorumProperties)
     properties.setProperty("log.dirs", availableDirs.mkString(","))
     assertEquals("Unsupported feature: non.existent.feature. Supported features are: " +
-      "kraft.version, transaction.version",
+      "eligible.leader.replicas.version, group.version, kraft.version, transaction.version",
         assertThrows(classOf[FormatterException], () =>
           runFormatCommand(new ByteArrayOutputStream(), properties,
             Seq("--feature", "non.existent.feature=20"))).getMessage)
@@ -368,26 +357,12 @@ Found problem:
       "Failed to find content in output: " + stream.toString())
   }
 
-  @Test
-  def testFormatWithReleaseVersionDefault(): Unit = {
-    val availableDirs = Seq(TestUtils.tempDir())
-    val properties = new Properties()
-    properties.putAll(defaultStaticQuorumProperties)
-    properties.setProperty("log.dirs", availableDirs.mkString(","))
-    properties.setProperty("inter.broker.protocol.version", "3.7")
-    val stream = new ByteArrayOutputStream()
-    assertEquals(0, runFormatCommand(stream, properties))
-    assertTrue(stream.toString().contains("3.7-IV4"),
-      "Failed to find content in output: " + stream.toString())
-  }
-
   @Test
   def testFormatWithReleaseVersionDefaultAndReleaseVersion(): Unit = {
     val availableDirs = Seq(TestUtils.tempDir())
     val properties = new Properties()
     properties.putAll(defaultStaticQuorumProperties)
     properties.setProperty("log.dirs", availableDirs.mkString(","))
-    properties.setProperty("inter.broker.protocol.version", "3.7")
     val stream = new ByteArrayOutputStream()
     assertEquals(0, runFormatCommand(stream, properties, Seq(
       "--release-version", "3.6-IV0",
@@ -400,10 +375,7 @@ Found problem:
   def testFormatWithStandaloneFlagOnBrokerFails(): Unit = {
     val availableDirs = Seq(TestUtils.tempDir())
     val properties = new Properties()
-    properties.setProperty("process.roles", "broker")
-    properties.setProperty("node.id", "0")
-    properties.setProperty("controller.listener.names", "CONTROLLER")
-    properties.setProperty("controller.quorum.bootstrap.servers", "localhost:9093")
+    properties.putAll(defaultStaticQuorumProperties)
     properties.setProperty("log.dirs", availableDirs.mkString(","))
     val stream = new ByteArrayOutputStream()
     val arguments = ListBuffer[String]("--release-version", "3.9-IV0", "--standalone")
@@ -486,14 +458,19 @@ Found problem:
               Seq("--release-version", "3.9-IV0"))).getMessage)
   }
 
-  @Test
-  def testFormatWithNoInitialControllersSucceedsOnController(): Unit = {
+  @ParameterizedTest
+  @ValueSource(booleans = Array(false, true))
+  def testFormatWithNoInitialControllersSucceedsOnController(setKraftVersionFeature: Boolean): Unit = {
     val availableDirs = Seq(TestUtils.tempDir())
     val properties = new Properties()
     properties.putAll(defaultDynamicQuorumProperties)
     properties.setProperty("log.dirs", availableDirs.mkString(","))
     val stream = new ByteArrayOutputStream()
     val arguments = ListBuffer[String]("--release-version", "3.9-IV0", "--no-initial-controllers")
+    if (setKraftVersionFeature) {
+      arguments += "--feature"
+      arguments += "kraft.version=1"
+    }
     assertEquals(0, runFormatCommand(stream, properties, arguments.toSeq))
     assertTrue(stream.toString().
       contains("Formatting metadata directory %s".format(availableDirs.head)),
@@ -539,6 +516,185 @@ Found problem:
       "Failed to find content in output: " + stream.toString())
   }
 
+  private def runVersionMappingCommand(
+    stream: ByteArrayOutputStream,
+    releaseVersion: String
+  ): Int = {
+    // Prepare the arguments list
+    val arguments = ListBuffer[String]("version-mapping")
+
+    // Add the release version argument
+    if (releaseVersion != null) {
+      arguments += "--release-version"
+      arguments += releaseVersion
+    }
+
+    // Execute the StorageTool with the arguments
+    StorageTool.execute(arguments.toArray, new PrintStream(stream))
+  }
+
+  @Test
+  def testVersionMappingWithValidReleaseVersion(): Unit = {
+    val stream = new ByteArrayOutputStream()
+    // Test with a valid release version
+    assertEquals(0, runVersionMappingCommand(stream, MetadataVersion.MINIMUM_VERSION.toString))
+
+    val output = stream.toString()
+    val metadataVersion = MetadataVersion.MINIMUM_VERSION
+    // Check that the metadata version is correctly included in the output
+    assertTrue(output.contains(s"metadata.version=${metadataVersion.featureLevel()} (${metadataVersion.version()})"),
+      s"Output did not contain expected Metadata Version: $output"
+    )
+
+    for (feature <- Feature.PRODUCTION_FEATURES.asScala) {
+      val featureLevel = feature.defaultLevel(metadataVersion)
+      assertTrue(output.contains(s"${feature.featureName()}=$featureLevel"),
+        s"Output did not contain expected feature mapping: $output"
+      )
+    }
+  }
+
+  @Test
+  def testVersionMappingWithNoReleaseVersion(): Unit = {
+    val properties = new Properties()
+    properties.putAll(defaultStaticQuorumProperties)
+
+    val stream = new ByteArrayOutputStream()
+    assertEquals(0, runVersionMappingCommand(stream, null))
+
+    val output = stream.toString
+    val metadataVersion = MetadataVersion.latestProduction()
+    // Check that the metadata version is correctly included in the output
+    assertTrue(output.contains(s"metadata.version=${metadataVersion.featureLevel()} (${metadataVersion.version()})"),
+      s"Output did not contain expected Metadata Version: $output"
+    )
+
+    for (feature <- Feature.PRODUCTION_FEATURES.asScala) {
+      val featureLevel = feature.defaultLevel(metadataVersion)
+      assertTrue(output.contains(s"${feature.featureName()}=$featureLevel"),
+        s"Output did not contain expected feature mapping: $output"
+      )
+    }
+  }
+
+  @Test
+  def testVersionMappingWithInvalidReleaseVersion(): Unit = {
+    val properties = new Properties()
+    properties.putAll(defaultStaticQuorumProperties)
+
+    val stream = new ByteArrayOutputStream()
+    // Test with an invalid release version
+    val exception = assertThrows(classOf[TerseFailure], () => {
+      runVersionMappingCommand(stream, "2.9-IV2")
+    })
+
+    assertEquals("Unknown release version '2.9-IV2'." +
+      " Supported versions are: " + MetadataVersion.MINIMUM_VERSION.version +
+      " to " + MetadataVersion.LATEST_PRODUCTION.version, exception.getMessage
+    )
+
+    val exception2 = assertThrows(classOf[TerseFailure], () => {
+      runVersionMappingCommand(stream, "invalid")
+    })
+
+    assertEquals("Unknown release version 'invalid'." +
+      " Supported versions are: " + MetadataVersion.MINIMUM_VERSION.version +
+      " to " + MetadataVersion.LATEST_PRODUCTION.version, exception2.getMessage
+    )
+  }
+
+  private def runFeatureDependenciesCommand(
+    stream: ByteArrayOutputStream,
+    features: Seq[String]
+  ): Int = {
+    val arguments = ListBuffer[String]("feature-dependencies")
+    features.foreach(feature => {
+      arguments += "--feature"
+      arguments += feature
+    })
+    StorageTool.execute(arguments.toArray, new PrintStream(stream))
+  }
+
+  @Test
+  def testTestingFeatureDependencies(): Unit = {
+    val stream = new ByteArrayOutputStream()
+    val namespace = StorageTool.parseArguments(Array("feature-dependencies", "--feature", "test.feature.version=2"))
+
+    StorageTool.runFeatureDependenciesCommand(namespace, new PrintStream(stream), testingFeatures)
+
+    val output = stream.toString.trim
+    System.out.println(output)
+
+    val latestTestingVersion = MetadataVersion.latestTesting()
+    val latestTestingVersionString = s"metadata.version=${latestTestingVersion.featureLevel()} (${latestTestingVersion.version()})"
+
+    val expectedOutput =
+      s"""test.feature.version=2 requires:
+         |    $latestTestingVersionString
+         |""".stripMargin.trim
+
+    assertEquals(expectedOutput, output)
+  }
+
+  @Test
+  def testMultipleFeatureDependencies(): Unit = {
+    val stream = new ByteArrayOutputStream()
+    val features = Seq("transaction.version=2", "group.version=1")
+
+    assertEquals(0, runFeatureDependenciesCommand(stream, features))
+
+    val output = stream.toString.trim
+    System.out.println(output)
+
+    val expectedOutput =
+      s"""transaction.version=2 has no dependencies.
+         |group.version=1 has no dependencies.
+         |""".stripMargin.trim
+
+    assertEquals(expectedOutput, output)
+  }
+
+  @Test
+  def testHandleFeatureDependenciesForFeatureWithNoDependencies(): Unit = {
+    val stream = new ByteArrayOutputStream()
+    assertEquals(0, runFeatureDependenciesCommand(stream, Seq("metadata.version=17")))
+
+    val output = stream.toString.trim
+
+    assertEquals("metadata.version=17 (3.7-IV2) has no dependencies.", output)
+  }
+
+  @Test
+  def testHandleFeatureDependenciesForUnknownFeature(): Unit = {
+    val stream = new ByteArrayOutputStream()
+    val exception = assertThrows(classOf[TerseFailure], () => {
+      runFeatureDependenciesCommand(stream, Seq("unknown.feature.version=1"))
+    })
+
+    assertEquals("Unknown feature: unknown.feature.version", exception.getMessage)
+  }
+
+  @Test
+  def testHandleFeatureDependenciesForFeatureWithUnknownFeatureVersion(): Unit = {
+    val stream = new ByteArrayOutputStream()
+    val exception = assertThrows(classOf[TerseFailure], () => {
+      runFeatureDependenciesCommand(stream, Seq("transaction.version=1000"))
+    })
+
+    assertEquals("Feature level 1000 is not supported for feature transaction.version", exception.getMessage)
+  }
+
+  @Test
+  def testHandleFeatureDependenciesForInvalidVersionFormat(): Unit = {
+    val stream = new ByteArrayOutputStream()
+
+    val exception = assertThrows(classOf[TerseFailure], () => {
+      runFeatureDependenciesCommand(stream, Seq("metadata.version=invalid"))
+    })
+
+    assertEquals("Invalid version format: invalid for feature metadata.version", exception.getMessage)
+  }
+
   @Test
   def testBootstrapScramRecords(): Unit = {
     val availableDirs = Seq(TestUtils.tempDir())
@@ -556,7 +712,7 @@ Found problem:
 
     // Not doing full SCRAM record validation since that's covered elsewhere.
     // Just checking that we generate the correct number of records
-    val bootstrapMetadata = new BootstrapDirectory(availableDirs.head.toString, java.util.Optional.empty()).read
+    val bootstrapMetadata = new BootstrapDirectory(availableDirs.head.toString).read
     val scramRecords = bootstrapMetadata.records().asScala
       .filter(apiMessageAndVersion => apiMessageAndVersion.message().isInstanceOf[UserScramCredentialRecord])
       .map(apiMessageAndVersion => apiMessageAndVersion.message().asInstanceOf[UserScramCredentialRecord])
