@@ -107,6 +107,13 @@ public final class FieldSpec {
             if (!this.type.isArray() && !this.type.isStruct()) {
                 throw new RuntimeException("Non-array or Struct field " + name + " cannot have fields");
             }
+            // Check struct invariants
+            if (this.type.isStruct() || this.type.isStructArray()) {
+                new StructSpec(name,
+                    versions,
+                    Versions.NONE_STRING, // version deprecations not supported at field level
+                    fields);
+            }
         }
 
         if (flexibleVersions == null || flexibleVersions.isEmpty()) {
@@ -446,11 +453,15 @@ public final class FieldSpec {
         } else if (type.isRecords()) {
             return "null";
         } else if (type.isStruct()) {
-            if (!fieldDefault.isEmpty()) {
+            if (fieldDefault.equals("null")) {
+                validateNullDefault();
+                return "null";
+            } else if (!fieldDefault.isEmpty()) {
                 throw new RuntimeException("Invalid default for struct field " +
-                    name + ": custom defaults are not supported for struct fields.");
+                    name + ".  The only valid default for a struct field " +
+                    "is the empty struct or null.");
             }
-            return "new " + type.toString() + "()";
+            return "new " + type + "()";
         } else if (type.isArray()) {
             if (fieldDefault.equals("null")) {
                 validateNullDefault();
