@@ -253,7 +253,7 @@ public class ConsumerGroupMember extends ModernGroupMember {
     /**
      * The rebalance timeout provided by the member.
      */
-    private int rebalanceTimeoutMs;
+    private final int rebalanceTimeoutMs;
 
     /**
      * The subscription pattern configured by the member.
@@ -360,7 +360,7 @@ public class ConsumerGroupMember extends ModernGroupMember {
      */
     public Optional<Integer> classicProtocolSessionTimeout() {
         if (useClassicProtocol()) {
-            return Optional.ofNullable(classicMemberMetadata.sessionTimeoutMs());
+            return Optional.of(classicMemberMetadata.sessionTimeoutMs());
         } else {
             return Optional.empty();
         }
@@ -408,7 +408,8 @@ public class ConsumerGroupMember extends ModernGroupMember {
             .setInstanceId(instanceId)
             .setRackId(rackId)
             .setSubscribedTopicNames(subscribedTopicNames == null ? null : new ArrayList<>(subscribedTopicNames))
-            .setSubscribedTopicRegex(subscribedTopicRegex);
+            .setSubscribedTopicRegex(subscribedTopicRegex)
+            .setMemberType(useClassicProtocol() ? (byte) 0 : (byte) 1);
     }
 
     private static List<ConsumerGroupDescribeResponseData.TopicPartitions> topicPartitionsFromMap(
@@ -517,4 +518,12 @@ public class ConsumerGroupMember extends ModernGroupMember {
             ", classicMemberMetadata='" + classicMemberMetadata + '\'' +
             ')';
     }
+
+    public static String subscribedTopicRegexOrNull(ConsumerGroupMember member) {
+        if (member != null && member.subscribedTopicRegex() != null && !member.subscribedTopicRegex().isEmpty()) {
+            return member.subscribedTopicRegex();
+        } else {
+            return null;
+        }
+    }
 }
