@@ -51,7 +51,7 @@ public class JsonConverterBenchmark {
     private JsonConverter converter;
 
     @Param({"true", "false"})
-    private boolean afterBurnModule;
+    private boolean blackbirdModule;
 
     @State(Scope.Benchmark)
     public static class Data {
@@ -425,7 +425,7 @@ public class JsonConverterBenchmark {
     @Setup(Level.Trial)
     public void setup(BenchmarkParams params)  {
 
-        converter = new JsonConverter(Boolean.parseBoolean(params.getParam("afterBurnModule")));
+        converter = new JsonConverter(Boolean.parseBoolean(params.getParam("blackbirdModule")));
         converter.configure(Collections.emptyMap(), false);
     }
 
