@@ -19,14 +19,15 @@ package org.apache.kafka.controller;
 
 import org.apache.kafka.clients.admin.FeatureUpdate;
 import org.apache.kafka.common.metadata.FeatureLevelRecord;
-import org.apache.kafka.common.metadata.ZkMigrationStateRecord;
 import org.apache.kafka.common.protocol.Errors;
 import org.apache.kafka.common.requests.ApiError;
 import org.apache.kafka.common.utils.LogContext;
 import org.apache.kafka.metadata.FinalizedControllerFeatures;
 import org.apache.kafka.metadata.VersionRange;
-import org.apache.kafka.metadata.migration.ZkMigrationState;
 import org.apache.kafka.server.common.ApiMessageAndVersion;
+import org.apache.kafka.server.common.EligibleLeaderReplicasVersion;
+import org.apache.kafka.server.common.Feature;
+import org.apache.kafka.server.common.KRaftVersion;
 import org.apache.kafka.server.common.MetadataVersion;
 import org.apache.kafka.server.mutable.BoundedList;
 import org.apache.kafka.timeline.SnapshotRegistry;
@@ -43,7 +44,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Optional;
-import java.util.TreeMap;
 import java.util.function.Consumer;
 
 import static org.apache.kafka.common.metadata.MetadataRecordType.FEATURE_LEVEL_RECORD;
@@ -56,7 +56,6 @@ public class FeatureControlManager {
         private SnapshotRegistry snapshotRegistry = null;
         private QuorumFeatures quorumFeatures = null;
         private MetadataVersion metadataVersion = MetadataVersion.latestProduction();
-        private MetadataVersion minimumBootstrapVersion = MetadataVersion.MINIMUM_BOOTSTRAP_VERSION;
         private ClusterFeatureSupportDescriber clusterSupportDescriber = new ClusterFeatureSupportDescriber() {
             @Override
             public Iterator<Entry<Integer, Map<String, VersionRange>>> brokerSupported() {
@@ -89,11 +88,6 @@ public class FeatureControlManager {
             return this;
         }
 
-        Builder setMinimumBootstrapVersion(MetadataVersion minimumBootstrapVersion) {
-            this.minimumBootstrapVersion = minimumBootstrapVersion;
-            return this;
-        }
-
         Builder setClusterFeatureSupportDescriber(ClusterFeatureSupportDescriber clusterSupportDescriber) {
             this.clusterSupportDescriber = clusterSupportDescriber;
             return this;
@@ -105,7 +99,7 @@ public class FeatureControlManager {
             if (quorumFeatures == null) {
                 Map<String, VersionRange> localSupportedFeatures = new HashMap<>();
                 localSupportedFeatures.put(MetadataVersion.FEATURE_NAME, VersionRange.of(
-                        MetadataVersion.MINIMUM_KRAFT_VERSION.featureLevel(),
+                        MetadataVersion.MINIMUM_VERSION.featureLevel(),
                         MetadataVersion.latestProduction().featureLevel()));
                 quorumFeatures = new QuorumFeatures(0, localSupportedFeatures, Collections.singletonList(0));
             }
@@ -114,7 +108,6 @@ public class FeatureControlManager {
                 quorumFeatures,
                 snapshotRegistry,
                 metadataVersion,
-                minimumBootstrapVersion,
                 clusterSupportDescriber
             );
         }
@@ -137,16 +130,6 @@ public class FeatureControlManager {
      */
     private final TimelineObject<MetadataVersion> metadataVersion;
 
-    /**
-     * The current ZK migration state
-     */
-    private final TimelineObject<ZkMigrationState> migrationControlState;
-
-    /**
-     * The minimum bootstrap version that we can't downgrade before.
-     */
-    private final MetadataVersion minimumBootstrapVersion;
-
     /**
      * Gives information about the supported versions in the cluster.
      */
@@ -157,35 +140,39 @@ public class FeatureControlManager {
         QuorumFeatures quorumFeatures,
         SnapshotRegistry snapshotRegistry,
         MetadataVersion metadataVersion,
-        MetadataVersion minimumBootstrapVersion,
         ClusterFeatureSupportDescriber clusterSupportDescriber
     ) {
         this.log = logContext.logger(FeatureControlManager.class);
         this.quorumFeatures = quorumFeatures;
         this.finalizedVersions = new TimelineHashMap<>(snapshotRegistry, 0);
         this.metadataVersion = new TimelineObject<>(snapshotRegistry, metadataVersion);
-        this.minimumBootstrapVersion = minimumBootstrapVersion;
-        this.migrationControlState = new TimelineObject<>(snapshotRegistry, ZkMigrationState.NONE);
         this.clusterSupportDescriber = clusterSupportDescriber;
     }
 
-    ControllerResult<Map<String, ApiError>> updateFeatures(
+    ControllerResult<ApiError> updateFeatures(
         Map<String, Short> updates,
         Map<String, FeatureUpdate.UpgradeType> upgradeTypes,
         boolean validateOnly
     ) {
-        TreeMap<String, ApiError> results = new TreeMap<>();
         List<ApiMessageAndVersion> records =
                 BoundedList.newArrayBacked(MAX_RECORDS_PER_USER_OP);
+
+        Map<String, Short> proposedUpdatedVersions = new HashMap<>(finalizedVersions);
+        proposedUpdatedVersions.put(MetadataVersion.FEATURE_NAME, metadataVersion.get().featureLevel());
+        proposedUpdatedVersions.putAll(updates);
+
         for (Entry<String, Short> entry : updates.entrySet()) {
-            results.put(entry.getKey(), updateFeature(entry.getKey(), entry.getValue(),
-                upgradeTypes.getOrDefault(entry.getKey(), FeatureUpdate.UpgradeType.UPGRADE), records));
+            ApiError error = updateFeature(entry.getKey(), entry.getValue(),
+                upgradeTypes.getOrDefault(entry.getKey(), FeatureUpdate.UpgradeType.UPGRADE), records, proposedUpdatedVersions);
+            if (!error.error().equals(Errors.NONE)) {
+                return ControllerResult.of(Collections.emptyList(), error);
+            }
         }
 
         if (validateOnly) {
-            return ControllerResult.of(Collections.emptyList(), results);
+            return ControllerResult.of(Collections.emptyList(), ApiError.NONE);
         } else {
-            return ControllerResult.atomicOf(records, results);
+            return ControllerResult.atomicOf(records, ApiError.NONE);
         }
     }
 
@@ -193,15 +180,12 @@ public class FeatureControlManager {
         return metadataVersion.get();
     }
 
-    ZkMigrationState zkMigrationState() {
-        return migrationControlState.get();
-    }
-
     private ApiError updateFeature(
         String featureName,
         short newVersion,
         FeatureUpdate.UpgradeType upgradeType,
-        List<ApiMessageAndVersion> records
+        List<ApiMessageAndVersion> records,
+        Map<String, Short> proposedUpdatedVersions
     ) {
         if (upgradeType.equals(FeatureUpdate.UpgradeType.UNKNOWN)) {
             return invalidUpdateVersion(featureName, newVersion,
@@ -241,6 +225,15 @@ public class FeatureControlManager {
             // Perform additional checks if we're updating metadata.version
             return updateMetadataVersion(newVersion, upgradeType.equals(FeatureUpdate.UpgradeType.UNSAFE_DOWNGRADE), records::add);
         } else {
+            // Validate dependencies for features that are not metadata.version
+            try {
+                Feature.validateVersion(
+                    // Allow unstable feature versions is true because the version range is already checked above.
+                    Feature.featureFromName(featureName).fromFeatureLevel(newVersion, true),
+                    proposedUpdatedVersions);
+            } catch (IllegalArgumentException e) {
+                return invalidUpdateVersion(featureName, newVersion, e.getMessage());
+            }
             records.add(new ApiMessageAndVersion(new FeatureLevelRecord().
                 setName(featureName).
                 setFeatureLevel(newVersion), (short) 0));
@@ -318,26 +311,14 @@ public class FeatureControlManager {
         Consumer<ApiMessageAndVersion> recordConsumer
     ) {
         MetadataVersion currentVersion = metadataVersion();
-        ZkMigrationState zkMigrationState = zkMigrationState();
         final MetadataVersion newVersion;
         try {
             newVersion = MetadataVersion.fromFeatureLevel(newVersionLevel);
         } catch (IllegalArgumentException e) {
-            return invalidMetadataVersion(newVersionLevel, "Unknown metadata.version.");
+            return invalidMetadataVersion(newVersionLevel, "Valid versions are from "
+                + MetadataVersion.MINIMUM_VERSION.featureLevel() + " to " + MetadataVersion.latestTesting().featureLevel() + ".");
         }
 
-        // Don't allow metadata.version changes while we're migrating
-        if (zkMigrationState.inProgress()) {
-            return invalidMetadataVersion(newVersionLevel, "Unable to modify metadata.version while a " +
-                "ZK migration is in progress.");
-        }
-
-        // We cannot set a version earlier than IBP_3_3_IV0, since that was the first version that contained
-        // FeatureLevelRecord itself.
-        if (newVersion.isLessThan(minimumBootstrapVersion)) {
-            return invalidMetadataVersion(newVersionLevel, "Unable to set a metadata.version less than " +
-                    minimumBootstrapVersion);
-        }
         if (newVersion.isLessThan(currentVersion)) {
             // This is a downgrade
             boolean metadataChanged = MetadataVersion.checkIfMetadataChanged(currentVersion, newVersion);
@@ -379,16 +360,6 @@ public class FeatureControlManager {
         return new FinalizedControllerFeatures(features, epoch);
     }
 
-    /**
-     * Tests if the controller should be preventing metadata updates due to being in the PRE_MIGRATION
-     * state. If the controller does not yet support migrations (before 3.4-IV0), then the migration state
-     * will be NONE and this will return false. Once the controller has been upgraded to a version that supports
-     * migrations, then this method checks if the migration state is equal to PRE_MIGRATION.
-     */
-    boolean inPreMigrationMode() {
-        return migrationControlState.get().equals(ZkMigrationState.PRE_MIGRATION);
-    }
-
     public void replay(FeatureLevelRecord record) {
         VersionRange range = quorumFeatures.localSupportedFeature(record.name());
         if (!range.contains(record.featureLevel())) {
@@ -399,6 +370,10 @@ public class FeatureControlManager {
             MetadataVersion mv = MetadataVersion.fromFeatureLevel(record.featureLevel());
             metadataVersion.set(mv);
             log.info("Replayed a FeatureLevelRecord setting metadata.version to {}", mv);
+        } else if (record.name().equals(KRaftVersion.FEATURE_NAME)) {
+            // KAFKA-18979 - Skip any feature level record for kraft.version. This has two benefits:
+            // 1. It removes from snapshots any FeatureLevelRecord for kraft.version that was incorrectly written to the log
+            // 2. Allows ApiVersions to report the correct finalized kraft.version
         } else {
             if (record.featureLevel() == 0) {
                 finalizedVersions.remove(record.name());
@@ -411,20 +386,12 @@ public class FeatureControlManager {
         }
     }
 
-    public void replay(ZkMigrationStateRecord record) {
-        ZkMigrationState newState = ZkMigrationState.of(record.zkMigrationState());
-        ZkMigrationState previousState = migrationControlState.get();
-        if (previousState.equals(newState)) {
-            log.debug("Replayed a ZkMigrationStateRecord which did not alter the state from {}.",
-                    previousState);
-        } else {
-            migrationControlState.set(newState);
-            log.info("Replayed a ZkMigrationStateRecord changing the migration state from {} to {}.",
-                    previousState, newState);
-        }
-    }
-
     boolean isControllerId(int nodeId) {
         return quorumFeatures.isControllerId(nodeId);
     }
+
+    boolean isElrFeatureEnabled() {
+        return finalizedVersions.getOrDefault(EligibleLeaderReplicasVersion.FEATURE_NAME, (short) 0) >=
+            EligibleLeaderReplicasVersion.ELRV_1.featureLevel();
+    }
 }
