@@ -29,28 +29,27 @@ import java.util.function.Consumer;
 public final class ImageWriterOptions {
     public static class Builder {
         private MetadataVersion metadataVersion;
-        private MetadataVersion requestedMetadataVersion;
         private Consumer<UnwritableMetadataException> lossHandler = e -> {
             throw e;
         };
+        private boolean isEligibleLeaderReplicasEnabled = false;
 
-        public Builder() {
-            this.metadataVersion = MetadataVersion.latestProduction();
+        public Builder(MetadataVersion metadataVersion) {
+            this.metadataVersion = metadataVersion;
         }
 
         public Builder(MetadataImage image) {
-            this.metadataVersion = image.features().metadataVersion();
+            this.metadataVersion = image.features().metadataVersionOrThrow();
+            this.isEligibleLeaderReplicasEnabled = image.features().isElrEnabled();
         }
 
         public Builder setMetadataVersion(MetadataVersion metadataVersion) {
-            this.requestedMetadataVersion = metadataVersion;
-            if (metadataVersion.isLessThan(MetadataVersion.MINIMUM_BOOTSTRAP_VERSION)) {
-                // When writing an image, all versions less than 3.3-IV0 are treated as 3.0-IV1.
-                // This is because those versions don't support FeatureLevelRecord.
-                this.metadataVersion = MetadataVersion.MINIMUM_KRAFT_VERSION;
-            } else {
-                this.metadataVersion = metadataVersion;
-            }
+            this.metadataVersion = metadataVersion;
+            return this;
+        }
+
+        public Builder setEligibleLeaderReplicasEnabled(boolean isEligibleLeaderReplicasEnabled) {
+            this.isEligibleLeaderReplicasEnabled = isEligibleLeaderReplicasEnabled;
             return this;
         }
 
@@ -58,8 +57,8 @@ public final class ImageWriterOptions {
             return metadataVersion;
         }
 
-        public MetadataVersion requestedMetadataVersion() {
-            return requestedMetadataVersion;
+        public boolean isEligibleLeaderReplicasEnabled() {
+            return isEligibleLeaderReplicasEnabled;
         }
 
         public Builder setLossHandler(Consumer<UnwritableMetadataException> lossHandler) {
@@ -68,29 +67,32 @@ public final class ImageWriterOptions {
         }
 
         public ImageWriterOptions build() {
-            return new ImageWriterOptions(metadataVersion, lossHandler, requestedMetadataVersion);
+            return new ImageWriterOptions(metadataVersion, lossHandler, isEligibleLeaderReplicasEnabled);
         }
     }
 
     private final MetadataVersion metadataVersion;
-    private final MetadataVersion requestedMetadataVersion;
     private final Consumer<UnwritableMetadataException> lossHandler;
+    private final boolean isEligibleLeaderReplicasEnabled;
 
     private ImageWriterOptions(
         MetadataVersion metadataVersion,
         Consumer<UnwritableMetadataException> lossHandler,
-        MetadataVersion orgMetadataVersion
+        boolean isEligibleLeaderReplicasEnabled
     ) {
         this.metadataVersion = metadataVersion;
         this.lossHandler = lossHandler;
-        this.requestedMetadataVersion = orgMetadataVersion;
+        this.isEligibleLeaderReplicasEnabled = isEligibleLeaderReplicasEnabled;
     }
 
     public MetadataVersion metadataVersion() {
         return metadataVersion;
     }
+    public boolean isEligibleLeaderReplicasEnabled() {
+        return isEligibleLeaderReplicasEnabled;
+    }
 
     public void handleLoss(String loss) {
-        lossHandler.accept(new UnwritableMetadataException(requestedMetadataVersion, loss));
+        lossHandler.accept(new UnwritableMetadataException(metadataVersion, loss));
     }
 }
