@@ -21,11 +21,12 @@ import org.apache.kafka.common.DirectoryId;
 import org.apache.kafka.common.TopicPartition;
 import org.apache.kafka.common.Uuid;
 import org.apache.kafka.common.errors.InvalidReplicaDirectoriesException;
-import org.apache.kafka.common.message.LeaderAndIsrRequestData.LeaderAndIsrPartitionState;
 import org.apache.kafka.common.metadata.PartitionChangeRecord;
 import org.apache.kafka.common.metadata.PartitionRecord;
+import org.apache.kafka.common.requests.LeaderAndIsrRequest;
 import org.apache.kafka.image.writer.ImageWriterOptions;
 import org.apache.kafka.server.common.ApiMessageAndVersion;
+
 import org.slf4j.Logger;
 
 import java.util.Arrays;
@@ -41,7 +42,7 @@ public class PartitionRegistration {
     /**
      * A builder class which creates a PartitionRegistration.
      */
-    static public class Builder {
+    public static class Builder {
         private int[] replicas;
         private Uuid[] directories;
         private int[] isr;
@@ -164,8 +165,8 @@ public class PartitionRegistration {
     public final int leaderEpoch;
     public final int partitionEpoch;
 
-    public static boolean electionWasClean(int newLeader, int[] isr) {
-        return newLeader == NO_LEADER || Replicas.contains(isr, newLeader);
+    public static boolean electionWasClean(int newLeader, int[] isr, int[] elr) {
+        return newLeader == NO_LEADER || Replicas.contains(isr, newLeader) || Replicas.contains(elr, newLeader);
     }
 
     private static List<Uuid> checkDirectories(PartitionRecord record) {
@@ -212,7 +213,7 @@ public class PartitionRegistration {
             record.leaderEpoch(),
             record.partitionEpoch(),
             Replicas.toArray(record.eligibleLeaderReplicas()),
-            Replicas.toArray(record.lastKnownELR()));
+            Replicas.toArray(record.lastKnownElr()));
     }
 
     private PartitionRegistration(int[] replicas, Uuid[] directories, int[] isr, int[] removingReplicas,
@@ -264,7 +265,7 @@ public class PartitionRegistration {
         LeaderRecoveryState newLeaderRecoveryState = leaderRecoveryState.changeTo(record.leaderRecoveryState());
 
         int[] newElr = (record.eligibleLeaderReplicas() == null) ? elr : Replicas.toArray(record.eligibleLeaderReplicas());
-        int[] newLastKnownElr = (record.lastKnownELR() == null) ? lastKnownElr : Replicas.toArray(record.lastKnownELR());
+        int[] newLastKnownElr = (record.lastKnownElr() == null) ? lastKnownElr : Replicas.toArray(record.lastKnownElr());
         return new PartitionRegistration(newReplicas,
             newDirectories,
             newIsr,
@@ -346,7 +347,7 @@ public class PartitionRegistration {
     }
 
     public void maybeLogPartitionChange(Logger log, String description, PartitionRegistration prev) {
-        if (!electionWasClean(leader, prev.isr)) {
+        if (!electionWasClean(leader, prev.isr, prev.elr)) {
             log.info("UNCLEAN partition change for {}: {}", description, diff(prev));
         } else if (log.isDebugEnabled()) {
             log.debug("partition change for {}: {}", description, diff(prev));
@@ -386,11 +387,16 @@ public class PartitionRegistration {
             setLeaderRecoveryState(leaderRecoveryState.value()).
             setLeaderEpoch(leaderEpoch).
             setPartitionEpoch(partitionEpoch);
-        if (options.metadataVersion().isElrSupported()) {
+        if (options.isEligibleLeaderReplicasEnabled()) {
             // The following are tagged fields, we should only set them when there are some contents, in order to save
             // spaces.
             if (elr.length > 0) record.setEligibleLeaderReplicas(Replicas.toList(elr));
-            if (lastKnownElr.length > 0) record.setLastKnownELR(Replicas.toList(lastKnownElr));
+            if (lastKnownElr.length > 0) record.setLastKnownElr(Replicas.toList(lastKnownElr));
+        }
+
+        if (options.metadataVersion() == null) {
+            options.handleLoss("the metadata version");
+            return new ApiMessageAndVersion(record, (short) 0);
         }
         if (options.metadataVersion().isDirectoryAssignmentSupported()) {
             record.setDirectories(Uuid.toList(directories));
@@ -405,9 +411,9 @@ public class PartitionRegistration {
         return new ApiMessageAndVersion(record, options.metadataVersion().partitionRecordVersion());
     }
 
-    public LeaderAndIsrPartitionState toLeaderAndIsrPartitionState(TopicPartition tp,
-                                                                   boolean isNew) {
-        return new LeaderAndIsrPartitionState().
+    public LeaderAndIsrRequest.PartitionState toLeaderAndIsrPartitionState(TopicPartition tp,
+                                                                           boolean isNew) {
+        return new LeaderAndIsrRequest.PartitionState().
             setTopicName(tp.topic()).
             setPartitionIndex(tp.partition()).
             setControllerEpoch(-1).
@@ -448,20 +454,18 @@ public class PartitionRegistration {
 
     @Override
     public String toString() {
-        StringBuilder builder = new StringBuilder("PartitionRegistration(");
-        builder.append("replicas=").append(Arrays.toString(replicas));
-        builder.append(", directories=").append(Arrays.toString(directories));
-        builder.append(", isr=").append(Arrays.toString(isr));
-        builder.append(", removingReplicas=").append(Arrays.toString(removingReplicas));
-        builder.append(", addingReplicas=").append(Arrays.toString(addingReplicas));
-        builder.append(", elr=").append(Arrays.toString(elr));
-        builder.append(", lastKnownElr=").append(Arrays.toString(lastKnownElr));
-        builder.append(", leader=").append(leader);
-        builder.append(", leaderRecoveryState=").append(leaderRecoveryState);
-        builder.append(", leaderEpoch=").append(leaderEpoch);
-        builder.append(", partitionEpoch=").append(partitionEpoch);
-        builder.append(")");
-        return builder.toString();
+        return "PartitionRegistration(" + "replicas=" + Arrays.toString(replicas) +
+                ", directories=" + Arrays.toString(directories) +
+                ", isr=" + Arrays.toString(isr) +
+                ", removingReplicas=" + Arrays.toString(removingReplicas) +
+                ", addingReplicas=" + Arrays.toString(addingReplicas) +
+                ", elr=" + Arrays.toString(elr) +
+                ", lastKnownElr=" + Arrays.toString(lastKnownElr) +
+                ", leader=" + leader +
+                ", leaderRecoveryState=" + leaderRecoveryState +
+                ", leaderEpoch=" + leaderEpoch +
+                ", partitionEpoch=" + partitionEpoch +
+                ")";
     }
 
     public boolean hasSameAssignment(PartitionRegistration registration) {
