@@ -17,6 +17,9 @@
 
 package org.apache.kafka.metadata.placement;
 
+import org.apache.kafka.common.errors.InvalidReplicationFactorException;
+import org.apache.kafka.metadata.OptionalStringComparator;
+
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
@@ -27,9 +30,6 @@ import java.util.Optional;
 import java.util.Random;
 import java.util.stream.Collectors;
 
-import org.apache.kafka.common.errors.InvalidReplicationFactorException;
-import org.apache.kafka.metadata.OptionalStringComparator;
-
 
 /**
  * The striped replica placer.
@@ -120,7 +120,7 @@ public class StripedReplicaPlacer implements ReplicaPlacer {
      * A list of brokers that we can iterate through.
      */
     static class BrokerList {
-        final static BrokerList EMPTY = new BrokerList();
+        static final BrokerList EMPTY = new BrokerList();
         private final List<Integer> brokers = new ArrayList<>(0);
 
         /**
