@@ -29,8 +29,8 @@ import org.apache.kafka.raft.DynamicVoters;
 import org.apache.kafka.raft.KafkaRaftClient;
 import org.apache.kafka.raft.VoterSet;
 import org.apache.kafka.server.common.ApiMessageAndVersion;
+import org.apache.kafka.server.common.Feature;
 import org.apache.kafka.server.common.FeatureVersion;
-import org.apache.kafka.server.common.Features;
 import org.apache.kafka.server.common.KRaftVersion;
 import org.apache.kafka.server.common.MetadataVersion;
 import org.apache.kafka.snapshot.FileRawSnapshotWriter;
@@ -52,7 +52,6 @@ import java.util.Optional;
 import java.util.OptionalInt;
 import java.util.TreeMap;
 import java.util.TreeSet;
-import java.util.stream.Collectors;
 
 import static org.apache.kafka.common.internals.Topic.CLUSTER_METADATA_TOPIC_PARTITION;
 import static org.apache.kafka.server.common.KRaftVersion.KRAFT_VERSION_0;
@@ -70,7 +69,7 @@ public class Formatter {
     /**
      * The features that are supported.
      */
-    private List<Features> supportedFeatures = Features.PRODUCTION_FEATURES;
+    private List<Feature> supportedFeatures = Feature.PRODUCTION_FEATURES;
 
     /**
      * The current node id.
@@ -127,20 +126,20 @@ public class Formatter {
     /**
      * The metadata log directory.
      */
-    private String metadataLogDirectory = null;
+    private Optional<String> metadataLogDirectory = Optional.empty();
 
     /**
      * The initial KIP-853 voters.
      */
     private Optional<DynamicVoters> initialControllers = Optional.empty();
-    private boolean hasDynamicQuorum = false;
+    private boolean noInitialControllersFlag = false;
 
     public Formatter setPrintStream(PrintStream printStream) {
         this.printStream = printStream;
         return this;
     }
 
-    public Formatter setSupportedFeatures(List<Features> supportedFeatures) {
+    public Formatter setSupportedFeatures(List<Feature> supportedFeatures) {
         this.supportedFeatures = supportedFeatures;
         return this;
     }
@@ -170,6 +169,10 @@ public class Formatter {
         return this;
     }
 
+    public Collection<String> directories() {
+        return directories;
+    }
+
     public Formatter setReleaseVersion(MetadataVersion releaseVersion) {
         this.releaseVersion = releaseVersion;
         return this;
@@ -201,6 +204,11 @@ public class Formatter {
     }
 
     public Formatter setMetadataLogDirectory(String metadataLogDirectory) {
+        this.metadataLogDirectory = Optional.of(metadataLogDirectory);
+        return this;
+    }
+
+    public Formatter setMetadataLogDirectory(Optional<String> metadataLogDirectory) {
         this.metadataLogDirectory = metadataLogDirectory;
         return this;
     }
@@ -210,8 +218,8 @@ public class Formatter {
         return this;
     }
 
-    public Formatter setHasDynamicQuorum(boolean hasDynamicQuorum) {
-        this.hasDynamicQuorum = hasDynamicQuorum;
+    public Formatter setNoInitialControllersFlag(boolean noInitialControllersFlag) {
+        this.noInitialControllersFlag = noInitialControllersFlag;
         return this;
     }
 
@@ -220,7 +228,7 @@ public class Formatter {
     }
 
     boolean hasDynamicQuorum() {
-        return hasDynamicQuorum;
+        return initialControllers.isPresent() || noInitialControllersFlag;
     }
 
     public BootstrapMetadata bootstrapMetadata() {
@@ -240,13 +248,12 @@ public class Formatter {
         if (controllerListenerName == null) {
             throw new FormatterException("You must specify the name of the initial controller listener.");
         }
-        if (metadataLogDirectory == null) {
-            throw new FormatterException("You must specify the metadata log directory.");
-        }
-        if (!directories.contains(metadataLogDirectory)) {
-            throw new FormatterException("The specified metadata log directory, " + metadataLogDirectory +
+        metadataLogDirectory.ifPresent(d -> {
+            if (!directories.contains(d)) {
+                throw new FormatterException("The specified metadata log directory, " + d +
                     " was not one of the given directories: " + directories);
-        }
+            }
+        });
         releaseVersion = calculateEffectiveReleaseVersion();
         featureLevels = calculateEffectiveFeatureLevels();
         this.bootstrapMetadata = calculateBootstrapMetadata();
@@ -277,10 +284,6 @@ public class Formatter {
     }
 
     MetadataVersion verifyReleaseVersion(MetadataVersion metadataVersion) {
-        if (!metadataVersion.isKRaftSupported()) {
-            throw new FormatterException(MetadataVersion.FEATURE_NAME + " " + metadataVersion +
-                " is too old to be supported.");
-        }
         if (!unstableFeatureVersionsEnabled) {
             if (!metadataVersion.isProduction()) {
                 throw new FormatterException(MetadataVersion.FEATURE_NAME + " " + metadataVersion +
@@ -291,7 +294,7 @@ public class Formatter {
     }
 
     Map<String, Short> calculateEffectiveFeatureLevels() {
-        Map<String, Features> nameToSupportedFeature = new TreeMap<>();
+        Map<String, Feature> nameToSupportedFeature = new TreeMap<>();
         supportedFeatures.forEach(feature -> nameToSupportedFeature.put(feature.featureName(), feature));
         Map<String, Short> newFeatureLevels = new TreeMap<>();
         // Verify that all specified features are known to us.
@@ -301,8 +304,7 @@ public class Formatter {
             if (!featureName.equals(MetadataVersion.FEATURE_NAME)) {
                 if (!nameToSupportedFeature.containsKey(featureName)) {
                     throw new FormatterException("Unsupported feature: " + featureName +
-                            ". Supported features are: " + nameToSupportedFeature.keySet().stream().
-                            collect(Collectors.joining(", ")));
+                            ". Supported features are: " + String.join(", ", nameToSupportedFeature.keySet()));
                 }
             }
             newFeatureLevels.put(featureName, level);
@@ -315,7 +317,7 @@ public class Formatter {
                     Optional.ofNullable(newFeatureLevels.get(KRaftVersion.FEATURE_NAME))));
             } else if (!newFeatureLevels.containsKey(supportedFeature.featureName())) {
                 newFeatureLevels.put(supportedFeature.featureName(),
-                    supportedFeature.defaultValue(releaseVersion));
+                    supportedFeature.defaultLevel(releaseVersion));
             }
         });
         // Verify that the specified features support the given levels. This requires the full
@@ -324,10 +326,10 @@ public class Formatter {
             String featureName = entry.getKey();
             if (!featureName.equals(MetadataVersion.FEATURE_NAME)) {
                 short level = entry.getValue();
-                Features supportedFeature = nameToSupportedFeature.get(featureName);
+                Feature supportedFeature = nameToSupportedFeature.get(featureName);
                 FeatureVersion featureVersion =
                     supportedFeature.fromFeatureLevel(level, unstableFeatureVersionsEnabled);
-                Features.validateVersion(featureVersion, newFeatureLevels);
+                Feature.validateVersion(featureVersion, newFeatureLevels);
             }
         }
         return newFeatureLevels;
@@ -336,8 +338,8 @@ public class Formatter {
     /**
      * Calculate the effective feature level for kraft.version. In order to keep existing
      * command-line invocations of StorageTool working, we default this to 0 if no dynamic
-     * voter quorum arguments were provided. As a convenience, if the static voters config is
-     * empty, we set the latest kraft.version. (Currently there is only 1 non-zero version).
+     * voter quorum arguments were provided. As a convenience, if dynamic voter quorum arguments
+     * were passed, we set the latest kraft.version. (Currently there is only 1 non-zero version).
      *
      * @param configuredKRaftVersionLevel   The configured level for kraft.version
      * @return                              The effective feature level.
@@ -346,21 +348,15 @@ public class Formatter {
         if (configuredKRaftVersionLevel.isPresent()) {
             if (configuredKRaftVersionLevel.get() == 0) {
                 if (hasDynamicQuorum()) {
-                    throw new FormatterException(
-                        "Cannot set kraft.version to 0 if controller.quorum.voters is empty and one of the flags " +
-                        "--standalone, --initial-controllers, or --no-initial-controllers is used. For dynamic " +
-                        "controllers support, try removing the --feature flag for kraft.version."
-                    );
+                    throw new FormatterException("Cannot set kraft.version to " +
+                        configuredKRaftVersionLevel.get() + " if KIP-853 configuration is present. " +
+                            "Try removing the --feature flag for kraft.version.");
                 }
             } else {
                 if (!hasDynamicQuorum()) {
-                    throw new FormatterException(
-                        "Cannot set kraft.version to " + configuredKRaftVersionLevel.get() +
-                        " unless controller.quorum.voters is empty and one of the flags --standalone, " +
-                        "--initial-controllers, or --no-initial-controllers is used. " +
-                        "For dynamic controllers support, try using one of --standalone, --initial-controllers, " +
-                        "or --no-initial-controllers and removing controller.quorum.voters."
-                    );
+                    throw new FormatterException("Cannot set kraft.version to " +
+                        configuredKRaftVersionLevel.get() + " unless KIP-853 configuration is present. " +
+                            "Try removing the --feature flag for kraft.version.");
                 }
             }
             return configuredKRaftVersionLevel.get();
@@ -416,7 +412,7 @@ public class Formatter {
             Map<String, DirectoryType> directoryTypes = new HashMap<>();
             for (String emptyLogDir : ensemble.emptyLogDirs()) {
                 DirectoryType directoryType = DirectoryType.calculate(emptyLogDir,
-                    metadataLogDirectory,
+                    metadataLogDirectory.orElse(""),
                     nodeId,
                     initialControllers);
                 directoryTypes.put(emptyLogDir, directoryType);
@@ -435,7 +431,7 @@ public class Formatter {
                     directoryTypes.get(writeLogDir).description(), writeLogDir,
                     MetadataVersion.FEATURE_NAME, releaseVersion);
                 Files.createDirectories(Paths.get(writeLogDir));
-                BootstrapDirectory bootstrapDirectory = new BootstrapDirectory(writeLogDir, Optional.empty());
+                BootstrapDirectory bootstrapDirectory = new BootstrapDirectory(writeLogDir);
                 bootstrapDirectory.writeBinaryFile(bootstrapMetadata);
                 if (directoryTypes.get(writeLogDir).isDynamicMetadataDirectory()) {
                     writeDynamicQuorumSnapshot(writeLogDir,
@@ -485,7 +481,7 @@ public class Formatter {
         ) {
             if (!logDir.equals(metadataLogDirectory)) {
                 return LOG_DIRECTORY;
-            } else if (!initialControllers.isPresent()) {
+            } else if (initialControllers.isEmpty()) {
                 return STATIC_METADATA_DIRECTORY;
             } else if (initialControllers.get().voters().containsKey(nodeId)) {
                 return DYNAMIC_METADATA_VOTER_DIRECTORY;
