@@ -25,10 +25,13 @@ import org.apache.kafka.common.utils.LogContext;
 import org.apache.kafka.metadata.FinalizedControllerFeatures;
 import org.apache.kafka.metadata.RecordTestUtils;
 import org.apache.kafka.metadata.VersionRange;
-import org.apache.kafka.metadata.bootstrap.BootstrapMetadata;
-import org.apache.kafka.metadata.migration.ZkMigrationState;
 import org.apache.kafka.server.common.ApiMessageAndVersion;
+import org.apache.kafka.server.common.Feature;
+import org.apache.kafka.server.common.KRaftVersion;
 import org.apache.kafka.server.common.MetadataVersion;
+import org.apache.kafka.server.common.MetadataVersionTestUtils;
+import org.apache.kafka.server.common.TestFeatureVersion;
+import org.apache.kafka.server.common.TransactionVersion;
 import org.apache.kafka.timeline.SnapshotRegistry;
 
 import org.junit.jupiter.api.Disabled;
@@ -49,12 +52,12 @@ import java.util.Optional;
 import static java.util.Collections.emptyList;
 import static java.util.Collections.singletonMap;
 import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 
 @Timeout(value = 40)
 public class FeatureControlManagerTest {
 
-    @SuppressWarnings("unchecked")
     private static Map<String, VersionRange> rangeMap(Object... args) {
         Map<String, VersionRange> result = new HashMap<>();
         for (int i = 0; i < args.length; i += 3) {
@@ -77,7 +80,7 @@ public class FeatureControlManagerTest {
     }
 
     public static QuorumFeatures features(Object... args) {
-        Map<String, VersionRange> features = QuorumFeatures.defaultFeatureMap(true);
+        Map<String, VersionRange> features = QuorumFeatures.defaultSupportedFeatureMap(true);
         features.putAll(rangeMap(args));
         return new QuorumFeatures(0, features, emptyList());
     }
@@ -96,31 +99,36 @@ public class FeatureControlManagerTest {
     public void testUpdateFeatures() {
         SnapshotRegistry snapshotRegistry = new SnapshotRegistry(new LogContext());
         FeatureControlManager manager = new FeatureControlManager.Builder().
-            setQuorumFeatures(features("foo", 1, 2)).
+            setQuorumFeatures(features(TestFeatureVersion.FEATURE_NAME, 0, 2)).
             setSnapshotRegistry(snapshotRegistry).
-            setMetadataVersion(MetadataVersion.IBP_3_3_IV0).
+            setMetadataVersion(MetadataVersion.MINIMUM_VERSION).
             build();
-        snapshotRegistry.getOrCreateSnapshot(-1);
-        assertEquals(new FinalizedControllerFeatures(Collections.singletonMap("metadata.version", (short) 4), -1),
+        snapshotRegistry.idempotentCreateSnapshot(-1);
+        assertEquals(new FinalizedControllerFeatures(Collections.singletonMap("metadata.version", MetadataVersion.MINIMUM_VERSION.featureLevel()), -1),
             manager.finalizedFeatures(-1));
-        assertEquals(ControllerResult.atomicOf(emptyList(), Collections.
-                singletonMap("foo", new ApiError(Errors.INVALID_UPDATE_VERSION,
-                    "Invalid update version 3 for feature foo. Local controller 0 only supports versions 1-2"))),
-            manager.updateFeatures(updateMap("foo", 3),
-                Collections.singletonMap("foo", FeatureUpdate.UpgradeType.SAFE_DOWNGRADE),
+        assertEquals(ControllerResult.of(emptyList(), new ApiError(Errors.INVALID_UPDATE_VERSION,
+            "Invalid update version 3 for feature " + TestFeatureVersion.FEATURE_NAME + ". Local controller 0 only supports versions 0-2")),
+            manager.updateFeatures(updateMap(TestFeatureVersion.FEATURE_NAME, 3),
+                Collections.singletonMap(TestFeatureVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.SAFE_DOWNGRADE),
                 false));
-        ControllerResult<Map<String, ApiError>> result = manager.updateFeatures(
-                updateMap("foo", 2, "bar", 1), Collections.emptyMap(),
+        ControllerResult<ApiError> result = manager.updateFeatures(
+                updateMap(TestFeatureVersion.FEATURE_NAME, 1, "bar", 1), Collections.emptyMap(),
                 false);
-        Map<String, ApiError> expectedMap = new HashMap<>();
-        expectedMap.put("foo", ApiError.NONE);
-        expectedMap.put("bar", new ApiError(Errors.INVALID_UPDATE_VERSION,
-                "Invalid update version 1 for feature bar. Local controller 0 does not support this feature."));
-        assertEquals(expectedMap, result.response());
+        ApiError expectedError = new ApiError(Errors.INVALID_UPDATE_VERSION,
+            "Invalid update version 1 for feature bar. Local controller 0 does not support this feature.");
         List<ApiMessageAndVersion> expectedMessages = new ArrayList<>();
+        assertEquals(expectedError, result.response());
+        assertEquals(expectedMessages, result.records());
+
+        result = manager.updateFeatures(
+                updateMap(TestFeatureVersion.FEATURE_NAME, 1), Collections.emptyMap(),
+                false);
+        expectedError =  ApiError.NONE;
+        assertEquals(expectedError, result.response());
+        expectedMessages = new ArrayList<>();
         expectedMessages.add(new ApiMessageAndVersion(new FeatureLevelRecord().
-            setName("foo").setFeatureLevel((short) 2),
-            (short) 0));
+                setName(TestFeatureVersion.FEATURE_NAME).setFeatureLevel((short) 1),
+                (short) 0));
         assertEquals(expectedMessages, result.records());
     }
 
@@ -131,19 +139,43 @@ public class FeatureControlManagerTest {
         FeatureLevelRecord record = new FeatureLevelRecord().
             setName("foo").setFeatureLevel((short) 2);
 
-        snapshotRegistry.getOrCreateSnapshot(-1);
+        snapshotRegistry.idempotentCreateSnapshot(-1);
         FeatureControlManager manager = new FeatureControlManager.Builder().
                 setLogContext(logContext).
                 setQuorumFeatures(features("foo", 1, 2)).
                 setSnapshotRegistry(snapshotRegistry).
-                setMetadataVersion(MetadataVersion.IBP_3_3_IV0).
+                setMetadataVersion(MetadataVersion.MINIMUM_VERSION).
                 build();
         manager.replay(record);
-        snapshotRegistry.getOrCreateSnapshot(123);
-        assertEquals(new FinalizedControllerFeatures(versionMap("metadata.version", 4, "foo", 2), 123),
+        snapshotRegistry.idempotentCreateSnapshot(123);
+        assertEquals(
+            new FinalizedControllerFeatures(versionMap("metadata.version", MetadataVersion.MINIMUM_VERSION.featureLevel(), "foo", 2), 123),
             manager.finalizedFeatures(123));
     }
 
+    @Test
+    public void testReplayKraftVersionFeatureLevel() {
+        LogContext logContext = new LogContext();
+        SnapshotRegistry snapshotRegistry = new SnapshotRegistry(logContext);
+
+        snapshotRegistry.idempotentCreateSnapshot(-1);
+        FeatureControlManager manager = new FeatureControlManager.Builder().
+                setLogContext(logContext).
+                setQuorumFeatures(features("foo", 1, 2)).
+                setSnapshotRegistry(snapshotRegistry).
+                build();
+        manager.replay(new FeatureLevelRecord().setName(MetadataVersion.FEATURE_NAME).setFeatureLevel(MetadataVersion.MINIMUM_VERSION.featureLevel()));
+        // Replay a kraft.version feature level record and shot that the level doesn't get updated
+        manager.replay(new FeatureLevelRecord().setName(KRaftVersion.FEATURE_NAME).setFeatureLevel(KRaftVersion.LATEST_PRODUCTION.featureLevel()));
+        snapshotRegistry.idempotentCreateSnapshot(123);
+        assertEquals(
+            new FinalizedControllerFeatures(
+                versionMap("metadata.version", MetadataVersion.MINIMUM_VERSION.featureLevel()), 123
+            ),
+            manager.finalizedFeatures(123)
+        );
+    }
+
     static ClusterFeatureSupportDescriber createFakeClusterFeatureSupportDescriber(
         List<Map.Entry<Integer, Map<String, VersionRange>>> brokerRanges,
         List<Map.Entry<Integer, Map<String, VersionRange>>> controllerRanges
@@ -167,109 +199,94 @@ public class FeatureControlManagerTest {
         SnapshotRegistry snapshotRegistry = new SnapshotRegistry(logContext);
         FeatureControlManager manager = new FeatureControlManager.Builder().
             setLogContext(logContext).
-            setQuorumFeatures(features("foo", 1, 5, "bar", 0, 3)).
+            setQuorumFeatures(features("foo", 1, 5, TransactionVersion.FEATURE_NAME, 0, 3)).
             setSnapshotRegistry(snapshotRegistry).
             setClusterFeatureSupportDescriber(createFakeClusterFeatureSupportDescriber(
-                Collections.singletonList(new SimpleImmutableEntry<>(5, singletonMap("bar", VersionRange.of(0, 3)))),
+                Collections.singletonList(new SimpleImmutableEntry<>(5, Collections.singletonMap(TransactionVersion.FEATURE_NAME, VersionRange.of(0, 2)))),
                 emptyList())).
             build();
 
-        assertEquals(ControllerResult.atomicOf(emptyList(),
-            Collections.singletonMap("foo", new ApiError(Errors.INVALID_UPDATE_VERSION,
-                "Invalid update version 3 for feature foo. Broker 5 does not support this feature."))),
+        assertEquals(ControllerResult.of(emptyList(), new ApiError(Errors.INVALID_UPDATE_VERSION,
+            "Invalid update version 3 for feature foo. Broker 5 does not support this feature.")),
                     manager.updateFeatures(updateMap("foo", 3),
                         Collections.singletonMap("foo", FeatureUpdate.UpgradeType.SAFE_DOWNGRADE),
                         false));
 
-        ControllerResult<Map<String, ApiError>> result = manager.updateFeatures(
-            updateMap("bar", 3), Collections.emptyMap(), false);
-        assertEquals(Collections.singletonMap("bar", ApiError.NONE), result.response());
+        ControllerResult<ApiError> result = manager.updateFeatures(
+            updateMap(TransactionVersion.FEATURE_NAME, 2), Collections.emptyMap(), false);
+        assertEquals(ApiError.NONE, result.response());
         manager.replay((FeatureLevelRecord) result.records().get(0).message());
-        snapshotRegistry.getOrCreateSnapshot(3);
+        snapshotRegistry.idempotentCreateSnapshot(3);
 
-        assertEquals(ControllerResult.atomicOf(emptyList(), Collections.
-                singletonMap("bar", new ApiError(Errors.INVALID_UPDATE_VERSION,
-                    "Invalid update version 2 for feature bar. Can't downgrade the version of this feature " +
-                    "without setting the upgrade type to either safe or unsafe downgrade."))),
-            manager.updateFeatures(updateMap("bar", 2), Collections.emptyMap(), false));
+        assertEquals(ControllerResult.of(emptyList(), new ApiError(Errors.INVALID_UPDATE_VERSION,
+            "Invalid update version 1 for feature " + TransactionVersion.FEATURE_NAME + "." +
+            " Can't downgrade the version of this feature without setting the upgrade type to either safe or unsafe downgrade.")),
+            manager.updateFeatures(updateMap(TransactionVersion.FEATURE_NAME, 1), Collections.emptyMap(), false));
 
         assertEquals(
             ControllerResult.atomicOf(
                 Collections.singletonList(
                     new ApiMessageAndVersion(
                         new FeatureLevelRecord()
-                            .setName("bar")
-                            .setFeatureLevel((short) 2),
+                            .setName(TransactionVersion.FEATURE_NAME)
+                            .setFeatureLevel((short) 1),
                         (short) 0
                     )
                 ),
-                Collections.singletonMap("bar", ApiError.NONE)
+                ApiError.NONE
             ),
             manager.updateFeatures(
-                updateMap("bar", 2),
-                Collections.singletonMap("bar", FeatureUpdate.UpgradeType.SAFE_DOWNGRADE),
+                updateMap(TransactionVersion.FEATURE_NAME, 1),
+                Collections.singletonMap(TransactionVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.SAFE_DOWNGRADE),
                 false)
         );
     }
 
     @Test
-    public void testReplayRecords() throws Exception {
+    public void testReplayRecords() {
         LogContext logContext = new LogContext();
         SnapshotRegistry snapshotRegistry = new SnapshotRegistry(logContext);
         FeatureControlManager manager = new FeatureControlManager.Builder().
             setLogContext(logContext).
-            setQuorumFeatures(features("foo", 1, 5, "bar", 1, 2)).
+            setQuorumFeatures(features(TestFeatureVersion.FEATURE_NAME, 0, 5, TransactionVersion.FEATURE_NAME, 0, 2)).
             setSnapshotRegistry(snapshotRegistry).
-            setMetadataVersion(MetadataVersion.IBP_3_3_IV0).
+            setMetadataVersion(MetadataVersion.MINIMUM_VERSION).
             build();
-        ControllerResult<Map<String, ApiError>> result = manager.
-            updateFeatures(updateMap("foo", 5, "bar", 1), Collections.emptyMap(), false);
+        ControllerResult<ApiError> result = manager.
+            updateFeatures(updateMap(TestFeatureVersion.FEATURE_NAME, 1, TransactionVersion.FEATURE_NAME, 2), Collections.emptyMap(), false);
         RecordTestUtils.replayAll(manager, result.records());
-        assertEquals(MetadataVersion.IBP_3_3_IV0, manager.metadataVersion());
-        assertEquals(Optional.of((short) 5), manager.finalizedFeatures(Long.MAX_VALUE).get("foo"));
-        assertEquals(Optional.of((short) 1), manager.finalizedFeatures(Long.MAX_VALUE).get("bar"));
+        assertEquals(MetadataVersion.MINIMUM_VERSION, manager.metadataVersion());
+        assertEquals(Optional.of((short) 1), manager.finalizedFeatures(Long.MAX_VALUE).get(TestFeatureVersion.FEATURE_NAME));
+        assertEquals(Optional.of((short) 2), manager.finalizedFeatures(Long.MAX_VALUE).get(TransactionVersion.FEATURE_NAME));
         assertEquals(new HashSet<>(Arrays.asList(
-            MetadataVersion.FEATURE_NAME, "foo", "bar")),
+            MetadataVersion.FEATURE_NAME, TestFeatureVersion.FEATURE_NAME, TransactionVersion.FEATURE_NAME)),
                 manager.finalizedFeatures(Long.MAX_VALUE).featureNames());
     }
 
     private static final FeatureControlManager.Builder TEST_MANAGER_BUILDER1 =
         new FeatureControlManager.Builder().
             setQuorumFeatures(features(MetadataVersion.FEATURE_NAME,
-                MetadataVersion.IBP_3_3_IV0.featureLevel(), MetadataVersion.IBP_3_3_IV3.featureLevel())).
-            setMetadataVersion(MetadataVersion.IBP_3_3_IV2);
+                MetadataVersion.MINIMUM_VERSION.featureLevel(), MetadataVersion.IBP_3_6_IV0.featureLevel())).
+            setMetadataVersion(MetadataVersion.IBP_3_4_IV0);
 
     @Test
     public void testApplyMetadataVersionChangeRecord() {
         FeatureControlManager manager = TEST_MANAGER_BUILDER1.build();
+        MetadataVersion initialMetadataVersion = manager.metadataVersion();
         manager.replay(new FeatureLevelRecord().
             setName(MetadataVersion.FEATURE_NAME).
-            setFeatureLevel(MetadataVersion.IBP_3_3_IV3.featureLevel()));
-        assertEquals(MetadataVersion.IBP_3_3_IV3, manager.metadataVersion());
-    }
-
-    @Test
-    public void testCannotDowngradeToVersionBeforeMinimumSupportedKraftVersion() {
-        FeatureControlManager manager = TEST_MANAGER_BUILDER1.build();
-        assertEquals(ControllerResult.of(Collections.emptyList(),
-            singletonMap(MetadataVersion.FEATURE_NAME, new ApiError(Errors.INVALID_UPDATE_VERSION,
-                "Invalid update version 3 for feature metadata.version. Local controller 0 only " +
-                "supports versions 4-7"))),
-            manager.updateFeatures(
-                singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_2_IV0.featureLevel()),
-                singletonMap(MetadataVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.UNSAFE_DOWNGRADE),
-                true));
+            setFeatureLevel((short) (initialMetadataVersion.featureLevel() + 1)));
+        assertEquals(MetadataVersion.IBP_3_5_IV0, manager.metadataVersion());
     }
 
     @Test
     public void testCannotDowngradeToHigherVersion() {
         FeatureControlManager manager = TEST_MANAGER_BUILDER1.build();
-        assertEquals(ControllerResult.of(Collections.emptyList(),
-            singletonMap(MetadataVersion.FEATURE_NAME, new ApiError(Errors.INVALID_UPDATE_VERSION,
-                "Invalid update version 7 for feature metadata.version. Can't downgrade to a " +
-                "newer version."))),
+        assertEquals(ControllerResult.of(Collections.emptyList(), new ApiError(Errors.INVALID_UPDATE_VERSION,
+            "Invalid update version 9 for feature metadata.version. Can't downgrade to a " +
+            "newer version.")),
             manager.updateFeatures(
-                singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_3_IV3.featureLevel()),
+                singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_5_IV0.featureLevel()),
                 singletonMap(MetadataVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.SAFE_DOWNGRADE),
                 true));
     }
@@ -277,26 +294,27 @@ public class FeatureControlManagerTest {
     @Test
     public void testCannotUnsafeDowngradeToHigherVersion() {
         FeatureControlManager manager = TEST_MANAGER_BUILDER1.build();
-        assertEquals(ControllerResult.of(Collections.emptyList(),
-            singletonMap(MetadataVersion.FEATURE_NAME, new ApiError(Errors.INVALID_UPDATE_VERSION,
-                "Invalid update version 7 for feature metadata.version. Can't downgrade to a " +
-                "newer version."))),
+        assertEquals(ControllerResult.of(Collections.emptyList(), new ApiError(Errors.INVALID_UPDATE_VERSION,
+            "Invalid update version 9 for feature metadata.version. Can't downgrade to a " +
+            "newer version.")),
             manager.updateFeatures(
-                singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_3_IV3.featureLevel()),
+                singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_5_IV0.featureLevel()),
                 singletonMap(MetadataVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.UNSAFE_DOWNGRADE),
                 true));
     }
 
     @Test
     public void testCannotUpgradeToLowerVersion() {
-        FeatureControlManager manager = TEST_MANAGER_BUILDER1.build();
-        assertEquals(ControllerResult.of(Collections.emptyList(),
-            singletonMap(MetadataVersion.FEATURE_NAME, new ApiError(Errors.INVALID_UPDATE_VERSION,
-                "Invalid update version 4 for feature metadata.version. Can't downgrade the " +
-                "version of this feature without setting the upgrade type to either safe or " +
-                "unsafe downgrade."))),
+        FeatureControlManager manager = new FeatureControlManager.Builder().
+            setQuorumFeatures(features(MetadataVersion.FEATURE_NAME,
+                MetadataVersion.MINIMUM_VERSION.featureLevel(), MetadataVersion.IBP_3_6_IV0.featureLevel())).
+            setMetadataVersion(MetadataVersion.IBP_3_5_IV1).build();
+        assertEquals(ControllerResult.of(Collections.emptyList(), new ApiError(Errors.INVALID_UPDATE_VERSION,
+            "Invalid update version 9 for feature metadata.version. Can't downgrade the " +
+            "version of this feature without setting the upgrade type to either safe or " +
+            "unsafe downgrade.")),
             manager.updateFeatures(
-                singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_3_IV0.featureLevel()),
+                singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_5_IV0.featureLevel()),
                 singletonMap(MetadataVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.UPGRADE),
                 true));
     }
@@ -304,10 +322,9 @@ public class FeatureControlManagerTest {
     @Test
     public void testCanUpgradeToHigherVersion() {
         FeatureControlManager manager = TEST_MANAGER_BUILDER1.build();
-        assertEquals(ControllerResult.of(Collections.emptyList(),
-            singletonMap(MetadataVersion.FEATURE_NAME, ApiError.NONE)),
+        assertEquals(ControllerResult.of(Collections.emptyList(), ApiError.NONE),
             manager.updateFeatures(
-                singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_3_IV3.featureLevel()),
+                singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_5_IV0.featureLevel()),
                 singletonMap(MetadataVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.UPGRADE),
                 true));
     }
@@ -315,12 +332,11 @@ public class FeatureControlManagerTest {
     @Test
     public void testCannotUseSafeDowngradeIfMetadataChanged() {
         FeatureControlManager manager = TEST_MANAGER_BUILDER1.build();
-        assertEquals(ControllerResult.of(Collections.emptyList(),
-            singletonMap(MetadataVersion.FEATURE_NAME, new ApiError(Errors.INVALID_UPDATE_VERSION,
-                "Invalid metadata.version 4. Refusing to perform the requested downgrade because " +
-                "it might delete metadata information."))),
+        assertEquals(ControllerResult.of(Collections.emptyList(), new ApiError(Errors.INVALID_UPDATE_VERSION,
+            "Invalid metadata.version 7. Refusing to perform the requested downgrade because " +
+            "it might delete metadata information.")),
             manager.updateFeatures(
-                singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_3_IV0.featureLevel()),
+                singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_3_IV3.featureLevel()),
                 singletonMap(MetadataVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.SAFE_DOWNGRADE),
                 true));
     }
@@ -328,11 +344,10 @@ public class FeatureControlManagerTest {
     @Test
     public void testUnsafeDowngradeIsTemporarilyDisabled() {
         FeatureControlManager manager = TEST_MANAGER_BUILDER1.build();
-        assertEquals(ControllerResult.of(Collections.emptyList(),
-                        singletonMap(MetadataVersion.FEATURE_NAME, new ApiError(Errors.INVALID_UPDATE_VERSION,
-                                "Invalid metadata.version 4. Unsafe metadata downgrade is not supported in this version."))),
+        assertEquals(ControllerResult.of(Collections.emptyList(), new ApiError(Errors.INVALID_UPDATE_VERSION,
+            "Invalid metadata.version 7. Unsafe metadata downgrade is not supported in this version.")),
                 manager.updateFeatures(
-                        singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_3_IV0.featureLevel()),
+                        singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_3_IV3.featureLevel()),
                         singletonMap(MetadataVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.UNSAFE_DOWNGRADE),
                         true));
     }
@@ -341,10 +356,9 @@ public class FeatureControlManagerTest {
     @Test
     public void testCanUseUnsafeDowngradeIfMetadataChanged() {
         FeatureControlManager manager = TEST_MANAGER_BUILDER1.build();
-        assertEquals(ControllerResult.of(Collections.emptyList(),
-                        singletonMap(MetadataVersion.FEATURE_NAME, ApiError.NONE)),
+        assertEquals(ControllerResult.of(Collections.emptyList(), ApiError.NONE),
                 manager.updateFeatures(
-                        singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_3_IV0.featureLevel()),
+                        singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_3_IV3.featureLevel()),
                         singletonMap(MetadataVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.UNSAFE_DOWNGRADE),
                         true));
     }
@@ -353,30 +367,27 @@ public class FeatureControlManagerTest {
     public void testCanUseSafeDowngradeIfMetadataDidNotChange() {
         FeatureControlManager manager = new FeatureControlManager.Builder().
                 setQuorumFeatures(features(MetadataVersion.FEATURE_NAME,
-                        MetadataVersion.IBP_3_0_IV0.featureLevel(), MetadataVersion.IBP_3_3_IV1.featureLevel())).
-                setMetadataVersion(MetadataVersion.IBP_3_1_IV0).
-                setMinimumBootstrapVersion(MetadataVersion.IBP_3_0_IV0).
+                        MetadataVersion.MINIMUM_VERSION.featureLevel(), MetadataVersion.IBP_3_6_IV0.featureLevel())).
+                setMetadataVersion(MetadataVersion.IBP_3_5_IV0).
                 build();
-        assertEquals(ControllerResult.of(Collections.emptyList(),
-                        singletonMap(MetadataVersion.FEATURE_NAME, ApiError.NONE)),
+        assertEquals(ControllerResult.of(Collections.emptyList(), ApiError.NONE),
                 manager.updateFeatures(
-                        singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_0_IV1.featureLevel()),
+                        singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_4_IV0.featureLevel()),
                         singletonMap(MetadataVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.SAFE_DOWNGRADE),
                         true));
     }
 
     @Test
-    public void testCannotDowngradeBefore3_3_IV0() {
+    public void testCannotDowngradeBeforeMinimumKraftVersion() {
         FeatureControlManager manager = new FeatureControlManager.Builder().
             setQuorumFeatures(features(MetadataVersion.FEATURE_NAME,
-                MetadataVersion.IBP_3_0_IV0.featureLevel(), MetadataVersion.IBP_3_3_IV3.featureLevel())).
-            setMetadataVersion(MetadataVersion.IBP_3_3_IV0).
+                MetadataVersion.MINIMUM_VERSION.featureLevel(), MetadataVersion.latestTesting().featureLevel())).
+            setMetadataVersion(MetadataVersion.MINIMUM_VERSION).
             build();
-        assertEquals(ControllerResult.of(Collections.emptyList(),
-                        singletonMap(MetadataVersion.FEATURE_NAME, new ApiError(Errors.INVALID_UPDATE_VERSION,
-                        "Invalid metadata.version 3. Unable to set a metadata.version less than 3.3-IV0"))),
+        assertEquals(ControllerResult.of(Collections.emptyList(), new ApiError(Errors.INVALID_UPDATE_VERSION,
+            "Invalid update version 6 for feature metadata.version. Local controller 0 only supports versions 7-26")),
                 manager.updateFeatures(
-                        singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_2_IV0.featureLevel()),
+                        singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersionTestUtils.IBP_3_3_IV2_FEATURE_LEVEL),
                         singletonMap(MetadataVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.UNSAFE_DOWNGRADE),
                         true));
     }
@@ -385,70 +396,61 @@ public class FeatureControlManagerTest {
     public void testCreateFeatureLevelRecords() {
         Map<String, VersionRange> localSupportedFeatures = new HashMap<>();
         localSupportedFeatures.put(MetadataVersion.FEATURE_NAME, VersionRange.of(
-            MetadataVersion.IBP_3_0_IV1.featureLevel(), MetadataVersion.latestTesting().featureLevel()));
-        localSupportedFeatures.put("foo", VersionRange.of(0, 2));
+            MetadataVersion.MINIMUM_VERSION.featureLevel(), MetadataVersion.latestTesting().featureLevel()));
+        localSupportedFeatures.put(Feature.TEST_VERSION.featureName(), VersionRange.of(0, 2));
         FeatureControlManager manager = new FeatureControlManager.Builder().
             setQuorumFeatures(new QuorumFeatures(0, localSupportedFeatures, emptyList())).
             setClusterFeatureSupportDescriber(createFakeClusterFeatureSupportDescriber(
-                Collections.singletonList(new SimpleImmutableEntry<>(1, singletonMap("foo", VersionRange.of(0, 3)))),
+                Collections.singletonList(new SimpleImmutableEntry<>(1, Collections.singletonMap(Feature.TEST_VERSION.featureName(), VersionRange.of(0, 3)))),
                 emptyList())).
                 build();
-        ControllerResult<Map<String, ApiError>> result  = manager.updateFeatures(
-                Collections.singletonMap("foo", (short) 1),
-                Collections.singletonMap("foo", FeatureUpdate.UpgradeType.UPGRADE),
+        ControllerResult<ApiError> result  = manager.updateFeatures(
+                Collections.singletonMap(Feature.TEST_VERSION.featureName(), (short) 1),
+                Collections.singletonMap(Feature.TEST_VERSION.featureName(), FeatureUpdate.UpgradeType.UPGRADE),
                 false);
         assertEquals(ControllerResult.atomicOf(Collections.singletonList(new ApiMessageAndVersion(
-                new FeatureLevelRecord().setName("foo").setFeatureLevel((short) 1), (short) 0)),
-                        Collections.singletonMap("foo", ApiError.NONE)), result);
+                new FeatureLevelRecord().setName(Feature.TEST_VERSION.featureName()).setFeatureLevel((short) 1), (short) 0)),
+                ApiError.NONE), result);
         RecordTestUtils.replayAll(manager, result.records());
-        assertEquals(Optional.of((short) 1), manager.finalizedFeatures(Long.MAX_VALUE).get("foo"));
+        assertEquals(Optional.of((short) 1), manager.finalizedFeatures(Long.MAX_VALUE).get(Feature.TEST_VERSION.featureName()));
 
-        ControllerResult<Map<String, ApiError>> result2  = manager.updateFeatures(
-                Collections.singletonMap("foo", (short) 0),
-                Collections.singletonMap("foo", FeatureUpdate.UpgradeType.UNSAFE_DOWNGRADE),
+        ControllerResult<ApiError> result2  = manager.updateFeatures(
+                Collections.singletonMap(Feature.TEST_VERSION.featureName(), (short) 0),
+                Collections.singletonMap(Feature.TEST_VERSION.featureName(), FeatureUpdate.UpgradeType.UNSAFE_DOWNGRADE),
                 false);
         assertEquals(ControllerResult.atomicOf(Collections.singletonList(new ApiMessageAndVersion(
-                        new FeatureLevelRecord().setName("foo").setFeatureLevel((short) 0), (short) 0)),
-                Collections.singletonMap("foo", ApiError.NONE)), result2);
+            new FeatureLevelRecord().setName(Feature.TEST_VERSION.featureName()).setFeatureLevel((short) 0), (short) 0)),
+            ApiError.NONE), result2);
         RecordTestUtils.replayAll(manager, result2.records());
-        assertEquals(Optional.empty(), manager.finalizedFeatures(Long.MAX_VALUE).get("foo"));
+        assertEquals(Optional.empty(), manager.finalizedFeatures(Long.MAX_VALUE).get(Feature.TEST_VERSION.featureName()));
     }
 
     @Test
-    public void testNoMetadataVersionChangeDuringMigration() {
+    public void testUpgradeElrFeatureLevel() {
+        Map<String, VersionRange> localSupportedFeatures = new HashMap<>();
+        localSupportedFeatures.put(MetadataVersion.FEATURE_NAME, VersionRange.of(
+            MetadataVersion.IBP_4_0_IV1.featureLevel(), MetadataVersion.latestTesting().featureLevel()));
+        localSupportedFeatures.put(Feature.ELIGIBLE_LEADER_REPLICAS_VERSION.featureName(), VersionRange.of(0, 1));
         FeatureControlManager manager = new FeatureControlManager.Builder().
-            setQuorumFeatures(features(MetadataVersion.FEATURE_NAME,
-                    MetadataVersion.IBP_3_0_IV0.featureLevel(), MetadataVersion.IBP_3_5_IV1.featureLevel())).
-            setMetadataVersion(MetadataVersion.IBP_3_4_IV0).
+            setQuorumFeatures(new QuorumFeatures(0, localSupportedFeatures, emptyList())).
+            setClusterFeatureSupportDescriber(createFakeClusterFeatureSupportDescriber(
+                Collections.singletonList(new SimpleImmutableEntry<>(1, Collections.singletonMap(Feature.ELIGIBLE_LEADER_REPLICAS_VERSION.featureName(), VersionRange.of(0, 1)))),
+                emptyList())).
+            setMetadataVersion(MetadataVersion.IBP_4_0_IV1).
             build();
-        BootstrapMetadata bootstrapMetadata = BootstrapMetadata.fromVersion(MetadataVersion.IBP_3_4_IV0, "FeatureControlManagerTest");
-        RecordTestUtils.replayAll(manager, bootstrapMetadata.records());
-        RecordTestUtils.replayOne(manager, ZkMigrationState.PRE_MIGRATION.toRecord());
-
-        assertEquals(ControllerResult.of(Collections.emptyList(),
-            singletonMap(MetadataVersion.FEATURE_NAME, new ApiError(Errors.INVALID_UPDATE_VERSION,
-                "Invalid metadata.version 10. Unable to modify metadata.version while a ZK migration is in progress."))),
-            manager.updateFeatures(
-                singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_5_IV1.featureLevel()),
-                singletonMap(MetadataVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.UPGRADE),
-                true));
-
-        assertEquals(ControllerResult.of(Collections.emptyList(),
-                singletonMap(MetadataVersion.FEATURE_NAME, new ApiError(Errors.INVALID_UPDATE_VERSION,
-                "Invalid metadata.version 4. Unable to modify metadata.version while a ZK migration is in progress."))),
-            manager.updateFeatures(
-                singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_3_IV0.featureLevel()),
-                singletonMap(MetadataVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.SAFE_DOWNGRADE),
-                true));
-
-        // Complete the migration
-        RecordTestUtils.replayOne(manager, ZkMigrationState.POST_MIGRATION.toRecord());
-        ControllerResult<Map<String, ApiError>> result = manager.updateFeatures(
-            singletonMap(MetadataVersion.FEATURE_NAME, MetadataVersion.IBP_3_5_IV1.featureLevel()),
-            singletonMap(MetadataVersion.FEATURE_NAME, FeatureUpdate.UpgradeType.UPGRADE),
+        ControllerResult<ApiError> result = manager.updateFeatures(
+            Collections.singletonMap(Feature.ELIGIBLE_LEADER_REPLICAS_VERSION.featureName(), (short) 1),
+            Collections.singletonMap(Feature.ELIGIBLE_LEADER_REPLICAS_VERSION.featureName(), FeatureUpdate.UpgradeType.UPGRADE),
             false);
-        assertEquals(Errors.NONE, result.response().get(MetadataVersion.FEATURE_NAME).error());
+        assertTrue(result.response().isSuccess());
+        assertEquals(Collections.singletonList(new ApiMessageAndVersion(
+            new FeatureLevelRecord().
+                setName(Feature.ELIGIBLE_LEADER_REPLICAS_VERSION.featureName()).
+                setFeatureLevel((short) 1), (short) 0)),
+            result.records());
         RecordTestUtils.replayAll(manager, result.records());
-        assertEquals(MetadataVersion.IBP_3_5_IV1, manager.metadataVersion());
+        assertEquals(Optional.of((short) 1), manager.finalizedFeatures(Long.MAX_VALUE).
+            get(Feature.ELIGIBLE_LEADER_REPLICAS_VERSION.featureName()));
     }
+
 }
