@@ -22,13 +22,13 @@ import org.apache.kafka.common.metadata.FeatureLevelRecord;
 import org.apache.kafka.common.metadata.PartitionRecord;
 import org.apache.kafka.common.metadata.RegisterBrokerRecord;
 import org.apache.kafka.common.metadata.TopicRecord;
-import org.apache.kafka.common.metadata.ZkMigrationStateRecord;
 import org.apache.kafka.image.writer.ImageWriterOptions;
 import org.apache.kafka.image.writer.RecordListWriter;
 import org.apache.kafka.image.writer.UnwritableMetadataException;
 import org.apache.kafka.metadata.RecordTestUtils;
 import org.apache.kafka.server.common.ApiMessageAndVersion;
 import org.apache.kafka.server.common.MetadataVersion;
+
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.Timeout;
 
@@ -77,64 +77,13 @@ public class ImageDowngradeTest {
                 setFeatureLevel(metadataVersion.featureLevel()), (short) 0);
     }
 
-    /**
-     * Test downgrading to a MetadataVersion that doesn't support FeatureLevelRecord.
-     */
-    @Test
-    public void testPremodernVersion() {
-        writeWithExpectedLosses(MetadataVersion.IBP_3_2_IV0,
-            Arrays.asList(
-                "feature flag(s): foo.feature"),
-            Arrays.asList(
-                metadataVersionRecord(MetadataVersion.IBP_3_3_IV0),
-                TEST_RECORDS.get(0),
-                TEST_RECORDS.get(1),
-                new ApiMessageAndVersion(new FeatureLevelRecord().
-                        setName("foo.feature").
-                        setFeatureLevel((short) 4), (short) 0)),
-            Arrays.asList(
-                TEST_RECORDS.get(0),
-                TEST_RECORDS.get(1)));
-    }
-
-    /**
-     * Test downgrading to a MetadataVersion that doesn't support inControlledShutdown.
-     */
-    @Test
-    public void testPreControlledShutdownStateVersion() {
-        writeWithExpectedLosses(MetadataVersion.IBP_3_3_IV2,
-                Arrays.asList(
-                        "the inControlledShutdown state of one or more brokers"),
-                Arrays.asList(
-                        metadataVersionRecord(MetadataVersion.IBP_3_3_IV3),
-                        new ApiMessageAndVersion(new RegisterBrokerRecord().
-                            setBrokerId(123).
-                            setIncarnationId(Uuid.fromString("XgjKo16hRWeWrTui0iR5Nw")).
-                            setBrokerEpoch(456).
-                            setRack(null).
-                            setFenced(false).
-                            setInControlledShutdown(true), (short) 1),
-                        TEST_RECORDS.get(0),
-                        TEST_RECORDS.get(1)),
-                Arrays.asList(
-                        metadataVersionRecord(MetadataVersion.IBP_3_3_IV2),
-                        new ApiMessageAndVersion(new RegisterBrokerRecord().
-                            setBrokerId(123).
-                            setIncarnationId(Uuid.fromString("XgjKo16hRWeWrTui0iR5Nw")).
-                            setBrokerEpoch(456).
-                            setRack(null).
-                            setFenced(false), (short) 0),
-                        TEST_RECORDS.get(0),
-                        TEST_RECORDS.get(1)));
-    }
-
     /**
      * Test downgrading to a MetadataVersion that doesn't support ZK migration.
      */
     @Test
     public void testPreZkMigrationSupportVersion() {
         writeWithExpectedLosses(MetadataVersion.IBP_3_3_IV3,
-            Arrays.asList(
+            Collections.singletonList(
                 "the isMigratingZkBroker state of one or more brokers"),
             Arrays.asList(
                 metadataVersionRecord(MetadataVersion.IBP_3_4_IV0),
@@ -158,7 +107,8 @@ public class ImageDowngradeTest {
                     setFenced(false).
                     setInControlledShutdown(true), (short) 1),
                 TEST_RECORDS.get(0),
-                TEST_RECORDS.get(1)));
+                TEST_RECORDS.get(1))
+        );
     }
 
     @Test
@@ -167,22 +117,23 @@ public class ImageDowngradeTest {
         MetadataVersion inputMetadataVersion = outputMetadataVersion;
         PartitionRecord testPartitionRecord = (PartitionRecord) TEST_RECORDS.get(1).message();
         writeWithExpectedLosses(outputMetadataVersion,
-                Collections.singletonList("the directory assignment state of one or more replicas"),
-                Arrays.asList(
-                        metadataVersionRecord(inputMetadataVersion),
-                        TEST_RECORDS.get(0),
-                        new ApiMessageAndVersion(
-                                testPartitionRecord.duplicate().setDirectories(Arrays.asList(
-                                        Uuid.fromString("c7QfSi6xSIGQVh3Qd5RJxA"),
-                                        Uuid.fromString("rWaCHejCRRiptDMvW5Xw0g"))),
-                                (short) 2)),
-                Arrays.asList(
-                        metadataVersionRecord(outputMetadataVersion),
-                        new ApiMessageAndVersion(new ZkMigrationStateRecord(), (short) 0),
-                        TEST_RECORDS.get(0),
-                        new ApiMessageAndVersion(
-                                testPartitionRecord.duplicate().setDirectories(Collections.emptyList()),
-                                (short) 0)));
+            Collections.singletonList(
+                    "the directory assignment state of one or more replicas"),
+            Arrays.asList(
+                metadataVersionRecord(inputMetadataVersion),
+                TEST_RECORDS.get(0),
+                new ApiMessageAndVersion(
+                    testPartitionRecord.duplicate().setDirectories(Arrays.asList(
+                        Uuid.fromString("c7QfSi6xSIGQVh3Qd5RJxA"),
+                        Uuid.fromString("rWaCHejCRRiptDMvW5Xw0g"))),
+                    (short) 2)),
+            Arrays.asList(
+                metadataVersionRecord(outputMetadataVersion),
+                TEST_RECORDS.get(0),
+                new ApiMessageAndVersion(
+                    testPartitionRecord.duplicate().setDirectories(Collections.emptyList()),
+                    (short) 0))
+        );
     }
 
     private static void writeWithExpectedLosses(
@@ -196,8 +147,7 @@ public class ImageDowngradeTest {
         RecordTestUtils.replayAll(delta, inputs);
         MetadataImage image = delta.apply(MetadataProvenance.EMPTY);
         RecordListWriter writer = new RecordListWriter();
-        image.write(writer, new ImageWriterOptions.Builder().
-                setMetadataVersion(metadataVersion).
+        image.write(writer, new ImageWriterOptions.Builder(metadataVersion).
                 setLossHandler(lossConsumer).
                 build());
         assertEquals(expectedLosses, lossConsumer.losses, "Failed to get expected metadata losses.");
