@@ -18,11 +18,9 @@
 package org.apache.kafka.image.loader;
 
 import org.apache.kafka.common.Uuid;
-import org.apache.kafka.common.config.ConfigResource;
 import org.apache.kafka.common.message.SnapshotHeaderRecord;
 import org.apache.kafka.common.metadata.AbortTransactionRecord;
 import org.apache.kafka.common.metadata.BeginTransactionRecord;
-import org.apache.kafka.common.metadata.ConfigRecord;
 import org.apache.kafka.common.metadata.EndTransactionRecord;
 import org.apache.kafka.common.metadata.FeatureLevelRecord;
 import org.apache.kafka.common.metadata.PartitionRecord;
@@ -55,6 +53,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Optional;
 import java.util.OptionalLong;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutionException;
@@ -65,9 +64,8 @@ import java.util.stream.Collectors;
 
 import static java.util.Arrays.asList;
 import static java.util.Collections.singletonList;
-import static org.apache.kafka.server.common.MetadataVersion.IBP_3_3_IV1;
-import static org.apache.kafka.server.common.MetadataVersion.IBP_3_3_IV2;
 import static org.apache.kafka.server.common.MetadataVersion.IBP_3_5_IV0;
+import static org.apache.kafka.server.common.MetadataVersion.MINIMUM_VERSION;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
@@ -243,8 +241,7 @@ public class MetadataLoaderTest {
         boolean loadSnapshot,
         boolean sameObject
     ) throws Exception {
-        MockFaultHandler faultHandler =
-                new MockFaultHandler("testPublisherCannotBeInstalledMoreThanOnce");
+        MockFaultHandler faultHandler = new MockFaultHandler("testPublisherCannotBeInstalledMoreThanOnce");
         MockPublisher publisher = new MockPublisher();
         try (MetadataLoader loader = new MetadataLoader.Builder().
                 setFaultHandler(faultHandler).
@@ -253,14 +250,19 @@ public class MetadataLoaderTest {
             loader.installPublishers(singletonList(publisher)).get();
             if (loadSnapshot) {
                 MockSnapshotReader snapshotReader = new MockSnapshotReader(
-                    new MetadataProvenance(200, 100, 4000),
-                    singletonList(
+                    new MetadataProvenance(200, 100, 4000, true),
+                    List.of(
                         Batch.control(
                             200,
                             100,
                             4000,
                             10,
                             singletonList(new ControlRecord(ControlRecordType.SNAPSHOT_HEADER, new SnapshotHeaderRecord()))
+                        ),
+                        Batch.data(0, 0, 0, 0,
+                            singletonList(new ApiMessageAndVersion(new FeatureLevelRecord().
+                                setName(MetadataVersion.FEATURE_NAME).
+                                setFeatureLevel(MINIMUM_VERSION.featureLevel()), (short) 0))
                         )
                     )
                 );
@@ -306,11 +308,11 @@ public class MetadataLoaderTest {
             loader.installPublishers(publishers.subList(0, 2)).get();
             loader.removeAndClosePublisher(publishers.get(1)).get();
             MockSnapshotReader snapshotReader = MockSnapshotReader.fromRecordLists(
-                new MetadataProvenance(100, 50, 2000),
+                new MetadataProvenance(100, 50, 2000, true),
                 singletonList(singletonList(new ApiMessageAndVersion(
                     new FeatureLevelRecord().
                         setName(MetadataVersion.FEATURE_NAME).
-                        setFeatureLevel(IBP_3_3_IV2.featureLevel()), (short) 0))));
+                        setFeatureLevel(MINIMUM_VERSION.featureLevel()), (short) 0))));
             assertFalse(snapshotReader.closed);
             loader.handleLoadSnapshot(snapshotReader);
             loader.waitForAllEventsToBeHandled();
@@ -319,7 +321,7 @@ public class MetadataLoaderTest {
             loader.removeAndClosePublisher(publishers.get(0)).get();
         }
         assertTrue(publishers.get(0).closed);
-        assertEquals(IBP_3_3_IV2,
+        assertEquals(Optional.of(MINIMUM_VERSION),
                 publishers.get(0).latestImage.features().metadataVersion());
         assertTrue(publishers.get(1).closed);
         assertNull(publishers.get(1).latestImage);
@@ -342,20 +344,36 @@ public class MetadataLoaderTest {
                 setHighWaterMarkAccessor(() -> OptionalLong.of(0L)).
                 build()) {
             loader.installPublishers(publishers).get();
+            loader.handleCommit(
+                MockBatchReader.newSingleBatchReader(400, 50, List.of(
+                    new ApiMessageAndVersion(new FeatureLevelRecord()
+                        .setName(MetadataVersion.FEATURE_NAME)
+                        .setFeatureLevel(MINIMUM_VERSION.featureLevel()), (short) 0)))
+            );
             loadEmptySnapshot(loader, 200);
             publishers.get(0).firstPublish.get(10, TimeUnit.SECONDS);
             assertEquals(200L, loader.lastAppliedOffset());
             loadEmptySnapshot(loader, 300);
             assertEquals(300L, loader.lastAppliedOffset());
-            assertEquals(new SnapshotManifest(new MetadataProvenance(300, 100, 4000), 3000000L),
+            assertEquals(new SnapshotManifest(new MetadataProvenance(300, 100, 4000, true), 3000000L),
                 publishers.get(0).latestSnapshotManifest);
-            assertEquals(MetadataVersion.MINIMUM_KRAFT_VERSION,
+            assertEquals(MINIMUM_VERSION,
                 loader.metrics().currentMetadataVersion());
         }
         assertTrue(publishers.get(0).closed);
-        assertEquals(MetadataVersion.IBP_3_0_IV1,
-                publishers.get(0).latestImage.features().metadataVersion());
-        assertTrue(publishers.get(0).latestImage.isEmpty());
+        assertEquals(Optional.of(MINIMUM_VERSION), publishers.get(0).latestImage.features().metadataVersion());
+        var latestImage = publishers.get(0).latestImage;
+        assertFalse(latestImage.isEmpty());
+        assertFalse(latestImage.features().isEmpty());
+        assertTrue(latestImage.features().finalizedVersions().isEmpty());
+        assertTrue(latestImage.cluster().isEmpty());
+        assertTrue(latestImage.topics().isEmpty());
+        assertTrue(latestImage.cluster().isEmpty());
+        assertTrue(latestImage.configs().isEmpty());
+        assertTrue(latestImage.producerIds().isEmpty());
+        assertTrue(latestImage.acls().isEmpty());
+        assertTrue(latestImage.scram().isEmpty());
+        assertTrue(latestImage.delegationTokens().isEmpty());
         faultHandler.maybeRethrowFirstException();
     }
 
@@ -364,7 +382,7 @@ public class MetadataLoaderTest {
         long offset
     ) throws Exception {
         MockSnapshotReader snapshotReader = new MockSnapshotReader(
-            new MetadataProvenance(offset, 100, 4000),
+            new MetadataProvenance(offset, 100, 4000, true),
             singletonList(
                 Batch.control(
                     200,
@@ -482,14 +500,14 @@ public class MetadataLoaderTest {
         assertTrue(publishers.get(0).closed);
         assertEquals(
             LogDeltaManifest.newBuilder()
-                .provenance(new MetadataProvenance(300, 100, 4000))
+                .provenance(new MetadataProvenance(300, 100, 4000, true))
                 .leaderAndEpoch(LeaderAndEpoch.UNKNOWN)
                 .numBatches(1)
                 .elapsedNs(0L)
                 .numBytes(10)
                 .build(),
             publishers.get(0).latestLogDeltaManifest);
-        assertEquals(MetadataVersion.IBP_3_3_IV1,
+        assertEquals(Optional.of(MINIMUM_VERSION),
             publishers.get(0).latestImage.features().metadataVersion());
         faultHandler.maybeRethrowFirstException();
     }
@@ -508,10 +526,10 @@ public class MetadataLoaderTest {
                 build()) {
             loader.installPublishers(publishers).get();
             loader.handleLoadSnapshot(MockSnapshotReader.fromRecordLists(
-                new MetadataProvenance(200, 100, 4000), asList(
+                new MetadataProvenance(200, 100, 4000, true), asList(
                     singletonList(new ApiMessageAndVersion(new FeatureLevelRecord().
                         setName(MetadataVersion.FEATURE_NAME).
-                        setFeatureLevel(IBP_3_3_IV1.featureLevel()), (short) 0)),
+                        setFeatureLevel(MINIMUM_VERSION.featureLevel()), (short) 0)),
                     singletonList(new ApiMessageAndVersion(new TopicRecord().
                         setName("foo").
                         setTopicId(Uuid.fromString("Uum7sfhHQP-obSvfywmNUA")), (short) 0))
@@ -531,8 +549,7 @@ public class MetadataLoaderTest {
         for (int i = 0; i < 2; i++) {
             assertTrue(publishers.get(i).closed);
             assertTrue(publishers.get(i).closed);
-            assertEquals(IBP_3_3_IV1,
-                    publishers.get(i).latestImage.features().metadataVersion());
+            assertEquals(Optional.of(MINIMUM_VERSION), publishers.get(i).latestImage.features().metadataVersion());
         }
         faultHandler.maybeRethrowFirstException();
     }
@@ -578,10 +595,10 @@ public class MetadataLoaderTest {
         long offset
     ) throws Exception {
         loader.handleLoadSnapshot(MockSnapshotReader.fromRecordLists(
-                new MetadataProvenance(offset, 100, 4000), asList(
+                new MetadataProvenance(offset, 100, 4000, true), asList(
                         singletonList(new ApiMessageAndVersion(new FeatureLevelRecord().
                                 setName(MetadataVersion.FEATURE_NAME).
-                                setFeatureLevel(IBP_3_3_IV1.featureLevel()), (short) 0)),
+                                setFeatureLevel(MINIMUM_VERSION.featureLevel()), (short) 0)),
                         singletonList(new ApiMessageAndVersion(new TopicRecord().
                                 setName("foo").
                                 setTopicId(Uuid.fromString("Uum7sfhHQP-obSvfywmNUA")), (short) 0))
@@ -594,10 +611,10 @@ public class MetadataLoaderTest {
         long offset
     ) throws Exception {
         loader.handleLoadSnapshot(MockSnapshotReader.fromRecordLists(
-                new MetadataProvenance(offset, 100, 4000), asList(
+                new MetadataProvenance(offset, 100, 4000, true), asList(
                         singletonList(new ApiMessageAndVersion(new FeatureLevelRecord().
                                 setName(MetadataVersion.FEATURE_NAME).
-                                setFeatureLevel(IBP_3_3_IV2.featureLevel()), (short) 0)),
+                                setFeatureLevel(MetadataVersion.latestProduction().featureLevel()), (short) 0)),
                         singletonList(new ApiMessageAndVersion(new TopicRecord().
                                 setName("bar").
                                 setTopicId(Uuid.fromString("VcL2Mw-cT4aL6XV9VujzoQ")), (short) 0))
@@ -625,13 +642,13 @@ public class MetadataLoaderTest {
 
             loadTestSnapshot(loader, 200);
             assertEquals(200L, loader.lastAppliedOffset());
-            assertEquals(IBP_3_3_IV1.featureLevel(),
+            assertEquals(MINIMUM_VERSION.featureLevel(),
                 loader.metrics().currentMetadataVersion().featureLevel());
             assertFalse(publishers.get(0).latestDelta.image().isEmpty());
 
             loadTestSnapshot2(loader, 400);
             assertEquals(400L, loader.lastAppliedOffset());
-            assertEquals(IBP_3_3_IV2.featureLevel(),
+            assertEquals(MetadataVersion.latestProduction().featureLevel(),
                 loader.metrics().currentMetadataVersion().featureLevel());
 
             // Make sure the topic in the initial snapshot was overwritten by loading the new snapshot.
@@ -661,6 +678,12 @@ public class MetadataLoaderTest {
                 setHighWaterMarkAccessor(() -> OptionalLong.of(0)).
                 build()) {
             loader.installPublishers(publishers).get();
+            loader.handleCommit(
+                MockBatchReader.newSingleBatchReader(400, 50, List.of(
+                    new ApiMessageAndVersion(new FeatureLevelRecord()
+                        .setName(MetadataVersion.FEATURE_NAME)
+                        .setFeatureLevel(MINIMUM_VERSION.featureLevel()), (short) 0)))
+            );
             loader.waitForAllEventsToBeHandled();
 
             loader.handleCommit(
@@ -722,6 +745,12 @@ public class MetadataLoaderTest {
                 setHighWaterMarkAccessor(() -> OptionalLong.of(0)).
                 build()) {
             loader.installPublishers(publishers).get();
+            loader.handleCommit(
+                MockBatchReader.newSingleBatchReader(400, 50, List.of(
+                    new ApiMessageAndVersion(new FeatureLevelRecord().
+                        setName(MetadataVersion.FEATURE_NAME).
+                        setFeatureLevel(MINIMUM_VERSION.featureLevel()), (short) 0)))
+            );
             loader.waitForAllEventsToBeHandled();
 
             loader.handleCommit(
@@ -753,6 +782,12 @@ public class MetadataLoaderTest {
                 setHighWaterMarkAccessor(() -> OptionalLong.of(0)).
                 build()) {
             loader.installPublishers(publishers).get();
+            loader.handleCommit(
+                MockBatchReader.newSingleBatchReader(400, 50, List.of(
+                    new ApiMessageAndVersion(new FeatureLevelRecord().
+                        setName(MetadataVersion.FEATURE_NAME).
+                        setFeatureLevel(MINIMUM_VERSION.featureLevel()), (short) 0)))
+            );
             loader.waitForAllEventsToBeHandled();
 
             loader.handleCommit(
@@ -769,7 +804,7 @@ public class MetadataLoaderTest {
 
             // loading a snapshot discards any in-flight transaction
             loader.handleLoadSnapshot(MockSnapshotReader.fromRecordLists(
-                new MetadataProvenance(600, 101, 4000), singletonList(
+                new MetadataProvenance(600, 101, 4000, true), singletonList(
                     singletonList(new ApiMessageAndVersion(new TopicRecord().
                         setName("foo").
                         setTopicId(Uuid.fromString("Uum7sfhHQP-obSvfywmNUA")), (short) 0))
@@ -809,12 +844,10 @@ public class MetadataLoaderTest {
             loader.handleCommit(
                 MockBatchReader.newSingleBatchReader(0, 1, singletonList(
                     // Any record will work here
-                    new ApiMessageAndVersion(new ConfigRecord()
-                        .setResourceType(ConfigResource.Type.BROKER.id())
-                        .setResourceName("3000")
-                        .setName("foo")
-                        .setValue("bar"), (short) 0)
-                )));
+                    new ApiMessageAndVersion(new FeatureLevelRecord()
+                        .setName(MetadataVersion.FEATURE_NAME)
+                        .setFeatureLevel(MINIMUM_VERSION.featureLevel()), (short) 0)))
+            );
             firstPublish.get(30, TimeUnit.SECONDS);
 
             assertFalse(capturedImages.isEmpty());
