@@ -29,15 +29,19 @@ import org.apache.kafka.metadata.properties.MetaProperties;
 import org.apache.kafka.metadata.properties.MetaPropertiesEnsemble;
 import org.apache.kafka.raft.DynamicVoters;
 import org.apache.kafka.server.common.ApiMessageAndVersion;
-import org.apache.kafka.server.common.Features;
-import org.apache.kafka.server.common.KRaftVersion;
+import org.apache.kafka.server.common.EligibleLeaderReplicasVersion;
+import org.apache.kafka.server.common.Feature;
+import org.apache.kafka.server.common.GroupVersion;
 import org.apache.kafka.server.common.MetadataVersion;
 import org.apache.kafka.server.common.TestFeatureVersion;
+import org.apache.kafka.server.common.TransactionVersion;
 import org.apache.kafka.test.TestUtils;
 
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.Timeout;
 import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.Arguments;
+import org.junit.jupiter.params.provider.MethodSource;
 import org.junit.jupiter.params.provider.ValueSource;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -53,9 +57,11 @@ import java.util.List;
 import java.util.Optional;
 import java.util.OptionalInt;
 import java.util.stream.Collectors;
+import java.util.stream.Stream;
 
 import static org.apache.kafka.metadata.storage.ScramParserTest.TEST_SALT;
 import static org.apache.kafka.metadata.storage.ScramParserTest.TEST_SALTED_PASSWORD;
+import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotEquals;
@@ -142,7 +148,7 @@ public class FormatterTest {
             assertEquals(Optional.of(DEFAULT_CLUSTER_ID.toString()), ensemble.clusterId());
             assertEquals(new HashSet<>(testEnv.directories), ensemble.logDirProps().keySet());
             BootstrapMetadata bootstrapMetadata =
-                new BootstrapDirectory(testEnv.directory(0), Optional.empty()).read();
+                new BootstrapDirectory(testEnv.directory(0)).read();
             assertEquals(MetadataVersion.latestProduction(), bootstrapMetadata.metadataVersion());
         }
     }
@@ -188,40 +194,6 @@ public class FormatterTest {
         }
     }
 
-    @Test
-    public void testStandaloneWithIgnoreFormatted() throws Exception {
-        try (TestEnv testEnv = new TestEnv(1)) {
-            FormatterContext formatter1 = testEnv.newFormatter();
-            String originalDirectoryId = Uuid.randomUuid().toString();
-            String newDirectoryId = Uuid.randomUuid().toString();
-            formatter1.formatter
-                .setInitialControllers(DynamicVoters.parse("1@localhost:8020:" + originalDirectoryId))
-                .setHasDynamicQuorum(true)
-                .run();
-            assertEquals("Formatting dynamic metadata voter directory " + testEnv.directory(0) +
-                    " with metadata.version " + MetadataVersion.latestProduction() + ".",
-                formatter1.output().trim());
-            assertMetadataDirectoryId(testEnv, Uuid.fromString(originalDirectoryId));
-
-            FormatterContext formatter2 = testEnv.newFormatter();
-            formatter2.formatter
-                .setIgnoreFormatted(true)
-                .setInitialControllers(DynamicVoters.parse("1@localhost:8020:" + newDirectoryId))
-                .run();
-            assertEquals("All of the log directories are already formatted.",
-                formatter2.output().trim());
-            assertMetadataDirectoryId(testEnv, Uuid.fromString(originalDirectoryId));
-        }
-    }
-
-    private void assertMetadataDirectoryId(TestEnv testEnv, Uuid expectedDirectoryId) throws Exception {
-        MetaPropertiesEnsemble ensemble = new MetaPropertiesEnsemble.Loader().
-            addLogDirs(testEnv.directories).
-            load();
-        MetaProperties logDirProps0 = ensemble.logDirProps().get(testEnv.directory(0));
-        assertEquals(expectedDirectoryId, logDirProps0.directoryId().get());
-    }
-
     @Test
     public void testOneDirectoryFormattedAndOthersNotFormatted() throws Exception {
         try (TestEnv testEnv = new TestEnv(2)) {
@@ -257,7 +229,7 @@ public class FormatterTest {
                 " with metadata.version " + MetadataVersion.IBP_3_5_IV0 + ".",
                     formatter1.output().trim());
             BootstrapMetadata bootstrapMetadata =
-                new BootstrapDirectory(testEnv.directory(0), Optional.empty()).read();
+                new BootstrapDirectory(testEnv.directory(0)).read();
             assertEquals(MetadataVersion.IBP_3_5_IV0, bootstrapMetadata.metadataVersion());
             assertEquals(1, bootstrapMetadata.records().size());
         }
@@ -284,7 +256,7 @@ public class FormatterTest {
                 " with metadata.version " + MetadataVersion.latestTesting() + ".",
                     formatter1.output().trim());
             BootstrapMetadata bootstrapMetadata =
-                    new BootstrapDirectory(testEnv.directory(0), Optional.empty()).read();
+                    new BootstrapDirectory(testEnv.directory(0)).read();
             assertEquals(MetadataVersion.latestTesting(), bootstrapMetadata.metadataVersion());
         }
     }
@@ -334,7 +306,7 @@ public class FormatterTest {
                 " with metadata.version " + MetadataVersion.IBP_3_8_IV0 + ".",
                     formatter1.output().trim());
             BootstrapMetadata bootstrapMetadata =
-                new BootstrapDirectory(testEnv.directory(0), Optional.empty()).read();
+                new BootstrapDirectory(testEnv.directory(0)).read();
             assertEquals(MetadataVersion.IBP_3_8_IV0, bootstrapMetadata.metadataVersion());
             List<ApiMessageAndVersion> scramRecords = bootstrapMetadata.records().stream().
                 filter(r -> r.message() instanceof UserScramCredentialRecord).
@@ -365,21 +337,27 @@ public class FormatterTest {
     public void testFeatureFlag(short version) throws Exception {
         try (TestEnv testEnv = new TestEnv(1)) {
             FormatterContext formatter1 = testEnv.newFormatter();
-            formatter1.formatter.setSupportedFeatures(Arrays.asList(Features.values()));
+            formatter1.formatter.setSupportedFeatures(Feature.TEST_AND_PRODUCTION_FEATURES);
             formatter1.formatter.setFeatureLevel(TestFeatureVersion.FEATURE_NAME, version);
             formatter1.formatter.run();
             BootstrapMetadata bootstrapMetadata =
-                new BootstrapDirectory(testEnv.directory(0), Optional.empty()).read();
+                new BootstrapDirectory(testEnv.directory(0)).read();
             List<ApiMessageAndVersion> expected = new ArrayList<>();
             expected.add(new ApiMessageAndVersion(new FeatureLevelRecord().
                 setName(MetadataVersion.FEATURE_NAME).
                 setFeatureLevel(MetadataVersion.latestProduction().featureLevel()),
                     (short) 0));
+            expected.add(new ApiMessageAndVersion(new FeatureLevelRecord().
+                setName(GroupVersion.FEATURE_NAME).
+                setFeatureLevel(GroupVersion.GV_1.featureLevel()), (short) 0));
             if (version > 0) {
                 expected.add(new ApiMessageAndVersion(new FeatureLevelRecord().
                     setName(TestFeatureVersion.FEATURE_NAME).
                     setFeatureLevel(version), (short) 0));
             }
+            expected.add(new ApiMessageAndVersion(new FeatureLevelRecord().
+                setName(TransactionVersion.FEATURE_NAME).
+                setFeatureLevel(TransactionVersion.TV_2.featureLevel()), (short) 0));
             assertEquals(expected, bootstrapMetadata.records());
         }
     }
@@ -388,10 +366,11 @@ public class FormatterTest {
     public void testInvalidFeatureFlag() throws Exception {
         try (TestEnv testEnv = new TestEnv(2)) {
             FormatterContext formatter1 = testEnv.newFormatter();
-            formatter1.formatter.setSupportedFeatures(Arrays.asList(Features.values()));
+            formatter1.formatter.setSupportedFeatures(Feature.TEST_AND_PRODUCTION_FEATURES);
             formatter1.formatter.setFeatureLevel("nonexistent.feature", (short) 1);
             assertEquals("Unsupported feature: nonexistent.feature. Supported features " +
-                    "are: kraft.version, test.feature.version, transaction.version",
+                    "are: eligible.leader.replicas.version, group.version, kraft.version, " +
+                    "test.feature.version, transaction.version",
                 assertThrows(FormatterException.class,
                     () -> formatter1.formatter.run()).
                         getMessage());
@@ -404,14 +383,13 @@ public class FormatterTest {
         try (TestEnv testEnv = new TestEnv(2)) {
             FormatterContext formatter1 = testEnv.newFormatter();
             if (specifyKRaftVersion) {
-                formatter1.formatter.setFeatureLevel(KRaftVersion.FEATURE_NAME, (short) 1);
+                formatter1.formatter.setFeatureLevel("kraft.version", (short) 1);
             }
             formatter1.formatter.setUnstableFeatureVersionsEnabled(true);
             formatter1.formatter.setInitialControllers(DynamicVoters.
                 parse("1@localhost:8020:4znU-ou9Taa06bmEJxsjnw"));
-            formatter1.formatter.setHasDynamicQuorum(true);
             formatter1.formatter.run();
-            assertEquals((short) 1, formatter1.formatter.featureLevels.get(KRaftVersion.FEATURE_NAME));
+            assertEquals((short) 1, formatter1.formatter.featureLevels.getOrDefault("kraft.version", (short) 0));
             assertEquals(Arrays.asList(
                 String.format("Formatting data directory %s with %s %s.",
                     testEnv.directory(1),
@@ -438,77 +416,90 @@ public class FormatterTest {
     public void testFormatWithInitialVotersFailsWithOlderKraftVersion() throws Exception {
         try (TestEnv testEnv = new TestEnv(2)) {
             FormatterContext formatter1 = testEnv.newFormatter();
-            formatter1.formatter.setFeatureLevel(KRaftVersion.FEATURE_NAME, (short) 0);
+            formatter1.formatter.setFeatureLevel("kraft.version", (short) 0);
             formatter1.formatter.setUnstableFeatureVersionsEnabled(true);
             formatter1.formatter.setInitialControllers(DynamicVoters.
                     parse("1@localhost:8020:4znU-ou9Taa06bmEJxsjnw"));
-            formatter1.formatter.setHasDynamicQuorum(true);
             assertTrue(formatter1.formatter.hasDynamicQuorum());
-            assertEquals(
-                "Cannot set kraft.version to 0 if controller.quorum.voters is empty " +
-                "and one of the flags --standalone, --initial-controllers, or --no-initial-controllers is used. " +
-                "For dynamic controllers support, try removing the --feature flag for kraft.version.",
-                assertThrows(FormatterException.class, formatter1.formatter::run).getMessage()
-            );
+            assertEquals("Cannot set kraft.version to 0 if KIP-853 configuration is present. " +
+                "Try removing the --feature flag for kraft.version.",
+                    assertThrows(FormatterException.class,
+                        () -> formatter1.formatter.run()).getMessage());
         }
     }
 
     @Test
-    public void testFormatWithStaticQuorumFailsWithNewerKraftVersion() throws Exception {
+    public void testFormatWithoutInitialVotersFailsWithNewerKraftVersion() throws Exception {
         try (TestEnv testEnv = new TestEnv(2)) {
             FormatterContext formatter1 = testEnv.newFormatter();
-            formatter1.formatter.setFeatureLevel(KRaftVersion.FEATURE_NAME, (short) 1);
+            formatter1.formatter.setFeatureLevel("kraft.version", (short) 1);
             formatter1.formatter.setUnstableFeatureVersionsEnabled(true);
             assertFalse(formatter1.formatter.hasDynamicQuorum());
-            assertEquals(
-                "Cannot set kraft.version to 1 unless controller.quorum.voters is empty and " +
-                "one of the flags --standalone, --initial-controllers, or --no-initial-controllers is used. " +
-                "For dynamic controllers support, try using one of --standalone, --initial-controllers, " +
-                "or --no-initial-controllers and removing controller.quorum.voters.",
-                assertThrows(FormatterException.class, formatter1.formatter::run).getMessage()
-            );
+            assertEquals("Cannot set kraft.version to 1 unless KIP-853 configuration is present. " +
+                "Try removing the --feature flag for kraft.version.",
+                    assertThrows(FormatterException.class,
+                        () -> formatter1.formatter.run()).getMessage());
         }
     }
 
     @Test
-    public void testFormatWithInitialVotersWithOlderMetadataVersion() throws Exception {
+    public void testFormatWithInitialVotersFailsWithOlderMetadataVersion() throws Exception {
         try (TestEnv testEnv = new TestEnv(2)) {
             FormatterContext formatter1 = testEnv.newFormatter();
             formatter1.formatter.setReleaseVersion(MetadataVersion.IBP_3_8_IV0);
-            formatter1.formatter.setFeatureLevel(KRaftVersion.FEATURE_NAME, (short) 1);
+            formatter1.formatter.setFeatureLevel("kraft.version", (short) 1);
             formatter1.formatter.setInitialControllers(DynamicVoters.
                     parse("1@localhost:8020:4znU-ou9Taa06bmEJxsjnw"));
             formatter1.formatter.setUnstableFeatureVersionsEnabled(true);
-            formatter1.formatter.setHasDynamicQuorum(true);
-            formatter1.formatter.run();
-            assertEquals((short) 1, formatter1.formatter.featureLevels.get(KRaftVersion.FEATURE_NAME));
+            assertEquals("kraft.version could not be set to 1 because it depends on " +
+                "metadata.version level 21",
+                    assertThrows(IllegalArgumentException.class,
+                        () -> formatter1.formatter.run()).getMessage());
+        }
+    }
+
+    private static Stream<Arguments> elrTestMetadataVersions() {
+        return Stream.of(
+            MetadataVersion.IBP_3_9_IV0,
+            MetadataVersion.IBP_4_0_IV0,
+            MetadataVersion.IBP_4_0_IV1 // ELR minimal MV
+        ).map(Arguments::of);
+    }
+
+    @ParameterizedTest
+    @MethodSource("elrTestMetadataVersions")
+    public void testFormatElrEnabledWithMetadataVersions(MetadataVersion metadataVersion) throws Exception {
+        try (TestEnv testEnv = new TestEnv(2)) {
+            FormatterContext formatter1 = testEnv.newFormatter();
+            formatter1.formatter.setReleaseVersion(metadataVersion);
+            formatter1.formatter.setFeatureLevel(EligibleLeaderReplicasVersion.FEATURE_NAME, (short) 1);
+            formatter1.formatter.setInitialControllers(DynamicVoters.
+                parse("1@localhost:8020:4znU-ou9Taa06bmEJxsjnw"));
+            if (metadataVersion.isAtLeast(MetadataVersion.IBP_4_0_IV1)) {
+                assertDoesNotThrow(() -> formatter1.formatter.run());
+            } else {
+                assertEquals("eligible.leader.replicas.version could not be set to 1 because it depends on " +
+                    "metadata.version level 23",
+                    assertThrows(IllegalArgumentException.class,
+                        () -> formatter1.formatter.run()).getMessage());
+            }
         }
     }
 
     @ParameterizedTest
     @ValueSource(booleans = {false, true})
-    public void testFormatWithNoInitialControllersWithOlderMetadataVersion(boolean hasDynamicQuorum) throws Exception {
+    public void testFormatWithNoInitialControllers(boolean specifyKRaftVersion) throws Exception {
         try (TestEnv testEnv = new TestEnv(2)) {
             FormatterContext formatter1 = testEnv.newFormatter();
-            formatter1.formatter.setReleaseVersion(MetadataVersion.IBP_3_8_IV0);
-            formatter1.formatter.setHasDynamicQuorum(hasDynamicQuorum);
-            formatter1.formatter.run();
-            if (hasDynamicQuorum) {
-                assertEquals((short) 1, formatter1.formatter.featureLevels.get(KRaftVersion.FEATURE_NAME));
-            } else {
-                assertEquals((short) 0, formatter1.formatter.featureLevels.get(KRaftVersion.FEATURE_NAME));
+            if (specifyKRaftVersion) {
+                formatter1.formatter.setFeatureLevel("kraft.version", (short) 1);
             }
-        }
-    }
-
-    @Test
-    public void testFormatWithNoInitialControllers() throws Exception {
-        try (TestEnv testEnv = new TestEnv(2)) {
-            FormatterContext formatter1 = testEnv.newFormatter();
             formatter1.formatter.setUnstableFeatureVersionsEnabled(true);
-            assertFalse(formatter1.formatter.hasDynamicQuorum());
+            formatter1.formatter.setNoInitialControllersFlag(true);
+            assertTrue(formatter1.formatter.hasDynamicQuorum());
+
             formatter1.formatter.run();
-            assertEquals((short) 0, formatter1.formatter.featureLevels.get(KRaftVersion.FEATURE_NAME));
+            assertEquals((short) 1, formatter1.formatter.featureLevels.getOrDefault("kraft.version", (short) 0));
             assertEquals(Arrays.asList(
                     String.format("Formatting data directory %s with %s %s.",
                         testEnv.directory(1),
@@ -528,4 +519,34 @@ public class FormatterTest {
             assertNotNull(logDirProps1);
         }
     }
+
+    @Test
+    public void testFormatWithoutNoInitialControllersFailsWithNewerKraftVersion() throws Exception {
+        try (TestEnv testEnv = new TestEnv(2)) {
+            FormatterContext formatter1 = testEnv.newFormatter();
+            formatter1.formatter.setFeatureLevel("kraft.version", (short) 1);
+            formatter1.formatter.setUnstableFeatureVersionsEnabled(true);
+            formatter1.formatter.setNoInitialControllersFlag(false);
+            assertFalse(formatter1.formatter.hasDynamicQuorum());
+            assertEquals("Cannot set kraft.version to 1 unless KIP-853 configuration is present. " +
+                    "Try removing the --feature flag for kraft.version.",
+                assertThrows(FormatterException.class,
+                    formatter1.formatter::run).getMessage());
+        }
+    }
+
+    @Test
+    public void testFormatWithNoInitialControllersFailsWithOlderKraftVersion() throws Exception {
+        try (TestEnv testEnv = new TestEnv(2)) {
+            FormatterContext formatter1 = testEnv.newFormatter();
+            formatter1.formatter.setFeatureLevel("kraft.version", (short) 0);
+            formatter1.formatter.setUnstableFeatureVersionsEnabled(true);
+            formatter1.formatter.setNoInitialControllersFlag(true);
+            assertTrue(formatter1.formatter.hasDynamicQuorum());
+            assertEquals("Cannot set kraft.version to 0 if KIP-853 configuration is present. " +
+                    "Try removing the --feature flag for kraft.version.",
+                assertThrows(FormatterException.class,
+                    formatter1.formatter::run).getMessage());
+        }
+    }
 }
