@@ -20,38 +20,33 @@ import org.apache.kafka.common.Uuid;
 import org.apache.kafka.common.utils.LogContext;
 import org.apache.kafka.common.utils.Time;
 import org.apache.kafka.common.utils.Timer;
+import org.apache.kafka.raft.internals.EpochElection;
 
 import org.slf4j.Logger;
 
-import java.util.HashMap;
-import java.util.Map;
 import java.util.Optional;
-import java.util.Set;
-import java.util.stream.Collectors;
-import java.util.stream.Stream;
 
-public class CandidateState implements EpochState {
+public class CandidateState implements NomineeState {
     private final int localId;
     private final Uuid localDirectoryId;
     private final int epoch;
     private final int retries;
-    private final Map<Integer, VoterState> voteStates = new HashMap<>();
+    private final EpochElection epochElection;
     private final Optional<LogOffsetMetadata> highWatermark;
     private final int electionTimeoutMs;
     private final Timer electionTimer;
     private final Timer backoffTimer;
     private final Logger log;
 
+    private boolean isBackingOff;
     /**
      * The lifetime of a candidate state is the following.
      *
-     *  1. Once started, it would keep record of the received votes.
-     *  2. If majority votes granted, it can then end its life and will be replaced by a leader state;
-     *  3. If majority votes rejected or election timed out, it would transit into a backing off phase;
-     *     after the backoff phase completes, it would end its left and be replaced by a new candidate state with bumped retry.
+     *  1. Once started, it will send vote requests and keep record of the received vote responses.
+     *  2. If majority votes granted, it will transition to leader state.
+     *  3. If majority votes rejected, it will transition to prospective after a backoff phase.
+     *  4. If election times out, it will transition immediately to prospective.
      */
-    private boolean isBackingOff;
-
     protected CandidateState(
         Time time,
         int localId,
@@ -85,26 +80,8 @@ public class CandidateState implements EpochState {
         this.backoffTimer = time.timer(0);
         this.log = logContext.logger(CandidateState.class);
 
-        for (ReplicaKey voter : voters.voterKeys()) {
-            voteStates.put(voter.id(), new VoterState(voter));
-        }
-        voteStates.get(localId).setState(State.GRANTED);
-    }
-
-    public int localId() {
-        return localId;
-    }
-
-    public int majoritySize() {
-        return voteStates.size() / 2 + 1;
-    }
-
-    private long numGranted() {
-        return votersInState(State.GRANTED).count();
-    }
-
-    private long numUnrecorded() {
-        return votersInState(State.UNRECORDED).count();
+        this.epochElection = new EpochElection(voters.voterKeys());
+        epochElection.recordVote(localId, true);
     }
 
     /**
@@ -118,69 +95,27 @@ public class CandidateState implements EpochState {
         return retries;
     }
 
-    /**
-     * Check whether we have received enough votes to conclude the election and become leader.
-     *
-     * @return true if at least a majority of nodes have granted the vote
-     */
-    public boolean isVoteGranted() {
-        return numGranted() >= majoritySize();
+    @Override
+    public EpochElection epochElection() {
+        return epochElection;
     }
 
-    /**
-     * Check if we have received enough rejections that it is no longer possible to reach a
-     * majority of grants.
-     *
-     * @return true if the vote is rejected, false if the vote is already or can still be granted
-     */
-    public boolean isVoteRejected() {
-        return numGranted() + numUnrecorded() < majoritySize();
-    }
-
-    /**
-     * Record a granted vote from one of the voters.
-     *
-     * @param remoteNodeId The id of the voter
-     * @return true if the voter had not been previously recorded
-     * @throws IllegalArgumentException if the remote node is not a voter or if the vote had already been
-     *         rejected by this node
-     */
+    @Override
     public boolean recordGrantedVote(int remoteNodeId) {
-        VoterState voterState = voteStates.get(remoteNodeId);
-        if (voterState == null) {
-            throw new IllegalArgumentException("Attempt to grant vote to non-voter " + remoteNodeId);
-        } else if (voterState.state().equals(State.REJECTED)) {
+        if (epochElection().isRejectedVoter(remoteNodeId)) {
             throw new IllegalArgumentException("Attempt to grant vote from node " + remoteNodeId +
                 " which previously rejected our request");
         }
-
-        boolean recorded = voterState.state().equals(State.UNRECORDED);
-        voterState.setState(State.GRANTED);
-
-        return recorded;
+        return epochElection().recordVote(remoteNodeId, true);
     }
 
-    /**
-     * Record a rejected vote from one of the voters.
-     *
-     * @param remoteNodeId The id of the voter
-     * @return true if the rejected vote had not been previously recorded
-     * @throws IllegalArgumentException if the remote node is not a voter or if the vote had already been
-     *         granted by this node
-     */
+    @Override
     public boolean recordRejectedVote(int remoteNodeId) {
-        VoterState voterState = voteStates.get(remoteNodeId);
-        if (voterState == null) {
-            throw new IllegalArgumentException("Attempt to reject vote to non-voter " + remoteNodeId);
-        } else if (voterState.state().equals(State.GRANTED)) {
+        if (epochElection().isGrantedVoter(remoteNodeId)) {
             throw new IllegalArgumentException("Attempt to reject vote from node " + remoteNodeId +
                 " which previously granted our request");
         }
-
-        boolean recorded = voterState.state().equals(State.UNRECORDED);
-        voterState.setState(State.REJECTED);
-
-        return recorded;
+        return epochElection().recordVote(remoteNodeId, false);
     }
 
     /**
@@ -192,41 +127,7 @@ public class CandidateState implements EpochState {
         this.isBackingOff = true;
     }
 
-    /**
-     * Get the set of voters which have not been counted as granted or rejected yet.
-     *
-     * @return The set of unrecorded voters
-     */
-    public Set<ReplicaKey> unrecordedVoters() {
-        return votersInState(State.UNRECORDED).collect(Collectors.toSet());
-    }
-
-    /**
-     * Get the set of voters that have granted our vote requests.
-     *
-     * @return The set of granting voters, which should always contain the ID of the candidate
-     */
-    public Set<Integer> grantingVoters() {
-        return votersInState(State.GRANTED).map(ReplicaKey::id).collect(Collectors.toSet());
-    }
-
-    /**
-     * Get the set of voters that have rejected our candidacy.
-     *
-     * @return The set of rejecting voters
-     */
-    public Set<Integer> rejectingVoters() {
-        return votersInState(State.REJECTED).map(ReplicaKey::id).collect(Collectors.toSet());
-    }
-
-    private Stream<ReplicaKey> votersInState(State state) {
-        return voteStates
-            .values()
-            .stream()
-            .filter(voterState -> voterState.state().equals(state))
-            .map(VoterState::replicaKey);
-    }
-
+    @Override
     public boolean hasElectionTimeoutExpired(long currentTimeMs) {
         electionTimer.update(currentTimeMs);
         return electionTimer.isExpired();
@@ -245,6 +146,7 @@ public class CandidateState implements EpochState {
         return backoffTimer.remainingMs();
     }
 
+    @Override
     public long remainingElectionTimeMs(long currentTimeMs) {
         electionTimer.update(currentTimeMs);
         return electionTimer.remainingMs();
@@ -255,7 +157,7 @@ public class CandidateState implements EpochState {
         return ElectionState.withVotedCandidate(
             epoch,
             ReplicaKey.of(localId, localDirectoryId),
-            voteStates.keySet()
+            epochElection.voterIds()
         );
     }
 
@@ -276,15 +178,22 @@ public class CandidateState implements EpochState {
 
     @Override
     public boolean canGrantVote(
-        ReplicaKey candidateKey,
-        boolean isLogUpToDate
+        ReplicaKey replicaKey,
+        boolean isLogUpToDate,
+        boolean isPreVote
     ) {
-        // Still reject vote request even candidateId = localId, Although the candidate votes for
+        if (isPreVote && isLogUpToDate) {
+            return true;
+        }
+        // Reject standard vote requests even if replicaId = localId, although the replica votes for
         // itself, this vote is implicit and not "granted".
         log.debug(
-            "Rejecting vote request from candidate ({}) since we are already candidate in epoch {}",
-            candidateKey,
-            epoch
+            "Rejecting Vote request (preVote={}) from replica ({}) since we are in CandidateState in epoch {} " +
+                "and the replica's log is up-to-date={}",
+            isPreVote,
+            replicaKey,
+            epoch,
+            isLogUpToDate
         );
         return false;
     }
@@ -292,13 +201,13 @@ public class CandidateState implements EpochState {
     @Override
     public String toString() {
         return String.format(
-            "CandidateState(localId=%d, localDirectoryId=%s,epoch=%d, retries=%d, voteStates=%s, " +
+            "CandidateState(localId=%d, localDirectoryId=%s, epoch=%d, retries=%d, epochElection=%s, " +
             "highWatermark=%s, electionTimeoutMs=%d)",
             localId,
             localDirectoryId,
             epoch,
             retries,
-            voteStates,
+            epochElection(),
             highWatermark,
             electionTimeoutMs
         );
@@ -311,31 +220,4 @@ public class CandidateState implements EpochState {
 
     @Override
     public void close() {}
-
-    private static final class VoterState {
-        private final ReplicaKey replicaKey;
-        private State state = State.UNRECORDED;
-
-        private VoterState(ReplicaKey replicaKey) {
-            this.replicaKey = replicaKey;
-        }
-
-        public State state() {
-            return state;
-        }
-
-        public void setState(State state) {
-            this.state = state;
-        }
-
-        public ReplicaKey replicaKey() {
-            return replicaKey;
-        }
-    }
-
-    private enum State {
-        UNRECORDED,
-        GRANTED,
-        REJECTED
-    }
 }
