@@ -23,8 +23,6 @@ import org.apache.kafka.common.message.LeaderChangeMessage;
 import org.apache.kafka.common.message.SnapshotHeaderRecord;
 import org.apache.kafka.common.protocol.ObjectSerializationCache;
 import org.apache.kafka.common.protocol.Writable;
-import org.apache.kafka.common.record.AbstractRecords;
-import org.apache.kafka.common.record.CompressionType;
 import org.apache.kafka.common.record.ControlRecordUtils;
 import org.apache.kafka.common.record.DefaultRecord;
 import org.apache.kafka.common.record.MemoryRecordsBuilder;
@@ -33,24 +31,20 @@ import org.apache.kafka.common.record.TimestampType;
 import org.apache.kafka.common.utils.ByteUtils;
 import org.apache.kafka.common.utils.MockTime;
 import org.apache.kafka.common.utils.Utils;
-import org.apache.kafka.raft.errors.UnexpectedBaseOffsetException;
+
 import org.junit.jupiter.api.Test;
-import org.junit.jupiter.params.ParameterizedTest;
-import org.junit.jupiter.params.provider.ValueSource;
 import org.mockito.Mockito;
 
 import java.nio.ByteBuffer;
 import java.util.Collections;
 import java.util.List;
-import java.util.OptionalLong;
 import java.util.concurrent.CountDownLatch;
-import java.util.stream.Collectors;
-import java.util.stream.Stream;
 
 import static java.util.Arrays.asList;
 import static java.util.Collections.singletonList;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotEquals;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
@@ -58,6 +52,7 @@ class BatchAccumulatorTest {
     private final MemoryPool memoryPool = Mockito.mock(MemoryPool.class);
     private final MockTime time = new MockTime();
     private final StringSerde serde = new StringSerde();
+    private final int maxNumberOfBatches = 10;
 
     private BatchAccumulator<String> buildAccumulator(
         int leaderEpoch,
@@ -70,6 +65,7 @@ class BatchAccumulatorTest {
             baseOffset,
             lingerMs,
             maxBatchSize,
+            maxNumberOfBatches,
             memoryPool,
             time,
             Compression.NONE,
@@ -108,102 +104,98 @@ class BatchAccumulatorTest {
 
     @Test
     public void testForceDrain() {
-        asList(APPEND, APPEND_ATOMIC).forEach(appender -> {
-            int leaderEpoch = 17;
-            long baseOffset = 157;
-            int lingerMs = 50;
-            int maxBatchSize = 512;
+        int leaderEpoch = 17;
+        long baseOffset = 157;
+        int lingerMs = 50;
+        int maxBatchSize = 512;
 
-            Mockito.when(memoryPool.tryAllocate(maxBatchSize))
-                .thenReturn(ByteBuffer.allocate(maxBatchSize));
+        Mockito.when(memoryPool.tryAllocate(maxBatchSize))
+            .thenReturn(ByteBuffer.allocate(maxBatchSize));
 
-            BatchAccumulator<String> acc = buildAccumulator(
-                leaderEpoch,
-                baseOffset,
-                lingerMs,
-                maxBatchSize
-            );
+        BatchAccumulator<String> acc = buildAccumulator(
+            leaderEpoch,
+            baseOffset,
+            lingerMs,
+            maxBatchSize
+        );
 
-            List<String> records = asList("a", "b", "c", "d", "e", "f", "g", "h", "i");
+        List<String> records = asList("a", "b", "c", "d", "e", "f", "g", "h", "i");
 
-            // Append records
-            assertEquals(baseOffset, appender.call(acc, leaderEpoch, records.subList(0, 1)));
-            assertEquals(baseOffset + 2, appender.call(acc, leaderEpoch, records.subList(1, 3)));
-            assertEquals(baseOffset + 5, appender.call(acc, leaderEpoch, records.subList(3, 6)));
-            assertEquals(baseOffset + 7, appender.call(acc, leaderEpoch, records.subList(6, 8)));
-            assertEquals(baseOffset + 8, appender.call(acc, leaderEpoch, records.subList(8, 9)));
+        // Append records
+        assertEquals(baseOffset, acc.append(leaderEpoch, records.subList(0, 1), false));
+        assertEquals(baseOffset + 2, acc.append(leaderEpoch, records.subList(1, 3), false));
+        assertEquals(baseOffset + 5, acc.append(leaderEpoch, records.subList(3, 6), false));
+        assertEquals(baseOffset + 7, acc.append(leaderEpoch, records.subList(6, 8), false));
+        assertEquals(baseOffset + 8, acc.append(leaderEpoch, records.subList(8, 9), false));
 
-            assertFalse(acc.needsDrain(time.milliseconds()));
-            acc.forceDrain();
-            assertTrue(acc.needsDrain(time.milliseconds()));
-            assertEquals(0, acc.timeUntilDrain(time.milliseconds()));
+        assertFalse(acc.needsDrain(time.milliseconds()));
+        acc.forceDrain();
+        assertTrue(acc.needsDrain(time.milliseconds()));
+        assertEquals(0, acc.timeUntilDrain(time.milliseconds()));
 
-            // Drain completed batches
-            List<BatchAccumulator.CompletedBatch<String>> batches = acc.drain();
+        // Drain completed batches
+        List<BatchAccumulator.CompletedBatch<String>> batches = acc.drain();
 
-            assertEquals(1, batches.size());
-            assertFalse(acc.needsDrain(time.milliseconds()));
-            assertEquals(Long.MAX_VALUE - time.milliseconds(), acc.timeUntilDrain(time.milliseconds()));
+        assertEquals(1, batches.size());
+        assertFalse(acc.needsDrain(time.milliseconds()));
+        assertEquals(Long.MAX_VALUE - time.milliseconds(), acc.timeUntilDrain(time.milliseconds()));
 
-            BatchAccumulator.CompletedBatch<String> batch = batches.get(0);
-            assertEquals(records, batch.records.get());
-            assertEquals(baseOffset, batch.baseOffset);
-            assertEquals(time.milliseconds(), batch.appendTimestamp());
-        });
+        BatchAccumulator.CompletedBatch<String> batch = batches.get(0);
+        assertEquals(records, batch.records.get());
+        assertEquals(baseOffset, batch.baseOffset);
+        assertEquals(time.milliseconds(), batch.appendTimestamp());
     }
 
     @Test
     public void testForceDrainBeforeAppendLeaderChangeMessage() {
-        asList(APPEND, APPEND_ATOMIC).forEach(appender -> {
-            int leaderEpoch = 17;
-            long baseOffset = 157;
-            int lingerMs = 50;
-            int maxBatchSize = 512;
+        int leaderEpoch = 17;
+        long baseOffset = 157;
+        int lingerMs = 50;
+        int maxBatchSize = 512;
 
-            Mockito.when(memoryPool.tryAllocate(maxBatchSize))
-                .thenReturn(ByteBuffer.allocate(maxBatchSize));
-            Mockito.when(memoryPool.tryAllocate(256))
-                .thenReturn(ByteBuffer.allocate(256));
+        Mockito.when(memoryPool.tryAllocate(maxBatchSize))
+            .thenReturn(ByteBuffer.allocate(maxBatchSize));
+        Mockito.when(memoryPool.tryAllocate(256))
+            .thenReturn(ByteBuffer.allocate(256));
 
-            BatchAccumulator<String> acc = buildAccumulator(
-                leaderEpoch,
-                baseOffset,
-                lingerMs,
-                maxBatchSize
-            );
+        BatchAccumulator<String> acc = buildAccumulator(
+            leaderEpoch,
+            baseOffset,
+            lingerMs,
+            maxBatchSize
+        );
 
-            List<String> records = asList("a", "b", "c", "d", "e", "f", "g", "h", "i");
+        List<String> records = asList("a", "b", "c", "d", "e", "f", "g", "h", "i");
 
-            // Append records
-            assertEquals(baseOffset, appender.call(acc, leaderEpoch, records.subList(0, 1)));
-            assertEquals(baseOffset + 2, appender.call(acc, leaderEpoch, records.subList(1, 3)));
-            assertEquals(baseOffset + 5, appender.call(acc, leaderEpoch, records.subList(3, 6)));
-            assertEquals(baseOffset + 7, appender.call(acc, leaderEpoch, records.subList(6, 8)));
-            assertEquals(baseOffset + 8, appender.call(acc, leaderEpoch, records.subList(8, 9)));
+        // Append records
+        assertEquals(baseOffset, acc.append(leaderEpoch, records.subList(0, 1), false));
+        assertEquals(baseOffset + 2, acc.append(leaderEpoch, records.subList(1, 3), false));
+        assertEquals(baseOffset + 5, acc.append(leaderEpoch, records.subList(3, 6), false));
+        assertEquals(baseOffset + 7, acc.append(leaderEpoch, records.subList(6, 8), false));
+        assertEquals(baseOffset + 8, acc.append(leaderEpoch, records.subList(8, 9), false));
 
-            assertFalse(acc.needsDrain(time.milliseconds()));
+        assertFalse(acc.needsDrain(time.milliseconds()));
 
-            // Append a leader change message
-            acc.appendLeaderChangeMessage(new LeaderChangeMessage(), time.milliseconds());
+        // Append a leader change message
+        acc.appendLeaderChangeMessage(new LeaderChangeMessage(), time.milliseconds());
 
-            assertTrue(acc.needsDrain(time.milliseconds()));
+        assertTrue(acc.needsDrain(time.milliseconds()));
 
-            // Test that drain status is FINISHED
-            assertEquals(0, acc.timeUntilDrain(time.milliseconds()));
+        // Test that drain status is FINISHED
+        assertEquals(0, acc.timeUntilDrain(time.milliseconds()));
 
-            // Drain completed batches
-            List<BatchAccumulator.CompletedBatch<String>> batches = acc.drain();
+        // Drain completed batches
+        List<BatchAccumulator.CompletedBatch<String>> batches = acc.drain();
 
-            // Should have 2 batches, one consisting of `records` and one `leaderChangeMessage`
-            assertEquals(2, batches.size());
-            assertFalse(acc.needsDrain(time.milliseconds()));
-            assertEquals(Long.MAX_VALUE - time.milliseconds(), acc.timeUntilDrain(time.milliseconds()));
+        // Should have 2 batches, one consisting of `records` and one `leaderChangeMessage`
+        assertEquals(2, batches.size());
+        assertFalse(acc.needsDrain(time.milliseconds()));
+        assertEquals(Long.MAX_VALUE - time.milliseconds(), acc.timeUntilDrain(time.milliseconds()));
 
-            BatchAccumulator.CompletedBatch<String> batch = batches.get(0);
-            assertEquals(records, batch.records.get());
-            assertEquals(baseOffset, batch.baseOffset);
-            assertEquals(time.milliseconds(), batch.appendTimestamp());
-        });
+        BatchAccumulator.CompletedBatch<String> batch = batches.get(0);
+        assertEquals(records, batch.records.get());
+        assertEquals(baseOffset, batch.baseOffset);
+        assertEquals(time.milliseconds(), batch.appendTimestamp());
     }
 
     @Test
@@ -243,7 +235,7 @@ class BatchAccumulatorTest {
         );
 
         time.sleep(15);
-        assertEquals(baseOffset, acc.append(leaderEpoch, singletonList("a"), OptionalLong.empty(), false));
+        assertEquals(baseOffset, acc.append(leaderEpoch, singletonList("a"), false));
         assertEquals(lingerMs, acc.timeUntilDrain(time.milliseconds()));
         assertFalse(acc.isEmpty());
 
@@ -275,7 +267,7 @@ class BatchAccumulatorTest {
             maxBatchSize
         );
 
-        assertEquals(baseOffset, acc.append(leaderEpoch, singletonList("a"), OptionalLong.empty(), false));
+        assertEquals(baseOffset, acc.append(leaderEpoch, singletonList("a"), false));
         time.sleep(lingerMs);
 
         List<BatchAccumulator.CompletedBatch<String>> batches = acc.drain();
@@ -304,94 +296,17 @@ class BatchAccumulatorTest {
             maxBatchSize
         );
 
-        assertEquals(baseOffset, acc.append(leaderEpoch, singletonList("a"), OptionalLong.empty(), false));
+        assertEquals(baseOffset, acc.append(leaderEpoch, singletonList("a"), false));
         acc.close();
         Mockito.verify(memoryPool).release(buffer);
     }
 
     @Test
     public void testSingleBatchAccumulation() {
-        asList(APPEND, APPEND_ATOMIC).forEach(appender -> {
-            int leaderEpoch = 17;
-            long baseOffset = 157;
-            int lingerMs = 50;
-            int maxBatchSize = 512;
-
-            Mockito.when(memoryPool.tryAllocate(maxBatchSize))
-                .thenReturn(ByteBuffer.allocate(maxBatchSize));
-
-            BatchAccumulator<String> acc = buildAccumulator(
-                leaderEpoch,
-                baseOffset,
-                lingerMs,
-                maxBatchSize
-            );
-
-            List<String> records = asList("a", "b", "c", "d", "e", "f", "g", "h", "i");
-            assertEquals(baseOffset, appender.call(acc, leaderEpoch, records.subList(0, 1)));
-            assertEquals(baseOffset + 2, appender.call(acc, leaderEpoch, records.subList(1, 3)));
-            assertEquals(baseOffset + 5, appender.call(acc, leaderEpoch, records.subList(3, 6)));
-            assertEquals(baseOffset + 7, appender.call(acc, leaderEpoch, records.subList(6, 8)));
-            assertEquals(baseOffset + 8, appender.call(acc, leaderEpoch, records.subList(8, 9)));
-
-            long expectedAppendTimestamp = time.milliseconds();
-            time.sleep(lingerMs);
-            assertTrue(acc.needsDrain(time.milliseconds()));
-
-            List<BatchAccumulator.CompletedBatch<String>> batches = acc.drain();
-            assertEquals(1, batches.size());
-            assertFalse(acc.needsDrain(time.milliseconds()));
-            assertEquals(Long.MAX_VALUE - time.milliseconds(), acc.timeUntilDrain(time.milliseconds()));
-
-            BatchAccumulator.CompletedBatch<String> batch = batches.get(0);
-            assertEquals(records, batch.records.get());
-            assertEquals(baseOffset, batch.baseOffset);
-            assertEquals(expectedAppendTimestamp, batch.appendTimestamp());
-        });
-    }
-
-    @Test
-    public void testMultipleBatchAccumulation() {
-        asList(APPEND, APPEND_ATOMIC).forEach(appender -> {
-            int leaderEpoch = 17;
-            long baseOffset = 157;
-            int lingerMs = 50;
-            int maxBatchSize = 256;
-
-            Mockito.when(memoryPool.tryAllocate(maxBatchSize))
-                .thenReturn(ByteBuffer.allocate(maxBatchSize));
-
-            BatchAccumulator<String> acc = buildAccumulator(
-                leaderEpoch,
-                baseOffset,
-                lingerMs,
-                maxBatchSize
-            );
-
-            // Append entries until we have 4 batches to drain (3 completed, 1 building)
-            while (acc.numCompletedBatches() < 3) {
-                appender.call(acc, leaderEpoch, singletonList("foo"));
-            }
-
-            List<BatchAccumulator.CompletedBatch<String>> batches = acc.drain();
-            assertEquals(4, batches.size());
-            assertTrue(batches.stream().allMatch(batch -> batch.data.sizeInBytes() <= maxBatchSize));
-        });
-    }
-
-    @Test
-    public void testRecordsAreSplit() {
         int leaderEpoch = 17;
         long baseOffset = 157;
         int lingerMs = 50;
-        String record = "a";
-        int numberOfRecords = 9;
-        int recordsPerBatch = 2;
-        int batchHeaderSize = AbstractRecords.recordBatchHeaderSizeInBytes(
-            RecordBatch.MAGIC_VALUE_V2,
-            CompressionType.NONE
-        );
-        int maxBatchSize = batchHeaderSize + recordsPerBatch * recordSizeInBytes(record, recordsPerBatch);
+        int maxBatchSize = 512;
 
         Mockito.when(memoryPool.tryAllocate(maxBatchSize))
             .thenReturn(ByteBuffer.allocate(maxBatchSize));
@@ -403,19 +318,53 @@ class BatchAccumulatorTest {
             maxBatchSize
         );
 
-        List<String> records = Stream
-            .generate(() -> record)
-            .limit(numberOfRecords)
-            .collect(Collectors.toList());
-        assertEquals(baseOffset + numberOfRecords - 1, acc.append(leaderEpoch, records, OptionalLong.empty(), false));
+        List<String> records = asList("a", "b", "c", "d", "e", "f", "g", "h", "i");
+        assertEquals(baseOffset, acc.append(leaderEpoch, records.subList(0, 1), false));
+        assertEquals(baseOffset + 2, acc.append(leaderEpoch, records.subList(1, 3), false));
+        assertEquals(baseOffset + 5, acc.append(leaderEpoch, records.subList(3, 6), false));
+        assertEquals(baseOffset + 7, acc.append(leaderEpoch, records.subList(6, 8), false));
+        assertEquals(baseOffset + 8, acc.append(leaderEpoch, records.subList(8, 9), false));
 
+        long expectedAppendTimestamp = time.milliseconds();
         time.sleep(lingerMs);
         assertTrue(acc.needsDrain(time.milliseconds()));
 
         List<BatchAccumulator.CompletedBatch<String>> batches = acc.drain();
-        // ceilingDiv(records.size(), recordsPerBatch)
-        int expectedBatches = (records.size() + recordsPerBatch - 1) / recordsPerBatch;
-        assertEquals(expectedBatches, batches.size());
+        assertEquals(1, batches.size());
+        assertFalse(acc.needsDrain(time.milliseconds()));
+        assertEquals(Long.MAX_VALUE - time.milliseconds(), acc.timeUntilDrain(time.milliseconds()));
+
+        BatchAccumulator.CompletedBatch<String> batch = batches.get(0);
+        assertEquals(records, batch.records.get());
+        assertEquals(baseOffset, batch.baseOffset);
+        assertEquals(expectedAppendTimestamp, batch.appendTimestamp());
+    }
+
+    @Test
+    public void testMultipleBatchAccumulation() {
+        int leaderEpoch = 17;
+        long baseOffset = 157;
+        int lingerMs = 50;
+        int maxBatchSize = 256;
+        int numberOfRecords = 100;
+
+        Mockito.when(memoryPool.tryAllocate(maxBatchSize))
+            .thenReturn(ByteBuffer.allocate(maxBatchSize));
+
+        BatchAccumulator<String> acc = buildAccumulator(
+            leaderEpoch,
+            baseOffset,
+            lingerMs,
+            maxBatchSize
+        );
+
+        // Append enough records so that multiple batches get created
+        for (int records = 0; records < numberOfRecords; records++) {
+            acc.append(leaderEpoch, singletonList("foo"), false);
+        }
+
+        List<BatchAccumulator.CompletedBatch<String>> batches = acc.drain();
+        assertTrue(batches.size() > 1, () -> String.format("number of batches is %s", batches.size()));
         assertTrue(batches.stream().allMatch(batch -> batch.data.sizeInBytes() <= maxBatchSize));
     }
 
@@ -443,6 +392,7 @@ class BatchAccumulatorTest {
         long baseOffset = 157;
         int lingerMs = 50;
         int maxBatchSize = 256;
+        int maxNumberOfBatches = 10;
 
         StringSerde serde = Mockito.spy(new StringSerde());
         BatchAccumulator<String> acc = new BatchAccumulator<>(
@@ -450,6 +400,7 @@ class BatchAccumulatorTest {
             baseOffset,
             lingerMs,
             maxBatchSize,
+            maxNumberOfBatches,
             memoryPool,
             time,
             Compression.NONE,
@@ -462,7 +413,7 @@ class BatchAccumulatorTest {
         // Do the first append outside the thread to start the linger timer
         Mockito.when(memoryPool.tryAllocate(maxBatchSize))
             .thenReturn(ByteBuffer.allocate(maxBatchSize));
-        acc.append(leaderEpoch, singletonList("a"), OptionalLong.empty(), false);
+        acc.append(leaderEpoch, singletonList("a"), false);
 
         // Let the serde block to simulate a slow append
         Mockito.doAnswer(invocation -> {
@@ -477,7 +428,7 @@ class BatchAccumulatorTest {
             Mockito.any(Writable.class)
         );
 
-        Thread appendThread = new Thread(() -> acc.append(leaderEpoch, singletonList("b"), OptionalLong.empty(), false));
+        Thread appendThread = new Thread(() -> acc.append(leaderEpoch, singletonList("b"), false));
         appendThread.start();
 
         // Attempt to drain while the append thread is holding the lock
@@ -493,10 +444,10 @@ class BatchAccumulatorTest {
         List<BatchAccumulator.CompletedBatch<String>> drained = acc.drain();
         assertEquals(1, drained.size());
         assertEquals(Long.MAX_VALUE - time.milliseconds(), acc.timeUntilDrain(time.milliseconds()));
-        drained.stream().forEach(completedBatch -> {
-            completedBatch.data.batches().forEach(recordBatch -> {
-                assertEquals(leaderEpoch, recordBatch.partitionLeaderEpoch()); });
-        });
+        drained.forEach(completedBatch ->
+            completedBatch.data.batches().forEach(recordBatch ->
+                assertEquals(leaderEpoch, recordBatch.partitionLeaderEpoch()))
+        );
 
         acc.close();
     }
@@ -515,52 +466,6 @@ class BatchAccumulatorTest {
         return ByteUtils.sizeOfVarint(recordSizeInBytes) + recordSizeInBytes;
     }
 
-    interface Appender {
-        Long call(BatchAccumulator<String> acc, int epoch, List<String> records);
-    }
-
-    static final Appender APPEND_ATOMIC = (acc, epoch, records) ->
-            acc.append(epoch, records, OptionalLong.empty(), true);
-
-    static final Appender APPEND = (acc, epoch, records) ->
-            acc.append(epoch, records, OptionalLong.empty(), false);
-
-    @ParameterizedTest
-    @ValueSource(booleans = {false, true})
-    public void testAppendWithRequiredBaseOffset(boolean correctOffset) {
-        int leaderEpoch = 17;
-        long baseOffset = 157;
-        int lingerMs = 50;
-        int maxBatchSize = 512;
-
-        ByteBuffer buffer = ByteBuffer.allocate(maxBatchSize);
-        Mockito.when(memoryPool.tryAllocate(maxBatchSize))
-                .thenReturn(buffer);
-
-        BatchAccumulator<String> acc = buildAccumulator(
-                leaderEpoch,
-                baseOffset,
-                lingerMs,
-                maxBatchSize
-        );
-
-        if (correctOffset) {
-            assertEquals(baseOffset, acc.append(leaderEpoch,
-                    singletonList("a"),
-                    OptionalLong.of(baseOffset),
-                    true));
-        } else {
-            assertEquals("Wanted base offset 156, but the next offset was 157",
-                assertThrows(UnexpectedBaseOffsetException.class, () -> {
-                    acc.append(leaderEpoch,
-                        singletonList("a"),
-                        OptionalLong.of(baseOffset - 1),
-                        true);
-                }).getMessage());
-        }
-        acc.close();
-    }
-
     @Test
     public void testMultipleControlRecords() {
         int leaderEpoch = 17;
@@ -579,9 +484,16 @@ class BatchAccumulatorTest {
                 maxBatchSize
             )
         ) {
-            acc.appendControlMessages((offset, epoch, buf) -> {
+            acc.appendControlMessages((offset, epoch, compression, buf) -> {
                 long now = 1234;
-                try (MemoryRecordsBuilder builder = controlRecordsBuilder(offset, epoch, now, buf)) {
+                try (MemoryRecordsBuilder builder = controlRecordsBuilder(
+                        offset,
+                        epoch,
+                        compression,
+                        now,
+                        buf
+                    )
+                ) {
                     builder.appendSnapshotHeaderMessage(
                         now,
                         new SnapshotHeaderRecord()
@@ -623,9 +535,16 @@ class BatchAccumulatorTest {
         Mockito.when(memoryPool.tryAllocate(maxBatchSize))
                 .thenReturn(buffer);
 
-        BatchAccumulator.MemoryRecordsCreator creator = (offset, epoch, buf) -> {
+        BatchAccumulator.MemoryRecordsCreator creator = (offset, epoch, compression, buf) -> {
             long now = 1234;
-            try (MemoryRecordsBuilder builder = controlRecordsBuilder(offset + 1, epoch, now, buf)) {
+            try (MemoryRecordsBuilder builder = controlRecordsBuilder(
+                    offset + 1,
+                    epoch,
+                    compression,
+                    now,
+                    buf
+                )
+            ) {
                 builder.appendSnapshotHeaderMessage(
                     now,
                     new SnapshotHeaderRecord()
@@ -659,9 +578,16 @@ class BatchAccumulatorTest {
         Mockito.when(memoryPool.tryAllocate(maxBatchSize))
                 .thenReturn(buffer);
 
-        BatchAccumulator.MemoryRecordsCreator creator = (offset, epoch, buf) -> {
+        BatchAccumulator.MemoryRecordsCreator creator = (offset, epoch, compression, buf) -> {
             long now = 1234;
-            try (MemoryRecordsBuilder builder = controlRecordsBuilder(offset, epoch + 1, now, buf)) {
+            try (MemoryRecordsBuilder builder = controlRecordsBuilder(
+                    offset,
+                    epoch + 1,
+                    compression,
+                    now,
+                    buf
+                )
+            ) {
                 builder.appendSnapshotHeaderMessage(
                     now,
                     new SnapshotHeaderRecord()
@@ -695,9 +621,16 @@ class BatchAccumulatorTest {
         Mockito.when(memoryPool.tryAllocate(maxBatchSize))
                 .thenReturn(buffer);
 
-        BatchAccumulator.MemoryRecordsCreator creator = (offset, epoch, buf) -> {
+        BatchAccumulator.MemoryRecordsCreator creator = (offset, epoch, compression, buf) -> {
             long now = 1234;
-            try (MemoryRecordsBuilder builder = controlRecordsBuilder(offset, epoch, now, buf)) {
+            try (MemoryRecordsBuilder builder = controlRecordsBuilder(
+                    offset,
+                    epoch,
+                    compression,
+                    now,
+                    buf
+                )
+            ) {
                 // Create a control batch without any records
                 return builder.build();
             }
@@ -714,16 +647,111 @@ class BatchAccumulatorTest {
         }
     }
 
+    @Test
+    void testDelayedDrain() {
+        int leaderEpoch = 17;
+        long baseOffset = 157;
+        int lingerMs = 50;
+        int maxBatchSize = 256;
+
+        Mockito.when(memoryPool.tryAllocate(maxBatchSize))
+            .thenReturn(ByteBuffer.allocate(maxBatchSize));
+
+        BatchAccumulator<String> acc = buildAccumulator(
+            leaderEpoch,
+            baseOffset,
+            lingerMs,
+            maxBatchSize
+        );
+
+        List<String> records = asList("a", "b", "c", "d", "e", "f", "g");
+        assertEquals(baseOffset, acc.append(leaderEpoch, records.subList(0, 1), true));
+        acc.forceDrain();
+        assertEquals(baseOffset + 2, acc.append(leaderEpoch, records.subList(1, 3), true));
+        acc.forceDrain();
+        assertEquals(baseOffset + 5, acc.append(leaderEpoch, records.subList(3, 6), true));
+        acc.forceDrain();
+
+        assertFalse(acc.needsDrain(time.milliseconds()));
+        assertNotEquals(0, acc.timeUntilDrain(time.milliseconds()));
+        assertEquals(0, acc.drain().size());
+
+        acc.allowDrain();
+        assertTrue(acc.needsDrain(time.milliseconds()));
+        assertEquals(0, acc.timeUntilDrain(time.milliseconds()));
+        assertEquals(3, acc.drain().size());
+    }
+
+    @Test
+    void testDelayedDrainAreReleased() {
+        int leaderEpoch = 17;
+        long baseOffset = 157;
+        int lingerMs = 50;
+        int maxBatchSize = 256;
+
+        ByteBuffer buffer = ByteBuffer.allocate(maxBatchSize);
+        Mockito.when(memoryPool.tryAllocate(maxBatchSize))
+            .thenReturn(buffer);
+
+        BatchAccumulator<String> acc = buildAccumulator(
+            leaderEpoch,
+            baseOffset,
+            lingerMs,
+            maxBatchSize
+        );
+
+        List<String> records = asList("a", "b", "c", "d", "e", "f", "g");
+        assertEquals(baseOffset, acc.append(leaderEpoch, records.subList(0, 1), true));
+        acc.forceDrain();
+
+        acc.close();
+        Mockito.verify(memoryPool).release(buffer);
+    }
+
+    @Test
+    public void testMaxNumberOfBatches() {
+        int leaderEpoch = 17;
+        long baseOffset = 157;
+        int lingerMs = 50;
+        int maxBatchSize = 256;
+        int maxNumberOfBatches = 2;
+
+        Mockito.when(memoryPool.tryAllocate(maxBatchSize))
+            .thenReturn(ByteBuffer.allocate(maxBatchSize));
+
+        StringSerde serde = Mockito.spy(new StringSerde());
+        BatchAccumulator<String> acc = new BatchAccumulator<>(
+            leaderEpoch,
+            baseOffset,
+            lingerMs,
+            maxBatchSize,
+            maxNumberOfBatches,
+            memoryPool,
+            time,
+            Compression.NONE,
+            serde
+        );
+
+        List<String> records = asList("a", "b", "c", "d", "e", "f", "g");
+        assertEquals(baseOffset, acc.append(leaderEpoch, records.subList(0, 1), true));
+        acc.forceDrain();
+        assertEquals(baseOffset + 2, acc.append(leaderEpoch, records.subList(1, 3), true));
+        acc.forceDrain();
+
+        assertThrows(IllegalStateException.class, () -> acc.append(leaderEpoch, records.subList(3, 6), true));
+    }
+
     private static MemoryRecordsBuilder controlRecordsBuilder(
         long baseOffset,
         int epoch,
+        Compression compression,
         long now,
         ByteBuffer buffer
     ) {
         return new MemoryRecordsBuilder(
             buffer,
             RecordBatch.CURRENT_MAGIC_VALUE,
-            Compression.NONE,
+            compression,
             TimestampType.CREATE_TIME,
             baseOffset,
             now,
