@@ -29,7 +29,7 @@ import org.apache.kafka.raft.QuorumState;
 import org.apache.kafka.raft.ReplicaKey;
 import org.apache.kafka.raft.VoterSet;
 import org.apache.kafka.raft.VoterSetTest;
-import org.apache.kafka.server.common.Features;
+import org.apache.kafka.server.common.Feature;
 import org.apache.kafka.server.common.KRaftVersion;
 
 import org.junit.jupiter.api.AfterEach;
@@ -45,6 +45,7 @@ import java.util.Random;
 import java.util.stream.IntStream;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNull;
 
 public class KafkaRaftMetricsTest {
 
@@ -86,13 +87,14 @@ public class KafkaRaftMetricsTest {
             localDirectoryId,
             mockPartitionState,
             voterSet.listeners(localId),
-            Features.KRAFT_VERSION.supportedVersionRange(),
+            Feature.KRAFT_VERSION.supportedVersionRange(),
             electionTimeoutMs,
             fetchTimeoutMs,
             new MockQuorumStateStore(),
             time,
             new LogContext("kafka-raft-metrics-test"),
-            random
+            random,
+            raftMetrics
         );
     }
 
@@ -116,6 +118,7 @@ public class KafkaRaftMetricsTest {
     public void shouldRecordVoterQuorumState(KRaftVersion kraftVersion) {
         boolean withDirectoryId = kraftVersion.featureLevel() > 0;
         Map<Integer, VoterSet.VoterNode> voterMap = VoterSetTest.voterMap(IntStream.of(1, 2), withDirectoryId);
+        Uuid voter2DirectoryId = voterMap.get(2).voterKey().directoryId().orElse(Uuid.ZERO_UUID);
         voterMap.put(
             localId,
             VoterSetTest.voterNode(
@@ -126,14 +129,28 @@ public class KafkaRaftMetricsTest {
             )
         );
         VoterSet voters = VoterSetTest.voterSet(voterMap);
+        raftMetrics = new KafkaRaftMetrics(metrics, "raft");
         QuorumState state = buildQuorumState(voters, kraftVersion);
 
+        raftMetrics.initialize(state);
         state.initialize(new OffsetAndEpoch(0L, 0));
-        raftMetrics = new KafkaRaftMetrics(metrics, "raft", state);
 
+        // unattached
         assertEquals("unattached", getMetric(metrics, "current-state").metricValue());
-        assertEquals((double) -1L, getMetric(metrics, "current-leader").metricValue());
-        assertEquals((double) -1L, getMetric(metrics, "current-vote").metricValue());
+        assertEquals((double) -1, getMetric(metrics, "current-leader").metricValue());
+        assertEquals((double) -1, getMetric(metrics, "current-vote").metricValue());
+        assertEquals(
+            Uuid.ZERO_UUID.toString(),
+            getMetric(metrics, "current-vote-directory-id").metricValue()
+        );
+        assertEquals((double) 0, getMetric(metrics, "current-epoch").metricValue());
+        assertEquals((double) -1, getMetric(metrics, "high-watermark").metricValue());
+
+        // prospective
+        state.transitionToProspective();
+        assertEquals("prospective", getMetric(metrics, "current-state").metricValue());
+        assertEquals((double) -1, getMetric(metrics, "current-leader").metricValue());
+        assertEquals((double) -1, getMetric(metrics, "current-vote").metricValue());
         assertEquals(
             Uuid.ZERO_UUID.toString(),
             getMetric(metrics, "current-vote-directory-id").metricValue()
@@ -141,17 +158,59 @@ public class KafkaRaftMetricsTest {
         assertEquals((double) 0, getMetric(metrics, "current-epoch").metricValue());
         assertEquals((double) -1L, getMetric(metrics, "high-watermark").metricValue());
 
+        // prospective with votedKey
+        state.prospectiveAddVotedState(0, ReplicaKey.of(1, ReplicaKey.NO_DIRECTORY_ID));
+        assertEquals("prospective-voted", getMetric(metrics, "current-state").metricValue());
+        assertEquals((double) -1, getMetric(metrics, "current-leader").metricValue());
+        assertEquals((double) 1, getMetric(metrics, "current-vote").metricValue());
+        assertEquals(
+            Uuid.ZERO_UUID.toString(),
+            getMetric(metrics, "current-vote-directory-id").metricValue()
+        );
+        assertEquals((double) 0, getMetric(metrics, "current-epoch").metricValue());
+        assertEquals((double) -1L, getMetric(metrics, "high-watermark").metricValue());
+
+        // follower with votedKey and leader
+        state.transitionToFollower(0, 2, voters.listeners(2));
+        assertEquals("follower", getMetric(metrics, "current-state").metricValue());
+        assertEquals((double) 2, getMetric(metrics, "current-leader").metricValue());
+        assertEquals((double) 1, getMetric(metrics, "current-vote").metricValue());
+        assertEquals(
+            Uuid.ZERO_UUID.toString(),
+            getMetric(metrics, "current-vote-directory-id").metricValue()
+        );
+        assertEquals((double) 0, getMetric(metrics, "current-epoch").metricValue());
+        assertEquals((double) -1L, getMetric(metrics, "high-watermark").metricValue());
+
+        // follower with updated HW
+        state.followerStateOrThrow().updateHighWatermark(OptionalLong.of(5L));
+        assertEquals((double) 5L, getMetric(metrics, "high-watermark").metricValue());
+
+        // prospective with votedKey and leader
+        state.transitionToProspective();
+        assertEquals("prospective-voted", getMetric(metrics, "current-state").metricValue());
+        assertEquals((double) 2, getMetric(metrics, "current-leader").metricValue());
+        assertEquals((double) 1, getMetric(metrics, "current-vote").metricValue());
+        assertEquals(
+            Uuid.ZERO_UUID.toString(),
+            getMetric(metrics, "current-vote-directory-id").metricValue()
+        );
+        assertEquals((double) 0, getMetric(metrics, "current-epoch").metricValue());
+        assertEquals((double) 5L, getMetric(metrics, "high-watermark").metricValue());
+
+        // candidate
         state.transitionToCandidate();
         assertEquals("candidate", getMetric(metrics, "current-state").metricValue());
-        assertEquals((double) -1L, getMetric(metrics, "current-leader").metricValue());
+        assertEquals((double) -1, getMetric(metrics, "current-leader").metricValue());
         assertEquals((double) localId, getMetric(metrics, "current-vote").metricValue());
         assertEquals(
             localDirectoryId.toString(),
             getMetric(metrics, "current-vote-directory-id").metricValue()
         );
         assertEquals((double) 1, getMetric(metrics, "current-epoch").metricValue());
-        assertEquals((double) -1L, getMetric(metrics, "high-watermark").metricValue());
+        assertEquals((double) 5L, getMetric(metrics, "high-watermark").metricValue());
 
+        // leader
         state.candidateStateOrThrow().recordGrantedVote(1);
         state.transitionToLeader(2L, accumulator);
         assertEquals("leader", getMetric(metrics, "current-state").metricValue());
@@ -162,16 +221,18 @@ public class KafkaRaftMetricsTest {
             getMetric(metrics, "current-vote-directory-id").metricValue()
         );
         assertEquals((double) 1, getMetric(metrics, "current-epoch").metricValue());
-        assertEquals((double) -1L, getMetric(metrics, "high-watermark").metricValue());
+        assertEquals((double) -1L, getMetric(metrics, "high-watermark").metricValue()); // todo, bug fix
 
-        state.leaderStateOrThrow().updateLocalState(new LogOffsetMetadata(5L), voters);
+        // leader with updated HW
+        state.leaderStateOrThrow().updateLocalState(new LogOffsetMetadata(10L), voters);
         state.leaderStateOrThrow().updateReplicaState(
             voterMap.get(1).voterKey(),
             0,
-            new LogOffsetMetadata(5L)
+            new LogOffsetMetadata(10L)
         );
-        assertEquals((double) 5L, getMetric(metrics, "high-watermark").metricValue());
+        assertEquals((double) 10L, getMetric(metrics, "high-watermark").metricValue());
 
+        // follower
         state.transitionToFollower(2, 1, voters.listeners(1));
         assertEquals("follower", getMetric(metrics, "current-state").metricValue());
         assertEquals((double) 1, getMetric(metrics, "current-leader").metricValue());
@@ -181,25 +242,25 @@ public class KafkaRaftMetricsTest {
             getMetric(metrics, "current-vote-directory-id").metricValue()
         );
         assertEquals((double) 2, getMetric(metrics, "current-epoch").metricValue());
-        assertEquals((double) 5L, getMetric(metrics, "high-watermark").metricValue());
-
-        state.followerStateOrThrow().updateHighWatermark(OptionalLong.of(10L));
         assertEquals((double) 10L, getMetric(metrics, "high-watermark").metricValue());
 
-        state.transitionToUnattachedVotedState(3, ReplicaKey.of(2, ReplicaKey.NO_DIRECTORY_ID));
-        assertEquals("voted", getMetric(metrics, "current-state").metricValue());
+        // unattached with votedKey
+        state.transitionToUnattached(3, OptionalInt.empty());
+        state.unattachedAddVotedState(3, ReplicaKey.of(2, voter2DirectoryId));
+        assertEquals("unattached-voted", getMetric(metrics, "current-state").metricValue());
         assertEquals((double) -1, getMetric(metrics, "current-leader").metricValue());
         assertEquals((double) 2, getMetric(metrics, "current-vote").metricValue());
         assertEquals(
-            Uuid.ZERO_UUID.toString(),
+            voter2DirectoryId.toString(),
             getMetric(metrics, "current-vote-directory-id").metricValue()
         );
         assertEquals((double) 3, getMetric(metrics, "current-epoch").metricValue());
         assertEquals((double) 10L, getMetric(metrics, "high-watermark").metricValue());
 
-        state.transitionToUnattached(4);
+        // unattached with leader without votedKey
+        state.transitionToUnattached(4, OptionalInt.of(1));
         assertEquals("unattached", getMetric(metrics, "current-state").metricValue());
-        assertEquals((double) -1, getMetric(metrics, "current-leader").metricValue());
+        assertEquals((double) 1, getMetric(metrics, "current-leader").metricValue());
         assertEquals((double) -1, getMetric(metrics, "current-vote").metricValue());
         assertEquals(
             Uuid.ZERO_UUID.toString(),
@@ -216,9 +277,10 @@ public class KafkaRaftMetricsTest {
         VoterSet voters = VoterSetTest.voterSet(
             VoterSetTest.voterMap(IntStream.of(1, 2, 3), withDirectoryId)
         );
+        raftMetrics = new KafkaRaftMetrics(metrics, "raft");
         QuorumState state = buildQuorumState(voters, kraftVersion);
+        raftMetrics.initialize(state);
         state.initialize(new OffsetAndEpoch(0L, 0));
-        raftMetrics = new KafkaRaftMetrics(metrics, "raft", state);
 
         assertEquals("unattached", getMetric(metrics, "current-state").metricValue());
         assertEquals((double) -1L, getMetric(metrics, "current-leader").metricValue());
@@ -244,7 +306,7 @@ public class KafkaRaftMetricsTest {
         state.followerStateOrThrow().updateHighWatermark(OptionalLong.of(10L));
         assertEquals((double) 10L, getMetric(metrics, "high-watermark").metricValue());
 
-        state.transitionToUnattached(4);
+        state.transitionToUnattached(4, OptionalInt.empty());
         assertEquals("unattached", getMetric(metrics, "current-state").metricValue());
         assertEquals((double) -1, getMetric(metrics, "current-leader").metricValue());
         assertEquals((double) -1, getMetric(metrics, "current-vote").metricValue());
@@ -259,9 +321,10 @@ public class KafkaRaftMetricsTest {
     @ParameterizedTest
     @EnumSource(value = KRaftVersion.class)
     public void shouldRecordLogEnd(KRaftVersion kraftVersion) {
+        raftMetrics = new KafkaRaftMetrics(metrics, "raft");
         QuorumState state = buildQuorumState(localStandaloneVoterSet(kraftVersion), kraftVersion);
+        raftMetrics.initialize(state);
         state.initialize(new OffsetAndEpoch(0L, 0));
-        raftMetrics = new KafkaRaftMetrics(metrics, "raft", state);
 
         assertEquals((double) 0L, getMetric(metrics, "log-end-offset").metricValue());
         assertEquals((double) 0, getMetric(metrics, "log-end-epoch").metricValue());
@@ -275,9 +338,10 @@ public class KafkaRaftMetricsTest {
     @ParameterizedTest
     @EnumSource(value = KRaftVersion.class)
     public void shouldRecordNumUnknownVoterConnections(KRaftVersion kraftVersion) {
+        raftMetrics = new KafkaRaftMetrics(metrics, "raft");
         QuorumState state = buildQuorumState(localStandaloneVoterSet(kraftVersion), kraftVersion);
+        raftMetrics.initialize(state);
         state.initialize(new OffsetAndEpoch(0L, 0));
-        raftMetrics = new KafkaRaftMetrics(metrics, "raft", state);
 
         assertEquals((double) 0, getMetric(metrics, "number-unknown-voter-connections").metricValue());
 
@@ -289,9 +353,10 @@ public class KafkaRaftMetricsTest {
     @ParameterizedTest
     @EnumSource(value = KRaftVersion.class)
     public void shouldRecordPollIdleRatio(KRaftVersion kraftVersion) {
+        raftMetrics = new KafkaRaftMetrics(metrics, "raft");
         QuorumState state = buildQuorumState(localStandaloneVoterSet(kraftVersion), kraftVersion);
+        raftMetrics.initialize(state);
         state.initialize(new OffsetAndEpoch(0L, 0));
-        raftMetrics = new KafkaRaftMetrics(metrics, "raft", state);
 
         // First recording is discarded (in order to align the interval of measurement)
         raftMetrics.updatePollStart(time.milliseconds());
@@ -362,9 +427,10 @@ public class KafkaRaftMetricsTest {
     @ParameterizedTest
     @EnumSource(value = KRaftVersion.class)
     public void shouldRecordLatency(KRaftVersion kraftVersion) {
+        raftMetrics = new KafkaRaftMetrics(metrics, "raft");
         QuorumState state = buildQuorumState(localStandaloneVoterSet(kraftVersion), kraftVersion);
+        raftMetrics.initialize(state);
         state.initialize(new OffsetAndEpoch(0L, 0));
-        raftMetrics = new KafkaRaftMetrics(metrics, "raft", state);
 
         raftMetrics.updateElectionStartMs(time.milliseconds());
         time.sleep(1000L);
@@ -394,9 +460,10 @@ public class KafkaRaftMetricsTest {
     @ParameterizedTest
     @EnumSource(value = KRaftVersion.class)
     public void shouldRecordRate(KRaftVersion kraftVersion) {
+        raftMetrics = new KafkaRaftMetrics(metrics, "raft");
         QuorumState state = buildQuorumState(localStandaloneVoterSet(kraftVersion), kraftVersion);
+        raftMetrics.initialize(state);
         state.initialize(new OffsetAndEpoch(0L, 0));
-        raftMetrics = new KafkaRaftMetrics(metrics, "raft", state);
 
         raftMetrics.updateAppendRecords(12);
         assertEquals(0.4, getMetric(metrics, "append-records-rate").metricValue());
@@ -411,6 +478,54 @@ public class KafkaRaftMetricsTest {
         assertEquals(2.4, getMetric(metrics, "fetch-records-rate").metricValue());
     }
 
+    @ParameterizedTest
+    @EnumSource(value = KRaftVersion.class)
+    public void testNumberOfVoters(KRaftVersion kraftVersion) {
+        raftMetrics = new KafkaRaftMetrics(metrics, "raft");
+        QuorumState state = buildQuorumState(localStandaloneVoterSet(kraftVersion), kraftVersion);
+        raftMetrics.initialize(state);
+        state.initialize(new OffsetAndEpoch(0L, 0));
+
+        assertEquals(0, getMetric(metrics, "number-of-voters").metricValue());
+
+        raftMetrics.updateNumVoters(3);
+        assertEquals(3, getMetric(metrics, "number-of-voters").metricValue());
+    }
+
+    @ParameterizedTest
+    @EnumSource(value = KRaftVersion.class)
+    public void testLeaderMetrics(KRaftVersion kraftVersion) {
+        raftMetrics = new KafkaRaftMetrics(metrics, "raft");
+        QuorumState state = buildQuorumState(localStandaloneVoterSet(kraftVersion), kraftVersion);
+        raftMetrics.initialize(state);
+        state.initialize(new OffsetAndEpoch(0L, 0));
+
+        assertNull(getMetric(metrics, "number-of-observers"));
+        assertNull(getMetric(metrics, "uncommitted-voter-change"));
+
+        raftMetrics.addLeaderMetrics();
+
+        assertEquals(0, getMetric(metrics, "number-of-observers").metricValue());
+        assertEquals(0, getMetric(metrics, "uncommitted-voter-change").metricValue());
+
+        raftMetrics.updateNumObservers(1);
+        raftMetrics.updateUncommittedVoterChange(true);
+
+        assertEquals(1, getMetric(metrics, "number-of-observers").metricValue());
+        assertEquals(1, getMetric(metrics, "uncommitted-voter-change").metricValue());
+
+        raftMetrics.removeLeaderMetrics();
+
+        assertNull(getMetric(metrics, "number-of-observers"));
+        assertNull(getMetric(metrics, "uncommitted-voter-change"));
+
+        // Check that these metrics are reset to default values when added back
+        raftMetrics.addLeaderMetrics();
+
+        assertEquals(0, getMetric(metrics, "number-of-observers").metricValue());
+        assertEquals(0, getMetric(metrics, "uncommitted-voter-change").metricValue());
+    }
+
     private KafkaMetric getMetric(final Metrics metrics, final String name) {
         return metrics.metrics().get(metrics.metricName(name, "raft-metrics"));
     }
