@@ -24,8 +24,6 @@ import org.junit.jupiter.api.Timeout;
 
 import java.util.Arrays;
 
-import static org.hamcrest.MatcherAssert.assertThat;
-import static org.hamcrest.Matchers.containsInAnyOrder;
 import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
@@ -65,10 +63,8 @@ public class TimelineHashSetTest {
         assertFalse(set.removeAll(Arrays.asList("d")));
         registry.getOrCreateSnapshot(2);
         assertTrue(set.removeAll(Arrays.asList("c")));
-        assertThat(TimelineHashMapTest.iteratorToList(set.iterator(2)),
-            containsInAnyOrder("a", "b", "c"));
-        assertThat(TimelineHashMapTest.iteratorToList(set.iterator()),
-            containsInAnyOrder("a", "b"));
+        assertTrue(TimelineHashMapTest.iteratorToList(set.iterator(2)).containsAll(Arrays.asList("a", "b", "c")));
+        assertTrue(TimelineHashMapTest.iteratorToList(set.iterator()).containsAll(Arrays.asList("a", "b")));
         assertEquals(2, set.size());
         assertEquals(3, set.size(2));
         set.clear();
@@ -101,14 +97,10 @@ public class TimelineHashSetTest {
         assertFalse(set.containsAll(Arrays.asList("abc", "def", "xyz")));
         assertTrue(set.removeAll(Arrays.asList("def", "ghi", "xyz")));
         registry.getOrCreateSnapshot(5);
-        assertThat(TimelineHashMapTest.iteratorToList(set.iterator(5)),
-            containsInAnyOrder("abc", "jkl"));
-        assertThat(TimelineHashMapTest.iteratorToList(set.iterator()),
-            containsInAnyOrder("abc", "jkl"));
+        assertTrue(TimelineHashMapTest.iteratorToList(set.iterator(5)).containsAll(Arrays.asList("abc", "jkl")));
+        assertTrue(TimelineHashMapTest.iteratorToList(set.iterator()).containsAll(Arrays.asList("abc", "jkl")));
         set.removeIf(e -> e.startsWith("a"));
-        assertThat(TimelineHashMapTest.iteratorToList(set.iterator()),
-            containsInAnyOrder("jkl"));
-        assertThat(TimelineHashMapTest.iteratorToList(set.iterator(5)),
-            containsInAnyOrder("abc", "jkl"));
+        assertTrue(TimelineHashMapTest.iteratorToList(set.iterator()).contains("jkl"));
+        assertTrue(TimelineHashMapTest.iteratorToList(set.iterator(5)).containsAll(Arrays.asList("abc", "jkl")));
     }
 }
