@@ -33,12 +33,14 @@ public class VerificationStateEntry {
     private final VerificationGuard verificationGuard;
     private int lowestSequence;
     private short epoch;
+    private boolean supportsEpochBump;
 
-    public VerificationStateEntry(long timestamp, int sequence, short epoch) {
+    public VerificationStateEntry(long timestamp, int sequence, short epoch, boolean supportsEpochBump) {
         this.timestamp = timestamp;
         this.verificationGuard = new VerificationGuard();
         this.lowestSequence = sequence;
         this.epoch = epoch;
+        this.supportsEpochBump = supportsEpochBump;
     }
 
     public long timestamp() {
@@ -57,6 +59,10 @@ public class VerificationStateEntry {
         return epoch;
     }
 
+    public boolean supportsEpochBump() {
+        return supportsEpochBump;
+    }
+
     /**
      * An OutOfOrderSequence loop can happen for any idempotent/transactional producer when a lower sequence fails with
      * a retriable error and a higher sequence is successfully written. The lower sequence will fail with
