@@ -19,6 +19,10 @@ package org.apache.kafka.streams.kstream;
 import org.apache.kafka.streams.processor.api.Processor;
 import org.apache.kafka.streams.processor.api.Record;
 
+/**
+ * @deprecated Since 4.0 and should not be used any longer.
+ */
+@Deprecated
 public class ForeachProcessor<K, V> implements Processor<K, V, Void, Void> {
 
     private final ForeachAction<K, V> action;
