@@ -16,8 +16,10 @@
  */
 package org.apache.kafka.streams.processor;
 
+import java.time.Duration;
+
 /**
- * Cancellable interface returned in {@link ProcessorContext#schedule(long, PunctuationType, Punctuator)}.
+ * Cancellable interface returned in {@link org.apache.kafka.streams.processor.api.ProcessorContext#schedule(Duration, PunctuationType, Punctuator)}.
  *
  * @see Punctuator
  */
