@@ -96,7 +96,6 @@ public class GlobalStateUpdateTask implements GlobalStateMaintainer {
         }
         initTopology();
         processorContext.initialize();
-        flushState();
         lastFlush = time.milliseconds();
         return stateMgr.changelogOffsets();
     }
@@ -139,9 +138,6 @@ public class GlobalStateUpdateTask implements GlobalStateMaintainer {
     }
 
     public void close(final boolean wipeStateStore) throws IOException {
-        if (!wipeStateStore) {
-            flushState();
-        }
         stateMgr.close();
         if (wipeStateStore) {
             try {
