@@ -16,22 +16,41 @@
  */
 package org.apache.kafka.streams.processor.internals;
 
+import java.util.Objects;
+
 public class Stamped<V> implements Comparable {
 
     public final V value;
     public final long timestamp;
 
-    public Stamped(V value, long timestamp) {
+    Stamped(final V value, final long timestamp) {
         this.value = value;
         this.timestamp = timestamp;
     }
 
     @Override
-    public int compareTo(Object other) {
-        long otherTimestamp = ((Stamped<?>) other).timestamp;
+    public int compareTo(final Object other) {
+        final long otherTimestamp = ((Stamped<?>) other).timestamp;
 
-        if (timestamp < otherTimestamp) return -1;
-        else if (timestamp > otherTimestamp) return 1;
+        if (timestamp < otherTimestamp) {
+            return -1;
+        } else if (timestamp > otherTimestamp) {
+            return 1;
+        }
         return 0;
     }
+
+    @Override
+    public boolean equals(final Object other) {
+        if (other == null || getClass() != other.getClass()) {
+            return false;
+        }
+        final long otherTimestamp = ((Stamped<?>) other).timestamp;
+        return timestamp == otherTimestamp;
+    }
+
+    @Override
+    public int hashCode() {
+        return Objects.hash(timestamp);
+    }
 }
