@@ -78,13 +78,10 @@ public class StreamsMetadataState {
     }
 
     public String toString(final String indent) {
-        final StringBuilder builder = new StringBuilder();
-        builder.append(indent).append("GlobalMetadata: ").append(allMetadata).append("\n");
-        builder.append(indent).append("GlobalStores: ").append(globalStores).append("\n");
-        builder.append(indent).append("My HostInfo: ").append(thisHost).append("\n");
-        builder.append(indent).append("PartitionsByTopic: ").append(partitionsByTopic).append("\n");
-
-        return builder.toString();
+        return indent + "GlobalMetadata: " + allMetadata + "\n" +
+                indent + "GlobalStores: " + globalStores + "\n" +
+                indent + "My HostInfo: " + thisHost + "\n" +
+                indent + "PartitionsByTopic: " + partitionsByTopic + "\n";
     }
 
     /**
@@ -102,7 +99,7 @@ public class StreamsMetadataState {
      *
      * @return all the {@link StreamsMetadata}s in a {@link KafkaStreams} application
      */
-    public Collection<StreamsMetadata> getAllMetadata() {
+    public Collection<StreamsMetadata> allMetadata() {
         return Collections.unmodifiableList(allMetadata);
     }
 
@@ -112,10 +109,10 @@ public class StreamsMetadataState {
      * @param storeName the storeName to find metadata for
      * @return A collection of {@link StreamsMetadata} that have the provided storeName
      */
-    public synchronized Collection<StreamsMetadata> getAllMetadataForStore(final String storeName) {
+    public synchronized Collection<StreamsMetadata> allMetadataForStore(final String storeName) {
         Objects.requireNonNull(storeName, "storeName cannot be null");
         if (topologyMetadata.hasNamedTopologies()) {
-            throw new IllegalArgumentException("Cannot invoke the getAllMetadataForStore(storeName) method when"
+            throw new IllegalArgumentException("Cannot invoke the allMetadataForStore(storeName) method when"
                                                    + "using named topologies, please use the overload that accepts"
                                                    + "a topologyName parameter to identify the correct store");
         }
@@ -149,7 +146,7 @@ public class StreamsMetadataState {
      * @param topologyName the storeName to find metadata for
      * @return A collection of {@link StreamsMetadata} that have the provided storeName
      */
-    public synchronized Collection<StreamsMetadata> getAllMetadataForStore(final String storeName, final String topologyName) {
+    public synchronized Collection<StreamsMetadata> allMetadataForStore(final String storeName, final String topologyName) {
         Objects.requireNonNull(storeName, "storeName cannot be null");
         Objects.requireNonNull(topologyName, "topologyName cannot be null");
 
@@ -173,7 +170,7 @@ public class StreamsMetadataState {
         return results;
     }
 
-    public synchronized Collection<StreamsMetadata> getAllMetadataForTopology(final String topologyName) {
+    public synchronized Collection<StreamsMetadata> allMetadataForTopology(final String topologyName) {
         Objects.requireNonNull(topologyName, "topologyName cannot be null");
 
         if (!isInitialized()) {
@@ -193,7 +190,7 @@ public class StreamsMetadataState {
     /**
      * Find the {@link KeyQueryMetadata}s for a given storeName and key. This method will use the
      * {@link DefaultStreamPartitioner} to locate the store. If a custom partitioner has been used
-     * please use {@link StreamsMetadataState#getKeyQueryMetadataForKey(String, Object, StreamPartitioner)} instead.
+     * please use {@link StreamsMetadataState#keyQueryMetadataForKey(String, Object, StreamPartitioner)} instead.
      *
      * Note: the key may not exist in the {@link org.apache.kafka.streams.processor.StateStore},
      * this method provides a way of finding which {@link KeyQueryMetadata} it would exist on.
@@ -206,32 +203,32 @@ public class StreamsMetadataState {
      * if streams is (re-)initializing or {@code null} if the corresponding topic cannot be found,
      * or null if no matching metadata could be found.
      */
-    public synchronized <K> KeyQueryMetadata getKeyQueryMetadataForKey(final String storeName,
-                                                                       final K key,
-                                                                       final Serializer<K> keySerializer) {
+    public synchronized <K> KeyQueryMetadata keyQueryMetadataForKey(final String storeName,
+                                                                    final K key,
+                                                                    final Serializer<K> keySerializer) {
         Objects.requireNonNull(keySerializer, "keySerializer can't be null");
         if (topologyMetadata.hasNamedTopologies()) {
-            throw new IllegalArgumentException("Cannot invoke the getKeyQueryMetadataForKey(storeName, key, keySerializer)"
+            throw new IllegalArgumentException("Cannot invoke the KeyQueryMetadataForKey(storeName, key, keySerializer)"
                                                    + "method when using named topologies, please use the overload that"
                                                    + "accepts a topologyName parameter to identify the correct store");
         }
-        return getKeyQueryMetadataForKey(storeName,
-                                         key,
-                                         new DefaultStreamPartitioner<>(keySerializer));
+        return keyQueryMetadataForKey(storeName,
+                                      key,
+                                      new DefaultStreamPartitioner<>(keySerializer));
     }
 
     /**
-     * See {@link StreamsMetadataState#getKeyQueryMetadataForKey(String, Object, Serializer)}
+     * See {@link StreamsMetadataState#keyQueryMetadataForKey(String, Object, Serializer)}
      */
-    public synchronized <K> KeyQueryMetadata getKeyQueryMetadataForKey(final String storeName,
-                                                                       final K key,
-                                                                       final Serializer<K> keySerializer,
-                                                                       final String topologyName) {
+    public synchronized <K> KeyQueryMetadata keyQueryMetadataForKey(final String storeName,
+                                                                    final K key,
+                                                                    final Serializer<K> keySerializer,
+                                                                    final String topologyName) {
         Objects.requireNonNull(keySerializer, "keySerializer can't be null");
-        return getKeyQueryMetadataForKey(storeName,
-                                         key,
-                                         new DefaultStreamPartitioner<>(keySerializer),
-                                         topologyName);
+        return keyQueryMetadataForKey(storeName,
+                                      key,
+                                      new DefaultStreamPartitioner<>(keySerializer),
+                                      topologyName);
     }
 
     /**
@@ -247,14 +244,14 @@ public class StreamsMetadataState {
      * @return The {@link KeyQueryMetadata} for the storeName and key or {@link KeyQueryMetadata#NOT_AVAILABLE}
      * if streams is (re-)initializing, or {@code null} if no matching metadata could be found.
      */
-    public synchronized <K> KeyQueryMetadata getKeyQueryMetadataForKey(final String storeName,
-                                                                       final K key,
-                                                                       final StreamPartitioner<? super K, ?> partitioner) {
+    public synchronized <K> KeyQueryMetadata keyQueryMetadataForKey(final String storeName,
+                                                                    final K key,
+                                                                    final StreamPartitioner<? super K, ?> partitioner) {
         Objects.requireNonNull(storeName, "storeName can't be null");
         Objects.requireNonNull(key, "key can't be null");
         Objects.requireNonNull(partitioner, "partitioner can't be null");
         if (topologyMetadata.hasNamedTopologies()) {
-            throw new IllegalArgumentException("Cannot invoke the getKeyQueryMetadataForKey(storeName, key, partitioner)"
+            throw new IllegalArgumentException("Cannot invoke the keyQueryMetadataForKey(storeName, key, partitioner)"
                                                    + "method when using named topologies, please use the overload that"
                                                    + "accepts a topologyName parameter to identify the correct store");
         }
@@ -276,16 +273,16 @@ public class StreamsMetadataState {
         if (sourceTopicsInfo == null) {
             return null;
         }
-        return getKeyQueryMetadataForKey(storeName, key, partitioner, sourceTopicsInfo);
+        return keyQueryMetadataForKey(storeName, key, partitioner, sourceTopicsInfo);
     }
 
     /**
-     * See {@link StreamsMetadataState#getKeyQueryMetadataForKey(String, Object, StreamPartitioner)}
+     * See {@link StreamsMetadataState#keyQueryMetadataForKey(String, Object, StreamPartitioner)}
      */
-    public synchronized <K> KeyQueryMetadata getKeyQueryMetadataForKey(final String storeName,
-                                                                       final K key,
-                                                                       final StreamPartitioner<? super K, ?> partitioner,
-                                                                       final String topologyName) {
+    public synchronized <K> KeyQueryMetadata keyQueryMetadataForKey(final String storeName,
+                                                                    final K key,
+                                                                    final StreamPartitioner<? super K, ?> partitioner,
+                                                                    final String topologyName) {
         Objects.requireNonNull(storeName, "storeName can't be null");
         Objects.requireNonNull(key, "key can't be null");
         Objects.requireNonNull(partitioner, "partitioner can't be null");
@@ -300,7 +297,7 @@ public class StreamsMetadataState {
         if (sourceTopicsInfo == null) {
             return null;
         }
-        return getKeyQueryMetadataForKey(storeName, key, partitioner, sourceTopicsInfo, topologyName);
+        return keyQueryMetadataForKey(storeName, key, partitioner, sourceTopicsInfo, topologyName);
     }
 
     /**
@@ -385,9 +382,6 @@ public class StreamsMetadataState {
                         activeStoresOnHost.addAll(getStoresOnHost(storeToSourceTopics, activePartitionsOnHost));
                     }
 
-                    // TODO KAFKA-13281: when we add support for global stores with named topologies we will
-                    //  need to add the global stores to the activeStoresOnHost set
-
                     final Set<TopicPartition> standbyPartitionsOnHost = new HashSet<>();
                     final Set<String> standbyStoresOnHost = new HashSet<>();
                     if (standbyPartitionHostMap.containsKey(hostInfo)) {
@@ -478,10 +472,10 @@ public class StreamsMetadataState {
         return maybeMulticastPartitions.get().iterator().next();
     };
 
-    private <K> KeyQueryMetadata getKeyQueryMetadataForKey(final String storeName,
-                                                           final K key,
-                                                           final StreamPartitioner<? super K, ?> partitioner,
-                                                           final SourceTopicsInfo sourceTopicsInfo) {
+    private <K> KeyQueryMetadata keyQueryMetadataForKey(final String storeName,
+                                                        final K key,
+                                                        final StreamPartitioner<? super K, ?> partitioner,
+                                                        final SourceTopicsInfo sourceTopicsInfo) {
 
         final Integer partition = getPartition.apply(partitioner.partitions(sourceTopicsInfo.topicWithMostPartitions, key, null, sourceTopicsInfo.maxPartitions));
         final Set<TopicPartition> matchingPartitions = new HashSet<>();
@@ -511,11 +505,11 @@ public class StreamsMetadataState {
         return new KeyQueryMetadata(activeHost, standbyHosts, partition);
     }
 
-    private <K> KeyQueryMetadata getKeyQueryMetadataForKey(final String storeName,
-                                                           final K key,
-                                                           final StreamPartitioner<? super K, ?> partitioner,
-                                                           final SourceTopicsInfo sourceTopicsInfo,
-                                                           final String topologyName) {
+    private <K> KeyQueryMetadata keyQueryMetadataForKey(final String storeName,
+                                                        final K key,
+                                                        final StreamPartitioner<? super K, ?> partitioner,
+                                                        final SourceTopicsInfo sourceTopicsInfo,
+                                                        final String topologyName) {
         Objects.requireNonNull(topologyName, "topology name must not be null");
         final Integer partition = getPartition.apply(partitioner.partitions(sourceTopicsInfo.topicWithMostPartitions, key, null, sourceTopicsInfo.maxPartitions));
         final Set<TopicPartition> matchingPartitions = new HashSet<>();
@@ -566,8 +560,8 @@ public class StreamsMetadataState {
         return partitionsByTopic != null && !partitionsByTopic.isEmpty() && localMetadata.get() != null;
     }
 
-    public String getStoreForChangelogTopic(final String topicName) {
-        return topologyMetadata.getStoreForChangelogTopic(topicName);
+    public String storeForChangelogTopic(final String topicName) {
+        return topologyMetadata.storeForChangelogTopic(topicName);
     }
 
     private class SourceTopicsInfo {
