@@ -20,7 +20,6 @@ import org.apache.kafka.common.serialization.Serializer;
 import org.apache.kafka.common.utils.Bytes;
 import org.apache.kafka.streams.KeyValue;
 import org.apache.kafka.streams.kstream.internals.Change;
-import org.apache.kafka.streams.processor.ProcessorContext;
 import org.apache.kafka.streams.processor.StateStore;
 import org.apache.kafka.streams.processor.StateStoreContext;
 import org.apache.kafka.streams.processor.api.Record;
@@ -61,7 +60,7 @@ public class CachingKeyValueStore
     private CacheFlushListener<byte[], byte[]> flushListener;
     private boolean sendOldValues;
     private String cacheName;
-    private InternalProcessorContext<?, ?> context;
+    private InternalProcessorContext<?, ?> internalContext;
     private Thread streamThread;
     private final ReadWriteLock lock = new ReentrantReadWriteLock();
     private final Position position;
@@ -95,23 +94,16 @@ public class CachingKeyValueStore
         this.timestampedSchema = timestampedSchema;
     }
 
-    @SuppressWarnings("deprecation") // This can be removed when it's removed from the interface.
-    @Deprecated
     @Override
-    public void init(final ProcessorContext context,
-                     final StateStore root) {
-        initInternal(asInternalProcessorContext(context));
-        super.init(context, root);
-        // save the stream thread as we only ever want to trigger a flush
-        // when the stream thread is the current thread.
-        streamThread = Thread.currentThread();
-    }
-
-    @Override
-    public void init(final StateStoreContext context,
-                     final StateStore root) {
-        initInternal(asInternalProcessorContext(context));
-        super.init(context, root);
+    public void init(final StateStoreContext stateStoreContext, final StateStore root) {
+        internalContext = asInternalProcessorContext(stateStoreContext);
+        cacheName = ThreadCache.nameSpaceFromTaskIdAndStore(internalContext.taskId().toString(), name());
+        internalContext.registerCacheFlushListener(cacheName, entries -> {
+            for (final ThreadCache.DirtyEntry entry : entries) {
+                putAndMaybeForward(entry, internalContext);
+            }
+        });
+        super.init(stateStoreContext, root);
         // save the stream thread as we only ever want to trigger a flush
         // when the stream thread is the current thread.
         streamThread = Thread.currentThread();
@@ -145,7 +137,7 @@ public class CachingKeyValueStore
             result = wrapped().query(query, positionBound, config);
 
         } else {
-            final int partition = context.taskId().partition();
+            final int partition = internalContext.taskId().partition();
             final Lock lock = this.lock.readLock();
             lock.lock();
             try {
@@ -190,8 +182,8 @@ public class CachingKeyValueStore
         final Bytes key = keyQuery.getKey();
 
         synchronized (mergedPosition) {
-            if (context.cache() != null) {
-                final LRUCacheEntry lruCacheEntry = context.cache().get(cacheName, key);
+            if (internalContext.cache() != null) {
+                final LRUCacheEntry lruCacheEntry = internalContext.cache().get(cacheName, key);
                 if (lruCacheEntry != null) {
                     final byte[] rawValue;
                     if (timestampedSchema && !WrappedStateStore.isTimestamped(wrapped()) && !StoreQueryUtils.isAdapter(wrapped())) {
@@ -213,16 +205,6 @@ public class CachingKeyValueStore
         return result;
     }
 
-    private void initInternal(final InternalProcessorContext<?, ?> context) {
-        this.context = context;
-        this.cacheName = ThreadCache.nameSpaceFromTaskIdAndStore(context.taskId().toString(), name());
-        this.context.registerCacheFlushListener(cacheName, entries -> {
-            for (final ThreadCache.DirtyEntry entry : entries) {
-                putAndMaybeForward(entry, context);
-            }
-        });
-    }
-
     private void putAndMaybeForward(final ThreadCache.DirtyEntry entry,
                                     final InternalProcessorContext<?, ?> context) {
         if (flushListener != null) {
@@ -285,19 +267,19 @@ public class CachingKeyValueStore
     private void putInternal(final Bytes key,
                              final byte[] value) {
         synchronized (position) {
-            context.cache().put(
+            internalContext.cache().put(
                 cacheName,
                 key,
                 new LRUCacheEntry(
                     value,
-                    context.headers(),
+                    internalContext.headers(),
                     true,
-                    context.offset(),
-                    context.timestamp(),
-                    context.partition(),
-                    context.topic()));
+                    internalContext.offset(),
+                    internalContext.timestamp(),
+                    internalContext.partition(),
+                    internalContext.topic()));
 
-            StoreQueryUtils.updatePosition(position, context);
+            StoreQueryUtils.updatePosition(position, internalContext);
         }
     }
 
@@ -374,8 +356,8 @@ public class CachingKeyValueStore
 
     private byte[] getInternal(final Bytes key) {
         LRUCacheEntry entry = null;
-        if (context.cache() != null) {
-            entry = context.cache().get(cacheName, key);
+        if (internalContext.cache() != null) {
+            entry = internalContext.cache().get(cacheName, key);
         }
         if (entry == null) {
             final byte[] rawValue = wrapped().get(key);
@@ -385,7 +367,7 @@ public class CachingKeyValueStore
             // only update the cache if this call is on the streamThread
             // as we don't want other threads to trigger an eviction/flush
             if (Thread.currentThread().equals(streamThread)) {
-                context.cache().put(cacheName, key, new LRUCacheEntry(rawValue));
+                internalContext.cache().put(cacheName, key, new LRUCacheEntry(rawValue));
             }
             return rawValue;
         } else {
@@ -406,7 +388,7 @@ public class CachingKeyValueStore
 
         validateStoreOpen();
         final KeyValueIterator<Bytes, byte[]> storeIterator = wrapped().range(from, to);
-        final ThreadCache.MemoryLRUCacheBytesIterator cacheIterator = context.cache().range(cacheName, from, to);
+        final ThreadCache.MemoryLRUCacheBytesIterator cacheIterator = internalContext.cache().range(cacheName, from, to);
         return new MergedSortedCacheKeyValueBytesStoreIterator(cacheIterator, storeIterator, true);
     }
 
@@ -423,16 +405,15 @@ public class CachingKeyValueStore
 
         validateStoreOpen();
         final KeyValueIterator<Bytes, byte[]> storeIterator = wrapped().reverseRange(from, to);
-        final ThreadCache.MemoryLRUCacheBytesIterator cacheIterator = context.cache().reverseRange(cacheName, from, to);
+        final ThreadCache.MemoryLRUCacheBytesIterator cacheIterator = internalContext.cache().reverseRange(cacheName, from, to);
         return new MergedSortedCacheKeyValueBytesStoreIterator(cacheIterator, storeIterator, false);
     }
 
     @Override
     public KeyValueIterator<Bytes, byte[]> all() {
         validateStoreOpen();
-        final KeyValueIterator<Bytes, byte[]> storeIterator =
-            new DelegatingPeekingKeyValueIterator<>(this.name(), wrapped().all());
-        final ThreadCache.MemoryLRUCacheBytesIterator cacheIterator = context.cache().all(cacheName);
+        final KeyValueIterator<Bytes, byte[]> storeIterator = wrapped().all();
+        final ThreadCache.MemoryLRUCacheBytesIterator cacheIterator = internalContext.cache().all(cacheName);
         return new MergedSortedCacheKeyValueBytesStoreIterator(cacheIterator, storeIterator, true);
     }
 
@@ -442,16 +423,15 @@ public class CachingKeyValueStore
         final KeyValueIterator<Bytes, byte[]> storeIterator = wrapped().prefixScan(prefix, prefixKeySerializer);
         final Bytes from = Bytes.wrap(prefixKeySerializer.serialize(null, prefix));
         final Bytes to = Bytes.increment(from);
-        final ThreadCache.MemoryLRUCacheBytesIterator cacheIterator = context.cache().range(cacheName, from, to, false);
+        final ThreadCache.MemoryLRUCacheBytesIterator cacheIterator = internalContext.cache().range(cacheName, from, to, false);
         return new MergedSortedCacheKeyValueBytesStoreIterator(cacheIterator, storeIterator, true);
     }
 
     @Override
     public KeyValueIterator<Bytes, byte[]> reverseAll() {
         validateStoreOpen();
-        final KeyValueIterator<Bytes, byte[]> storeIterator =
-            new DelegatingPeekingKeyValueIterator<>(this.name(), wrapped().reverseAll());
-        final ThreadCache.MemoryLRUCacheBytesIterator cacheIterator = context.cache().reverseAll(cacheName);
+        final KeyValueIterator<Bytes, byte[]> storeIterator = wrapped().reverseAll();
+        final ThreadCache.MemoryLRUCacheBytesIterator cacheIterator = internalContext.cache().reverseAll(cacheName);
         return new MergedSortedCacheKeyValueBytesStoreIterator(cacheIterator, storeIterator, false);
     }
 
@@ -473,7 +453,7 @@ public class CachingKeyValueStore
         lock.writeLock().lock();
         try {
             validateStoreOpen();
-            context.cache().flush(cacheName);
+            internalContext.cache().flush(cacheName);
             wrapped().flush();
         } finally {
             lock.writeLock().unlock();
@@ -486,7 +466,7 @@ public class CachingKeyValueStore
         lock.writeLock().lock();
         try {
             validateStoreOpen();
-            context.cache().flush(cacheName);
+            internalContext.cache().flush(cacheName);
         } finally {
             lock.writeLock().unlock();
         }
@@ -498,7 +478,7 @@ public class CachingKeyValueStore
         lock.writeLock().lock();
         try {
             validateStoreOpen();
-            context.cache().clear(cacheName);
+            internalContext.cache().clear(cacheName);
         } finally {
             lock.writeLock().unlock();
         }
@@ -509,8 +489,8 @@ public class CachingKeyValueStore
         lock.writeLock().lock();
         try {
             final LinkedList<RuntimeException> suppressed = executeAll(
-                () -> context.cache().flush(cacheName),
-                () -> context.cache().close(cacheName),
+                () -> internalContext.cache().flush(cacheName),
+                () -> internalContext.cache().close(cacheName),
                 wrapped()::close
             );
             if (!suppressed.isEmpty()) {
