@@ -30,19 +30,20 @@ import org.apache.kafka.streams.kstream.internals.KTableImpl;
 import org.apache.kafka.streams.kstream.internals.SessionWindow;
 import org.apache.kafka.streams.kstream.internals.TimeWindow;
 import org.apache.kafka.streams.processor.StateStore;
-import org.apache.kafka.streams.processor.StateStoreContext;
 import org.apache.kafka.streams.processor.api.MockProcessorContext;
 import org.apache.kafka.streams.processor.api.Processor;
 import org.apache.kafka.streams.processor.api.Record;
 import org.apache.kafka.streams.processor.internals.ProcessorNode;
+import org.apache.kafka.streams.state.StoreBuilder;
 import org.apache.kafka.streams.state.internals.InMemoryTimeOrderedKeyValueChangeBuffer;
-import org.apache.kafka.test.MockInternalNewProcessorContext;
+import org.apache.kafka.test.MockInternalProcessorContext;
 
 import org.hamcrest.BaseMatcher;
 import org.hamcrest.Description;
 import org.hamcrest.Matcher;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.extension.ExtendWith;
+import org.mockito.Mockito;
 import org.mockito.junit.jupiter.MockitoExtension;
 import org.mockito.junit.jupiter.MockitoSettings;
 import org.mockito.quality.Strictness;
@@ -76,7 +77,7 @@ public class KTableSuppressProcessorTest {
 
     private static class Harness<K, V> {
         private final Processor<K, Change<V>, K, Change<V>> processor;
-        private final MockInternalNewProcessorContext<K, Change<V>> context;
+        private final MockInternalProcessorContext<K, Change<V>> context;
 
 
         Harness(final Suppressed<K> suppressed,
@@ -92,12 +93,12 @@ public class KTableSuppressProcessorTest {
             @SuppressWarnings("unchecked")
             final KTableImpl<K, ?, V> parent = mock(KTableImpl.class);
             final Processor<K, Change<V>, K, Change<V>> processor =
-                new KTableSuppressProcessorSupplier<>((SuppressedInternal<K>) suppressed, storeName, parent).get();
+                new KTableSuppressProcessorSupplier<>((SuppressedInternal<K>) suppressed, mockBuilderWithName(storeName), parent).get();
 
-            final MockInternalNewProcessorContext<K, Change<V>> context = new MockInternalNewProcessorContext<>();
-            context.setCurrentNode(new ProcessorNode("testNode"));
+            final MockInternalProcessorContext<K, Change<V>> context = new MockInternalProcessorContext<>();
+            context.setCurrentNode(new ProcessorNode<>("testNode"));
 
-            buffer.init((StateStoreContext) context, buffer);
+            buffer.init(context, buffer);
             processor.init(context);
 
             this.processor = processor;
@@ -109,7 +110,7 @@ public class KTableSuppressProcessorTest {
     public void zeroTimeLimitShouldImmediatelyEmit() {
         final Harness<String, Long> harness =
             new Harness<>(untilTimeLimit(ZERO, unbounded()), String(), Long());
-        final MockInternalNewProcessorContext<String, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<String, Change<Long>> context = harness.context;
 
         final long timestamp = ARBITRARY_LONG;
         context.setRecordMetadata("", 0, 0L);
@@ -119,7 +120,7 @@ public class KTableSuppressProcessorTest {
         harness.processor.process(new Record<>(key, value, timestamp));
 
         assertThat(context.forwarded(), hasSize(1));
-        final MockProcessorContext.CapturedForward capturedForward = context.forwarded().get(0);
+        final MockProcessorContext.CapturedForward<?, ?> capturedForward = context.forwarded().get(0);
         assertThat(capturedForward.record(), is(new Record<>(key, value, timestamp)));
     }
 
@@ -127,7 +128,7 @@ public class KTableSuppressProcessorTest {
     public void windowedZeroTimeLimitShouldImmediatelyEmit() {
         final Harness<Windowed<String>, Long> harness =
             new Harness<>(untilTimeLimit(ZERO, unbounded()), timeWindowedSerdeFrom(String.class, 100L), Long());
-        final MockInternalNewProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
 
         final long timestamp = ARBITRARY_LONG;
         context.setRecordMetadata("", 0, 0L);
@@ -137,7 +138,7 @@ public class KTableSuppressProcessorTest {
         harness.processor.process(new Record<>(key, value, timestamp));
 
         assertThat(context.forwarded(), hasSize(1));
-        final MockProcessorContext.CapturedForward capturedForward = context.forwarded().get(0);
+        final MockProcessorContext.CapturedForward<?, ?> capturedForward = context.forwarded().get(0);
         assertThat(capturedForward.record(), is(new Record<>(key, value, timestamp)));
     }
 
@@ -145,7 +146,7 @@ public class KTableSuppressProcessorTest {
     public void intermediateSuppressionShouldBufferAndEmitLater() {
         final Harness<String, Long> harness =
             new Harness<>(untilTimeLimit(ofMillis(1), unbounded()), String(), Long());
-        final MockInternalNewProcessorContext<String, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<String, Change<Long>> context = harness.context;
 
         final long timestamp = 0L;
         context.setRecordMetadata("topic", 0, 0);
@@ -160,7 +161,7 @@ public class KTableSuppressProcessorTest {
         harness.processor.process(new Record<>("tick", new Change<>(null, null), 1L));
 
         assertThat(context.forwarded(), hasSize(1));
-        final MockProcessorContext.CapturedForward capturedForward = context.forwarded().get(0);
+        final MockProcessorContext.CapturedForward<?, ?> capturedForward = context.forwarded().get(0);
         assertThat(capturedForward.record(), is(new Record<>(key, value, timestamp)));
     }
 
@@ -168,7 +169,7 @@ public class KTableSuppressProcessorTest {
     public void finalResultsSuppressionShouldBufferAndEmitAtGraceExpiration() {
         final Harness<Windowed<String>, Long> harness =
             new Harness<>(finalResults(ofMillis(1L)), timeWindowedSerdeFrom(String.class, 1L), Long());
-        final MockInternalNewProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
 
         final long windowStart = 99L;
         final long recordTime = 99L;
@@ -199,7 +200,7 @@ public class KTableSuppressProcessorTest {
         harness.processor.process(new Record<>(new Windowed<>("dummyKey2", new TimeWindow(windowStart3, windowEnd3)), ARBITRARY_CHANGE, recordTime3));
 
         assertThat(context.forwarded(), hasSize(1));
-        final MockProcessorContext.CapturedForward capturedForward = context.forwarded().get(0);
+        final MockProcessorContext.CapturedForward<?, ?> capturedForward = context.forwarded().get(0);
         assertThat(capturedForward.record(), is(new Record<>(key, value, recordTime)));
     }
 
@@ -212,7 +213,7 @@ public class KTableSuppressProcessorTest {
     public void finalResultsWithZeroGraceShouldStillBufferUntilTheWindowEnd() {
         final Harness<Windowed<String>, Long> harness =
             new Harness<>(finalResults(ofMillis(0L)), timeWindowedSerdeFrom(String.class, 100L), Long());
-        final MockInternalNewProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
 
         // note the record is in the past, but the window end is in the future, so we still have to buffer,
         // even though the grace period is 0.
@@ -230,7 +231,7 @@ public class KTableSuppressProcessorTest {
         harness.processor.process(new Record<>(new Windowed<>("dummyKey", new TimeWindow(windowEnd, windowEnd + 100L)), ARBITRARY_CHANGE, windowEnd));
 
         assertThat(context.forwarded(), hasSize(1));
-        final MockProcessorContext.CapturedForward capturedForward = context.forwarded().get(0);
+        final MockProcessorContext.CapturedForward<?, ?> capturedForward = context.forwarded().get(0);
         assertThat(capturedForward.record(), is(new Record<>(key, value, timestamp)));
     }
 
@@ -238,7 +239,7 @@ public class KTableSuppressProcessorTest {
     public void finalResultsWithZeroGraceAtWindowEndShouldImmediatelyEmit() {
         final Harness<Windowed<String>, Long> harness =
             new Harness<>(finalResults(ofMillis(0L)), timeWindowedSerdeFrom(String.class, 100L), Long());
-        final MockInternalNewProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
 
         final long timestamp = 100L;
         context.setRecordMetadata("", 0, 0L);
@@ -248,7 +249,7 @@ public class KTableSuppressProcessorTest {
         harness.processor.process(new Record<>(key, value, timestamp));
 
         assertThat(context.forwarded(), hasSize(1));
-        final MockProcessorContext.CapturedForward capturedForward = context.forwarded().get(0);
+        final MockProcessorContext.CapturedForward<?, ?> capturedForward = context.forwarded().get(0);
         assertThat(capturedForward.record(), is(new Record<>(key, value, timestamp)));
     }
 
@@ -260,7 +261,7 @@ public class KTableSuppressProcessorTest {
     public void finalResultsShouldDropTombstonesForTimeWindows() {
         final Harness<Windowed<String>, Long> harness =
             new Harness<>(finalResults(ofMillis(0L)), timeWindowedSerdeFrom(String.class, 100L), Long());
-        final MockInternalNewProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
 
         final long timestamp = 100L;
         context.setRecordMetadata("", 0, 0L);
@@ -281,7 +282,7 @@ public class KTableSuppressProcessorTest {
     public void finalResultsShouldDropTombstonesForSessionWindows() {
         final Harness<Windowed<String>, Long> harness =
             new Harness<>(finalResults(ofMillis(0L)), sessionWindowedSerdeFrom(String.class), Long());
-        final MockInternalNewProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
 
         final long timestamp = 100L;
         context.setRecordMetadata("", 0, 0L);
@@ -301,7 +302,7 @@ public class KTableSuppressProcessorTest {
     public void suppressShouldNotDropTombstonesForTimeWindows() {
         final Harness<Windowed<String>, Long> harness =
             new Harness<>(untilTimeLimit(ofMillis(0), maxRecords(0)), timeWindowedSerdeFrom(String.class, 100L), Long());
-        final MockInternalNewProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
 
         final long timestamp = 100L;
         final Headers headers = new RecordHeaders().add("k", "v".getBytes(StandardCharsets.UTF_8));
@@ -313,7 +314,7 @@ public class KTableSuppressProcessorTest {
         harness.processor.process(new Record<>(key, value, timestamp));
 
         assertThat(context.forwarded(), hasSize(1));
-        final MockProcessorContext.CapturedForward capturedForward = context.forwarded().get(0);
+        final MockProcessorContext.CapturedForward<?, ?> capturedForward = context.forwarded().get(0);
         assertThat(capturedForward.record(), is(new Record<>(key, value, timestamp, headers)));
     }
 
@@ -326,7 +327,7 @@ public class KTableSuppressProcessorTest {
     public void suppressShouldNotDropTombstonesForSessionWindows() {
         final Harness<Windowed<String>, Long> harness =
             new Harness<>(untilTimeLimit(ofMillis(0), maxRecords(0)), sessionWindowedSerdeFrom(String.class), Long());
-        final MockInternalNewProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<Windowed<String>, Change<Long>> context = harness.context;
 
         final long timestamp = 100L;
         context.setRecordMetadata("", 0, 0L);
@@ -336,7 +337,7 @@ public class KTableSuppressProcessorTest {
         harness.processor.process(new Record<>(key, value, timestamp));
 
         assertThat(context.forwarded(), hasSize(1));
-        final MockProcessorContext.CapturedForward capturedForward = context.forwarded().get(0);
+        final MockProcessorContext.CapturedForward<?, ?> capturedForward = context.forwarded().get(0);
         assertThat(capturedForward.record(), is(new Record<>(key, value, timestamp)));
     }
 
@@ -349,7 +350,7 @@ public class KTableSuppressProcessorTest {
     public void suppressShouldNotDropTombstonesForKTable() {
         final Harness<String, Long> harness =
             new Harness<>(untilTimeLimit(ofMillis(0), maxRecords(0)), String(), Long());
-        final MockInternalNewProcessorContext<String, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<String, Change<Long>> context = harness.context;
 
         final long timestamp = 100L;
         context.setRecordMetadata("", 0, 0L);
@@ -359,7 +360,7 @@ public class KTableSuppressProcessorTest {
         harness.processor.process(new Record<>(key, value, timestamp));
 
         assertThat(context.forwarded(), hasSize(1));
-        final MockProcessorContext.CapturedForward capturedForward = context.forwarded().get(0);
+        final MockProcessorContext.CapturedForward<?, ?> capturedForward = context.forwarded().get(0);
         assertThat(capturedForward.record(), is(new Record<>(key, value, timestamp)));
     }
 
@@ -367,7 +368,7 @@ public class KTableSuppressProcessorTest {
     public void suppressShouldEmitWhenOverRecordCapacity() {
         final Harness<String, Long> harness =
             new Harness<>(untilTimeLimit(Duration.ofDays(100), maxRecords(1)), String(), Long());
-        final MockInternalNewProcessorContext<String, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<String, Change<Long>> context = harness.context;
 
         final long timestamp = 100L;
         context.setRecordMetadata("", 0, 0L);
@@ -381,7 +382,7 @@ public class KTableSuppressProcessorTest {
         harness.processor.process(new Record<>("dummyKey", value, timestamp + 1));
 
         assertThat(context.forwarded(), hasSize(1));
-        final MockProcessorContext.CapturedForward capturedForward = context.forwarded().get(0);
+        final MockProcessorContext.CapturedForward<?, ?> capturedForward = context.forwarded().get(0);
         assertThat(capturedForward.record(), is(new Record<>(key, value, timestamp)));
     }
 
@@ -389,7 +390,7 @@ public class KTableSuppressProcessorTest {
     public void suppressShouldEmitWhenOverByteCapacity() {
         final Harness<String, Long> harness =
             new Harness<>(untilTimeLimit(Duration.ofDays(100), maxBytes(60L)), String(), Long());
-        final MockInternalNewProcessorContext<String, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<String, Change<Long>> context = harness.context;
 
         final long timestamp = 100L;
         context.setRecordMetadata("", 0, 0L);
@@ -403,7 +404,7 @@ public class KTableSuppressProcessorTest {
         harness.processor.process(new Record<>("dummyKey", value, timestamp + 1));
 
         assertThat(context.forwarded(), hasSize(1));
-        final MockProcessorContext.CapturedForward capturedForward = context.forwarded().get(0);
+        final MockProcessorContext.CapturedForward<?, ?> capturedForward = context.forwarded().get(0);
         assertThat(capturedForward.record(), is(new Record<>(key, value, timestamp)));
     }
 
@@ -411,12 +412,12 @@ public class KTableSuppressProcessorTest {
     public void suppressShouldShutDownWhenOverRecordCapacity() {
         final Harness<String, Long> harness =
             new Harness<>(untilTimeLimit(Duration.ofDays(100), maxRecords(1).shutDownWhenFull()), String(), Long());
-        final MockInternalNewProcessorContext<String, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<String, Change<Long>> context = harness.context;
 
         final long timestamp = 100L;
         context.setRecordMetadata("", 0, 0L);
         context.setTimestamp(timestamp);
-        context.setCurrentNode(new ProcessorNode("testNode"));
+        context.setCurrentNode(new ProcessorNode<>("testNode"));
         final String key = "hey";
         final Change<Long> value = new Change<>(null, ARBITRARY_LONG);
         harness.processor.process(new Record<>(key, value, timestamp));
@@ -435,12 +436,12 @@ public class KTableSuppressProcessorTest {
     public void suppressShouldShutDownWhenOverByteCapacity() {
         final Harness<String, Long> harness =
             new Harness<>(untilTimeLimit(Duration.ofDays(100), maxBytes(60L).shutDownWhenFull()), String(), Long());
-        final MockInternalNewProcessorContext<String, Change<Long>> context = harness.context;
+        final MockInternalProcessorContext<String, Change<Long>> context = harness.context;
 
         final long timestamp = 100L;
         context.setRecordMetadata("", 0, 0L);
         context.setTimestamp(timestamp);
-        context.setCurrentNode(new ProcessorNode("testNode"));
+        context.setCurrentNode(new ProcessorNode<>("testNode"));
         final String key = "hey";
         final Change<Long> value = new Change<>(null, ARBITRARY_LONG);
         harness.processor.process(new Record<>(key, value, timestamp));
@@ -461,7 +462,7 @@ public class KTableSuppressProcessorTest {
     }
 
     private static <E> Matcher<Collection<E>> hasSize(final int i) {
-        return new BaseMatcher<Collection<E>>() {
+        return new BaseMatcher<>() {
             @Override
             public void describeTo(final Description description) {
                 description.appendText("a collection of size " + i);
@@ -480,6 +481,7 @@ public class KTableSuppressProcessorTest {
         };
     }
 
+    @SuppressWarnings("resource")
     private static <K> Serde<Windowed<K>> timeWindowedSerdeFrom(final Class<K> rawType, final long windowSize) {
         final Serde<K> kSerde = Serdes.serdeFrom(rawType);
         return new Serdes.WrapperSerde<>(
@@ -487,4 +489,10 @@ public class KTableSuppressProcessorTest {
             new TimeWindowedDeserializer<>(kSerde.deserializer(), windowSize)
         );
     }
-}
\ No newline at end of file
+
+    private static StoreBuilder<?> mockBuilderWithName(final String name) {
+        final StoreBuilder<?> builder = Mockito.mock(StoreBuilder.class);
+        Mockito.when(builder.name()).thenReturn(name);
+        return builder;
+    }
+}
