@@ -28,37 +28,39 @@ import org.apache.kafka.streams.processor.internals.InternalTopologyBuilder.Topi
 import org.apache.kafka.streams.processor.internals.assignment.CopartitionedTopicsEnforcer;
 import org.apache.kafka.streams.processor.internals.testutil.DummyStreamsConfig;
 
-import org.junit.Before;
-import org.junit.Test;
-import org.junit.runner.RunWith;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.extension.ExtendWith;
 import org.mockito.Mock;
-import org.mockito.junit.MockitoJUnitRunner;
+import org.mockito.junit.jupiter.MockitoExtension;
+import org.mockito.junit.jupiter.MockitoSettings;
+import org.mockito.quality.Strictness;
 
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
 import static org.apache.kafka.common.utils.Utils.mkEntry;
 import static org.apache.kafka.common.utils.Utils.mkMap;
-import static org.apache.kafka.common.utils.Utils.mkSet;
 import static org.apache.kafka.streams.processor.internals.TopologyMetadata.UNNAMED_TOPOLOGY;
 import static org.apache.kafka.streams.processor.internals.assignment.AssignmentTestUtils.SUBTOPOLOGY_0;
 import static org.apache.kafka.streams.processor.internals.assignment.AssignmentTestUtils.SUBTOPOLOGY_1;
-
 import static org.hamcrest.CoreMatchers.equalTo;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.CoreMatchers.notNullValue;
 import static org.hamcrest.CoreMatchers.nullValue;
 import static org.hamcrest.MatcherAssert.assertThat;
-import static org.junit.Assert.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.eq;
 import static org.mockito.Mockito.verify;
 import static org.mockito.Mockito.when;
 
-@RunWith(MockitoJUnitRunner.StrictStubs.class)
+@ExtendWith(MockitoExtension.class)
+@MockitoSettings(strictness = Strictness.STRICT_STUBS)
 public class RepartitionTopicsTest {
 
     private static final String SOURCE_TOPIC_NAME1 = "source1";
@@ -80,8 +82,8 @@ public class RepartitionTopicsTest {
     private static final RepartitionTopicConfig REPARTITION_TOPIC_CONFIG2 =
         new RepartitionTopicConfig(REPARTITION_TOPIC_NAME2, TOPIC_CONFIG2, 2, true);
     private static final TopicsInfo TOPICS_INFO1 = new TopicsInfo(
-        mkSet(REPARTITION_TOPIC_NAME1),
-        mkSet(SOURCE_TOPIC_NAME1, SOURCE_TOPIC_NAME2),
+        Set.of(REPARTITION_TOPIC_NAME1),
+        Set.of(SOURCE_TOPIC_NAME1, SOURCE_TOPIC_NAME2),
         mkMap(
             mkEntry(REPARTITION_TOPIC_NAME1, REPARTITION_TOPIC_CONFIG1),
             mkEntry(REPARTITION_TOPIC_NAME2, REPARTITION_TOPIC_CONFIG2)
@@ -89,8 +91,8 @@ public class RepartitionTopicsTest {
         Collections.emptyMap()
     );
     private static final TopicsInfo TOPICS_INFO2 = new TopicsInfo(
-        mkSet(SINK_TOPIC_NAME1),
-        mkSet(REPARTITION_TOPIC_NAME1),
+        Set.of(SINK_TOPIC_NAME1),
+        Set.of(REPARTITION_TOPIC_NAME1),
         mkMap(mkEntry(REPARTITION_TOPIC_NAME1, REPARTITION_TOPIC_CONFIG1)),
         Collections.emptyMap()
     );
@@ -105,7 +107,7 @@ public class RepartitionTopicsTest {
     @Mock
     Cluster clusterMetadata;
 
-    @Before
+    @BeforeEach
     public void setup() {
         when(internalTopologyBuilder.hasNamedTopology()).thenReturn(false);
         when(internalTopologyBuilder.topologyName()).thenReturn(null);
@@ -115,8 +117,8 @@ public class RepartitionTopicsTest {
     public void shouldSetupRepartitionTopics() {
         when(internalTopologyBuilder.subtopologyToTopicsInfo())
             .thenReturn(mkMap(mkEntry(SUBTOPOLOGY_0, TOPICS_INFO1), mkEntry(SUBTOPOLOGY_1, TOPICS_INFO2)));
-        final Set<String> coPartitionGroup1 = mkSet(SOURCE_TOPIC_NAME1, SOURCE_TOPIC_NAME2);
-        final Set<String> coPartitionGroup2 = mkSet(REPARTITION_TOPIC_NAME1, REPARTITION_TOPIC_NAME2);
+        final Set<String> coPartitionGroup1 = Set.of(SOURCE_TOPIC_NAME1, SOURCE_TOPIC_NAME2);
+        final Set<String> coPartitionGroup2 = Set.of(REPARTITION_TOPIC_NAME1, REPARTITION_TOPIC_NAME2);
         final List<Set<String>> coPartitionGroups = Arrays.asList(coPartitionGroup1, coPartitionGroup2);
         when(internalTopologyBuilder.copartitionGroups()).thenReturn(coPartitionGroups);
         when(internalTopicManager.makeReady(
@@ -125,7 +127,7 @@ public class RepartitionTopicsTest {
                 mkEntry(REPARTITION_TOPIC_NAME2, REPARTITION_TOPIC_CONFIG2)
             ))
         ).thenReturn(Collections.emptySet());
-        setupCluster();
+        setupCluster(false);
         final RepartitionTopics repartitionTopics = new RepartitionTopics(
             new TopologyMetadata(internalTopologyBuilder, config),
             internalTopicManager,
@@ -154,10 +156,10 @@ public class RepartitionTopicsTest {
 
     @Test
     public void shouldReturnMissingSourceTopics() {
-        final Set<String> missingSourceTopics = mkSet(SOURCE_TOPIC_NAME1);
+        final Set<String> missingSourceTopics = Set.of(SOURCE_TOPIC_NAME1);
         when(internalTopologyBuilder.subtopologyToTopicsInfo())
             .thenReturn(mkMap(mkEntry(SUBTOPOLOGY_0, TOPICS_INFO1), mkEntry(SUBTOPOLOGY_1, TOPICS_INFO2)));
-        setupClusterWithMissingTopics(missingSourceTopics);
+        setupClusterWithMissingTopics(missingSourceTopics, false);
         final RepartitionTopics repartitionTopics = new RepartitionTopics(
             new TopologyMetadata(internalTopologyBuilder, config),
             internalTopicManager,
@@ -186,7 +188,7 @@ public class RepartitionTopicsTest {
                 mkEntry(SUBTOPOLOGY_0, TOPICS_INFO1),
                 mkEntry(SUBTOPOLOGY_1, setupTopicInfoWithRepartitionTopicWithoutPartitionCount(repartitionTopicConfigWithoutPartitionCount))
             ));
-        setupCluster();
+        setupCluster(false);
         final RepartitionTopics repartitionTopics = new RepartitionTopics(
             new TopologyMetadata(internalTopologyBuilder, config),
             internalTopicManager,
@@ -206,8 +208,8 @@ public class RepartitionTopicsTest {
         final RepartitionTopicConfig repartitionTopicConfigWithoutPartitionCount =
             new RepartitionTopicConfig(REPARTITION_WITHOUT_PARTITION_COUNT, TOPIC_CONFIG5);
         final TopicsInfo topicsInfo = new TopicsInfo(
-            mkSet(REPARTITION_WITHOUT_PARTITION_COUNT),
-            mkSet(SOURCE_TOPIC_NAME1),
+            Set.of(REPARTITION_WITHOUT_PARTITION_COUNT),
+            Set.of(SOURCE_TOPIC_NAME1),
             mkMap(
                 mkEntry(REPARTITION_WITHOUT_PARTITION_COUNT, repartitionTopicConfigWithoutPartitionCount)
             ),
@@ -218,7 +220,7 @@ public class RepartitionTopicsTest {
                 mkEntry(SUBTOPOLOGY_0, topicsInfo),
                 mkEntry(SUBTOPOLOGY_1, setupTopicInfoWithRepartitionTopicWithoutPartitionCount(repartitionTopicConfigWithoutPartitionCount))
             ));
-        setupClusterWithMissingPartitionCounts(mkSet(SOURCE_TOPIC_NAME1));
+        setupClusterWithMissingPartitionCounts(Set.of(SOURCE_TOPIC_NAME1), true);
         final RepartitionTopics repartitionTopics = new RepartitionTopics(
             new TopologyMetadata(internalTopologyBuilder, config),
             internalTopicManager,
@@ -241,8 +243,8 @@ public class RepartitionTopicsTest {
         final RepartitionTopicConfig repartitionTopicConfigWithoutPartitionCount =
             new RepartitionTopicConfig(REPARTITION_WITHOUT_PARTITION_COUNT, TOPIC_CONFIG5);
         final TopicsInfo topicsInfo = new TopicsInfo(
-            mkSet(REPARTITION_TOPIC_NAME1, REPARTITION_WITHOUT_PARTITION_COUNT),
-            mkSet(SOURCE_TOPIC_NAME1, REPARTITION_TOPIC_NAME2),
+            Set.of(REPARTITION_TOPIC_NAME1, REPARTITION_WITHOUT_PARTITION_COUNT),
+            Set.of(SOURCE_TOPIC_NAME1, REPARTITION_TOPIC_NAME2),
             mkMap(
                 mkEntry(REPARTITION_TOPIC_NAME1, REPARTITION_TOPIC_CONFIG1),
                 mkEntry(REPARTITION_TOPIC_NAME2, REPARTITION_TOPIC_CONFIG2),
@@ -263,7 +265,7 @@ public class RepartitionTopicsTest {
                 mkEntry(REPARTITION_WITHOUT_PARTITION_COUNT, repartitionTopicConfigWithoutPartitionCount)
             ))
         ).thenReturn(Collections.emptySet());
-        setupCluster();
+        setupCluster(true);
         final RepartitionTopics repartitionTopics = new RepartitionTopics(
             new TopologyMetadata(internalTopologyBuilder, config),
             internalTopicManager,
@@ -295,8 +297,8 @@ public class RepartitionTopicsTest {
         final RepartitionTopicConfig repartitionTopicConfigWithoutPartitionCount =
             new RepartitionTopicConfig(REPARTITION_WITHOUT_PARTITION_COUNT, TOPIC_CONFIG5);
         final TopicsInfo topicsInfo = new TopicsInfo(
-            mkSet(REPARTITION_TOPIC_NAME2, REPARTITION_WITHOUT_PARTITION_COUNT),
-            mkSet(SOURCE_TOPIC_NAME1, REPARTITION_TOPIC_NAME1),
+            Set.of(REPARTITION_TOPIC_NAME2, REPARTITION_WITHOUT_PARTITION_COUNT),
+            Set.of(SOURCE_TOPIC_NAME1, REPARTITION_TOPIC_NAME1),
             mkMap(
                 mkEntry(REPARTITION_TOPIC_NAME1, REPARTITION_TOPIC_CONFIG1),
                 mkEntry(REPARTITION_TOPIC_NAME2, REPARTITION_TOPIC_CONFIG2),
@@ -317,7 +319,7 @@ public class RepartitionTopicsTest {
                 mkEntry(REPARTITION_WITHOUT_PARTITION_COUNT, repartitionTopicConfigWithoutPartitionCount)
             ))
         ).thenReturn(Collections.emptySet());
-        setupCluster();
+        setupCluster(true);
         final RepartitionTopics repartitionTopics = new RepartitionTopics(
             new TopologyMetadata(internalTopologyBuilder, config),
             internalTopicManager,
@@ -348,14 +350,14 @@ public class RepartitionTopicsTest {
     @Test
     public void shouldNotSetupRepartitionTopicsWhenTopologyDoesNotContainAnyRepartitionTopics() {
         final TopicsInfo topicsInfo = new TopicsInfo(
-            mkSet(SINK_TOPIC_NAME1),
-            mkSet(SOURCE_TOPIC_NAME1),
+            Set.of(SINK_TOPIC_NAME1),
+            Set.of(SOURCE_TOPIC_NAME1),
             Collections.emptyMap(),
             Collections.emptyMap()
         );
         when(internalTopologyBuilder.subtopologyToTopicsInfo())
             .thenReturn(mkMap(mkEntry(SUBTOPOLOGY_0, topicsInfo)));
-        setupCluster();
+        setupCluster(false);
         final RepartitionTopics repartitionTopics = new RepartitionTopics(
             new TopologyMetadata(internalTopologyBuilder, config),
             internalTopicManager,
@@ -387,21 +389,24 @@ public class RepartitionTopicsTest {
         assertThat(repartitionTopicInfo.replicas(), is(new Node[0]));
     }
 
-    private void setupCluster() {
-        setupClusterWithMissingTopicsAndMissingPartitionCounts(Collections.emptySet(), Collections.emptySet());
+    private void setupCluster(final boolean mockPartitionCount) {
+        setupClusterWithMissingTopicsAndMissingPartitionCounts(Collections.emptySet(), Collections.emptySet(), mockPartitionCount);
     }
 
-    private void setupClusterWithMissingTopics(final Set<String> missingTopics) {
-        setupClusterWithMissingTopicsAndMissingPartitionCounts(missingTopics, Collections.emptySet());
+    private void setupClusterWithMissingTopics(final Set<String> missingTopics, final boolean mockPartitionCount) {
+        setupClusterWithMissingTopicsAndMissingPartitionCounts(missingTopics, Collections.emptySet(), mockPartitionCount);
     }
 
-    private void setupClusterWithMissingPartitionCounts(final Set<String> topicsWithMissingPartitionCounts) {
-        setupClusterWithMissingTopicsAndMissingPartitionCounts(Collections.emptySet(), topicsWithMissingPartitionCounts);
+    private void setupClusterWithMissingPartitionCounts(final Set<String> topicsWithMissingPartitionCounts, final boolean mockPartitionCount) {
+        setupClusterWithMissingTopicsAndMissingPartitionCounts(Collections.emptySet(),
+            topicsWithMissingPartitionCounts,
+            mockPartitionCount);
     }
 
     private void setupClusterWithMissingTopicsAndMissingPartitionCounts(final Set<String> missingTopics,
-                                                                        final Set<String> topicsWithMissingPartitionCounts) {
-        final Set<String> topics = mkSet(
+                                                                        final Set<String> topicsWithMissingPartitionCounts,
+                                                                        final boolean mockPartitionCount) {
+        final Set<String> topics = new HashSet<>(List.of(
             SOURCE_TOPIC_NAME1,
             SOURCE_TOPIC_NAME2,
             SOURCE_TOPIC_NAME3,
@@ -412,17 +417,19 @@ public class RepartitionTopicsTest {
             REPARTITION_TOPIC_NAME3,
             REPARTITION_TOPIC_NAME4,
             SOME_OTHER_TOPIC
-        );
+        ));
         topics.removeAll(missingTopics);
         when(clusterMetadata.topics()).thenReturn(topics);
-        when(clusterMetadata.partitionCountForTopic(SOURCE_TOPIC_NAME1))
-            .thenReturn(topicsWithMissingPartitionCounts.contains(SOURCE_TOPIC_NAME1) ? null : 3);
+        if (mockPartitionCount) {
+            when(clusterMetadata.partitionCountForTopic(SOURCE_TOPIC_NAME1))
+                .thenReturn(topicsWithMissingPartitionCounts.contains(SOURCE_TOPIC_NAME1) ? null : 3);
+        }
     }
 
     private TopicsInfo setupTopicInfoWithRepartitionTopicWithoutPartitionCount(final RepartitionTopicConfig repartitionTopicConfigWithoutPartitionCount) {
         return new TopicsInfo(
-            mkSet(SINK_TOPIC_NAME2),
-            mkSet(REPARTITION_TOPIC_NAME1, REPARTITION_WITHOUT_PARTITION_COUNT),
+            Set.of(SINK_TOPIC_NAME2),
+            Set.of(REPARTITION_TOPIC_NAME1, REPARTITION_WITHOUT_PARTITION_COUNT),
             mkMap(
                 mkEntry(REPARTITION_TOPIC_NAME1, REPARTITION_TOPIC_CONFIG1),
                 mkEntry(REPARTITION_WITHOUT_PARTITION_COUNT, repartitionTopicConfigWithoutPartitionCount)
