@@ -47,7 +47,6 @@ import java.util.Set;
 import java.util.function.Function;
 import java.util.stream.Collectors;
 
-import static org.apache.kafka.common.utils.Utils.mkSet;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
@@ -110,12 +109,12 @@ public class StreamsMetadataStateTest {
         hostTwo = new HostInfo("host-two", 9090);
         hostThree = new HostInfo("host-three", 7070);
         hostToActivePartitions = new HashMap<>();
-        hostToActivePartitions.put(hostOne, mkSet(topic1P0, topic2P1, topic4P0));
-        hostToActivePartitions.put(hostTwo, mkSet(topic2P0, topic1P1));
+        hostToActivePartitions.put(hostOne, Set.of(topic1P0, topic2P1, topic4P0));
+        hostToActivePartitions.put(hostTwo, Set.of(topic2P0, topic1P1));
         hostToActivePartitions.put(hostThree, Collections.singleton(topic3P0));
         hostToStandbyPartitions = new HashMap<>();
-        hostToStandbyPartitions.put(hostThree, mkSet(topic1P0, topic2P1, topic4P0));
-        hostToStandbyPartitions.put(hostOne, mkSet(topic2P0, topic1P1));
+        hostToStandbyPartitions.put(hostThree, Set.of(topic1P0, topic2P1, topic4P0));
+        hostToStandbyPartitions.put(hostOne, Set.of(topic2P0, topic1P1));
         hostToStandbyPartitions.put(hostTwo, Collections.singleton(topic3P0));
 
         partitionInfos = new HashMap<>();
@@ -130,18 +129,11 @@ public class StreamsMetadataStateTest {
         topologyMetadata.buildAndRewriteTopology();
         metadataState = new StreamsMetadataState(topologyMetadata, hostOne, logContext);
         metadataState.onChange(hostToActivePartitions, hostToStandbyPartitions, partitionInfos);
-        partitioner = (topic, key, value, numPartitions) -> 1;
-        storeNames = mkSet("table-one", "table-two", "merged-table", globalTable);
+        partitioner = (topic, key, value, numPartitions) -> Optional.of(Collections.singleton(1));
+        storeNames = Set.of("table-one", "table-two", "merged-table", globalTable);
     }
 
     static class MultiValuedPartitioner implements StreamPartitioner<String, Object> {
-
-        @Override
-        @Deprecated
-        public Integer partition(final String topic, final String key, final Object value, final int numPartitions) {
-            return null;
-        }
-
         @Override
         public Optional<Set<Integer>> partitions(final String topic, final String key, final Object value, final int numPartitions) {
             final Set<Integer> partitions = new HashSet<>();
@@ -157,29 +149,29 @@ public class StreamsMetadataStateTest {
             new TopologyMetadata(TopologyWrapper.getInternalTopologyBuilder(builder.build()), new DummyStreamsConfig()),
             hostOne,
             logContext
-        ).getAllMetadataForStore("store");
+        ).allMetadataForStore("store");
         assertEquals(0, metadata.size());
     }
 
     @Test
     public void shouldGetAllStreamInstances() {
         final StreamsMetadata one = new StreamsMetadataImpl(hostOne,
-            mkSet(globalTable, "table-one", "table-two", "merged-table"),
-            mkSet(topic1P0, topic2P1, topic4P0),
-            mkSet("table-one", "table-two", "merged-table"),
-            mkSet(topic2P0, topic1P1));
+            Set.of(globalTable, "table-one", "table-two", "merged-table"),
+            Set.of(topic1P0, topic2P1, topic4P0),
+            Set.of("table-one", "table-two", "merged-table"),
+            Set.of(topic2P0, topic1P1));
         final StreamsMetadata two = new StreamsMetadataImpl(hostTwo,
-            mkSet(globalTable, "table-two", "table-one", "merged-table"),
-            mkSet(topic2P0, topic1P1),
-            mkSet("table-three"),
-            mkSet(topic3P0));
+            Set.of(globalTable, "table-two", "table-one", "merged-table"),
+            Set.of(topic2P0, topic1P1),
+            Set.of("table-three"),
+            Set.of(topic3P0));
         final StreamsMetadata three = new StreamsMetadataImpl(hostThree,
-            mkSet(globalTable, "table-three"),
+            Set.of(globalTable, "table-three"),
             Collections.singleton(topic3P0),
-            mkSet("table-one", "table-two", "merged-table"),
-            mkSet(topic1P0, topic2P1, topic4P0));
+            Set.of("table-one", "table-two", "merged-table"),
+            Set.of(topic1P0, topic2P1, topic4P0));
 
-        final Collection<StreamsMetadata> actual = metadataState.getAllMetadata();
+        final Collection<StreamsMetadata> actual = metadataState.allMetadata();
         assertEquals(3, actual.size());
         assertTrue(actual.contains(one), "expected " + actual + " to contain " + one);
         assertTrue(actual.contains(two), "expected " + actual + " to contain " + two);
@@ -192,30 +184,30 @@ public class StreamsMetadataStateTest {
 
         final TopicPartition tp5 = new TopicPartition("topic-five", 1);
         final HostInfo hostFour = new HostInfo("host-four", 8080);
-        hostToActivePartitions.put(hostFour, mkSet(tp5));
+        hostToActivePartitions.put(hostFour, Set.of(tp5));
 
         metadataState.onChange(hostToActivePartitions, Collections.emptyMap(),
             Collections.singletonMap(tp5, new PartitionInfo("topic-five", 1, null, null, null)));
 
         final StreamsMetadata expected = new StreamsMetadataImpl(hostFour, Collections.singleton(globalTable),
                 Collections.singleton(tp5), Collections.emptySet(), Collections.emptySet());
-        final Collection<StreamsMetadata> actual = metadataState.getAllMetadata();
+        final Collection<StreamsMetadata> actual = metadataState.allMetadata();
         assertTrue(actual.contains(expected), "expected " + actual + " to contain " + expected);
     }
 
     @Test
     public void shouldGetInstancesForStoreName() {
         final StreamsMetadata one = new StreamsMetadataImpl(hostOne,
-            mkSet(globalTable, "table-one", "table-two", "merged-table"),
-            mkSet(topic1P0, topic2P1, topic4P0),
-            mkSet("table-one", "table-two", "merged-table"),
-            mkSet(topic2P0, topic1P1));
+            Set.of(globalTable, "table-one", "table-two", "merged-table"),
+            Set.of(topic1P0, topic2P1, topic4P0),
+            Set.of("table-one", "table-two", "merged-table"),
+            Set.of(topic2P0, topic1P1));
         final StreamsMetadata two = new StreamsMetadataImpl(hostTwo,
-            mkSet(globalTable, "table-two", "table-one", "merged-table"),
-            mkSet(topic2P0, topic1P1),
-            mkSet("table-three"),
-            mkSet(topic3P0));
-        final Collection<StreamsMetadata> actual = metadataState.getAllMetadataForStore("table-one");
+            Set.of(globalTable, "table-two", "table-one", "merged-table"),
+            Set.of(topic2P0, topic1P1),
+            Set.of("table-three"),
+            Set.of(topic3P0));
+        final Collection<StreamsMetadata> actual = metadataState.allMetadataForStore("table-one");
         final Map<HostInfo, StreamsMetadata> actualAsMap = actual.stream()
             .collect(Collectors.toMap(StreamsMetadata::hostInfo, Function.identity()));
         assertEquals(3, actual.size());
@@ -227,25 +219,25 @@ public class StreamsMetadataStateTest {
 
     @Test
     public void shouldThrowIfStoreNameIsNullOnGetAllInstancesWithStore() {
-        assertThrows(NullPointerException.class, () -> metadataState.getAllMetadataForStore(null));
+        assertThrows(NullPointerException.class, () -> metadataState.allMetadataForStore(null));
     }
 
     @Test
     public void shouldReturnEmptyCollectionOnGetAllInstancesWithStoreWhenStoreDoesntExist() {
-        final Collection<StreamsMetadata> actual = metadataState.getAllMetadataForStore("not-a-store");
+        final Collection<StreamsMetadata> actual = metadataState.allMetadataForStore("not-a-store");
         assertTrue(actual.isEmpty());
     }
 
     @Test
     public void shouldGetInstanceWithKey() {
         final TopicPartition tp4 = new TopicPartition("topic-three", 1);
-        hostToActivePartitions.put(hostTwo, mkSet(topic2P0, tp4));
+        hostToActivePartitions.put(hostTwo, Set.of(topic2P0, tp4));
 
         metadataState.onChange(hostToActivePartitions, hostToStandbyPartitions,
             Collections.singletonMap(tp4, new PartitionInfo("topic-three", 1, null, null, null)));
 
-        final KeyQueryMetadata expected = new KeyQueryMetadata(hostThree, mkSet(hostTwo), 0);
-        final KeyQueryMetadata actual = metadataState.getKeyQueryMetadataForKey("table-three",
+        final KeyQueryMetadata expected = new KeyQueryMetadata(hostThree, Set.of(hostTwo), 0);
+        final KeyQueryMetadata actual = metadataState.keyQueryMetadataForKey("table-three",
                                                                     "the-key",
                                                                     Serdes.String().serializer());
         assertEquals(expected, actual);
@@ -254,14 +246,14 @@ public class StreamsMetadataStateTest {
     @Test
     public void shouldGetInstanceWithKeyAndCustomPartitioner() {
         final TopicPartition tp4 = new TopicPartition("topic-three", 1);
-        hostToActivePartitions.put(hostTwo, mkSet(topic2P0, tp4));
+        hostToActivePartitions.put(hostTwo, Set.of(topic2P0, tp4));
 
         metadataState.onChange(hostToActivePartitions, hostToStandbyPartitions,
             Collections.singletonMap(tp4, new PartitionInfo("topic-three", 1, null, null, null)));
 
         final KeyQueryMetadata expected = new KeyQueryMetadata(hostTwo, Collections.emptySet(), 1);
 
-        final KeyQueryMetadata actual = metadataState.getKeyQueryMetadataForKey("table-three",
+        final KeyQueryMetadata actual = metadataState.keyQueryMetadataForKey("table-three",
                 "the-key",
                 partitioner);
         assertEquals(expected, actual);
@@ -271,13 +263,13 @@ public class StreamsMetadataStateTest {
     @Test
     public void shouldFailWhenIqQueriedWithCustomPartitionerReturningMultiplePartitions() {
         final TopicPartition tp4 = new TopicPartition("topic-three", 1);
-        hostToActivePartitions.put(hostTwo, mkSet(topic2P0, tp4));
+        hostToActivePartitions.put(hostTwo, Set.of(topic2P0, tp4));
 
         metadataState.onChange(hostToActivePartitions, hostToStandbyPartitions,
                 Collections.singletonMap(tp4, new PartitionInfo("topic-three", 1, null, null, null)));
 
 
-        assertThrows(IllegalArgumentException.class, () -> metadataState.getKeyQueryMetadataForKey("table-three",
+        assertThrows(IllegalArgumentException.class, () -> metadataState.keyQueryMetadataForKey("table-three",
                 "the-key",
                 new MultiValuedPartitioner()));
     }
@@ -285,29 +277,29 @@ public class StreamsMetadataStateTest {
     @Test
     public void shouldReturnNotAvailableWhenClusterIsEmpty() {
         metadataState.onChange(Collections.emptyMap(), Collections.emptyMap(), Collections.emptyMap());
-        final KeyQueryMetadata result = metadataState.getKeyQueryMetadataForKey("table-one", "a", Serdes.String().serializer());
+        final KeyQueryMetadata result = metadataState.keyQueryMetadataForKey("table-one", "a", Serdes.String().serializer());
         assertEquals(KeyQueryMetadata.NOT_AVAILABLE, result);
     }
 
     @Test
     public void shouldGetInstanceWithKeyWithMergedStreams() {
         final TopicPartition topic2P2 = new TopicPartition("topic-two", 2);
-        hostToActivePartitions.put(hostTwo, mkSet(topic2P0, topic1P1, topic2P2));
-        hostToStandbyPartitions.put(hostOne, mkSet(topic2P0, topic1P1, topic2P2));
+        hostToActivePartitions.put(hostTwo, Set.of(topic2P0, topic1P1, topic2P2));
+        hostToStandbyPartitions.put(hostOne, Set.of(topic2P0, topic1P1, topic2P2));
         metadataState.onChange(hostToActivePartitions, hostToStandbyPartitions,
                 Collections.singletonMap(topic2P2, new PartitionInfo("topic-two", 2, null, null, null)));
 
-        final KeyQueryMetadata expected = new KeyQueryMetadata(hostTwo, mkSet(hostOne), 2);
+        final KeyQueryMetadata expected = new KeyQueryMetadata(hostTwo, Set.of(hostOne), 2);
 
-        final KeyQueryMetadata actual = metadataState.getKeyQueryMetadataForKey("merged-table",  "the-key",
-            (topic, key, value, numPartitions) -> 2);
+        final KeyQueryMetadata actual = metadataState.keyQueryMetadataForKey("merged-table",  "the-key",
+            (topic, key, value, numPartitions) -> Optional.of(Collections.singleton(2)));
 
         assertEquals(expected, actual);
     }
 
     @Test
     public void shouldReturnNullOnGetWithKeyWhenStoreDoesntExist() {
-        final KeyQueryMetadata actual = metadataState.getKeyQueryMetadataForKey("not-a-store",
+        final KeyQueryMetadata actual = metadataState.keyQueryMetadataForKey("not-a-store",
                 "key",
                 Serdes.String().serializer());
         assertNull(actual);
@@ -315,28 +307,28 @@ public class StreamsMetadataStateTest {
 
     @Test
     public void shouldThrowWhenKeyIsNull() {
-        assertThrows(NullPointerException.class, () -> metadataState.getKeyQueryMetadataForKey("table-three", null, Serdes.String().serializer()));
+        assertThrows(NullPointerException.class, () -> metadataState.keyQueryMetadataForKey("table-three", null, Serdes.String().serializer()));
     }
 
     @Test
     public void shouldThrowWhenSerializerIsNull() {
-        assertThrows(NullPointerException.class, () -> metadataState.getKeyQueryMetadataForKey("table-three", "key", (Serializer<Object>) null));
+        assertThrows(NullPointerException.class, () -> metadataState.keyQueryMetadataForKey("table-three", "key", (Serializer<Object>) null));
     }
 
     @Test
     public void shouldThrowIfStoreNameIsNull() {
-        assertThrows(NullPointerException.class, () -> metadataState.getKeyQueryMetadataForKey(null, "key", Serdes.String().serializer()));
+        assertThrows(NullPointerException.class, () -> metadataState.keyQueryMetadataForKey(null, "key", Serdes.String().serializer()));
     }
 
     @SuppressWarnings("unchecked")
     @Test
     public void shouldThrowIfStreamPartitionerIsNull() {
-        assertThrows(NullPointerException.class, () -> metadataState.getKeyQueryMetadataForKey(null, "key", (StreamPartitioner) null));
+        assertThrows(NullPointerException.class, () -> metadataState.keyQueryMetadataForKey(null, "key", (StreamPartitioner) null));
     }
 
     @Test
     public void shouldHaveGlobalStoreInAllMetadata() {
-        final Collection<StreamsMetadata> metadata = metadataState.getAllMetadataForStore(globalTable);
+        final Collection<StreamsMetadata> metadata = metadataState.allMetadataForStore(globalTable);
         assertEquals(3, metadata.size());
         for (final StreamsMetadata streamsMetadata : metadata) {
             assertTrue(streamsMetadata.stateStoreNames().contains(globalTable));
@@ -355,7 +347,7 @@ public class StreamsMetadataStateTest {
 
     @Test
     public void shouldGetQueryMetadataForGlobalStoreWithKey() {
-        final KeyQueryMetadata metadata = metadataState.getKeyQueryMetadataForKey(globalTable, "key", Serdes.String().serializer());
+        final KeyQueryMetadata metadata = metadataState.keyQueryMetadataForKey(globalTable, "key", Serdes.String().serializer());
         assertEquals(hostOne, metadata.activeHost());
         assertTrue(metadata.standbyHosts().isEmpty());
     }
@@ -368,12 +360,12 @@ public class StreamsMetadataStateTest {
             logContext
         );
         streamsMetadataState.onChange(hostToActivePartitions, hostToStandbyPartitions, partitionInfos);
-        assertNotNull(streamsMetadataState.getKeyQueryMetadataForKey(globalTable, "key", Serdes.String().serializer()));
+        assertNotNull(streamsMetadataState.keyQueryMetadataForKey(globalTable, "key", Serdes.String().serializer()));
     }
 
     @Test
     public void shouldGetQueryMetadataForGlobalStoreWithKeyAndPartitioner() {
-        final KeyQueryMetadata metadata = metadataState.getKeyQueryMetadataForKey(globalTable, "key", partitioner);
+        final KeyQueryMetadata metadata = metadataState.keyQueryMetadataForKey(globalTable, "key", partitioner);
         assertEquals(hostOne, metadata.activeHost());
         assertTrue(metadata.standbyHosts().isEmpty());
     }
@@ -386,12 +378,12 @@ public class StreamsMetadataStateTest {
             logContext
         );
         streamsMetadataState.onChange(hostToActivePartitions, hostToStandbyPartitions, partitionInfos);
-        assertNotNull(streamsMetadataState.getKeyQueryMetadataForKey(globalTable, "key", partitioner));
+        assertNotNull(streamsMetadataState.keyQueryMetadataForKey(globalTable, "key", partitioner));
     }
 
     @Test
     public void shouldReturnAllMetadataThatRemainsValidAfterChange() {
-        final Collection<StreamsMetadata> allMetadata = metadataState.getAllMetadata();
+        final Collection<StreamsMetadata> allMetadata = metadataState.allMetadata();
         final Collection<StreamsMetadata> copy = new ArrayList<>(allMetadata);
         assertFalse(allMetadata.isEmpty(), "invalid test");
         metadataState.onChange(Collections.emptyMap(), Collections.emptyMap(), partitionInfos);
@@ -400,7 +392,7 @@ public class StreamsMetadataStateTest {
 
     @Test
     public void shouldNotReturnMutableReferenceToInternalAllMetadataCollection() {
-        final Collection<StreamsMetadata> allMetadata = metadataState.getAllMetadata();
+        final Collection<StreamsMetadata> allMetadata = metadataState.allMetadata();
         assertFalse(allMetadata.isEmpty(), "invalid test");
 
         try {
@@ -410,6 +402,6 @@ public class StreamsMetadataStateTest {
             // Or should fail.
         }
 
-        assertFalse(metadataState.getAllMetadata().isEmpty(), "encapsulation broken");
+        assertFalse(metadataState.allMetadata().isEmpty(), "encapsulation broken");
     }
 }
