@@ -29,7 +29,6 @@ import java.util.Set;
 
 import static org.apache.kafka.common.utils.Utils.mkEntry;
 import static org.apache.kafka.common.utils.Utils.mkMap;
-import static org.apache.kafka.common.utils.Utils.mkSet;
 import static org.apache.kafka.test.StreamsTestUtils.TaskBuilder.standbyTask;
 import static org.apache.kafka.test.StreamsTestUtils.TaskBuilder.statefulTask;
 import static org.apache.kafka.test.StreamsTestUtils.TaskBuilder.statelessTask;
@@ -54,11 +53,11 @@ public class TasksTest {
 
     @Test
     public void shouldCheckStateWhenRemoveTask() {
-        final StreamTask closedTask = statefulTask(TASK_0_0, mkSet(TOPIC_PARTITION_A_0)).inState(State.CLOSED).build();
-        final StandbyTask suspendedTask = standbyTask(TASK_0_1, mkSet(TOPIC_PARTITION_A_1)).inState(State.SUSPENDED).build();
+        final StreamTask closedTask = statefulTask(TASK_0_0, Set.of(TOPIC_PARTITION_A_0)).inState(State.CLOSED).build();
+        final StandbyTask suspendedTask = standbyTask(TASK_0_1, Set.of(TOPIC_PARTITION_A_1)).inState(State.SUSPENDED).build();
         final StreamTask runningTask = statelessTask(TASK_1_0).inState(State.RUNNING).build();
 
-        tasks.addActiveTasks(mkSet(closedTask, runningTask));
+        tasks.addActiveTasks(Set.of(closedTask, runningTask));
         tasks.addStandbyTasks(Collections.singletonList(suspendedTask));
 
         assertDoesNotThrow(() -> tasks.removeTask(closedTask));
@@ -68,21 +67,21 @@ public class TasksTest {
 
     @Test
     public void shouldKeepAddedTasks() {
-        final StreamTask statefulTask = statefulTask(TASK_0_0, mkSet(TOPIC_PARTITION_A_0)).build();
-        final StandbyTask standbyTask = standbyTask(TASK_0_1, mkSet(TOPIC_PARTITION_A_1)).build();
+        final StreamTask statefulTask = statefulTask(TASK_0_0, Set.of(TOPIC_PARTITION_A_0)).build();
+        final StandbyTask standbyTask = standbyTask(TASK_0_1, Set.of(TOPIC_PARTITION_A_1)).build();
         final StreamTask statelessTask = statelessTask(TASK_1_0).build();
 
-        tasks.addActiveTasks(mkSet(statefulTask, statelessTask));
+        tasks.addActiveTasks(Set.of(statefulTask, statelessTask));
         tasks.addStandbyTasks(Collections.singletonList(standbyTask));
 
         assertEquals(statefulTask, tasks.task(statefulTask.id()));
         assertEquals(statelessTask, tasks.task(statelessTask.id()));
         assertEquals(standbyTask, tasks.task(standbyTask.id()));
 
-        assertEquals(mkSet(statefulTask, statelessTask), new HashSet<>(tasks.activeTasks()));
-        assertEquals(mkSet(statefulTask, statelessTask, standbyTask), tasks.allTasks());
-        assertEquals(mkSet(statefulTask, standbyTask), tasks.tasks(mkSet(statefulTask.id(), standbyTask.id())));
-        assertEquals(mkSet(statefulTask.id(), statelessTask.id(), standbyTask.id()), tasks.allTaskIds());
+        assertEquals(Set.of(statefulTask, statelessTask), new HashSet<>(tasks.activeTasks()));
+        assertEquals(Set.of(statefulTask, statelessTask, standbyTask), tasks.allTasks());
+        assertEquals(Set.of(statefulTask, standbyTask), tasks.tasks(Set.of(statefulTask.id(), standbyTask.id())));
+        assertEquals(Set.of(statefulTask.id(), statelessTask.id(), standbyTask.id()), tasks.allTaskIds());
         assertEquals(
             mkMap(
                 mkEntry(statefulTask.id(), statefulTask),
@@ -98,44 +97,44 @@ public class TasksTest {
     @Test
     public void shouldDrainPendingTasksToCreate() {
         tasks.addPendingActiveTasksToCreate(mkMap(
-            mkEntry(new TaskId(0, 0, "A"), mkSet(TOPIC_PARTITION_A_0)),
-            mkEntry(new TaskId(0, 1, "A"), mkSet(TOPIC_PARTITION_A_1)),
-            mkEntry(new TaskId(0, 0, "B"), mkSet(TOPIC_PARTITION_B_0)),
-            mkEntry(new TaskId(0, 1, "B"), mkSet(TOPIC_PARTITION_B_1))
+            mkEntry(new TaskId(0, 0, "A"), Set.of(TOPIC_PARTITION_A_0)),
+            mkEntry(new TaskId(0, 1, "A"), Set.of(TOPIC_PARTITION_A_1)),
+            mkEntry(new TaskId(0, 0, "B"), Set.of(TOPIC_PARTITION_B_0)),
+            mkEntry(new TaskId(0, 1, "B"), Set.of(TOPIC_PARTITION_B_1))
         ));
 
         tasks.addPendingStandbyTasksToCreate(mkMap(
-            mkEntry(new TaskId(0, 0, "A"), mkSet(TOPIC_PARTITION_A_0)),
-            mkEntry(new TaskId(0, 1, "A"), mkSet(TOPIC_PARTITION_A_1)),
-            mkEntry(new TaskId(0, 0, "B"), mkSet(TOPIC_PARTITION_B_0)),
-            mkEntry(new TaskId(0, 1, "B"), mkSet(TOPIC_PARTITION_B_1))
+            mkEntry(new TaskId(0, 0, "A"), Set.of(TOPIC_PARTITION_A_0)),
+            mkEntry(new TaskId(0, 1, "A"), Set.of(TOPIC_PARTITION_A_1)),
+            mkEntry(new TaskId(0, 0, "B"), Set.of(TOPIC_PARTITION_B_0)),
+            mkEntry(new TaskId(0, 1, "B"), Set.of(TOPIC_PARTITION_B_1))
         ));
 
         assertEquals(mkMap(
-            mkEntry(new TaskId(0, 0, "A"), mkSet(TOPIC_PARTITION_A_0)),
-            mkEntry(new TaskId(0, 1, "A"), mkSet(TOPIC_PARTITION_A_1))
-        ), tasks.drainPendingActiveTasksForTopologies(mkSet("A")));
+            mkEntry(new TaskId(0, 0, "A"), Set.of(TOPIC_PARTITION_A_0)),
+            mkEntry(new TaskId(0, 1, "A"), Set.of(TOPIC_PARTITION_A_1))
+        ), tasks.drainPendingActiveTasksForTopologies(Set.of("A")));
 
         assertEquals(mkMap(
-            mkEntry(new TaskId(0, 0, "A"), mkSet(TOPIC_PARTITION_A_0)),
-            mkEntry(new TaskId(0, 1, "A"), mkSet(TOPIC_PARTITION_A_1))
-        ), tasks.drainPendingStandbyTasksForTopologies(mkSet("A")));
+            mkEntry(new TaskId(0, 0, "A"), Set.of(TOPIC_PARTITION_A_0)),
+            mkEntry(new TaskId(0, 1, "A"), Set.of(TOPIC_PARTITION_A_1))
+        ), tasks.drainPendingStandbyTasksForTopologies(Set.of("A")));
 
         tasks.clearPendingTasksToCreate();
 
-        assertEquals(Collections.emptyMap(), tasks.drainPendingActiveTasksForTopologies(mkSet("B")));
-        assertEquals(Collections.emptyMap(), tasks.drainPendingStandbyTasksForTopologies(mkSet("B")));
+        assertEquals(Collections.emptyMap(), tasks.drainPendingActiveTasksForTopologies(Set.of("B")));
+        assertEquals(Collections.emptyMap(), tasks.drainPendingStandbyTasksForTopologies(Set.of("B")));
     }
 
     @Test
     public void shouldVerifyIfPendingTaskToInitExist() {
         assertFalse(tasks.hasPendingTasksToInit());
 
-        final StreamTask activeTask = statefulTask(TASK_0_0, mkSet(TOPIC_PARTITION_B_0)).build();
+        final StreamTask activeTask = statefulTask(TASK_0_0, Set.of(TOPIC_PARTITION_B_0)).build();
         tasks.addPendingTasksToInit(Collections.singleton(activeTask));
         assertTrue(tasks.hasPendingTasksToInit());
 
-        final StandbyTask standbyTask = standbyTask(TASK_1_0, mkSet(TOPIC_PARTITION_A_1)).build();
+        final StandbyTask standbyTask = standbyTask(TASK_1_0, Set.of(TOPIC_PARTITION_A_1)).build();
         tasks.addPendingTasksToInit(Collections.singleton(standbyTask));
         assertTrue(tasks.hasPendingTasksToInit());
 
@@ -150,25 +149,25 @@ public class TasksTest {
 
     @Test
     public void shouldVerifyIfPendingActiveTaskToInitAreDrained() {
-        final StreamTask activeTask1 = statefulTask(TASK_0_0, mkSet(TOPIC_PARTITION_B_0)).build();
-        final StreamTask activeTask2 = statefulTask(TASK_0_1, mkSet(TOPIC_PARTITION_B_1)).build();
-        final StandbyTask standbyTask1 = standbyTask(TASK_1_0, mkSet(TOPIC_PARTITION_A_0)).build();
-        final StandbyTask standbyTask2 = standbyTask(TASK_1_1, mkSet(TOPIC_PARTITION_A_1)).build();
-        tasks.addPendingTasksToInit(mkSet(activeTask1, activeTask2, standbyTask1, standbyTask2));
+        final StreamTask activeTask1 = statefulTask(TASK_0_0, Set.of(TOPIC_PARTITION_B_0)).build();
+        final StreamTask activeTask2 = statefulTask(TASK_0_1, Set.of(TOPIC_PARTITION_B_1)).build();
+        final StandbyTask standbyTask1 = standbyTask(TASK_1_0, Set.of(TOPIC_PARTITION_A_0)).build();
+        final StandbyTask standbyTask2 = standbyTask(TASK_1_1, Set.of(TOPIC_PARTITION_A_1)).build();
+        tasks.addPendingTasksToInit(Set.of(activeTask1, activeTask2, standbyTask1, standbyTask2));
 
         final Set<Task> activeTasksToInit = tasks.drainPendingActiveTasksToInit();
         assertEquals(2, activeTasksToInit.size());
-        assertTrue(activeTasksToInit.containsAll(mkSet(activeTask1, activeTask2)));
-        assertFalse(activeTasksToInit.containsAll(mkSet(standbyTask1, standbyTask2)));
+        assertTrue(activeTasksToInit.containsAll(Set.of(activeTask1, activeTask2)));
+        assertFalse(activeTasksToInit.containsAll(Set.of(standbyTask1, standbyTask2)));
         assertEquals(2, tasks.pendingTasksToInit().size());
         assertTrue(tasks.hasPendingTasksToInit());
-        assertTrue(tasks.pendingTasksToInit().containsAll(mkSet(standbyTask1, standbyTask2)));
+        assertTrue(tasks.pendingTasksToInit().containsAll(Set.of(standbyTask1, standbyTask2)));
     }
 
     @Test
     public void shouldAddFailedTask() {
-        final StreamTask activeTask1 = statefulTask(TASK_0_0, mkSet(TOPIC_PARTITION_B_0)).build();
-        final StreamTask activeTask2 = statefulTask(TASK_0_1, mkSet(TOPIC_PARTITION_B_1)).build();
+        final StreamTask activeTask1 = statefulTask(TASK_0_0, Set.of(TOPIC_PARTITION_B_0)).build();
+        final StreamTask activeTask2 = statefulTask(TASK_0_1, Set.of(TOPIC_PARTITION_B_1)).build();
         tasks.addTask(activeTask2);
 
         tasks.addFailedTask(activeTask1);
@@ -183,7 +182,7 @@ public class TasksTest {
 
     @Test
     public void shouldRemoveFailedTask() {
-        final StreamTask activeTask1 = statefulTask(TASK_0_0, mkSet(TOPIC_PARTITION_B_0))
+        final StreamTask activeTask1 = statefulTask(TASK_0_0, Set.of(TOPIC_PARTITION_B_0))
             .inState(State.SUSPENDED).build();
         tasks.addFailedTask(activeTask1);
 
@@ -197,7 +196,7 @@ public class TasksTest {
 
     @Test
     public void shouldClearFailedTask() {
-        final StreamTask activeTask1 = statefulTask(TASK_0_0, mkSet(TOPIC_PARTITION_B_0))
+        final StreamTask activeTask1 = statefulTask(TASK_0_0, Set.of(TOPIC_PARTITION_B_0))
             .inState(State.SUSPENDED).build();
         tasks.addFailedTask(activeTask1);
 
@@ -208,27 +207,4 @@ public class TasksTest {
         tasks.addTask(activeTask1);
         assertTrue(tasks.allNonFailedTasks().contains(activeTask1));
     }
-
-    @Test
-    public void shouldClearAllPendingTasks() {
-        final StandbyTask task = standbyTask(TASK_0_0, mkSet(TOPIC_PARTITION_B_0))
-            .inState(State.CREATED).build();
-        tasks.addPendingTasksToInit(Collections.singleton(task));
-        final TaskId taskId1 = new TaskId(0, 0, "A");
-        tasks.addPendingActiveTasksToCreate(mkMap(
-            mkEntry(taskId1, mkSet(TOPIC_PARTITION_A_0))
-        ));
-        final TaskId taskId2 = new TaskId(0, 1, "A");
-        tasks.addPendingStandbyTasksToCreate(mkMap(
-            mkEntry(taskId2, mkSet(TOPIC_PARTITION_A_0))
-        ));
-
-        assertTrue(tasks.pendingTasksToInit().contains(task));
-        assertTrue(tasks.pendingActiveTasksToCreate().containsKey(taskId1));
-        assertTrue(tasks.pendingStandbyTasksToCreate().containsKey(taskId2));
-        tasks.clear();
-        assertTrue(tasks.pendingTasksToInit().isEmpty());
-        assertTrue(tasks.pendingActiveTasksToCreate().isEmpty());
-        assertTrue(tasks.pendingStandbyTasksToCreate().isEmpty());
-    }
 }
