@@ -21,7 +21,8 @@ import org.apache.kafka.streams.processor.assignment.KafkaStreamsAssignment.Assi
 
 import org.junit.jupiter.api.Test;
 
-import static org.apache.kafka.common.utils.Utils.mkSet;
+import java.util.Set;
+
 import static org.apache.kafka.streams.processor.internals.assignment.AssignmentTestUtils.TASK_0_0;
 import static org.apache.kafka.streams.processor.internals.assignment.AssignmentTestUtils.TASK_0_1;
 import static org.apache.kafka.streams.processor.internals.assignment.AssignmentTestUtils.TASK_0_2;
@@ -34,7 +35,7 @@ public class KafkaStreamsAssignmentTest {
     public void shouldHaveReadableString() {
         final KafkaStreamsAssignment assignment = KafkaStreamsAssignment.of(
             processIdForInt(1),
-            mkSet(
+            Set.of(
                 new AssignedTask(TASK_0_0, AssignedTask.Type.ACTIVE),
                 new AssignedTask(TASK_0_1, AssignedTask.Type.STANDBY),
                 new AssignedTask(TASK_0_2, AssignedTask.Type.ACTIVE)
