@@ -50,7 +50,6 @@ import java.util.stream.Collectors;
 
 import static org.apache.kafka.common.utils.Utils.mkEntry;
 import static org.apache.kafka.common.utils.Utils.mkMap;
-import static org.apache.kafka.common.utils.Utils.mkSet;
 import static org.apache.kafka.streams.processor.internals.assignment.AssignmentTestUtils.TASK_0_0;
 import static org.apache.kafka.streams.processor.internals.assignment.AssignmentTestUtils.TASK_0_1;
 import static org.apache.kafka.streams.processor.internals.assignment.AssignmentTestUtils.TASK_0_2;
@@ -74,8 +73,8 @@ public class TaskAssignmentUtilsTest {
         final AssignmentConfigs assignmentConfigs = defaultAssignmentConfigs(
             strategy, 100, 1, 1, Collections.emptyList());
         final Map<TaskId, TaskInfo> tasks = mkMap(
-            mkTaskInfo(TASK_0_0, true, mkSet("rack-2")),
-            mkTaskInfo(TASK_0_1, true, mkSet("rack-1"))
+            mkTaskInfo(TASK_0_0, true, Set.of("rack-2")),
+            mkTaskInfo(TASK_0_1, true, Set.of("rack-1"))
         );
         final Map<ProcessId, KafkaStreamsState> kafkaStreamsStates = mkMap(
             mkStreamState(1, 1, Optional.of("rack-1")),
@@ -92,15 +91,15 @@ public class TaskAssignmentUtilsTest {
         TaskAssignmentUtils.optimizeRackAwareActiveTasks(
             RackAwareOptimizationParams.of(applicationState), assignments);
         assertThat(assignments.size(), equalTo(2));
-        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(mkSet(TASK_0_1)));
-        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(mkSet(TASK_0_0)));
+        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(Set.of(TASK_0_1)));
+        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(Set.of(TASK_0_0)));
 
         // Repeated to make sure nothing gets shifted around after the first round of optimization.
         TaskAssignmentUtils.optimizeRackAwareActiveTasks(
             RackAwareOptimizationParams.of(applicationState), assignments);
         assertThat(assignments.size(), equalTo(2));
-        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(mkSet(TASK_0_1)));
-        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(mkSet(TASK_0_0)));
+        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(Set.of(TASK_0_1)));
+        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(Set.of(TASK_0_0)));
     }
 
     @Timeout(value = 30)
@@ -113,8 +112,8 @@ public class TaskAssignmentUtilsTest {
         final AssignmentConfigs assignmentConfigs = defaultAssignmentConfigs(
             strategy, 100, 1, 1, Collections.emptyList());
         final Map<TaskId, TaskInfo> tasks = mkMap(
-            mkTaskInfo(TASK_0_0, true, mkSet("rack-2")),
-            mkTaskInfo(TASK_0_1, true, mkSet("rack-3"))
+            mkTaskInfo(TASK_0_0, true, Set.of("rack-2")),
+            mkTaskInfo(TASK_0_1, true, Set.of("rack-3"))
         );
         final Map<ProcessId, KafkaStreamsState> kafkaStreamsStates = mkMap(
             mkStreamState(1, 2, Optional.of("rack-1")),
@@ -132,9 +131,9 @@ public class TaskAssignmentUtilsTest {
 
         TaskAssignmentUtils.optimizeRackAwareStandbyTasks(RackAwareOptimizationParams.of(applicationState), assignments);
         assertThat(assignments.size(), equalTo(3));
-        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(mkSet(TASK_0_0, TASK_0_1)));
-        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(mkSet(TASK_0_0)));
-        assertThat(assignments.get(processId(3)).tasks().keySet(), equalTo(mkSet(TASK_0_1)));
+        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(Set.of(TASK_0_0, TASK_0_1)));
+        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(Set.of(TASK_0_0)));
+        assertThat(assignments.get(processId(3)).tasks().keySet(), equalTo(Set.of(TASK_0_1)));
     }
 
     @Timeout(value = 30)
@@ -146,16 +145,16 @@ public class TaskAssignmentUtilsTest {
             mkTaskInfo(TASK_0_0, true)
         );
         final Map<ProcessId, KafkaStreamsState> kafkaStreamsStates = mkMap(
-            mkStreamState(1, 2, Optional.empty(), mkSet(), mkSet(), mkMap(
+            mkStreamState(1, 2, Optional.empty(), Set.of(), Set.of(), mkMap(
                 mkEntry("az", "1")
             )),
-            mkStreamState(2, 2, Optional.empty(), mkSet(), mkSet(), mkMap(
+            mkStreamState(2, 2, Optional.empty(), Set.of(), Set.of(), mkMap(
                 mkEntry("az", "1")
             )),
-            mkStreamState(3, 2, Optional.empty(), mkSet(), mkSet(), mkMap(
+            mkStreamState(3, 2, Optional.empty(), Set.of(), Set.of(), mkMap(
                 mkEntry("az", "2")
             )),
-            mkStreamState(4, 2, Optional.empty(), mkSet(), mkSet(), mkMap(
+            mkStreamState(4, 2, Optional.empty(), Set.of(), Set.of(), mkMap(
                 mkEntry("az", "3")
             ))
         );
@@ -168,12 +167,12 @@ public class TaskAssignmentUtilsTest {
 
         TaskAssignmentUtils.defaultStandbyTaskAssignment(applicationState, assignments);
         assertThat(assignments.size(), equalTo(4));
-        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(mkSet(TASK_0_0)));
+        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(Set.of(TASK_0_0)));
         assertThat(assignments.get(processId(1)).tasks().get(TASK_0_0).type(), equalTo(AssignedTask.Type.ACTIVE));
 
-        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(mkSet()));
-        assertThat(assignments.get(processId(3)).tasks().keySet(), equalTo(mkSet(TASK_0_0)));
-        assertThat(assignments.get(processId(4)).tasks().keySet(), equalTo(mkSet(TASK_0_0)));
+        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(Set.of()));
+        assertThat(assignments.get(processId(3)).tasks().keySet(), equalTo(Set.of(TASK_0_0)));
+        assertThat(assignments.get(processId(4)).tasks().keySet(), equalTo(Set.of(TASK_0_0)));
     }
 
     @Timeout(value = 30)
@@ -190,11 +189,11 @@ public class TaskAssignmentUtilsTest {
             mkTaskInfo(TASK_0_5, false)
         );
         final Map<ProcessId, KafkaStreamsState> kafkaStreamsStates = mkMap(
-            mkStreamState(1, 5, Optional.empty(), mkSet(), mkSet()),
-            mkStreamState(2, 5, Optional.empty(), mkSet(), mkSet()),
-            mkStreamState(3, 5, Optional.empty(), mkSet(), mkSet()),
-            mkStreamState(4, 5, Optional.empty(), mkSet(), mkSet()),
-            mkStreamState(5, 5, Optional.empty(), mkSet(), mkSet())
+            mkStreamState(1, 5, Optional.empty(), Set.of(), Set.of()),
+            mkStreamState(2, 5, Optional.empty(), Set.of(), Set.of()),
+            mkStreamState(3, 5, Optional.empty(), Set.of(), Set.of()),
+            mkStreamState(4, 5, Optional.empty(), Set.of(), Set.of()),
+            mkStreamState(5, 5, Optional.empty(), Set.of(), Set.of())
         );
         final ApplicationState applicationState = new TestApplicationState(
             assignmentConfigs, kafkaStreamsStates, tasks);
@@ -206,10 +205,10 @@ public class TaskAssignmentUtilsTest {
 
         TaskAssignmentUtils.defaultStandbyTaskAssignment(applicationState, assignments);
         assertThat(assignments.size(), equalTo(5));
-        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(mkSet(TASK_0_3, TASK_0_4, TASK_0_5)));
-        assertThat(assignments.get(processId(3)).tasks().keySet(), equalTo(mkSet(TASK_0_0)));
-        assertThat(assignments.get(processId(4)).tasks().keySet(), equalTo(mkSet(TASK_0_0)));
-        assertThat(assignments.get(processId(5)).tasks().keySet(), equalTo(mkSet(TASK_0_0)));
+        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(Set.of(TASK_0_3, TASK_0_4, TASK_0_5)));
+        assertThat(assignments.get(processId(3)).tasks().keySet(), equalTo(Set.of(TASK_0_0)));
+        assertThat(assignments.get(processId(4)).tasks().keySet(), equalTo(Set.of(TASK_0_0)));
+        assertThat(assignments.get(processId(5)).tasks().keySet(), equalTo(Set.of(TASK_0_0)));
     }
 
     @Timeout(value = 30)
@@ -222,20 +221,20 @@ public class TaskAssignmentUtilsTest {
         final AssignmentConfigs assignmentConfigs = defaultAssignmentConfigs(
             strategy, 100, 1, 2, Collections.singletonList("az"));
         final Map<TaskId, TaskInfo> tasks = mkMap(
-            mkTaskInfo(TASK_0_0, true, mkSet("r1")),
-            mkTaskInfo(TASK_0_1, true, mkSet("r1"))
+            mkTaskInfo(TASK_0_0, true, Set.of("r1")),
+            mkTaskInfo(TASK_0_1, true, Set.of("r1"))
         );
         final Map<ProcessId, KafkaStreamsState> kafkaStreamsStates = mkMap(
-            mkStreamState(1, 2, Optional.of("r1"), mkSet(), mkSet(), mkMap(
+            mkStreamState(1, 2, Optional.of("r1"), Set.of(), Set.of(), mkMap(
                 mkEntry("az", "1")
             )),
-            mkStreamState(2, 2, Optional.of("r1"), mkSet(), mkSet(), mkMap(
+            mkStreamState(2, 2, Optional.of("r1"), Set.of(), Set.of(), mkMap(
                 mkEntry("az", "2")
             )),
-            mkStreamState(3, 2, Optional.of("r1"), mkSet(), mkSet(), mkMap(
+            mkStreamState(3, 2, Optional.of("r1"), Set.of(), Set.of(), mkMap(
                 mkEntry("az", "3")
             )),
-            mkStreamState(4, 2, Optional.of("r1"), mkSet(), mkSet(), mkMap(
+            mkStreamState(4, 2, Optional.of("r1"), Set.of(), Set.of(), mkMap(
                 mkEntry("az", "2")
             ))
         );
@@ -263,10 +262,10 @@ public class TaskAssignmentUtilsTest {
 
         TaskAssignmentUtils.optimizeRackAwareStandbyTasks(RackAwareOptimizationParams.of(applicationState), assignments);
         assertThat(assignments.size(), equalTo(4));
-        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(mkSet(TASK_0_0, TASK_0_1)));
-        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(mkSet(TASK_0_0, TASK_0_1)));
-        assertThat(assignments.get(processId(3)).tasks().keySet(), equalTo(mkSet(TASK_0_0, TASK_0_1)));
-        assertThat(assignments.get(processId(4)).tasks().keySet(), equalTo(mkSet()));
+        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(Set.of(TASK_0_0, TASK_0_1)));
+        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(Set.of(TASK_0_0, TASK_0_1)));
+        assertThat(assignments.get(processId(3)).tasks().keySet(), equalTo(Set.of(TASK_0_0, TASK_0_1)));
+        assertThat(assignments.get(processId(4)).tasks().keySet(), equalTo(Set.of()));
     }
 
     @Timeout(value = 30)
@@ -279,9 +278,9 @@ public class TaskAssignmentUtilsTest {
         final AssignmentConfigs assignmentConfigs = defaultAssignmentConfigs(
             strategy, 100, 1, 1, Collections.emptyList());
         final Map<TaskId, TaskInfo> tasks = mkMap(
-            mkTaskInfo(TASK_0_0, true, mkSet("rack-1", "rack-2")),
-            mkTaskInfo(TASK_0_1, true, mkSet("rack-2", "rack-3")),
-            mkTaskInfo(TASK_0_2, true, mkSet("rack-3", "rack-4"))
+            mkTaskInfo(TASK_0_0, true, Set.of("rack-1", "rack-2")),
+            mkTaskInfo(TASK_0_1, true, Set.of("rack-2", "rack-3")),
+            mkTaskInfo(TASK_0_2, true, Set.of("rack-3", "rack-4"))
         );
         final Map<ProcessId, KafkaStreamsState> kafkaStreamsStates = mkMap(
             mkStreamState(1, 2, Optional.of("rack-1")),
@@ -299,13 +298,13 @@ public class TaskAssignmentUtilsTest {
 
         TaskAssignmentUtils.optimizeRackAwareActiveTasks(
             RackAwareOptimizationParams.of(applicationState)
-                .forTasks(new TreeSet<>(mkSet(TASK_0_0, TASK_0_1, TASK_0_2))),
+                .forTasks(new TreeSet<>(Set.of(TASK_0_0, TASK_0_1, TASK_0_2))),
             assignments
         );
         assertThat(assignments.size(), equalTo(3));
-        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(mkSet(TASK_0_0)));
-        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(mkSet(TASK_0_1)));
-        assertThat(assignments.get(processId(3)).tasks().keySet(), equalTo(mkSet(TASK_0_2)));
+        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(Set.of(TASK_0_0)));
+        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(Set.of(TASK_0_1)));
+        assertThat(assignments.get(processId(3)).tasks().keySet(), equalTo(Set.of(TASK_0_2)));
     }
 
     @Timeout(value = 30)
@@ -319,11 +318,11 @@ public class TaskAssignmentUtilsTest {
             mkTaskInfo(TASK_0_2, true)
         );
         final Map<ProcessId, KafkaStreamsState> kafkaStreamsStates = mkMap(
-            mkStreamState(1, 5, Optional.empty(), mkSet(TASK_0_0, TASK_0_1, TASK_0_2), mkSet()),
-            mkStreamState(2, 5, Optional.empty(), mkSet(), mkSet(TASK_0_0, TASK_0_1, TASK_0_2)),
-            mkStreamState(3, 5, Optional.empty(), mkSet(), mkSet()),
-            mkStreamState(4, 5, Optional.empty(), mkSet(), mkSet()),
-            mkStreamState(5, 5, Optional.empty(), mkSet(), mkSet())
+            mkStreamState(1, 5, Optional.empty(), Set.of(TASK_0_0, TASK_0_1, TASK_0_2), Set.of()),
+            mkStreamState(2, 5, Optional.empty(), Set.of(), Set.of(TASK_0_0, TASK_0_1, TASK_0_2)),
+            mkStreamState(3, 5, Optional.empty(), Set.of(), Set.of()),
+            mkStreamState(4, 5, Optional.empty(), Set.of(), Set.of()),
+            mkStreamState(5, 5, Optional.empty(), Set.of(), Set.of())
         );
         final ApplicationState applicationState = new TestApplicationState(
             assignmentConfigs, kafkaStreamsStates, tasks);
@@ -331,11 +330,11 @@ public class TaskAssignmentUtilsTest {
 
         final Map<ProcessId, KafkaStreamsAssignment> assignments = TaskAssignmentUtils.identityAssignment(applicationState);
         assertThat(assignments.size(), equalTo(5));
-        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(mkSet(TASK_0_0, TASK_0_1, TASK_0_2)));
-        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(mkSet(TASK_0_0, TASK_0_1, TASK_0_2)));
-        assertThat(assignments.get(processId(3)).tasks().keySet(), equalTo(mkSet()));
-        assertThat(assignments.get(processId(4)).tasks().keySet(), equalTo(mkSet()));
-        assertThat(assignments.get(processId(5)).tasks().keySet(), equalTo(mkSet()));
+        assertThat(assignments.get(processId(1)).tasks().keySet(), equalTo(Set.of(TASK_0_0, TASK_0_1, TASK_0_2)));
+        assertThat(assignments.get(processId(2)).tasks().keySet(), equalTo(Set.of(TASK_0_0, TASK_0_1, TASK_0_2)));
+        assertThat(assignments.get(processId(3)).tasks().keySet(), equalTo(Set.of()));
+        assertThat(assignments.get(processId(4)).tasks().keySet(), equalTo(Set.of()));
+        assertThat(assignments.get(processId(5)).tasks().keySet(), equalTo(Set.of()));
     }
 
     @Timeout(value = 30)
@@ -355,13 +354,13 @@ public class TaskAssignmentUtilsTest {
 
         // ****
         final org.apache.kafka.streams.processor.assignment.TaskAssignor.TaskAssignment noError = new org.apache.kafka.streams.processor.assignment.TaskAssignor.TaskAssignment(
-            mkSet(
-                KafkaStreamsAssignment.of(processId(1), mkSet(
+            Set.of(
+                KafkaStreamsAssignment.of(processId(1), Set.of(
                     new KafkaStreamsAssignment.AssignedTask(
                         new TaskId(1, 1), KafkaStreamsAssignment.AssignedTask.Type.ACTIVE
                     )
                 )),
-                KafkaStreamsAssignment.of(processId(2), mkSet())
+                KafkaStreamsAssignment.of(processId(2), Set.of())
             )
         );
         org.apache.kafka.streams.processor.assignment.TaskAssignor.AssignmentError error = TaskAssignmentUtils.validateTaskAssignment(applicationState, noError);
@@ -369,8 +368,8 @@ public class TaskAssignmentUtilsTest {
 
         // ****
         final org.apache.kafka.streams.processor.assignment.TaskAssignor.TaskAssignment missingProcessId = new org.apache.kafka.streams.processor.assignment.TaskAssignor.TaskAssignment(
-            mkSet(
-                KafkaStreamsAssignment.of(processId(1), mkSet(
+            Set.of(
+                KafkaStreamsAssignment.of(processId(1), Set.of(
                     new KafkaStreamsAssignment.AssignedTask(
                         new TaskId(1, 1), KafkaStreamsAssignment.AssignedTask.Type.ACTIVE
                     )
@@ -382,14 +381,14 @@ public class TaskAssignmentUtilsTest {
 
         // ****
         final org.apache.kafka.streams.processor.assignment.TaskAssignor.TaskAssignment unknownProcessId = new org.apache.kafka.streams.processor.assignment.TaskAssignor.TaskAssignment(
-            mkSet(
-                KafkaStreamsAssignment.of(processId(1), mkSet(
+            Set.of(
+                KafkaStreamsAssignment.of(processId(1), Set.of(
                     new KafkaStreamsAssignment.AssignedTask(
                         new TaskId(1, 1), KafkaStreamsAssignment.AssignedTask.Type.ACTIVE
                     )
                 )),
-                KafkaStreamsAssignment.of(processId(2), mkSet()),
-                KafkaStreamsAssignment.of(ProcessId.randomProcessId(), mkSet())
+                KafkaStreamsAssignment.of(processId(2), Set.of()),
+                KafkaStreamsAssignment.of(ProcessId.randomProcessId(), Set.of())
             )
         );
         error = TaskAssignmentUtils.validateTaskAssignment(applicationState, unknownProcessId);
@@ -397,13 +396,13 @@ public class TaskAssignmentUtilsTest {
 
         // ****
         final org.apache.kafka.streams.processor.assignment.TaskAssignor.TaskAssignment unknownTaskId = new org.apache.kafka.streams.processor.assignment.TaskAssignor.TaskAssignment(
-            mkSet(
-                KafkaStreamsAssignment.of(processId(1), mkSet(
+            Set.of(
+                KafkaStreamsAssignment.of(processId(1), Set.of(
                     new KafkaStreamsAssignment.AssignedTask(
                         new TaskId(1, 1), KafkaStreamsAssignment.AssignedTask.Type.ACTIVE
                     )
                 )),
-                KafkaStreamsAssignment.of(processId(2), mkSet(
+                KafkaStreamsAssignment.of(processId(2), Set.of(
                     new KafkaStreamsAssignment.AssignedTask(
                         new TaskId(13, 13), KafkaStreamsAssignment.AssignedTask.Type.ACTIVE
                     )
@@ -415,13 +414,13 @@ public class TaskAssignmentUtilsTest {
 
         // ****
         final org.apache.kafka.streams.processor.assignment.TaskAssignor.TaskAssignment activeTaskDuplicated = new org.apache.kafka.streams.processor.assignment.TaskAssignor.TaskAssignment(
-            mkSet(
-                KafkaStreamsAssignment.of(processId(1), mkSet(
+            Set.of(
+                KafkaStreamsAssignment.of(processId(1), Set.of(
                     new KafkaStreamsAssignment.AssignedTask(
                         new TaskId(1, 1), KafkaStreamsAssignment.AssignedTask.Type.ACTIVE
                     )
                 )),
-                KafkaStreamsAssignment.of(processId(2), mkSet(
+                KafkaStreamsAssignment.of(processId(2), Set.of(
                     new KafkaStreamsAssignment.AssignedTask(
                         new TaskId(1, 1), KafkaStreamsAssignment.AssignedTask.Type.ACTIVE
                     )
@@ -504,10 +503,9 @@ public class TaskAssignmentUtilsTest {
                                                                             final int client,
                                                                             final TaskId... taskIds) {
         final ProcessId processId = processId(client);
-        final Set<AssignedTask> assignedTasks = mkSet();
-        for (final TaskId taskId : taskIds) {
-            assignedTasks.add(new AssignedTask(taskId, taskType));
-        }
+        final Set<AssignedTask> assignedTasks = Arrays.stream(taskIds)
+                .map(taskId -> new AssignedTask(taskId, taskType))
+                .collect(Collectors.toSet());
         return mkEntry(
             processId,
             KafkaStreamsAssignment.of(
@@ -537,7 +535,7 @@ public class TaskAssignmentUtilsTest {
         if (!isStateful) {
             return mkEntry(
                 taskId,
-                new DefaultTaskInfo(taskId, false, mkSet(), mkSet())
+                new DefaultTaskInfo(taskId, false, Set.of(), Set.of())
             );
         }
 
@@ -559,7 +557,7 @@ public class TaskAssignmentUtilsTest {
             new DefaultTaskInfo(
                 taskId,
                 true,
-                mkSet(String.format("test-statestore-%d", taskId.subtopology())),
+                Set.of(String.format("test-statestore-%d", taskId.subtopology())),
                 partitions.stream().map(p -> (TaskTopicPartition) p).collect(Collectors.toSet())
             )
         );
