@@ -19,7 +19,7 @@ package org.apache.kafka.streams.state.internals;
 public class RocksDBTimestampedSegmentedBytesStoreTest
     extends AbstractRocksDBSegmentedBytesStoreTest<TimestampedSegment> {
 
-    private final static String METRICS_SCOPE = "metrics-scope";
+    private static final String METRICS_SCOPE = "metrics-scope";
 
     RocksDBTimestampedSegmentedBytesStore getBytesStore() {
         return new RocksDBTimestampedSegmentedBytesStore(
