@@ -16,7 +16,14 @@
  */
 package org.apache.kafka.streams.state.internals;
 
+import org.apache.kafka.clients.admin.Admin;
+import org.apache.kafka.clients.admin.MockAdminClient;
+import org.apache.kafka.clients.consumer.Consumer;
 import org.apache.kafka.clients.consumer.ConsumerConfig;
+import org.apache.kafka.clients.consumer.MockConsumer;
+import org.apache.kafka.clients.consumer.internals.AutoOffsetResetStrategy;
+import org.apache.kafka.clients.producer.MockProducer;
+import org.apache.kafka.clients.producer.Producer;
 import org.apache.kafka.common.PartitionInfo;
 import org.apache.kafka.common.TopicPartition;
 import org.apache.kafka.common.metrics.Metrics;
@@ -27,6 +34,7 @@ import org.apache.kafka.common.utils.Time;
 import org.apache.kafka.common.utils.Utils;
 import org.apache.kafka.streams.StoreQueryParameters;
 import org.apache.kafka.streams.StreamsConfig;
+import org.apache.kafka.streams.TopologyConfig;
 import org.apache.kafka.streams.TopologyWrapper;
 import org.apache.kafka.streams.errors.InvalidStateStoreException;
 import org.apache.kafka.streams.internals.StreamsConfigUtils;
@@ -46,7 +54,6 @@ import org.apache.kafka.streams.processor.internals.StreamThread;
 import org.apache.kafka.streams.processor.internals.StreamsProducer;
 import org.apache.kafka.streams.processor.internals.Task;
 import org.apache.kafka.streams.processor.internals.metrics.StreamsMetricsImpl;
-import org.apache.kafka.streams.TopologyConfig;
 import org.apache.kafka.streams.state.QueryableStoreTypes;
 import org.apache.kafka.streams.state.ReadOnlyKeyValueStore;
 import org.apache.kafka.streams.state.ReadOnlySessionStore;
@@ -56,16 +63,18 @@ import org.apache.kafka.streams.state.TimestampedKeyValueStore;
 import org.apache.kafka.streams.state.TimestampedWindowStore;
 import org.apache.kafka.streams.state.ValueAndTimestamp;
 import org.apache.kafka.test.MockApiProcessorSupplier;
-import org.apache.kafka.test.MockClientSupplier;
 import org.apache.kafka.test.MockStandbyUpdateListener;
 import org.apache.kafka.test.MockStateRestoreListener;
 import org.apache.kafka.test.TestUtils;
-import org.junit.After;
-import org.junit.Before;
-import org.junit.Test;
-import org.junit.runner.RunWith;
+
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.extension.ExtendWith;
 import org.mockito.Mock;
-import org.mockito.junit.MockitoJUnitRunner;
+import org.mockito.junit.jupiter.MockitoExtension;
+import org.mockito.junit.jupiter.MockitoSettings;
+import org.mockito.quality.Strictness;
 
 import java.io.File;
 import java.io.IOException;
@@ -78,18 +87,19 @@ import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 import java.util.Set;
-import java.util.UUID;
 
+import static org.apache.kafka.streams.internals.StreamsConfigUtils.ProcessingMode.AT_LEAST_ONCE;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.Matchers.instanceOf;
 import static org.hamcrest.Matchers.is;
 import static org.hamcrest.Matchers.not;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
 
-@RunWith(MockitoJUnitRunner.StrictStubs.class)
+@ExtendWith(MockitoExtension.class)
+@MockitoSettings(strictness = Strictness.STRICT_STUBS)
 public class StreamThreadStateStoreProviderTest {
 
     private StreamTask taskOne;
@@ -101,7 +111,7 @@ public class StreamThreadStateStoreProviderTest {
     private StreamThread threadMock;
     private Map<TaskId, Task> tasks;
 
-    @Before
+    @BeforeEach
     public void before() {
         final TopologyWrapper topology = new TopologyWrapper();
         topology.addSource("the-source", topicName);
@@ -155,9 +165,12 @@ public class StreamThreadStateStoreProviderTest {
         properties.put(StreamsConfig.STATE_DIR_CONFIG, stateDir.getPath());
 
         final StreamsConfig streamsConfig = new StreamsConfig(properties);
-        final MockClientSupplier clientSupplier = new MockClientSupplier();
-        configureClients(clientSupplier, "applicationId-kv-store-changelog");
-        configureClients(clientSupplier, "applicationId-window-store-changelog");
+        final MockConsumer<byte[], byte[]> mockConsumer = new MockConsumer<>(AutoOffsetResetStrategy.EARLIEST.name());
+        final MockConsumer<byte[], byte[]> mockRestoreConsumer = new MockConsumer<>(AutoOffsetResetStrategy.EARLIEST.name());
+        final MockProducer<byte[], byte[]> mockProducer = new MockProducer<>();
+        final MockAdminClient mockAdminClient = MockAdminClient.create().build();
+        configureClients(mockRestoreConsumer, mockAdminClient, "applicationId-kv-store-changelog");
+        configureClients(mockRestoreConsumer, mockAdminClient, "applicationId-window-store-changelog");
 
         final InternalTopologyBuilder internalTopologyBuilder = topology.getInternalBuilder(applicationId);
         final ProcessorTopology processorTopology = internalTopologyBuilder.buildTopology();
@@ -167,7 +180,10 @@ public class StreamThreadStateStoreProviderTest {
 
         taskOne = createStreamsTask(
             streamsConfig,
-            clientSupplier,
+            mockConsumer,
+            mockRestoreConsumer,
+            mockProducer,
+            mockAdminClient,
             processorTopology,
             new TaskId(0, 0));
         taskOne.initializeIfNeeded();
@@ -175,7 +191,10 @@ public class StreamThreadStateStoreProviderTest {
 
         final StreamTask taskTwo = createStreamsTask(
             streamsConfig,
-            clientSupplier,
+            mockConsumer,
+            mockRestoreConsumer,
+            mockProducer,
+            mockAdminClient,
             processorTopology,
             new TaskId(0, 1));
         taskTwo.initializeIfNeeded();
@@ -185,7 +204,7 @@ public class StreamThreadStateStoreProviderTest {
 
     }
 
-    @After
+    @AfterEach
     public void cleanUp() throws IOException {
         Utils.delete(stateDir);
     }
@@ -312,7 +331,7 @@ public class StreamThreadStateStoreProviderTest {
     @Test
     public void shouldThrowInvalidStoreExceptionIfKVStoreClosed() {
         mockThread(true);
-        taskOne.getStore("kv-store").close();
+        taskOne.store("kv-store").close();
         assertThrows(InvalidStateStoreException.class, () -> provider.stores(StoreQueryParameters.fromNameAndType("kv-store",
                 QueryableStoreTypes.keyValueStore())));
     }
@@ -320,7 +339,7 @@ public class StreamThreadStateStoreProviderTest {
     @Test
     public void shouldThrowInvalidStoreExceptionIfTsKVStoreClosed() {
         mockThread(true);
-        taskOne.getStore("timestamped-kv-store").close();
+        taskOne.store("timestamped-kv-store").close();
         assertThrows(InvalidStateStoreException.class, () -> provider.stores(StoreQueryParameters.fromNameAndType("timestamped-kv-store",
                 QueryableStoreTypes.timestampedKeyValueStore())));
     }
@@ -328,7 +347,7 @@ public class StreamThreadStateStoreProviderTest {
     @Test
     public void shouldThrowInvalidStoreExceptionIfWindowStoreClosed() {
         mockThread(true);
-        taskOne.getStore("window-store").close();
+        taskOne.store("window-store").close();
         assertThrows(InvalidStateStoreException.class, () -> provider.stores(StoreQueryParameters.fromNameAndType("window-store",
                 QueryableStoreTypes.windowStore())));
     }
@@ -336,7 +355,7 @@ public class StreamThreadStateStoreProviderTest {
     @Test
     public void shouldThrowInvalidStoreExceptionIfTsWindowStoreClosed() {
         mockThread(true);
-        taskOne.getStore("timestamped-window-store").close();
+        taskOne.store("timestamped-window-store").close();
         assertThrows(InvalidStateStoreException.class, () -> provider.stores(StoreQueryParameters.fromNameAndType("timestamped-window-store",
                 QueryableStoreTypes.timestampedWindowStore())));
     }
@@ -344,7 +363,7 @@ public class StreamThreadStateStoreProviderTest {
     @Test
     public void shouldThrowInvalidStoreExceptionIfSessionStoreClosed() {
         mockThread(true);
-        taskOne.getStore("session-store").close();
+        taskOne.store("session-store").close();
         assertThrows(InvalidStateStoreException.class, () -> provider.stores(StoreQueryParameters.fromNameAndType("session-store",
                 QueryableStoreTypes.sessionStore())));
     }
@@ -397,13 +416,16 @@ public class StreamThreadStateStoreProviderTest {
 
     @Test
     public void shouldThrowInvalidStoreExceptionIfNotAllStoresAvailable() {
-        mockThread(false);
+        when(threadMock.state()).thenReturn(StreamThread.State.PARTITIONS_ASSIGNED);
         assertThrows(InvalidStateStoreException.class, () -> provider.stores(StoreQueryParameters.fromNameAndType("kv-store",
                 QueryableStoreTypes.keyValueStore())));
     }
 
     private StreamTask createStreamsTask(final StreamsConfig streamsConfig,
-                                         final MockClientSupplier clientSupplier,
+                                         final Consumer<byte[], byte[]> consumer,
+                                         final Consumer<byte[], byte[]> restoreConsumer,
+                                         final Producer<byte[], byte[]> producer,
+                                         final Admin adminClient,
                                          final ProcessorTopology topology,
                                          final TaskId taskId) {
         final Metrics metrics = new Metrics();
@@ -419,8 +441,8 @@ public class StreamThreadStateStoreProviderTest {
                 new MockTime(),
                 streamsConfig,
                 logContext,
-                clientSupplier.adminClient,
-                clientSupplier.restoreConsumer,
+                adminClient,
+                restoreConsumer,
                 new MockStateRestoreListener(),
                 new MockStandbyUpdateListener()),
             topology.storeToChangelogTopic(),
@@ -430,20 +452,17 @@ public class StreamThreadStateStoreProviderTest {
             logContext,
             taskId,
             new StreamsProducer(
-                streamsConfig,
-                "threadId",
-                clientSupplier,
-                new TaskId(0, 0),
-                UUID.randomUUID(),
-                logContext,
-                Time.SYSTEM
+                producer,
+                AT_LEAST_ONCE,
+                Time.SYSTEM,
+                logContext
             ),
-            streamsConfig.defaultProductionExceptionHandler(),
+            streamsConfig.productionExceptionHandler(),
             new MockStreamsMetrics(metrics),
             topology
         );
         final StreamsMetricsImpl streamsMetrics = new MockStreamsMetrics(metrics);
-        final InternalProcessorContext context = new ProcessorContextImpl(
+        final InternalProcessorContext<?, ?> context = new ProcessorContextImpl(
             taskId,
             streamsConfig,
             stateManager,
@@ -454,7 +473,7 @@ public class StreamThreadStateStoreProviderTest {
             taskId,
             partitions,
             topology,
-            clientSupplier.consumer,
+            consumer,
             new TopologyConfig(null, streamsConfig, new Properties()).getTaskConfig(),
             streamsMetrics,
             stateDirectory,
@@ -475,25 +494,27 @@ public class StreamThreadStateStoreProviderTest {
         );
     }
 
-    private void configureClients(final MockClientSupplier clientSupplier, final String topic) {
+    private void configureClients(final MockConsumer<byte[], byte[]> restoreConsumer,
+                                  final MockAdminClient adminClient,
+                                  final String topic) {
         final List<PartitionInfo> partitions = Arrays.asList(
             new PartitionInfo(topic, 0, null, null, null),
             new PartitionInfo(topic, 1, null, null, null)
         );
-        clientSupplier.restoreConsumer.updatePartitions(topic, partitions);
+        restoreConsumer.updatePartitions(topic, partitions);
         final TopicPartition tp1 = new TopicPartition(topic, 0);
         final TopicPartition tp2 = new TopicPartition(topic, 1);
 
-        clientSupplier.restoreConsumer.assign(Arrays.asList(tp1, tp2));
+        restoreConsumer.assign(Arrays.asList(tp1, tp2));
 
         final Map<TopicPartition, Long> offsets = new HashMap<>();
         offsets.put(tp1, 0L);
         offsets.put(tp2, 0L);
 
-        clientSupplier.restoreConsumer.updateBeginningOffsets(offsets);
-        clientSupplier.restoreConsumer.updateEndOffsets(offsets);
+        restoreConsumer.updateBeginningOffsets(offsets);
+        restoreConsumer.updateEndOffsets(offsets);
 
-        clientSupplier.adminClient.updateBeginningOffsets(offsets);
-        clientSupplier.adminClient.updateEndOffsets(offsets);
+        adminClient.updateBeginningOffsets(offsets);
+        adminClient.updateEndOffsets(offsets);
     }
 }
