@@ -17,14 +17,20 @@
 package org.apache.kafka.streams;
 
 import org.apache.kafka.common.header.internals.RecordHeaders;
+import org.apache.kafka.common.metrics.MetricConfig;
+import org.apache.kafka.common.metrics.Metrics;
 import org.apache.kafka.common.serialization.Serdes;
+import org.apache.kafka.common.utils.Time;
+import org.apache.kafka.streams.kstream.Transformer;
 import org.apache.kafka.streams.processor.MockProcessorContext;
-import org.apache.kafka.streams.processor.MockProcessorContext.CapturedForward;
 import org.apache.kafka.streams.processor.ProcessorContext;
 import org.apache.kafka.streams.processor.PunctuationType;
 import org.apache.kafka.streams.processor.Punctuator;
+import org.apache.kafka.streams.processor.StateStore;
 import org.apache.kafka.streams.processor.TaskId;
 import org.apache.kafka.streams.processor.To;
+import org.apache.kafka.streams.processor.internals.InternalProcessorContext;
+import org.apache.kafka.streams.processor.internals.metrics.StreamsMetricsImpl;
 import org.apache.kafka.streams.state.KeyValueStore;
 import org.apache.kafka.streams.state.StoreBuilder;
 import org.apache.kafka.streams.state.Stores;
@@ -33,7 +39,9 @@ import org.junit.jupiter.api.Test;
 
 import java.io.File;
 import java.time.Duration;
+import java.util.HashMap;
 import java.util.Iterator;
+import java.util.Map;
 import java.util.Properties;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
@@ -41,25 +49,42 @@ import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.junit.jupiter.api.Assertions.fail;
+import static org.mockito.ArgumentMatchers.any;
+import static org.mockito.ArgumentMatchers.anyString;
+import static org.mockito.Mockito.doAnswer;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.when;
 
-@SuppressWarnings("deprecation") // this is a test of a deprecated API
+@Deprecated
 public class MockProcessorContextTest {
+
     @Test
     public void shouldCaptureOutputRecords() {
-        final org.apache.kafka.streams.processor.AbstractProcessor<String, Long> processor = new org.apache.kafka.streams.processor.AbstractProcessor<String, Long>() {
+        final Transformer<String, Long, KeyValue<String, Long>> transformer = new Transformer<>() {
+            private ProcessorContext context;
+
             @Override
-            public void process(final String key, final Long value) {
-                context().forward(key + value, key.length() + value);
+            public void init(final ProcessorContext context) {
+                this.context = context;
             }
+
+            @Override
+            public KeyValue<String, Long> transform(final String key, final Long value) {
+                context.forward(key + value, key.length() + value);
+                return null;
+            }
+
+            @Override
+            public void close() { }
         };
 
         final MockProcessorContext context = new MockProcessorContext();
-        processor.init(context);
+        transformer.init(context);
 
-        processor.process("foo", 5L);
-        processor.process("barbaz", 50L);
+        transformer.transform("foo", 5L);
+        transformer.transform("barbaz", 50L);
 
-        final Iterator<CapturedForward> forwarded = context.forwarded().iterator();
+        final Iterator<MockProcessorContext.CapturedForward> forwarded = context.forwarded().iterator();
         assertEquals(new KeyValue<>("foo5", 8L), forwarded.next().keyValue());
         assertEquals(new KeyValue<>("barbaz50", 56L), forwarded.next().keyValue());
         assertFalse(forwarded.hasNext());
@@ -71,21 +96,32 @@ public class MockProcessorContextTest {
 
     @Test
     public void shouldCaptureOutputRecordsUsingTo() {
-        final org.apache.kafka.streams.processor.AbstractProcessor<String, Long> processor = new org.apache.kafka.streams.processor.AbstractProcessor<String, Long>() {
+        final Transformer<String, Long, KeyValue<String, Long>> transformer = new Transformer<>() {
+            private ProcessorContext context;
+
             @Override
-            public void process(final String key, final Long value) {
-                context().forward(key + value, key.length() + value, To.all());
+            public void init(final ProcessorContext context) {
+                this.context = context;
             }
+
+            @Override
+            public KeyValue<String, Long> transform(final String key, final Long value) {
+                context.forward(key + value, key.length() + value);
+                return null;
+            }
+
+            @Override
+            public void close() { }
         };
 
         final MockProcessorContext context = new MockProcessorContext();
 
-        processor.init(context);
+        transformer.init(context);
 
-        processor.process("foo", 5L);
-        processor.process("barbaz", 50L);
+        transformer.transform("foo", 5L);
+        transformer.transform("barbaz", 50L);
 
-        final Iterator<CapturedForward> forwarded = context.forwarded().iterator();
+        final Iterator<MockProcessorContext.CapturedForward> forwarded = context.forwarded().iterator();
         assertEquals(new KeyValue<>("foo5", 8L), forwarded.next().keyValue());
         assertEquals(new KeyValue<>("barbaz50", 56L), forwarded.next().keyValue());
         assertFalse(forwarded.hasNext());
@@ -97,39 +133,50 @@ public class MockProcessorContextTest {
 
     @Test
     public void shouldCaptureRecordsOutputToChildByName() {
-        final org.apache.kafka.streams.processor.AbstractProcessor<String, Long> processor = new org.apache.kafka.streams.processor.AbstractProcessor<String, Long>() {
+        final Transformer<String, Long, KeyValue<String, Long>> transformer = new Transformer<>() {
+            private ProcessorContext context;
             private int count = 0;
 
             @Override
-            public void process(final String key, final Long value) {
+            public void init(final ProcessorContext context) {
+                this.context = context;
+            }
+
+            @Override
+            public KeyValue<String, Long> transform(final String key, final Long value) {
                 if (count == 0) {
-                    context().forward("start", -1L, To.all()); // broadcast
+                    context.forward("start", -1L, To.all()); // broadcast
                 }
                 final To toChild = count % 2 == 0 ? To.child("george") : To.child("pete");
-                context().forward(key + value, key.length() + value, toChild);
+                context.forward(key + value, key.length() + value, toChild);
                 count++;
+
+                return null;
             }
+
+            @Override
+            public void close() { }
         };
 
         final MockProcessorContext context = new MockProcessorContext();
 
-        processor.init(context);
+        transformer.init(context);
 
-        processor.process("foo", 5L);
-        processor.process("barbaz", 50L);
+        transformer.transform("foo", 5L);
+        transformer.transform("barbaz", 50L);
 
         {
-            final Iterator<CapturedForward> forwarded = context.forwarded().iterator();
+            final Iterator<MockProcessorContext.CapturedForward> forwarded = context.forwarded().iterator();
 
-            final CapturedForward forward1 = forwarded.next();
+            final MockProcessorContext.CapturedForward forward1 = forwarded.next();
             assertEquals(new KeyValue<>("start", -1L), forward1.keyValue());
             assertNull(forward1.childName());
 
-            final CapturedForward forward2 = forwarded.next();
+            final MockProcessorContext.CapturedForward forward2 = forwarded.next();
             assertEquals(new KeyValue<>("foo5", 8L), forward2.keyValue());
             assertEquals("george", forward2.childName());
 
-            final CapturedForward forward3 = forwarded.next();
+            final MockProcessorContext.CapturedForward forward3 = forwarded.next();
             assertEquals(new KeyValue<>("barbaz50", 56L), forward3.keyValue());
             assertEquals("pete", forward3.childName());
 
@@ -137,21 +184,21 @@ public class MockProcessorContextTest {
         }
 
         {
-            final Iterator<CapturedForward> forwarded = context.forwarded("george").iterator();
+            final Iterator<MockProcessorContext.CapturedForward> forwarded = context.forwarded("george").iterator();
             assertEquals(new KeyValue<>("start", -1L), forwarded.next().keyValue());
             assertEquals(new KeyValue<>("foo5", 8L), forwarded.next().keyValue());
             assertFalse(forwarded.hasNext());
         }
 
         {
-            final Iterator<CapturedForward> forwarded = context.forwarded("pete").iterator();
+            final Iterator<MockProcessorContext.CapturedForward> forwarded = context.forwarded("pete").iterator();
             assertEquals(new KeyValue<>("start", -1L), forwarded.next().keyValue());
             assertEquals(new KeyValue<>("barbaz50", 56L), forwarded.next().keyValue());
             assertFalse(forwarded.hasNext());
         }
 
         {
-            final Iterator<CapturedForward> forwarded = context.forwarded("steve").iterator();
+            final Iterator<MockProcessorContext.CapturedForward> forwarded = context.forwarded("steve").iterator();
             assertEquals(new KeyValue<>("start", -1L), forwarded.next().keyValue());
             assertFalse(forwarded.hasNext());
         }
@@ -159,27 +206,37 @@ public class MockProcessorContextTest {
 
     @Test
     public void shouldCaptureCommitsAndAllowReset() {
-        final org.apache.kafka.streams.processor.AbstractProcessor<String, Long> processor = new org.apache.kafka.streams.processor.AbstractProcessor<String, Long>() {
+        final Transformer<String, Long, ?> transformer = new Transformer<>() {
+            private ProcessorContext context;
             private int count = 0;
 
             @Override
-            public void process(final String key, final Long value) {
-                if (++count > 2) {
-                    context().commit();
-                }
+            public void init(final ProcessorContext context) {
+                this.context = context;
             }
+
+            @Override
+            public Object transform(final String key, final Long value) {
+                if (++count > 2) {
+                    context.commit();
+                }
+                return null;
+            }
+
+            @Override
+            public void close() { }
         };
 
         final MockProcessorContext context = new MockProcessorContext();
 
-        processor.init(context);
+        transformer.init(context);
 
-        processor.process("foo", 5L);
-        processor.process("barbaz", 50L);
+        transformer.transform("foo", 5L);
+        transformer.transform("barbaz", 50L);
 
         assertFalse(context.committed());
 
-        processor.process("foobar", 500L);
+        transformer.transform("foobar", 500L);
 
         assertTrue(context.committed());
 
@@ -190,30 +247,60 @@ public class MockProcessorContextTest {
 
     @Test
     public void shouldStoreAndReturnStateStores() {
-        final org.apache.kafka.streams.processor.AbstractProcessor<String, Long> processor = new org.apache.kafka.streams.processor.AbstractProcessor<String, Long>() {
+        final Transformer<String, Long, ?> transformer = new Transformer<>() {
+            private ProcessorContext context;
+
             @Override
-            public void process(final String key, final Long value) {
-                final KeyValueStore<String, Long> stateStore = context().getStateStore("my-state");
+            public void init(final ProcessorContext context) {
+                this.context = context;
+            }
+
+            @Override
+            public Object transform(final String key, final Long value) {
+                final KeyValueStore<String, Long> stateStore = context.getStateStore("my-state");
                 stateStore.put(key, (stateStore.get(key) == null ? 0 : stateStore.get(key)) + value);
                 stateStore.put("all", (stateStore.get("all") == null ? 0 : stateStore.get("all")) + value);
+
+                return null;
             }
+
+            @Override
+            public void close() { }
         };
 
-        final MockProcessorContext context = new MockProcessorContext();
-
         final StoreBuilder<KeyValueStore<String, Long>> storeBuilder = Stores.keyValueStoreBuilder(
-                Stores.inMemoryKeyValueStore("my-state"),
-                Serdes.String(),
-                Serdes.Long()).withLoggingDisabled();
+            Stores.inMemoryKeyValueStore("my-state"),
+            Serdes.String(),
+            Serdes.Long()).withLoggingDisabled();
 
         final KeyValueStore<String, Long> store = storeBuilder.build();
 
-        store.init(context, store);
+        final InternalProcessorContext<?, ?> mockInternalProcessorContext = mock(InternalProcessorContext.class);
+        final Map<String, StateStore> stateStores = new HashMap<>();
+        doAnswer(invocation -> {
+            final StateStore stateStore = invocation.getArgument(0);
+            stateStores.put(stateStore.name(), stateStore);
+            return null;
+        }).when(mockInternalProcessorContext).register(any(), any());
+        when(mockInternalProcessorContext.getStateStore(anyString())).thenAnswer(invocation -> {
+                final String name = invocation.getArgument(0);
+                return stateStores.get(name);
+            }
+        );
+        when(mockInternalProcessorContext.metrics()).thenReturn(new StreamsMetricsImpl(
+            new Metrics(new MetricConfig()),
+            Thread.currentThread().getName(),
+            "processId",
+            Time.SYSTEM
+        ));
+        when(mockInternalProcessorContext.taskId()).thenReturn(new TaskId(1, 1));
 
-        processor.init(context);
+        store.init(mockInternalProcessorContext, store);
 
-        processor.process("foo", 5L);
-        processor.process("bar", 50L);
+        transformer.init(mockInternalProcessorContext);
+
+        transformer.transform("foo", 5L);
+        transformer.transform("bar", 50L);
 
         assertEquals(5L, (long) store.get("foo"));
         assertEquals(50L, (long) store.get("bar"));
@@ -225,27 +312,39 @@ public class MockProcessorContextTest {
         final Properties config = new Properties();
         config.put(StreamsConfig.APPLICATION_ID_CONFIG, "testMetadata");
 
-        final org.apache.kafka.streams.processor.AbstractProcessor<String, Object> processor = new org.apache.kafka.streams.processor.AbstractProcessor<String, Object>() {
+        final Transformer<String, Long, KeyValue<String, Object>> transformer = new Transformer<>() {
+            private ProcessorContext context;
+
             @Override
-            public void process(final String key, final Object value) {
-                context().forward("appId", context().applicationId());
-                context().forward("taskId", context().taskId());
-
-                context().forward("topic", context().topic());
-                context().forward("partition", context().partition());
-                context().forward("offset", context().offset());
-                context().forward("timestamp", context().timestamp());
-
-                context().forward("key", key);
-                context().forward("value", value);
+            public void init(final ProcessorContext context) {
+                this.context = context;
             }
+
+            @Override
+            public KeyValue<String, Object> transform(final String key, final Long value) {
+                context.forward("appId", context.applicationId());
+                context.forward("taskId", context.taskId());
+
+                context.forward("topic", context.topic());
+                context.forward("partition", context.partition());
+                context.forward("offset", context.offset());
+                context.forward("timestamp", context.timestamp());
+
+                context.forward("key", key);
+                context.forward("value", value);
+
+                return null;
+            }
+
+            @Override
+            public void close() { }
         };
 
         final MockProcessorContext context = new MockProcessorContext(config);
-        processor.init(context);
+        transformer.init(context);
 
         try {
-            processor.process("foo", 5L);
+            transformer.transform("foo", 5L);
             fail("Should have thrown an exception.");
         } catch (final IllegalStateException expected) {
             // expected, since the record metadata isn't initialized
@@ -255,8 +354,8 @@ public class MockProcessorContextTest {
         context.setRecordMetadata("t1", 0, 0L, new RecordHeaders(), 0L);
 
         {
-            processor.process("foo", 5L);
-            final Iterator<CapturedForward> forwarded = context.forwarded().iterator();
+            transformer.transform("foo", 5L);
+            final Iterator<MockProcessorContext.CapturedForward> forwarded = context.forwarded().iterator();
             assertEquals(new KeyValue<>("appId", "testMetadata"), forwarded.next().keyValue());
             assertEquals(new KeyValue<>("taskId", new TaskId(0, 0)), forwarded.next().keyValue());
             assertEquals(new KeyValue<>("topic", "t1"), forwarded.next().keyValue());
@@ -276,8 +375,8 @@ public class MockProcessorContextTest {
         context.setCurrentStreamTimeMs(30L);
 
         {
-            processor.process("bar", 50L);
-            final Iterator<CapturedForward> forwarded = context.forwarded().iterator();
+            transformer.transform("bar", 50L);
+            final Iterator<MockProcessorContext.CapturedForward> forwarded = context.forwarded().iterator();
             assertEquals(new KeyValue<>("appId", "testMetadata"), forwarded.next().keyValue());
             assertEquals(new KeyValue<>("taskId", new TaskId(0, 0)), forwarded.next().keyValue());
             assertEquals(new KeyValue<>("topic", "t1"), forwarded.next().keyValue());
@@ -296,8 +395,8 @@ public class MockProcessorContextTest {
         context.setPartition(30);
 
         {
-            processor.process("baz", 500L);
-            final Iterator<CapturedForward> forwarded = context.forwarded().iterator();
+            transformer.transform("baz", 500L);
+            final Iterator<MockProcessorContext.CapturedForward> forwarded = context.forwarded().iterator();
             assertEquals(new KeyValue<>("appId", "testMetadata"), forwarded.next().keyValue());
             assertEquals(new KeyValue<>("taskId", new TaskId(0, 0)), forwarded.next().keyValue());
             assertEquals(new KeyValue<>("topic", "t2"), forwarded.next().keyValue());
@@ -311,7 +410,7 @@ public class MockProcessorContextTest {
 
     @Test
     public void shouldCapturePunctuator() {
-        final org.apache.kafka.streams.processor.Processor<String, Long> processor = new org.apache.kafka.streams.processor.Processor<String, Long>() {
+        final Transformer<String, Long, KeyValue<String, Long>> transformer = new Transformer<>() {
             @Override
             public void init(final ProcessorContext context) {
                 context.schedule(
@@ -322,17 +421,17 @@ public class MockProcessorContextTest {
             }
 
             @Override
-            public void process(final String key, final Long value) {
+            public KeyValue<String, Long> transform(final String key, final Long value) {
+                return null;
             }
 
             @Override
-            public void close() {
-            }
+            public void close() { }
         };
 
         final MockProcessorContext context = new MockProcessorContext();
 
-        processor.init(context);
+        transformer.init(context);
 
         final MockProcessorContext.CapturedPunctuator capturedPunctuator = context.scheduledPunctuators().get(0);
         assertEquals(1000L, capturedPunctuator.getIntervalMs());
@@ -345,12 +444,13 @@ public class MockProcessorContextTest {
         assertTrue(context.committed());
     }
 
+    @SuppressWarnings("resource")
     @Test
     public void fullConstructorShouldSetAllExpectedAttributes() {
         final Properties config = new Properties();
         config.put(StreamsConfig.APPLICATION_ID_CONFIG, "testFullConstructor");
-        config.put(StreamsConfig.DEFAULT_KEY_SERDE_CLASS_CONFIG, Serdes.String().getClass());
-        config.put(StreamsConfig.DEFAULT_VALUE_SERDE_CLASS_CONFIG, Serdes.Long().getClass());
+        config.put(StreamsConfig.DEFAULT_KEY_SERDE_CLASS_CONFIG, Serdes.StringSerde.class);
+        config.put(StreamsConfig.DEFAULT_VALUE_SERDE_CLASS_CONFIG, Serdes.LongSerde.class);
 
         final File dummyFile = new File("");
         final MockProcessorContext context = new MockProcessorContext(config, new TaskId(1, 1), dummyFile);
@@ -359,8 +459,8 @@ public class MockProcessorContextTest {
         assertEquals(new TaskId(1, 1), context.taskId());
         assertEquals("testFullConstructor", context.appConfigs().get(StreamsConfig.APPLICATION_ID_CONFIG));
         assertEquals("testFullConstructor", context.appConfigsWithPrefix("application.").get("id"));
-        assertEquals(Serdes.String().getClass(), context.keySerde().getClass());
-        assertEquals(Serdes.Long().getClass(), context.valueSerde().getClass());
+        assertEquals(Serdes.StringSerde.class, context.keySerde().getClass());
+        assertEquals(Serdes.LongSerde.class, context.valueSerde().getClass());
         assertEquals(dummyFile, context.stateDir());
     }
 }
