@@ -13,5 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from kafka import KafkaService
-from util import TopicPartition
+from .kafka import KafkaService
+from .util import TopicPartition
+from .config import KafkaConfig
