@@ -29,14 +29,16 @@ class ReplicaVerificationTool(KafkaPathResolverMixin, BackgroundThreadService):
             "collect_default": False}
     }
 
-    def __init__(self, context, num_nodes, kafka, topic, report_interval_ms, security_protocol="PLAINTEXT", stop_timeout_sec=30):
+    def __init__(self, context, num_nodes, kafka, topic, report_interval_ms, security_protocol="PLAINTEXT",
+                 stop_timeout_sec=30, tls_version=None):
         super(ReplicaVerificationTool, self).__init__(context, num_nodes)
 
         self.kafka = kafka
         self.topic = topic
         self.report_interval_ms = report_interval_ms
         self.security_protocol = security_protocol
-        self.security_config = SecurityConfig(self.context, security_protocol)
+        self.tls_version = tls_version
+        self.security_config = SecurityConfig(self.context, security_protocol, tls_version=tls_version)
         self.partition_lag = {}
         self.stop_timeout_sec = stop_timeout_sec
 
@@ -69,15 +71,14 @@ class ReplicaVerificationTool(KafkaPathResolverMixin, BackgroundThreadService):
         return lag
 
     def start_cmd(self, node):
-        cmd = self.path.script("kafka-run-class.sh", node)
-        cmd += " %s" % self.java_class_name()
-        cmd += " --broker-list %s --topic-white-list %s --time -2 --report-interval-ms %s" % (self.kafka.bootstrap_servers(self.security_protocol), self.topic, self.report_interval_ms)
+        cmd = self.path.script("kafka-replica-verification.sh", node)
+        cmd += " --broker-list %s --topics-include %s --time -2 --report-interval-ms %s" % (self.kafka.bootstrap_servers(self.security_protocol), self.topic, self.report_interval_ms)
 
         cmd += " 2>> /mnt/replica_verification_tool.log | tee -a /mnt/replica_verification_tool.log &"
         return cmd
 
     def stop_node(self, node):
-        node.account.kill_java_processes(self.java_class_name(), clean_shutdown=True,
+        node.account.kill_java_processes("ReplicaVerificationTool", clean_shutdown=True,
                                          allow_fail=True)
 
         stopped = self.wait_node(node, timeout_sec=self.stop_timeout_sec)
@@ -85,9 +86,6 @@ class ReplicaVerificationTool(KafkaPathResolverMixin, BackgroundThreadService):
                         (str(node.account), str(self.stop_timeout_sec))
 
     def clean_node(self, node):
-        node.account.kill_java_processes(self.java_class_name(), clean_shutdown=False,
+        node.account.kill_java_processes("ReplicaVerificationTool", clean_shutdown=False,
                                          allow_fail=True)
         node.account.ssh("rm -rf /mnt/replica_verification_tool.log", allow_fail=False)
-
-    def java_class_name(self):
-        return "kafka.tools.ReplicaVerificationTool"
