@@ -26,31 +26,20 @@ from ducktape.tests.test import TestContext
 from kafkatest.services.zookeeper import ZookeeperService
 from kafkatest.services.kafka import KafkaService, quorum
 from ducktape.tests.test import Test
-from kafkatest.version import DEV_BRANCH, LATEST_0_10_0, LATEST_0_10_1, LATEST_0_10_2, LATEST_0_11_0, LATEST_1_0, \
-    LATEST_1_1, LATEST_2_0, LATEST_2_1, LATEST_2_2, LATEST_2_3, LATEST_2_4, LATEST_2_5, LATEST_2_6, LATEST_2_7, \
-    LATEST_2_8, LATEST_3_0, LATEST_3_1, LATEST_3_2, LATEST_3_3, LATEST_3_4, LATEST_3_5, LATEST_3_6, LATEST_3_7, \
-    V_0_11_0_0, V_0_10_1_0, KafkaVersion
+from kafkatest.version import DEV_BRANCH, \
+    LATEST_2_1, LATEST_2_2, LATEST_2_3, LATEST_2_4, LATEST_2_5, LATEST_2_6, LATEST_2_7, LATEST_2_8, \
+    LATEST_3_0, LATEST_3_1, LATEST_3_2, LATEST_3_3, LATEST_3_4, LATEST_3_5, LATEST_3_6, LATEST_3_7, LATEST_3_8, LATEST_3_9, KafkaVersion
 
 def get_broker_features(broker_version):
     features = {}
-    if broker_version < V_0_10_1_0:
-        features["create-topics-supported"] = False
-        features["offsets-for-times-supported"] = False
-        features["cluster-id-supported"] = False
-        features["expect-record-too-large-exception"] = True
-    else:
-        features["create-topics-supported"] = True
-        features["offsets-for-times-supported"] = True
-        features["cluster-id-supported"] = True
-        features["expect-record-too-large-exception"] = False
-    if broker_version < V_0_11_0_0:
-        features["describe-acls-supported"] = False
-        features["describe-configs-supported"] = False
-        features["idempotent-producer-supported"] = False
-    else:
-        features["describe-acls-supported"] = True
-        features["describe-configs-supported"] = True
-        features["idempotent-producer-supported"] = True
+    features["create-topics-supported"] = True
+    features["offsets-for-times-supported"] = True
+    features["cluster-id-supported"] = True
+    features["expect-record-too-large-exception"] = False
+
+    features["describe-acls-supported"] = True
+    features["describe-configs-supported"] = True
+    features["idempotent-producer-supported"] = True
     return features
 
 def run_command(node, cmd, ssh_log_file):
@@ -64,7 +53,6 @@ def run_command(node, cmd, ssh_log_file):
             print(e, flush=True)
             raise
 
-
 class ClientCompatibilityFeaturesTest(Test):
     """
     Tests clients for the presence or absence of specific features when communicating with brokers with various
@@ -118,13 +106,6 @@ class ClientCompatibilityFeaturesTest(Test):
 
     @cluster(num_nodes=7)
     @matrix(broker_version=[str(DEV_BRANCH)], metadata_quorum=quorum.all_non_upgrade)
-    @parametrize(broker_version=str(LATEST_0_10_0))
-    @parametrize(broker_version=str(LATEST_0_10_1))
-    @parametrize(broker_version=str(LATEST_0_10_2))
-    @parametrize(broker_version=str(LATEST_0_11_0))
-    @parametrize(broker_version=str(LATEST_1_0))
-    @parametrize(broker_version=str(LATEST_1_1))
-    @parametrize(broker_version=str(LATEST_2_0))
     @parametrize(broker_version=str(LATEST_2_1))
     @parametrize(broker_version=str(LATEST_2_2))
     @parametrize(broker_version=str(LATEST_2_3))
@@ -136,15 +117,20 @@ class ClientCompatibilityFeaturesTest(Test):
     @parametrize(broker_version=str(LATEST_3_0))
     @parametrize(broker_version=str(LATEST_3_1))
     @parametrize(broker_version=str(LATEST_3_2))
-    @parametrize(broker_version=str(LATEST_3_3))
-    @parametrize(broker_version=str(LATEST_3_4))
-    @parametrize(broker_version=str(LATEST_3_5))
-    @parametrize(broker_version=str(LATEST_3_6))
-    @parametrize(broker_version=str(LATEST_3_7))
+    @parametrize(broker_version=str(LATEST_3_3), metadata_quorum=quorum.isolated_kraft)
+    @parametrize(broker_version=str(LATEST_3_4), metadata_quorum=quorum.isolated_kraft)
+    @parametrize(broker_version=str(LATEST_3_5), metadata_quorum=quorum.isolated_kraft)
+    @parametrize(broker_version=str(LATEST_3_6), metadata_quorum=quorum.isolated_kraft)
+    @parametrize(broker_version=str(LATEST_3_7), metadata_quorum=quorum.isolated_kraft)
+    @parametrize(broker_version=str(LATEST_3_8), metadata_quorum=quorum.isolated_kraft)
+    @parametrize(broker_version=str(LATEST_3_9), metadata_quorum=quorum.isolated_kraft)
     def run_compatibility_test(self, broker_version, metadata_quorum=quorum.zk):
         if self.zk:
             self.zk.start()
         self.kafka.set_version(KafkaVersion(broker_version))
+        if metadata_quorum == quorum.isolated_kraft:
+            for node in self.kafka.controller_quorum.nodes:
+                node.version = KafkaVersion(broker_version)
         self.kafka.start()
         features = get_broker_features(broker_version)
         self.invoke_compatibility_program(features)
