@@ -18,7 +18,6 @@ from ducktape.mark.resource import cluster
 from ducktape.utils.util import wait_until
 
 from kafkatest.services.kafka import config_property
-from kafkatest.services.zookeeper import ZookeeperService
 from kafkatest.services.kafka import KafkaService, quorum, consumer_group, TopicPartition
 from kafkatest.services.verifiable_producer import VerifiableProducer
 from kafkatest.services.console_consumer import ConsoleConsumer
@@ -35,22 +34,21 @@ class ReassignPartitionsTest(ProduceConsumeValidateTest):
     """
 
     def __init__(self, test_context):
-        self.num_zk = 1
         """:type test_context: ducktape.tests.test.TestContext"""
         super(ReassignPartitionsTest, self).__init__(test_context=test_context)
 
         self.topic = "test_topic"
         self.num_partitions = 20
-        self.zk = ZookeeperService(test_context, self.num_zk) if quorum.for_test(test_context) == quorum.zk else None
         # We set the min.insync.replicas to match the replication factor because
         # it makes the test more stringent. If min.isr = 2 and
         # replication.factor=3, then the test would tolerate the failure of
         # reassignment for upto one replica per partition, which is not
         # desirable for this test in particular.
-        self.kafka = KafkaService(test_context, num_nodes=4, zk=self.zk,
+        self.kafka = KafkaService(test_context, num_nodes=4, zk=None,
                                   server_prop_overrides=[
                                       [config_property.LOG_ROLL_TIME_MS, "5000"],
-                                      [config_property.LOG_RETENTION_CHECK_INTERVAL_MS, "5000"]
+                                      [config_property.LOG_RETENTION_CHECK_INTERVAL_MS, "5000"],
+                                      [config_property.LOG_INITIAL_TASK_DELAY, "5000"]
                                   ],
                                   topics={self.topic: {
                                       "partitions": self.num_partitions,
@@ -59,16 +57,12 @@ class ReassignPartitionsTest(ProduceConsumeValidateTest):
                                           "min.insync.replicas": 3,
                                       }}
                                   },
-                                  controller_num_nodes_override=self.num_zk)
+                                  controller_num_nodes_override=1)
         self.timeout_sec = 60
         self.producer_throughput = 1000
         self.num_producers = 1
         self.num_consumers = 1
 
-    def setUp(self):
-        if self.zk:
-            self.zk.start()
-
     def min_cluster_size(self):
         # Override this since we're adding services outside of the constructor
         return super(ReassignPartitionsTest, self).min_cluster_size() + self.num_producers + self.num_consumers
@@ -104,7 +98,7 @@ class ReassignPartitionsTest(ProduceConsumeValidateTest):
         # ensure all partitions have data so we don't hit OutOfOrderExceptions due to broker restarts
         wait_until(check_all_partitions,
                    timeout_sec=60,
-                   err_msg="Failed to produce to all partitions in 30s")
+                   err_msg="Failed to produce to all partitions in 60s")
 
         # send reassign partitions command
         self.kafka.execute_reassign_partitions(partition_info)
@@ -137,17 +131,17 @@ class ReassignPartitionsTest(ProduceConsumeValidateTest):
         self.logger.info("Seeded topic with %d messages which will be deleted" %\
                          producer.num_acked)
         # Since the configured check interval is 5 seconds, we wait another
-        # 6 seconds to ensure that at least one more cleaning so that the last
-        # segment is deleted. An altenate to using timeouts is to poll each
+        # 12 seconds to ensure that at least one more cleaning so that the last
+        # segment is deleted. An alternate to using timeouts is to poll each
         # partition until the log start offset matches the end offset. The
         # latter is more robust.
-        time.sleep(6)
+        time.sleep(12)
 
     @cluster(num_nodes=8)
     @matrix(
         bounce_brokers=[True, False],
         reassign_from_offset_zero=[True, False],
-        metadata_quorum=[quorum.zk, quorum.isolated_kraft],
+        metadata_quorum=[quorum.isolated_kraft],
         use_new_coordinator=[False]
     )
     @matrix(
@@ -159,7 +153,7 @@ class ReassignPartitionsTest(ProduceConsumeValidateTest):
     )
     def test_reassign_partitions(self, bounce_brokers, reassign_from_offset_zero, metadata_quorum, use_new_coordinator=False, group_protocol=None):
         """Reassign partitions tests.
-        Setup: 1 zk, 4 kafka nodes, 1 topic with partitions=20, replication-factor=3,
+        Setup: 1 controller, 4 kafka nodes, 1 topic with partitions=20, replication-factor=3,
         and min.insync.replicas=3
 
             - Produce messages in the background
