@@ -61,7 +61,7 @@ class SecurityTest(EndToEndTest):
     @matrix(
         security_protocol=['PLAINTEXT'],
         interbroker_security_protocol=['SSL'],
-        metadata_quorum=[quorum.zk, quorum.isolated_kraft],
+        metadata_quorum=[quorum.isolated_kraft],
         use_new_coordinator=[False]
     )
     @matrix(
@@ -74,7 +74,7 @@ class SecurityTest(EndToEndTest):
     @matrix(
         security_protocol=['SSL'],
         interbroker_security_protocol=['PLAINTEXT'],
-        metadata_quorum=[quorum.zk, quorum.isolated_kraft],
+        metadata_quorum=[quorum.isolated_kraft],
         use_new_coordinator=[False]
     )
     @matrix(
@@ -84,7 +84,7 @@ class SecurityTest(EndToEndTest):
         use_new_coordinator=[True],
         group_protocol=consumer_group.all_group_protocols
     )
-    def test_client_ssl_endpoint_validation_failure(self, security_protocol, interbroker_security_protocol, metadata_quorum=quorum.zk, use_new_coordinator=False, group_protocol=None):
+    def test_client_ssl_endpoint_validation_failure(self, security_protocol, interbroker_security_protocol, metadata_quorum, use_new_coordinator=False, group_protocol=None):
         """
         Test that invalid hostname in certificate results in connection failures.
         When security_protocol=SSL, client SSL handshakes are expected to fail due to hostname verification failure.
@@ -99,10 +99,6 @@ class SecurityTest(EndToEndTest):
         SecurityConfig.ssl_stores = TestSslStores(self.test_context.local_scratch_dir,
                                                   valid_hostname=True)
 
-        self.create_zookeeper_if_necessary()
-        if self.zk:
-            self.zk.start()
-
         self.create_kafka(security_protocol=security_protocol,
                           interbroker_security_protocol=interbroker_security_protocol)
         if self.kafka.quorum_info.using_kraft and interbroker_security_protocol == 'SSL':
@@ -157,7 +153,7 @@ class SecurityTest(EndToEndTest):
 
     @cluster(num_nodes=2)
     @matrix(
-        metadata_quorum=[quorum.zk, quorum.isolated_kraft],
+        metadata_quorum=[quorum.isolated_kraft],
         use_new_coordinator=[False]
     )
     @matrix(
@@ -165,26 +161,17 @@ class SecurityTest(EndToEndTest):
         use_new_coordinator=[True],
         group_protocol=consumer_group.all_group_protocols
     )
-    def test_quorum_ssl_endpoint_validation_failure(self, metadata_quorum=quorum.zk, use_new_coordinator=False, group_protocol=None):
+    def test_quorum_ssl_endpoint_validation_failure(self, metadata_quorum, use_new_coordinator=False, group_protocol=None):
         """
         Test that invalid hostname in ZooKeeper or KRaft Controller results in broker inability to start.
         """
-        # Start ZooKeeper/KRaft Controller with valid hostnames in the certs' SANs
+        # Start KRaft Controller with valid hostnames in the certs' SANs
         # so that we can start Kafka
         SecurityConfig.ssl_stores = TestSslStores(self.test_context.local_scratch_dir,
                                                   valid_hostname=True)
 
-        self.create_zookeeper_if_necessary(num_nodes=1,
-                                           zk_client_port = False,
-                                           zk_client_secure_port = True,
-                                           zk_tls_encrypt_only = True,
-                                           )
-        if self.zk:
-            self.zk.start()
-
         self.create_kafka(num_nodes=1,
                           interbroker_security_protocol='SSL', # also sets the broker-to-kraft-controller security protocol for the KRaft case
-                          zk_client_secure=True, # ignored if we aren't using ZooKeeper
                           )
         self.kafka.start()
 
@@ -194,10 +181,7 @@ class SecurityTest(EndToEndTest):
         self.kafka.stop_node(self.kafka.nodes[0])
 
         SecurityConfig.ssl_stores.valid_hostname = False
-        if quorum.for_test(self.test_context) == quorum.zk:
-            self.kafka.zk.restart_cluster()
-        else:
-            self.kafka.isolated_controller_quorum.restart_cluster()
+        self.kafka.isolated_controller_quorum.restart_cluster()
 
         try:
             self.kafka.start_node(self.kafka.nodes[0], timeout_sec=30)
