@@ -39,21 +39,19 @@ class StreamsEosTest(KafkaTest):
         self.test_context = test_context
 
     @cluster(num_nodes=9)
-    @matrix(processing_guarantee=["exactly_once", "exactly_once_v2"],
-            metadata_quorum=[quorum.isolated_kraft])
-    def test_rebalance_simple(self, processing_guarantee, metadata_quorum):
-        self.run_rebalance(StreamsEosTestJobRunnerService(self.test_context, self.kafka, processing_guarantee),
-                           StreamsEosTestJobRunnerService(self.test_context, self.kafka, processing_guarantee),
-                           StreamsEosTestJobRunnerService(self.test_context, self.kafka, processing_guarantee),
+    @matrix(metadata_quorum=[quorum.combined_kraft])
+    def test_rebalance_simple(self, metadata_quorum):
+        self.run_rebalance(StreamsEosTestJobRunnerService(self.test_context, self.kafka),
+                           StreamsEosTestJobRunnerService(self.test_context, self.kafka),
+                           StreamsEosTestJobRunnerService(self.test_context, self.kafka),
                            StreamsEosTestVerifyRunnerService(self.test_context, self.kafka))
 
     @cluster(num_nodes=9)
-    @matrix(processing_guarantee=["exactly_once", "exactly_once_v2"],
-            metadata_quorum=[quorum.isolated_kraft])
-    def test_rebalance_complex(self, processing_guarantee, metadata_quorum):
-        self.run_rebalance(StreamsComplexEosTestJobRunnerService(self.test_context, self.kafka, processing_guarantee),
-                           StreamsComplexEosTestJobRunnerService(self.test_context, self.kafka, processing_guarantee),
-                           StreamsComplexEosTestJobRunnerService(self.test_context, self.kafka, processing_guarantee),
+    @matrix(metadata_quorum=[quorum.combined_kraft])
+    def test_rebalance_complex(self, metadata_quorum):
+        self.run_rebalance(StreamsComplexEosTestJobRunnerService(self.test_context, self.kafka),
+                           StreamsComplexEosTestJobRunnerService(self.test_context, self.kafka),
+                           StreamsComplexEosTestJobRunnerService(self.test_context, self.kafka),
                            StreamsComplexEosTestVerifyRunnerService(self.test_context, self.kafka))
 
     def run_rebalance(self, processor1, processor2, processor3, verifier):
@@ -83,21 +81,19 @@ class StreamsEosTest(KafkaTest):
         verifier.node.account.ssh("grep ALL-RECORDS-DELIVERED %s" % verifier.STDOUT_FILE, allow_fail=False)
 
     @cluster(num_nodes=9)
-    @matrix(processing_guarantee=["exactly_once", "exactly_once_v2"],
-            metadata_quorum=[quorum.isolated_kraft])
-    def test_failure_and_recovery(self, processing_guarantee, metadata_quorum):
-        self.run_failure_and_recovery(StreamsEosTestJobRunnerService(self.test_context, self.kafka, processing_guarantee),
-                                      StreamsEosTestJobRunnerService(self.test_context, self.kafka, processing_guarantee),
-                                      StreamsEosTestJobRunnerService(self.test_context, self.kafka, processing_guarantee),
+    @matrix(metadata_quorum=[quorum.combined_kraft])
+    def test_failure_and_recovery(self, metadata_quorum):
+        self.run_failure_and_recovery(StreamsEosTestJobRunnerService(self.test_context, self.kafka),
+                                      StreamsEosTestJobRunnerService(self.test_context, self.kafka),
+                                      StreamsEosTestJobRunnerService(self.test_context, self.kafka),
                                       StreamsEosTestVerifyRunnerService(self.test_context, self.kafka))
 
     @cluster(num_nodes=9)
-    @matrix(processing_guarantee=["exactly_once", "exactly_once_v2"],
-            metadata_quorum=[quorum.isolated_kraft])
-    def test_failure_and_recovery_complex(self, processing_guarantee, metadata_quorum):
-        self.run_failure_and_recovery(StreamsComplexEosTestJobRunnerService(self.test_context, self.kafka, processing_guarantee),
-                                      StreamsComplexEosTestJobRunnerService(self.test_context, self.kafka, processing_guarantee),
-                                      StreamsComplexEosTestJobRunnerService(self.test_context, self.kafka, processing_guarantee),
+    @matrix(metadata_quorum=[quorum.combined_kraft])
+    def test_failure_and_recovery_complex(self, metadata_quorum):
+        self.run_failure_and_recovery(StreamsComplexEosTestJobRunnerService(self.test_context, self.kafka),
+                                      StreamsComplexEosTestJobRunnerService(self.test_context, self.kafka),
+                                      StreamsComplexEosTestJobRunnerService(self.test_context, self.kafka),
                                       StreamsComplexEosTestVerifyRunnerService(self.test_context, self.kafka))
 
     def run_failure_and_recovery(self, processor1, processor2, processor3, verifier):
