@@ -21,7 +21,6 @@ from kafkatest.services.kafka import KafkaService, quorum
 from kafkatest.services.streams import StreamsOptimizedUpgradeTestService
 from kafkatest.services.streams import StreamsResetter
 from kafkatest.services.verifiable_producer import VerifiableProducer
-from kafkatest.services.zookeeper import ZookeeperService
 from kafkatest.tests.streams.utils import stop_processors
 
 class StreamsOptimizedTest(Test):
@@ -46,13 +45,8 @@ class StreamsOptimizedTest(Test):
             self.join_topic: {'partitions': 6}
         }
 
-        self.zookeeper = (
-            ZookeeperService(self.test_context, 1)
-            if quorum.for_test(self.test_context) == quorum.zk
-            else None
-        )
-        self.kafka = KafkaService(self.test_context, num_nodes=3,
-                                  zk=self.zookeeper, topics=self.topics, controller_num_nodes_override=1)
+        self.kafka = KafkaService(self.test_context, num_nodes=3, controller_num_nodes_override=1,
+                                  zk=None, topics=self.topics)
 
         self.producer = VerifiableProducer(self.test_context,
                                            1,
@@ -62,10 +56,8 @@ class StreamsOptimizedTest(Test):
                                            acks=1)
 
     @cluster(num_nodes=9)
-    @matrix(metadata_quorum=[quorum.isolated_kraft])
+    @matrix(metadata_quorum=[quorum.combined_kraft])
     def test_upgrade_optimized_topology(self, metadata_quorum):
-        if self.zookeeper:
-            self.zookeeper.start()
         self.kafka.start()
 
         processor1 = StreamsOptimizedUpgradeTestService(self.test_context, self.kafka)
@@ -111,8 +103,6 @@ class StreamsOptimizedTest(Test):
         self.logger.info("teardown")
         self.producer.stop()
         self.kafka.stop()
-        if self.zookeeper:
-            self.zookeeper.stop()
 
     def reset_application(self):
         resetter = StreamsResetter(self.test_context, self.kafka, topic = self.input_topic, applicationId = 'StreamsOptimizedTest')
