@@ -51,7 +51,7 @@ class StreamsStandbyTask(BaseStreamsTest):
     def test_standby_tasks_rebalance(self, metadata_quorum, use_new_coordinator=False):
         # TODO KIP-441: consider rewriting the test for HighAvailabilityTaskAssignor
         configs = self.get_configs(
-            ",sourceTopic=%s,sinkTopic1=%s,sinkTopic2=%s,internal.task.assignor.class=org.apache.kafka.streams.processor.internals.assignment.StickyTaskAssignor" % (
+            ",sourceTopic=%s,sinkTopic1=%s,sinkTopic2=%s,internal.task.assignor.class=org.apache.kafka.streams.processor.internals.assignment.LegacyStickyTaskAssignor" % (
             self.streams_source_topic,
             self.streams_sink_topic_1,
             self.streams_sink_topic_2
