@@ -19,7 +19,6 @@ from ducktape.tests.test import Test
 from kafkatest.services.kafka import KafkaService, quorum
 from kafkatest.services.streams import StaticMemberTestService
 from kafkatest.services.verifiable_producer import VerifiableProducer
-from kafkatest.services.zookeeper import ZookeeperService
 from kafkatest.tests.streams.utils import verify_stopped, stop_processors, verify_running, extract_generation_from_logs, extract_generation_id
 
 class StreamsStaticMembershipTest(Test):
@@ -39,13 +38,8 @@ class StreamsStaticMembershipTest(Test):
             self.input_topic: {'partitions': 18},
         }
 
-        self.zookeeper = (
-            ZookeeperService(self.test_context, 1)
-            if quorum.for_test(self.test_context) == quorum.zk
-            else None
-        )
         self.kafka = KafkaService(self.test_context, num_nodes=3,
-                                  zk=self.zookeeper, topics=self.topics, controller_num_nodes_override=1)
+                                  zk=None, topics=self.topics, controller_num_nodes_override=1)
 
         self.producer = VerifiableProducer(self.test_context,
                                            1,
@@ -57,8 +51,6 @@ class StreamsStaticMembershipTest(Test):
     @cluster(num_nodes=8)
     @matrix(metadata_quorum=[quorum.isolated_kraft], use_new_coordinator=[True, False])
     def test_rolling_bounces_will_not_trigger_rebalance_under_static_membership(self, metadata_quorum, use_new_coordinator=False):
-        if self.zookeeper:
-            self.zookeeper.start()
         self.kafka.start()
 
         numThreads = 3
@@ -104,8 +96,6 @@ class StreamsStaticMembershipTest(Test):
 
         self.producer.stop()
         self.kafka.stop(timeout_sec=120)
-        if self.zookeeper:
-            self.zookeeper.stop()
 
     def verify_processing(self, processors):
         for processor in processors:
