@@ -31,14 +31,14 @@ import org.apache.kafka.common.utils.Exit;
 import org.apache.kafka.common.utils.Utils;
 import org.apache.kafka.server.util.CommandDefaultOptions;
 import org.apache.kafka.server.util.CommandLineUtils;
-import org.apache.kafka.server.util.PartitionFilter;
-import org.apache.kafka.server.util.PartitionFilter.PartitionRangeFilter;
-import org.apache.kafka.server.util.PartitionFilter.PartitionsSetFilter;
-import org.apache.kafka.server.util.PartitionFilter.UniquePartitionFilter;
-import org.apache.kafka.server.util.TopicFilter.IncludeList;
-import org.apache.kafka.server.util.TopicPartitionFilter;
-import org.apache.kafka.server.util.TopicPartitionFilter.CompositeTopicPartitionFilter;
-import org.apache.kafka.server.util.TopicPartitionFilter.TopicFilterAndPartitionFilter;
+import org.apache.kafka.tools.filter.PartitionFilter;
+import org.apache.kafka.tools.filter.PartitionFilter.PartitionRangeFilter;
+import org.apache.kafka.tools.filter.PartitionFilter.PartitionsSetFilter;
+import org.apache.kafka.tools.filter.PartitionFilter.UniquePartitionFilter;
+import org.apache.kafka.tools.filter.TopicFilter.IncludeList;
+import org.apache.kafka.tools.filter.TopicPartitionFilter;
+import org.apache.kafka.tools.filter.TopicPartitionFilter.CompositeTopicPartitionFilter;
+import org.apache.kafka.tools.filter.TopicPartitionFilter.TopicFilterAndPartitionFilter;
 
 import java.io.IOException;
 import java.util.Arrays;
@@ -101,18 +101,13 @@ public class GetOffsetShell {
         private final OptionSpec<String> partitionsOpt;
         private final OptionSpec<String> timeOpt;
         private final OptionSpec<String> commandConfigOpt;
-        private final OptionSpec<String> effectiveBrokerListOpt;
+        private final OptionSpec<String> bootstrapServerOpt;
         private final OptionSpecBuilder excludeInternalTopicsOpt;
 
         public GetOffsetShellOptions(String[] args) throws TerseException {
             super(args);
 
-            OptionSpec<String> brokerListOpt = parser.accepts("broker-list", "DEPRECATED, use --bootstrap-server instead; ignored if --bootstrap-server is specified. The server(s) to connect to in the form HOST1:PORT1,HOST2:PORT2.")
-                    .withRequiredArg()
-                    .describedAs("HOST1:PORT1,...,HOST3:PORT3")
-                    .ofType(String.class);
-            OptionSpec<String> bootstrapServerOpt = parser.accepts("bootstrap-server", "REQUIRED. The server(s) to connect to in the form HOST1:PORT1,HOST2:PORT2.")
-                    .requiredUnless("broker-list")
+            bootstrapServerOpt = parser.accepts("bootstrap-server", "REQUIRED. The server(s) to connect to in the form HOST1:PORT1,HOST2:PORT2.")
                     .withRequiredArg()
                     .describedAs("HOST1:PORT1,...,HOST3:PORT3")
                     .ofType(String.class);
@@ -151,17 +146,11 @@ public class GetOffsetShell {
                 throw new TerseException(e.getMessage());
             }
 
-            if (options.has(bootstrapServerOpt)) {
-                effectiveBrokerListOpt = bootstrapServerOpt;
-            } else {
-                effectiveBrokerListOpt = brokerListOpt;
-            }
-
             CommandLineUtils.maybePrintHelpOrVersion(this, USAGE_TEXT);
 
-            CommandLineUtils.checkRequiredArgs(parser, options, effectiveBrokerListOpt);
+            CommandLineUtils.checkRequiredArgs(parser, options, bootstrapServerOpt);
 
-            String brokerList = options.valueOf(effectiveBrokerListOpt);
+            String brokerList = options.valueOf(bootstrapServerOpt);
 
             try {
                 ToolsUtils.validateBootstrapServer(brokerList);
@@ -207,7 +196,7 @@ public class GetOffsetShell {
         }
 
         public String effectiveBrokerListOpt() {
-            return options.valueOf(effectiveBrokerListOpt);
+            return options.valueOf(bootstrapServerOpt);
         }
 
         public boolean hasExcludeInternalTopicsOpt() {
