@@ -16,8 +16,6 @@
  */
 package org.apache.kafka.tools.consumer;
 
-import joptsimple.OptionException;
-import joptsimple.OptionSpec;
 import org.apache.kafka.clients.consumer.ConsumerConfig;
 import org.apache.kafka.common.MessageFormatter;
 import org.apache.kafka.common.requests.ListOffsetsRequest;
@@ -41,12 +39,14 @@ import java.util.Random;
 import java.util.Set;
 import java.util.stream.Collectors;
 
+import joptsimple.OptionException;
+import joptsimple.OptionSpec;
+
 public final class ConsoleConsumerOptions extends CommandDefaultOptions {
 
     private static final Random RANDOM = new Random();
 
     private final OptionSpec<String> topicOpt;
-    private final OptionSpec<String> whitelistOpt;
     private final OptionSpec<String> includeOpt;
     private final OptionSpec<Integer> partitionIdOpt;
     private final OptionSpec<String> offsetOpt;
@@ -75,11 +75,6 @@ public final class ConsoleConsumerOptions extends CommandDefaultOptions {
                 .withRequiredArg()
                 .describedAs("topic")
                 .ofType(String.class);
-        whitelistOpt = parser.accepts("whitelist",
-                        "DEPRECATED, use --include instead; ignored if --include specified. Regular expression specifying list of topics to include for consumption.")
-                .withRequiredArg()
-                .describedAs("Java regex (String)")
-                .ofType(String.class);
         includeOpt = parser.accepts("include",
                         "Regular expression specifying list of topics to include for consumption.")
                 .withRequiredArg()
@@ -113,6 +108,7 @@ public final class ConsoleConsumerOptions extends CommandDefaultOptions {
                             " print.timestamp=true|false\n" +
                             " print.key=true|false\n" +
                             " print.offset=true|false\n" +
+                            " print.epoch=true|false\n" +
                             " print.partition=true|false\n" +
                             " print.headers=true|false\n" +
                             " print.value=true|false\n" +
@@ -192,10 +188,9 @@ public final class ConsoleConsumerOptions extends CommandDefaultOptions {
     private void checkRequiredArgs() {
         List<Optional<String>> topicOrFilterArgs = new ArrayList<>(Arrays.asList(topicArg(), includedTopicsArg()));
         topicOrFilterArgs.removeIf(arg -> !arg.isPresent());
-        // user need to specify value for either --topic or one of the include filters options (--include or --whitelist)
+        // user need to specify value for either --topic or --include options)
         if (topicOrFilterArgs.size() != 1) {
-            CommandLineUtils.printUsageAndExit(parser, "Exactly one of --include/--topic is required. " +
-                    (options.has(whitelistOpt) ? "--whitelist is DEPRECATED use --include instead; ignored if --include specified." : ""));
+            CommandLineUtils.printUsageAndExit(parser, "Exactly one of --include/--topic is required. ");
         }
 
         if (partitionArg().isPresent()) {
@@ -332,7 +327,7 @@ public final class ConsoleConsumerOptions extends CommandDefaultOptions {
     private MessageFormatter buildFormatter() {
         MessageFormatter formatter = null;
         try {
-            Class<?> messageFormatterClass = Class.forName(convertDeprecatedClass(options.valueOf(messageFormatterOpt)));
+            Class<?> messageFormatterClass = Class.forName(options.valueOf(messageFormatterOpt));
             formatter = (MessageFormatter) messageFormatterClass.getDeclaredConstructor().newInstance();
 
             Properties formatterArgs = formatterArgs();
@@ -349,25 +344,6 @@ public final class ConsoleConsumerOptions extends CommandDefaultOptions {
         return formatter;
     }
 
-    private static String convertDeprecatedClass(String className) {
-        switch (className) {
-            case "kafka.tools.DefaultMessageFormatter":
-                System.err.println("WARNING: kafka.tools.DefaultMessageFormatter is deprecated and will be removed in the next major release. " +
-                        "Please use org.apache.kafka.tools.consumer.DefaultMessageFormatter instead");
-                return DefaultMessageFormatter.class.getName();
-            case "kafka.tools.LoggingMessageFormatter":
-                System.err.println("WARNING: kafka.tools.LoggingMessageFormatter is deprecated and will be removed in the next major release. " +
-                        "Please use org.apache.kafka.tools.consumer.LoggingMessageFormatter instead");
-                return LoggingMessageFormatter.class.getName();
-            case "kafka.tools.NoOpMessageFormatter":
-                System.err.println("WARNING: kafka.tools.NoOpMessageFormatter is deprecated and will be removed in the next major release. " +
-                        "Please use org.apache.kafka.tools.consumer.NoOpMessageFormatter instead");
-                return NoOpMessageFormatter.class.getName();
-            default:
-                return className;
-        }
-    }
-
     Properties consumerProps() {
         return consumerProps;
     }
@@ -414,7 +390,7 @@ public final class ConsoleConsumerOptions extends CommandDefaultOptions {
     Optional<String> includedTopicsArg() {
         return options.has(includeOpt)
                 ? Optional.of(options.valueOf(includeOpt))
-                : Optional.ofNullable(options.valueOf(whitelistOpt));
+                : Optional.empty();
     }
 
     Properties formatterArgs() throws IOException {
