@@ -16,11 +16,6 @@
  */
 package org.apache.kafka.tools;
 
-import kafka.test.ClusterInstance;
-import kafka.test.annotation.ClusterConfigProperty;
-import kafka.test.annotation.ClusterTest;
-import kafka.test.annotation.ClusterTestDefaults;
-import kafka.test.junit.ClusterTestExtensions;
 import kafka.utils.TestUtils;
 
 import org.apache.kafka.clients.admin.Admin;
@@ -29,9 +24,12 @@ import org.apache.kafka.clients.admin.CreateTopicsResult;
 import org.apache.kafka.clients.admin.NewTopic;
 import org.apache.kafka.common.TopicPartition;
 import org.apache.kafka.common.errors.UnknownTopicOrPartitionException;
+import org.apache.kafka.common.test.ClusterInstance;
+import org.apache.kafka.common.test.api.ClusterConfigProperty;
+import org.apache.kafka.common.test.api.ClusterTest;
+import org.apache.kafka.common.test.api.ClusterTestDefaults;
 import org.apache.kafka.server.common.AdminCommandFailedException;
 
-import org.junit.jupiter.api.extension.ExtendWith;
 import org.mockito.ArgumentCaptor;
 import org.mockito.MockedStatic;
 import org.mockito.Mockito;
@@ -51,7 +49,7 @@ import java.util.Properties;
 import java.util.Set;
 import java.util.concurrent.ExecutionException;
 
-import scala.collection.JavaConverters;
+import scala.jdk.javaapi.CollectionConverters;
 
 import static java.util.Arrays.asList;
 import static java.util.Collections.singletonList;
@@ -61,14 +59,10 @@ import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
 
-@SuppressWarnings("deprecation")
-@ExtendWith(value = ClusterTestExtensions.class)
 @ClusterTestDefaults(brokers = 3, serverProperties = {
     @ClusterConfigProperty(key = "auto.create.topics.enable", value = "false"),
     @ClusterConfigProperty(key = "auto.leader.rebalance.enable", value = "false"),
     @ClusterConfigProperty(key = "controlled.shutdown.enable", value = "true"),
-    @ClusterConfigProperty(key = "controlled.shutdown.max.retries", value = "1"),
-    @ClusterConfigProperty(key = "controlled.shutdown.retry.backoff.ms", value = "1000"),
     @ClusterConfigProperty(key = "offsets.topic.replication.factor", value = "2")
 })
 public class LeaderElectionCommandTest {
@@ -87,30 +81,31 @@ public class LeaderElectionCommandTest {
         List<Integer> assignment = asList(broker2, broker3);
 
         cluster.waitForReadyBrokers();
-        Admin client = cluster.createAdminClient();
+        try (Admin client = cluster.admin()) {
 
-        createTopic(client, topic, Collections.singletonMap(partition, assignment));
+            createTopic(client, topic, Collections.singletonMap(partition, assignment));
 
-        TopicPartition topicPartition = new TopicPartition(topic, partition);
+            TopicPartition topicPartition = new TopicPartition(topic, partition);
 
-        TestUtils.assertLeader(client, topicPartition, broker2);
-        cluster.shutdownBroker(broker3);
-        TestUtils.waitForBrokersOutOfIsr(client,
-                JavaConverters.asScalaBuffer(singletonList(topicPartition)).toSet(),
-                JavaConverters.asScalaBuffer(singletonList(broker3)).toSet()
-        );
-        cluster.shutdownBroker(broker2);
-        TestUtils.assertNoLeader(client, topicPartition);
-        cluster.startBroker(broker3);
-        TestUtils.waitForOnlineBroker(client, broker3);
+            TestUtils.assertLeader(client, topicPartition, broker2);
+            cluster.shutdownBroker(broker3);
+            TestUtils.waitForBrokersOutOfIsr(client,
+                    CollectionConverters.asScala(singletonList(topicPartition)).toSet(),
+                    CollectionConverters.asScala(singletonList(broker3)).toSet()
+            );
+            cluster.shutdownBroker(broker2);
+            TestUtils.assertNoLeader(client, topicPartition);
+            cluster.startBroker(broker3);
+            TestUtils.waitForOnlineBroker(client, broker3);
 
-        assertEquals(0, LeaderElectionCommand.mainNoExit(
-            "--bootstrap-server", cluster.bootstrapServers(),
-            "--election-type", "unclean",
-            "--all-topic-partitions"
-        ));
+            assertEquals(0, LeaderElectionCommand.mainNoExit(
+                    "--bootstrap-server", cluster.bootstrapServers(),
+                    "--election-type", "unclean",
+                    "--all-topic-partitions"
+            ));
 
-        TestUtils.assertLeader(client, topicPartition, broker3);
+            TestUtils.assertLeader(client, topicPartition, broker3);
+        }
     }
 
     @ClusterTest
@@ -143,31 +138,32 @@ public class LeaderElectionCommandTest {
         List<Integer> assignment = asList(broker2, broker3);
 
         cluster.waitForReadyBrokers();
-        Admin client = cluster.createAdminClient();
-        createTopic(client, topic, Collections.singletonMap(partition, assignment));
+        try (Admin client = cluster.admin()) {
+            createTopic(client, topic, Collections.singletonMap(partition, assignment));
 
-        TopicPartition topicPartition = new TopicPartition(topic, partition);
+            TopicPartition topicPartition = new TopicPartition(topic, partition);
 
-        TestUtils.assertLeader(client, topicPartition, broker2);
+            TestUtils.assertLeader(client, topicPartition, broker2);
 
-        cluster.shutdownBroker(broker3);
-        TestUtils.waitForBrokersOutOfIsr(client,
-            JavaConverters.asScalaBuffer(singletonList(topicPartition)).toSet(),
-            JavaConverters.asScalaBuffer(singletonList(broker3)).toSet()
-        );
-        cluster.shutdownBroker(broker2);
-        TestUtils.assertNoLeader(client, topicPartition);
-        cluster.startBroker(broker3);
-        TestUtils.waitForOnlineBroker(client, broker3);
+            cluster.shutdownBroker(broker3);
+            TestUtils.waitForBrokersOutOfIsr(client,
+                    CollectionConverters.asScala(singletonList(topicPartition)).toSet(),
+                    CollectionConverters.asScala(singletonList(broker3)).toSet()
+            );
+            cluster.shutdownBroker(broker2);
+            TestUtils.assertNoLeader(client, topicPartition);
+            cluster.startBroker(broker3);
+            TestUtils.waitForOnlineBroker(client, broker3);
 
-        assertEquals(0, LeaderElectionCommand.mainNoExit(
-            "--bootstrap-server", cluster.bootstrapServers(),
-            "--election-type", "unclean",
-            "--topic", topic,
-            "--partition", Integer.toString(partition)
-        ));
+            assertEquals(0, LeaderElectionCommand.mainNoExit(
+                    "--bootstrap-server", cluster.bootstrapServers(),
+                    "--election-type", "unclean",
+                    "--topic", topic,
+                    "--partition", Integer.toString(partition)
+            ));
 
-        TestUtils.assertLeader(client, topicPartition, broker3);
+            TestUtils.assertLeader(client, topicPartition, broker3);
+        }
     }
 
     @ClusterTest
@@ -180,32 +176,33 @@ public class LeaderElectionCommandTest {
         Map<Integer, List<Integer>> partitionAssignment = new HashMap<>();
         partitionAssignment.put(partition, assignment);
 
-        Admin client = cluster.createAdminClient();
-        createTopic(client, topic, partitionAssignment);
+        try (Admin client = cluster.admin()) {
+            createTopic(client, topic, partitionAssignment);
 
-        TopicPartition topicPartition = new TopicPartition(topic, partition);
+            TopicPartition topicPartition = new TopicPartition(topic, partition);
 
-        TestUtils.assertLeader(client, topicPartition, broker2);
+            TestUtils.assertLeader(client, topicPartition, broker2);
 
-        cluster.shutdownBroker(broker3);
-        TestUtils.waitForBrokersOutOfIsr(client,
-            JavaConverters.asScalaBuffer(singletonList(topicPartition)).toSet(),
-            JavaConverters.asScalaBuffer(singletonList(broker3)).toSet()
-        );
-        cluster.shutdownBroker(broker2);
-        TestUtils.assertNoLeader(client, topicPartition);
-        cluster.startBroker(broker3);
-        TestUtils.waitForOnlineBroker(client, broker3);
+            cluster.shutdownBroker(broker3);
+            TestUtils.waitForBrokersOutOfIsr(client,
+                    CollectionConverters.asScala(singletonList(topicPartition)).toSet(),
+                    CollectionConverters.asScala(singletonList(broker3)).toSet()
+            );
+            cluster.shutdownBroker(broker2);
+            TestUtils.assertNoLeader(client, topicPartition);
+            cluster.startBroker(broker3);
+            TestUtils.waitForOnlineBroker(client, broker3);
 
-        Path topicPartitionPath = tempTopicPartitionFile(singletonList(topicPartition));
+            Path topicPartitionPath = tempTopicPartitionFile(singletonList(topicPartition));
 
-        assertEquals(0, LeaderElectionCommand.mainNoExit(
-            "--bootstrap-server", cluster.bootstrapServers(),
-            "--election-type", "unclean",
-            "--path-to-json-file", topicPartitionPath.toString()
-        ));
+            assertEquals(0, LeaderElectionCommand.mainNoExit(
+                    "--bootstrap-server", cluster.bootstrapServers(),
+                    "--election-type", "unclean",
+                    "--path-to-json-file", topicPartitionPath.toString()
+            ));
 
-        TestUtils.assertLeader(client, topicPartition, broker3);
+            TestUtils.assertLeader(client, topicPartition, broker3);
+        }
     }
 
     @ClusterTest
@@ -215,30 +212,31 @@ public class LeaderElectionCommandTest {
         List<Integer> assignment = asList(broker2, broker3);
 
         cluster.waitForReadyBrokers();
-        Admin client = cluster.createAdminClient();
-        Map<Integer, List<Integer>> partitionAssignment = new HashMap<>();
-        partitionAssignment.put(partition, assignment);
+        try (Admin client = cluster.admin()) {
+            Map<Integer, List<Integer>> partitionAssignment = new HashMap<>();
+            partitionAssignment.put(partition, assignment);
 
-        createTopic(client, topic, partitionAssignment);
+            createTopic(client, topic, partitionAssignment);
 
-        TopicPartition topicPartition = new TopicPartition(topic, partition);
+            TopicPartition topicPartition = new TopicPartition(topic, partition);
 
-        TestUtils.assertLeader(client, topicPartition, broker2);
+            TestUtils.assertLeader(client, topicPartition, broker2);
 
-        cluster.shutdownBroker(broker2);
-        TestUtils.assertLeader(client, topicPartition, broker3);
-        cluster.startBroker(broker2);
-        TestUtils.waitForBrokersInIsr(client, topicPartition,
-            JavaConverters.asScalaBuffer(singletonList(broker2)).toSet()
-        );
+            cluster.shutdownBroker(broker2);
+            TestUtils.assertLeader(client, topicPartition, broker3);
+            cluster.startBroker(broker2);
+            TestUtils.waitForBrokersInIsr(client, topicPartition,
+                    CollectionConverters.asScala(singletonList(broker2)).toSet()
+            );
 
-        assertEquals(0, LeaderElectionCommand.mainNoExit(
-            "--bootstrap-server", cluster.bootstrapServers(),
-            "--election-type", "preferred",
-            "--all-topic-partitions"
-        ));
+            assertEquals(0, LeaderElectionCommand.mainNoExit(
+                    "--bootstrap-server", cluster.bootstrapServers(),
+                    "--election-type", "preferred",
+                    "--all-topic-partitions"
+            ));
 
-        TestUtils.assertLeader(client, topicPartition, broker2);
+            TestUtils.assertLeader(client, topicPartition, broker2);
+        }
     }
 
     @ClusterTest
@@ -262,28 +260,31 @@ public class LeaderElectionCommandTest {
         List<Integer> assignment1 = asList(broker3, broker2);
 
         cluster.waitForReadyBrokers();
-        Admin client = cluster.createAdminClient();
-        Map<Integer, List<Integer>> partitionAssignment = new HashMap<>();
-        partitionAssignment.put(partition0, assignment0);
-        partitionAssignment.put(partition1, assignment1);
+        TopicPartition topicPartition0;
+        TopicPartition topicPartition1;
+        try (Admin client = cluster.admin()) {
+            Map<Integer, List<Integer>> partitionAssignment = new HashMap<>();
+            partitionAssignment.put(partition0, assignment0);
+            partitionAssignment.put(partition1, assignment1);
 
-        createTopic(client, topic, partitionAssignment);
+            createTopic(client, topic, partitionAssignment);
 
-        TopicPartition topicPartition0 = new TopicPartition(topic, partition0);
-        TopicPartition topicPartition1 = new TopicPartition(topic, partition1);
+            topicPartition0 = new TopicPartition(topic, partition0);
+            topicPartition1 = new TopicPartition(topic, partition1);
 
-        TestUtils.assertLeader(client, topicPartition0, broker2);
-        TestUtils.assertLeader(client, topicPartition1, broker3);
+            TestUtils.assertLeader(client, topicPartition0, broker2);
+            TestUtils.assertLeader(client, topicPartition1, broker3);
 
-        cluster.shutdownBroker(broker2);
-        TestUtils.assertLeader(client, topicPartition0, broker3);
-        cluster.startBroker(broker2);
-        TestUtils.waitForBrokersInIsr(client, topicPartition0,
-            JavaConverters.asScalaBuffer(singletonList(broker2)).toSet()
-        );
-        TestUtils.waitForBrokersInIsr(client, topicPartition1,
-            JavaConverters.asScalaBuffer(singletonList(broker2)).toSet()
-        );
+            cluster.shutdownBroker(broker2);
+            TestUtils.assertLeader(client, topicPartition0, broker3);
+            cluster.startBroker(broker2);
+            TestUtils.waitForBrokersInIsr(client, topicPartition0,
+                    CollectionConverters.asScala(singletonList(broker2)).toSet()
+            );
+            TestUtils.waitForBrokersInIsr(client, topicPartition1,
+                    CollectionConverters.asScala(singletonList(broker2)).toSet()
+            );
+        }
 
         Path topicPartitionPath = tempTopicPartitionFile(asList(topicPartition0, topicPartition1));
         String output = ToolsTestUtils.captureStandardOut(() ->
