@@ -73,35 +73,6 @@ public class ConsoleConsumerOptionsTest {
         assertTrue(config.fromBeginning());
     }
 
-    @Test
-    public void shouldParseWhitelistArgument() throws IOException {
-        String[] args = new String[]{
-            "--bootstrap-server", "localhost:9092",
-            "--whitelist", "whitelistTest*",
-            "--from-beginning"
-        };
-
-        ConsoleConsumerOptions config = new ConsoleConsumerOptions(args);
-
-        assertEquals("localhost:9092", config.bootstrapServer());
-        assertEquals("whitelistTest*", config.includedTopicsArg().orElse(""));
-        assertTrue(config.fromBeginning());
-    }
-
-    @Test
-    public void shouldIgnoreWhitelistArgumentIfIncludeSpecified() throws IOException {
-        String[] args = new String[]{
-            "--bootstrap-server", "localhost:9092",
-            "--include", "includeTest*",
-            "--whitelist", "whitelistTest*",
-            "--from-beginning"
-        };
-        ConsoleConsumerOptions config = new ConsoleConsumerOptions(args);
-        assertEquals("localhost:9092", config.bootstrapServer());
-        assertEquals("includeTest*", config.includedTopicsArg().orElse(""));
-        assertTrue(config.fromBeginning());
-    }
-
     @Test
     public void shouldParseValidSimpleConsumerValidConfigWithNumericOffset() throws IOException {
         String[] args = new String[]{
@@ -537,25 +508,6 @@ public class ConsoleConsumerOptionsTest {
         }
     }
 
-    @Test
-    public void shouldExitIfTopicAndWhitelistSpecified() {
-        Exit.setExitProcedure((code, message) -> {
-            throw new IllegalArgumentException(message);
-        });
-
-        String[] args = new String[]{
-            "--bootstrap-server", "localhost:9092",
-            "--topic", "test",
-            "--whitelist", "whitelistTest*"
-        };
-
-        try {
-            assertThrows(IllegalArgumentException.class, () -> new ConsoleConsumerOptions(args));
-        } finally {
-            Exit.resetExitProcedure();
-        }
-    }
-
     @Test
     public void testClientIdOverride() throws IOException {
         String[] args = new String[]{
@@ -648,57 +600,15 @@ public class ConsoleConsumerOptionsTest {
     }
 
     @Test
-    public void testParseDeprecatedFormatter() throws Exception {
-        String[] deprecatedDefaultMessageFormatter = generateArgsForFormatter("kafka.tools.DefaultMessageFormatter");
-        assertInstanceOf(DefaultMessageFormatter.class, new ConsoleConsumerOptions(deprecatedDefaultMessageFormatter).formatter());
+    public void testParseFormatter() throws Exception {
+        String[] defaultMessageFormatter = generateArgsForFormatter("org.apache.kafka.tools.consumer.DefaultMessageFormatter");
+        assertInstanceOf(DefaultMessageFormatter.class, new ConsoleConsumerOptions(defaultMessageFormatter).formatter());
 
-        String[] deprecatedLoggingMessageFormatter = generateArgsForFormatter("kafka.tools.LoggingMessageFormatter");
-        assertInstanceOf(LoggingMessageFormatter.class, new ConsoleConsumerOptions(deprecatedLoggingMessageFormatter).formatter());
+        String[] loggingMessageFormatter = generateArgsForFormatter("org.apache.kafka.tools.consumer.LoggingMessageFormatter");
+        assertInstanceOf(LoggingMessageFormatter.class, new ConsoleConsumerOptions(loggingMessageFormatter).formatter());
 
-        String[] deprecatedNoOpMessageFormatter = generateArgsForFormatter("kafka.tools.NoOpMessageFormatter");
-        assertInstanceOf(NoOpMessageFormatter.class, new ConsoleConsumerOptions(deprecatedNoOpMessageFormatter).formatter());
-    }
-
-    @SuppressWarnings("deprecation")
-    @Test
-    public void testNewAndDeprecateTransactionLogMessageFormatter() throws Exception {
-        String[] deprecatedTransactionLogMessageFormatter = 
-                generateArgsForFormatter("kafka.coordinator.transaction.TransactionLog$TransactionLogMessageFormatter");
-        assertInstanceOf(kafka.coordinator.transaction.TransactionLog.TransactionLogMessageFormatter.class, 
-                new ConsoleConsumerOptions(deprecatedTransactionLogMessageFormatter).formatter());
-
-        String[] transactionLogMessageFormatter = 
-                generateArgsForFormatter("org.apache.kafka.tools.consumer.TransactionLogMessageFormatter");
-        assertInstanceOf(TransactionLogMessageFormatter.class, 
-                new ConsoleConsumerOptions(transactionLogMessageFormatter).formatter());
-    }
-
-    @SuppressWarnings("deprecation")
-    @Test
-    public void testNewAndDeprecateOffsetsMessageFormatter() throws Exception {
-        String[] deprecatedOffsetsMessageFormatter = 
-                generateArgsForFormatter("kafka.coordinator.group.GroupMetadataManager$OffsetsMessageFormatter");
-        assertInstanceOf(kafka.coordinator.group.GroupMetadataManager.OffsetsMessageFormatter.class,
-                new ConsoleConsumerOptions(deprecatedOffsetsMessageFormatter).formatter());
-
-        String[] offsetsMessageFormatter = 
-                generateArgsForFormatter("org.apache.kafka.tools.consumer.OffsetsMessageFormatter");
-        assertInstanceOf(OffsetsMessageFormatter.class,
-                new ConsoleConsumerOptions(offsetsMessageFormatter).formatter());
-    }
-
-    @SuppressWarnings("deprecation")
-    @Test
-    public void testNewAndDeprecateGroupMetadataMessageFormatter() throws Exception {
-        String[] deprecatedGroupMetadataMessageFormatter =
-                generateArgsForFormatter("kafka.coordinator.group.GroupMetadataManager$GroupMetadataMessageFormatter");
-        assertInstanceOf(kafka.coordinator.group.GroupMetadataManager.GroupMetadataMessageFormatter.class,
-                new ConsoleConsumerOptions(deprecatedGroupMetadataMessageFormatter).formatter());
-
-        String[] groupMetadataMessageFormatter =
-                generateArgsForFormatter("org.apache.kafka.tools.consumer.GroupMetadataMessageFormatter");
-        assertInstanceOf(GroupMetadataMessageFormatter.class,
-                new ConsoleConsumerOptions(groupMetadataMessageFormatter).formatter());
+        String[] noOpMessageFormatter = generateArgsForFormatter("org.apache.kafka.tools.consumer.NoOpMessageFormatter");
+        assertInstanceOf(NoOpMessageFormatter.class, new ConsoleConsumerOptions(noOpMessageFormatter).formatter());
     }
     
     private String[] generateArgsForFormatter(String formatter) {
