@@ -16,11 +16,6 @@
  */
 package org.apache.kafka.tools.consumer.group;
 
-import kafka.test.ClusterConfig;
-import kafka.test.ClusterInstance;
-import kafka.test.annotation.ClusterTemplate;
-import kafka.test.junit.ClusterTestExtensions;
-
 import org.apache.kafka.clients.CommonClientConfigs;
 import org.apache.kafka.clients.admin.Admin;
 import org.apache.kafka.clients.admin.AdminClientConfig;
@@ -29,16 +24,18 @@ import org.apache.kafka.clients.admin.NewTopic;
 import org.apache.kafka.clients.consumer.GroupProtocol;
 import org.apache.kafka.clients.consumer.KafkaConsumer;
 import org.apache.kafka.clients.consumer.RangeAssignor;
-import org.apache.kafka.common.ConsumerGroupState;
+import org.apache.kafka.common.GroupState;
 import org.apache.kafka.common.GroupType;
 import org.apache.kafka.common.TopicPartition;
 import org.apache.kafka.common.serialization.StringDeserializer;
+import org.apache.kafka.common.test.ClusterInstance;
+import org.apache.kafka.common.test.api.ClusterConfig;
+import org.apache.kafka.common.test.api.ClusterTemplate;
 import org.apache.kafka.test.TestUtils;
 import org.apache.kafka.tools.ToolsTestUtils;
 
 import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.Test;
-import org.junit.jupiter.api.extension.ExtendWith;
 
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -64,9 +61,8 @@ import static org.apache.kafka.clients.consumer.ConsumerConfig.GROUP_PROTOCOL_CO
 import static org.apache.kafka.clients.consumer.ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG;
 import static org.apache.kafka.clients.consumer.ConsumerConfig.PARTITION_ASSIGNMENT_STRATEGY_CONFIG;
 import static org.apache.kafka.clients.consumer.ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG;
-import static org.apache.kafka.common.utils.Utils.mkSet;
 
-@ExtendWith(ClusterTestExtensions.class)
+
 public class ListConsumerGroupTest {
     private static final String TOPIC_PREFIX = "test.topic.";
     private static final String TOPIC_PARTITIONS_GROUP_PREFIX = "test.topic.partitions.group.";
@@ -82,10 +78,6 @@ public class ListConsumerGroupTest {
         return ConsumerGroupCommandTestUtils.generator();
     }
 
-    private static List<ClusterConfig> consumerProtocolOnlyGenerator() {
-        return ConsumerGroupCommandTestUtils.forConsumerGroupCoordinator();
-    }
-
     private List<GroupProtocol> supportedGroupProtocols() {
         return new ArrayList<>(clusterInstance.supportedGroupProtocols());
     }
@@ -103,7 +95,7 @@ public class ListConsumerGroupTest {
             try (AutoCloseable topicPartitionsConsumerGroupExecutor = consumerGroupClosable(topicPartitionsGroup, Collections.singleton(new TopicPartition(topic, 0)));
                  AutoCloseable topicConsumerGroupExecutor = consumerGroupClosable(GroupProtocol.CLASSIC, topicGroup, topic);
                  AutoCloseable protocolConsumerGroupExecutor = consumerGroupClosable(groupProtocol, protocolGroup, topic);
-                 ConsumerGroupCommand.ConsumerGroupService service = getConsumerGroupService(new String[]{"--bootstrap-server", clusterInstance.bootstrapServers(), "--list"});
+                 ConsumerGroupCommand.ConsumerGroupService service = getConsumerGroupService(new String[]{"--bootstrap-server", clusterInstance.bootstrapServers(), "--list"})
             ) {
                 Set<String> expectedGroups = set(Arrays.asList(topicPartitionsGroup, topicGroup, protocolGroup));
                 final AtomicReference<Set> foundGroups = new AtomicReference<>();
@@ -136,50 +128,50 @@ public class ListConsumerGroupTest {
 
             try (AutoCloseable topicPartitionsConsumerGroupExecutor = consumerGroupClosable(topicPartitionsGroup, Collections.singleton(new TopicPartition(topic, 0)));
                  AutoCloseable protocolConsumerGroupExecutor = consumerGroupClosable(groupProtocol, protocolGroup, topic);
-                 ConsumerGroupCommand.ConsumerGroupService service = getConsumerGroupService(new String[]{"--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--state"});
+                 ConsumerGroupCommand.ConsumerGroupService service = getConsumerGroupService(new String[]{"--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--state"})
             ) {
-                Set<ConsumerGroupListing> expectedListing = mkSet(
+                Set<ConsumerGroupListing> expectedListing = Set.of(
                         new ConsumerGroupListing(
                                 topicPartitionsGroup,
-                                true,
-                                Optional.of(ConsumerGroupState.EMPTY),
-                                Optional.of(GroupType.CLASSIC)
+                                Optional.of(GroupState.EMPTY),
+                                Optional.of(GroupType.CLASSIC),
+                                true
                         ),
                         new ConsumerGroupListing(
                                 protocolGroup,
-                                false,
-                                Optional.of(ConsumerGroupState.STABLE),
-                                Optional.of(GroupType.parse(groupProtocol.name()))
+                                Optional.of(GroupState.STABLE),
+                                Optional.of(GroupType.parse(groupProtocol.name())),
+                                false
                         )
                 );
 
                 assertGroupListing(
                         service,
                         Collections.emptySet(),
-                        EnumSet.allOf(ConsumerGroupState.class),
+                        EnumSet.allOf(GroupState.class),
                         expectedListing
                 );
 
-                expectedListing = mkSet(
+                expectedListing = Set.of(
                         new ConsumerGroupListing(
                                 protocolGroup,
-                                false,
-                                Optional.of(ConsumerGroupState.STABLE),
-                                Optional.of(GroupType.parse(groupProtocol.name()))
+                                Optional.of(GroupState.STABLE),
+                                Optional.of(GroupType.parse(groupProtocol.name())),
+                                false
                         )
                 );
 
                 assertGroupListing(
                         service,
                         Collections.emptySet(),
-                        mkSet(ConsumerGroupState.STABLE),
+                        Set.of(GroupState.STABLE),
                         expectedListing
                 );
 
                 assertGroupListing(
                         service,
                         Collections.emptySet(),
-                        mkSet(ConsumerGroupState.PREPARING_REBALANCE),
+                        Set.of(GroupState.PREPARING_REBALANCE),
                         Collections.emptySet()
                 );
             }
@@ -199,20 +191,20 @@ public class ListConsumerGroupTest {
 
         try (AutoCloseable topicPartitionsConsumerGroupExecutor = consumerGroupClosable(topicPartitionsGroup, Collections.singleton(new TopicPartition(topic, 0)));
              AutoCloseable protocolConsumerGroupExecutor = consumerGroupClosable(groupProtocol, protocolGroup, topic);
-             ConsumerGroupCommand.ConsumerGroupService service = getConsumerGroupService(new String[]{"--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--state"});
+             ConsumerGroupCommand.ConsumerGroupService service = getConsumerGroupService(new String[]{"--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--state"})
         ) {
-            Set<ConsumerGroupListing> expectedListing = mkSet(
+            Set<ConsumerGroupListing> expectedListing = Set.of(
                     new ConsumerGroupListing(
                             topicPartitionsGroup,
-                            true,
-                            Optional.of(ConsumerGroupState.EMPTY),
-                            Optional.of(GroupType.CLASSIC)
+                            Optional.of(GroupState.EMPTY),
+                            Optional.of(GroupType.CLASSIC),
+                            true
                     ),
                     new ConsumerGroupListing(
                             protocolGroup,
-                            false,
-                            Optional.of(ConsumerGroupState.STABLE),
-                            Optional.of(GroupType.CLASSIC)
+                            Optional.of(GroupState.STABLE),
+                            Optional.of(GroupType.CLASSIC),
+                            false
                     )
             );
 
@@ -229,21 +221,21 @@ public class ListConsumerGroupTest {
             // New Group Coordinator returns groups according to the filter.
             assertGroupListing(
                     service,
-                    mkSet(GroupType.CONSUMER),
+                    Set.of(GroupType.CONSUMER),
                     Collections.emptySet(),
                     Collections.emptySet()
             );
 
             assertGroupListing(
                     service,
-                    mkSet(GroupType.CLASSIC),
+                    Set.of(GroupType.CLASSIC),
                     Collections.emptySet(),
                     expectedListing
             );
         }
     }
 
-    @ClusterTemplate("consumerProtocolOnlyGenerator")
+    @ClusterTemplate("defaultGenerator")
     public void testListConsumerGroupsWithTypesConsumerProtocol() throws Exception {
         GroupProtocol groupProtocol = GroupProtocol.CONSUMER;
         String topic = TOPIC_PREFIX + groupProtocol.name;
@@ -255,29 +247,29 @@ public class ListConsumerGroupTest {
         try (AutoCloseable topicPartitionsConsumerGroupExecutor = consumerGroupClosable(topicPartitionsGroup, Collections.singleton(new TopicPartition(topic, 0)));
              AutoCloseable topicConsumerGroupExecutor = consumerGroupClosable(GroupProtocol.CLASSIC, topicGroup, topic);
              AutoCloseable protocolConsumerGroupExecutor = consumerGroupClosable(groupProtocol, protocolGroup, topic);
-             ConsumerGroupCommand.ConsumerGroupService service = getConsumerGroupService(new String[]{"--bootstrap-server", clusterInstance.bootstrapServers(), "--list"});
+             ConsumerGroupCommand.ConsumerGroupService service = getConsumerGroupService(new String[]{"--bootstrap-server", clusterInstance.bootstrapServers(), "--list"})
         ) {
 
 
             // No filters explicitly mentioned. Expectation is that all groups are returned.
-            Set<ConsumerGroupListing> expectedListing = mkSet(
+            Set<ConsumerGroupListing> expectedListing = Set.of(
                     new ConsumerGroupListing(
                             topicPartitionsGroup,
-                            true,
-                            Optional.of(ConsumerGroupState.EMPTY),
-                            Optional.of(GroupType.CLASSIC)
+                            Optional.of(GroupState.EMPTY),
+                            Optional.of(GroupType.CLASSIC),
+                            true
                     ),
                     new ConsumerGroupListing(
                             topicGroup,
-                            false,
-                            Optional.of(ConsumerGroupState.STABLE),
-                            Optional.of(GroupType.CLASSIC)
+                            Optional.of(GroupState.STABLE),
+                            Optional.of(GroupType.CLASSIC),
+                            false
                     ),
                     new ConsumerGroupListing(
                             protocolGroup,
-                            false,
-                            Optional.of(ConsumerGroupState.STABLE),
-                            Optional.of(GroupType.CONSUMER)
+                            Optional.of(GroupState.STABLE),
+                            Optional.of(GroupType.CONSUMER),
+                            false
                     )
             );
 
@@ -290,40 +282,40 @@ public class ListConsumerGroupTest {
 
             // When group type is mentioned:
             // New Group Coordinator returns groups according to the filter.
-            expectedListing = mkSet(
+            expectedListing = Set.of(
                     new ConsumerGroupListing(
                             protocolGroup,
-                            false,
-                            Optional.of(ConsumerGroupState.STABLE),
-                            Optional.of(GroupType.CONSUMER)
+                            Optional.of(GroupState.STABLE),
+                            Optional.of(GroupType.CONSUMER),
+                            false
                     )
             );
 
             assertGroupListing(
                     service,
-                    mkSet(GroupType.CONSUMER),
+                    Set.of(GroupType.CONSUMER),
                     Collections.emptySet(),
                     expectedListing
             );
 
-            expectedListing = mkSet(
+            expectedListing = Set.of(
                     new ConsumerGroupListing(
                             topicPartitionsGroup,
-                            true,
-                            Optional.of(ConsumerGroupState.EMPTY),
-                            Optional.of(GroupType.CLASSIC)
+                            Optional.of(GroupState.EMPTY),
+                            Optional.of(GroupType.CLASSIC),
+                            true
                     ),
                     new ConsumerGroupListing(
                             topicGroup,
-                            false,
-                            Optional.of(ConsumerGroupState.STABLE),
-                            Optional.of(GroupType.CLASSIC)
+                            Optional.of(GroupState.STABLE),
+                            Optional.of(GroupType.CLASSIC),
+                            false
                     )
             );
 
             assertGroupListing(
                     service,
-                    mkSet(GroupType.CLASSIC),
+                    Set.of(GroupType.CLASSIC),
                     Collections.emptySet(),
                     expectedListing
             );
@@ -344,7 +336,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list"),
                     Collections.emptyList(),
-                    mkSet(
+                    Set.of(
                             Collections.singletonList(protocolGroup),
                             Collections.singletonList(topicPartitionsGroup)
                     )
@@ -353,7 +345,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--state"),
                     Arrays.asList("GROUP", "STATE"),
-                    mkSet(
+                    Set.of(
                             Arrays.asList(protocolGroup, "Stable"),
                             Arrays.asList(topicPartitionsGroup, "Empty")
                     )
@@ -362,7 +354,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--type"),
                     Arrays.asList("GROUP", "TYPE"),
-                    mkSet(
+                    Set.of(
                             Arrays.asList(protocolGroup, "Classic"),
                             Arrays.asList(topicPartitionsGroup, "Classic")
                     )
@@ -371,7 +363,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--type", "--state"),
                     Arrays.asList("GROUP", "TYPE", "STATE"),
-                    mkSet(
+                    Set.of(
                             Arrays.asList(protocolGroup, "Classic", "Stable"),
                             Arrays.asList(topicPartitionsGroup, "Classic", "Empty")
                     )
@@ -380,7 +372,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--state", "Stable"),
                     Arrays.asList("GROUP", "STATE"),
-                    mkSet(
+                    Set.of(
                             Arrays.asList(protocolGroup, "Stable")
                     )
             );
@@ -389,7 +381,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--state", "stable"),
                     Arrays.asList("GROUP", "STATE"),
-                    mkSet(
+                    Set.of(
                             Arrays.asList(protocolGroup, "Stable")
                     )
             );
@@ -397,7 +389,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--type", "Classic"),
                     Arrays.asList("GROUP", "TYPE"),
-                    mkSet(
+                    Set.of(
                             Arrays.asList(protocolGroup, "Classic"),
                             Arrays.asList(topicPartitionsGroup, "Classic")
                     )
@@ -407,7 +399,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--type", "classic"),
                     Arrays.asList("GROUP", "TYPE"),
-                    mkSet(
+                    Set.of(
                             Arrays.asList(protocolGroup, "Classic"),
                             Arrays.asList(topicPartitionsGroup, "Classic")
                     )
@@ -415,7 +407,7 @@ public class ListConsumerGroupTest {
         }
     }
 
-    @ClusterTemplate("consumerProtocolOnlyGenerator")
+    @ClusterTemplate("defaultGenerator")
     public void testListGroupCommandConsumerProtocol() throws Exception {
         GroupProtocol groupProtocol = GroupProtocol.CONSUMER;
         String topic = TOPIC_PREFIX + groupProtocol.name;
@@ -429,7 +421,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list"),
                     Collections.emptyList(),
-                    mkSet(
+                    Set.of(
                             Collections.singletonList(protocolGroup),
                             Collections.singletonList(topicPartitionsGroup)
                     )
@@ -438,7 +430,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--state"),
                     Arrays.asList("GROUP", "STATE"),
-                    mkSet(
+                    Set.of(
                             Arrays.asList(protocolGroup, "Stable"),
                             Arrays.asList(topicPartitionsGroup, "Empty")
                     )
@@ -447,7 +439,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--type"),
                     Arrays.asList("GROUP", "TYPE"),
-                    mkSet(
+                    Set.of(
                             Arrays.asList(protocolGroup, "Consumer"),
                             Arrays.asList(topicPartitionsGroup, "Classic")
                     )
@@ -456,7 +448,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--type", "--state"),
                     Arrays.asList("GROUP", "TYPE", "STATE"),
-                    mkSet(
+                    Set.of(
                             Arrays.asList(protocolGroup, "Consumer", "Stable"),
                             Arrays.asList(topicPartitionsGroup, "Classic", "Empty")
                     )
@@ -465,7 +457,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--type", "consumer"),
                     Arrays.asList("GROUP", "TYPE"),
-                    mkSet(
+                    Set.of(
                             Arrays.asList(protocolGroup, "Consumer")
                     )
             );
@@ -473,7 +465,7 @@ public class ListConsumerGroupTest {
             validateListOutput(
                     Arrays.asList("--bootstrap-server", clusterInstance.bootstrapServers(), "--list", "--type", "consumer", "--state", "Stable"),
                     Arrays.asList("GROUP", "TYPE", "STATE"),
-                    mkSet(
+                    Set.of(
                             Arrays.asList(protocolGroup, "Consumer", "Stable")
                     )
             );
@@ -516,7 +508,9 @@ public class ListConsumerGroupTest {
         configs.put(KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getName());
         configs.put(VALUE_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class.getName());
         configs.put(GROUP_PROTOCOL_CONFIG, groupProtocol);
-        configs.put(PARTITION_ASSIGNMENT_STRATEGY_CONFIG, RangeAssignor.class.getName());
+        if (GroupProtocol.CLASSIC.name.equalsIgnoreCase(groupProtocol)) {
+            configs.put(PARTITION_ASSIGNMENT_STRATEGY_CONFIG, RangeAssignor.class.getName());
+        }
 
         configs.putAll(customConfigs);
         return configs;
@@ -553,20 +547,20 @@ public class ListConsumerGroupTest {
     /**
      * Validates the consumer group listings returned against expected values using specified filters.
      *
-     * @param service           The service to list consumer groups.
-     * @param typeFilterSet     Filters for group types, empty for no filter.
-     * @param stateFilterSet    Filters for group states, empty for no filter.
-     * @param expectedListing   Expected consumer group listings.
+     * @param service                The service to list consumer groups.
+     * @param typeFilterSet          Filters for group types, empty for no filter.
+     * @param groupStateFilterSet    Filters for group states, empty for no filter.
+     * @param expectedListing        Expected consumer group listings.
      */
     private static void assertGroupListing(
         ConsumerGroupCommand.ConsumerGroupService service,
         Set<GroupType> typeFilterSet,
-        Set<ConsumerGroupState> stateFilterSet,
+        Set<GroupState> groupStateFilterSet,
         Set<ConsumerGroupListing> expectedListing
     ) throws Exception {
         final AtomicReference<Set<ConsumerGroupListing>> foundListing = new AtomicReference<>();
         TestUtils.waitForCondition(() -> {
-            foundListing.set(set(service.listConsumerGroupsWithFilters(set(typeFilterSet), set(stateFilterSet))));
+            foundListing.set(set(service.listConsumerGroupsWithFilters(set(typeFilterSet), set(groupStateFilterSet))));
             return Objects.equals(set(expectedListing), foundListing.get());
         }, () -> "Expected to show groups " + expectedListing + ", but found " + foundListing.get() + ".");
     }
@@ -619,29 +613,29 @@ public class ListConsumerGroupTest {
 class ListConsumerGroupUnitTest {
     @Test
     public void testConsumerGroupStatesFromString() {
-        Set<ConsumerGroupState> result = ConsumerGroupCommand.consumerGroupStatesFromString("Stable");
-        Assertions.assertEquals(ListConsumerGroupTest.set(Collections.singleton(ConsumerGroupState.STABLE)), result);
+        Set<GroupState> result = ConsumerGroupCommand.groupStatesFromString("Stable");
+        Assertions.assertEquals(ListConsumerGroupTest.set(Collections.singleton(GroupState.STABLE)), result);
 
-        result = ConsumerGroupCommand.consumerGroupStatesFromString("Stable, PreparingRebalance");
-        Assertions.assertEquals(ListConsumerGroupTest.set(Arrays.asList(ConsumerGroupState.STABLE, ConsumerGroupState.PREPARING_REBALANCE)), result);
+        result = ConsumerGroupCommand.groupStatesFromString("Stable, PreparingRebalance");
+        Assertions.assertEquals(ListConsumerGroupTest.set(Arrays.asList(GroupState.STABLE, GroupState.PREPARING_REBALANCE)), result);
 
-        result = ConsumerGroupCommand.consumerGroupStatesFromString("Dead,CompletingRebalance,");
-        Assertions.assertEquals(ListConsumerGroupTest.set(Arrays.asList(ConsumerGroupState.DEAD, ConsumerGroupState.COMPLETING_REBALANCE)), result);
+        result = ConsumerGroupCommand.groupStatesFromString("Dead,CompletingRebalance,");
+        Assertions.assertEquals(ListConsumerGroupTest.set(Arrays.asList(GroupState.DEAD, GroupState.COMPLETING_REBALANCE)), result);
 
-        result = ConsumerGroupCommand.consumerGroupStatesFromString("stable");
-        Assertions.assertEquals(ListConsumerGroupTest.set(Collections.singletonList(ConsumerGroupState.STABLE)), result);
+        result = ConsumerGroupCommand.groupStatesFromString("stable");
+        Assertions.assertEquals(ListConsumerGroupTest.set(Collections.singletonList(GroupState.STABLE)), result);
 
-        result = ConsumerGroupCommand.consumerGroupStatesFromString("stable, assigning");
-        Assertions.assertEquals(ListConsumerGroupTest.set(Arrays.asList(ConsumerGroupState.STABLE, ConsumerGroupState.ASSIGNING)), result);
+        result = ConsumerGroupCommand.groupStatesFromString("stable, assigning");
+        Assertions.assertEquals(ListConsumerGroupTest.set(Arrays.asList(GroupState.STABLE, GroupState.ASSIGNING)), result);
 
-        result = ConsumerGroupCommand.consumerGroupStatesFromString("dead,reconciling,");
-        Assertions.assertEquals(ListConsumerGroupTest.set(Arrays.asList(ConsumerGroupState.DEAD, ConsumerGroupState.RECONCILING)), result);
+        result = ConsumerGroupCommand.groupStatesFromString("dead,reconciling,");
+        Assertions.assertEquals(ListConsumerGroupTest.set(Arrays.asList(GroupState.DEAD, GroupState.RECONCILING)), result);
 
-        Assertions.assertThrows(IllegalArgumentException.class, () -> ConsumerGroupCommand.consumerGroupStatesFromString("bad, wrong"));
+        Assertions.assertThrows(IllegalArgumentException.class, () -> ConsumerGroupCommand.groupStatesFromString("bad, wrong"));
 
-        Assertions.assertThrows(IllegalArgumentException.class, () -> ConsumerGroupCommand.consumerGroupStatesFromString("  bad, Stable"));
+        Assertions.assertThrows(IllegalArgumentException.class, () -> ConsumerGroupCommand.groupStatesFromString("  bad, Stable"));
 
-        Assertions.assertThrows(IllegalArgumentException.class, () -> ConsumerGroupCommand.consumerGroupStatesFromString("   ,   ,"));
+        Assertions.assertThrows(IllegalArgumentException.class, () -> ConsumerGroupCommand.groupStatesFromString("   ,   ,"));
     }
 
     @Test
