@@ -17,11 +17,6 @@
 
 package org.apache.kafka.trogdor.coordinator;
 
-import com.fasterxml.jackson.core.JsonProcessingException;
-import com.fasterxml.jackson.databind.JsonNode;
-import com.fasterxml.jackson.databind.node.JsonNodeFactory;
-import com.fasterxml.jackson.databind.node.LongNode;
-import com.fasterxml.jackson.databind.node.ObjectNode;
 import org.apache.kafka.common.KafkaException;
 import org.apache.kafka.common.errors.InvalidRequestException;
 import org.apache.kafka.common.utils.Scheduler;
@@ -46,6 +41,13 @@ import org.apache.kafka.trogdor.rest.WorkerReceiving;
 import org.apache.kafka.trogdor.rest.WorkerState;
 import org.apache.kafka.trogdor.task.TaskController;
 import org.apache.kafka.trogdor.task.TaskSpec;
+
+import com.fasterxml.jackson.core.JsonProcessingException;
+import com.fasterxml.jackson.databind.JsonNode;
+import com.fasterxml.jackson.databind.node.JsonNodeFactory;
+import com.fasterxml.jackson.databind.node.LongNode;
+import com.fasterxml.jackson.databind.node.ObjectNode;
+
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -149,23 +151,23 @@ public final class TaskManager {
         /**
          * The task id.
          */
-        final private String id;
+        private final String id;
 
         /**
          * The original task specification as submitted when the task was created.
          */
-        final private TaskSpec originalSpec;
+        private final TaskSpec originalSpec;
 
         /**
          * The effective task specification.
          * The start time will be adjusted to reflect the time when the task was submitted.
          */
-        final private TaskSpec spec;
+        private final TaskSpec spec;
 
         /**
          * The task controller.
          */
-        final private TaskController controller;
+        private final TaskController controller;
 
         /**
          * The task state.
